//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: AboutBox.c
//---------------------------------------------------------------------------
// Purpose: dialog box with program information
//---------------------------------------------------------------------------

#include <qdialog.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtextedit.h>


#include <stdio.h>

#include "AboutBox.h"
#include "IconLibrary.h"
#include "ApplicationFrame.h"
#include "ApplicationStorage.h"
#include "QtUtilities.h"

//-----------------------------------------------------------------------------
                                                 
AboutBox::AboutBox(QWidget *parent) : 
                          QDialog(parent,"About",TRUE)
{
   PixelBuffer *PericoBuffer;
   QPushButton *ok;
   QGridLayout *grid;
   QBoxLayout  *topLayout;
   QTextEdit   *w_TextEdit;
   QPalette    pal;
   QPixmap     *Perico;
   QLabel      *m_PericoImageLabel;
   QColor      aColor;
   char        versionString[20];
   char        buildString[30];

   // load perico image to get its background pixel
   PericoBuffer = g_Application->m_pc_IconLibrary->GetIcon("Perico");
   if(PericoBuffer != NULL)
   {
      // get background
      aColor = PericoBuffer->Get(0,0);
      // buffer is not needed anymore
      delete PericoBuffer;
      pal = this->palette();
      // set background of widget to the image background
      pal.setColor( QColorGroup::Background, aColor );
      pal.setColor( QColorGroup::Button, aColor );
   
      setPalette( pal );
   }
   
   setCaption(tr("About Perico"));

   Perico = g_Application->m_pc_IconLibrary->GetPixmap("Perico");

   topLayout = new QVBoxLayout( this, 2 );

   // add title in the top
   topLayout->addWidget(new QLabel(" ",this));
   topLayout->addWidget(new QLabel("Perico",this),0,AlignHCenter);
   topLayout->addWidget(new QLabel(" ",this));

   // make the image of perico
   m_PericoImageLabel = new QLabel(" ",this);
   m_PericoImageLabel->setPixmap(*Perico);

   topLayout->addWidget(m_PericoImageLabel,0,AlignHCenter);
   topLayout->addWidget(new QLabel(" ",this));

   // make information about program
   sprintf(versionString,"%10s",PERICO_VERSION);
   sprintf(buildString  ,"%s",   __DATE__ );

   grid = new QGridLayout( topLayout, 4, 2,12);
   grid->addWidget(new QLabel(tr("Version"),this),0,0);
   grid->addWidget(new QLabel(tr("Build on"),this),1,0);
   grid->addWidget(new QLabel(tr("Author"),this),2,0);
   grid->addWidget(new QLabel(tr("Licence"),this),3,0);
   grid->addWidget(new QLabel(versionString,this),0,1);
   grid->addWidget(new QLabel(buildString,this),1,1);
   grid->addWidget(new QLabel("Peter Jodda / peter AT jodda.de",this),2,1);
   grid->addWidget(new QLabel("GPL",this),3,1);

   w_TextEdit =new QTextEdit(this);
   w_TextEdit->setMinimumWidth(300);   
   w_TextEdit->setMinimumHeight(g_Application->m_pc_QtUtility->GetMaximalFontHeight()*10);
   topLayout->addWidget(w_TextEdit);
   w_TextEdit->setReadOnly(true);
   QtUtility::SetWidgetReadOnlyColor(w_TextEdit);

   w_TextEdit->append("If you like this program send a postcard to\n");
   w_TextEdit->append("Peter Jodda, Hugoweg 25,\n29221 Celle, Germany\n\n");

   w_TextEdit->append("Special thanks to: \n");
   w_TextEdit->append("Dries Pruimboom for the comic algorithm.\n");
   w_TextEdit->append("Ewald Kr�er for the Xenomorph filters.\n");
   

   // --- the ok button
   ok = new QPushButton( tr("OK"), this );
   connect( ok, 
            SIGNAL(clicked()), 
            SLOT(accept()) );

   topLayout->addWidget(ok,20,AlignCenter);
   delete Perico;

} // Aboutbox constructor

