//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: Autosave.c
//---------------------------------------------------------------------------
// Purpose: Automatic creation of safety-copies
//---------------------------------------------------------------------------

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>


#include "Autosave.h"
#include "MainFrame.h"
#include "ApplicationStorage.h"
#include "BaseData.h"
#include "BaseDataStrings.h"
#include "FileLoader.h"

//===========================================================================
//
//===========================================================================


Autosave::Autosave(MainFrame *pc_MainFrame)
{
   bool b_Success;
   int  i;

   m_pc_MainFrame =pc_MainFrame;
   
   // allocate filename history buffer
   m_i_IntervalMinutes      = 5;
   m_i_AutosaveHistoryCount = 2;
   m_b_Enabled              = true;
   // get the autosave enabled
   b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_ENABLED,
                                  m_b_Enabled);
   // get the autosave interval
   b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_INTERVAL,
                                  m_i_IntervalMinutes);
   // get the autosave number of images
   b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_NUMBER_OF_IMAGES,
                                  m_i_AutosaveHistoryCount);

   // get the global temp directory path
   m_qs_Path = g_Application->m_qs_TempDirectory;
   
   m_qs_Path +="/Autosave";

   // create it, in case of non existent
   FileLoader::MakeDirectory((const char *) m_qs_Path);
      
   m_qsa_AutosaveHistory = new QString[m_i_AutosaveHistoryCount];

   // set strings empty
   for(i=0;i<m_i_AutosaveHistoryCount;i++)
   {
      m_qsa_AutosaveHistory[i]="";
   }

   
   // init with current time
   m_tt_LastSave = time(NULL);

   // name prefix is PID and the hex mainframe ptr.
   m_qs_NamePrefix.sprintf("%08x%08x",
                           (int)getpid(),
                           (int)pc_MainFrame
                           );
   
} // Autosave

//===========================================================================
//
//===========================================================================

Autosave::~Autosave()
{
   // at a regulare end delete the autosave history.
   // only keep it at abnormal termination
   int i;

   
   // set strings empty
   for(i=0;i<m_i_AutosaveHistoryCount;i++)
   {
      // now ad the name into the save history
      if( !m_qsa_AutosaveHistory[i].isEmpty())
      {
         // delete the oldest one
         unlink(m_qsa_AutosaveHistory[i]);
         m_qsa_AutosaveHistory[i]="";
      }
   // move all one up
   }

   
} // ~Autosave

//===========================================================================
//
//===========================================================================

void Autosave::TimerEvent()
{
   time_t tt_CurrentTime;

   if(!m_b_Enabled)
   {
      return;
   }

   
   tt_CurrentTime = time(NULL);
   if(  (tt_CurrentTime - m_tt_LastSave) < (m_i_IntervalMinutes*60)   )
   {
      // time between last too short.
      return;
   }


   // here do the autosave
   DoSave();

   // remember this time   
   m_tt_LastSave = time(NULL);
   
} // TimerEvent()

//===========================================================================
//
//===========================================================================

void Autosave::SetPath(QString qs_Path)
{
   m_qs_Path = qs_Path;

} // SetPath()


//===========================================================================
//
//===========================================================================

void Autosave::DoSave()
{
   QString qs_Filename;
   QString qs_FullPath;
   time_t  tt_CurrentTime;
   struct tm tm_Time;
   int i;

   tt_CurrentTime = time(NULL);

   memcpy(&tm_Time,localtime(&tt_CurrentTime),sizeof(struct tm));

   qs_Filename.sprintf("%s-%04d%02d%02d_%02d%02d%02d.perico",
            (const char *) m_qs_NamePrefix,
            tm_Time.tm_year+1900,
            tm_Time.tm_mon+1,
            tm_Time.tm_mday,
            tm_Time.tm_hour,
            tm_Time.tm_min,
            tm_Time.tm_sec);
   // fprintf(stderr,"Filename is %s\n",(const char *) qs_Filename);

   qs_FullPath = m_qs_Path+"/"+qs_Filename;
   m_pc_MainFrame->SaveFile(qs_FullPath);            
   // fprintf(stderr,"Path is %s\n",(const char *) qs_FullPath);

   
   // now ad the name into the save history
   if( !m_qsa_AutosaveHistory[m_i_AutosaveHistoryCount-1].isEmpty())
   {
      // delete the oldest one
      unlink(m_qsa_AutosaveHistory[m_i_AutosaveHistoryCount-1]);
      m_qsa_AutosaveHistory[m_i_AutosaveHistoryCount-1]="";
   }
   // move all one up
   
   for(i= (m_i_AutosaveHistoryCount-1) ;
       i>0;
       i--)
   {
      // reverse loop - copy element
      m_qsa_AutosaveHistory[i] = m_qsa_AutosaveHistory[i-1];
      
   }
   // store the latest at the first position
   m_qsa_AutosaveHistory[i] = qs_FullPath;

} // DoSave()

//===========================================================================
//
//===========================================================================

void Autosave::SetEnabled(bool b_Enabled)
{
   m_b_Enabled = b_Enabled;
} // SetEnabled()

//===========================================================================
//
//===========================================================================



