//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: AutosaveInfoDialog.c
//---------------------------------------------------------------------------
// Purpose: Infobox to show availiability of autosave images
//---------------------------------------------------------------------------
#include <qmainwindow.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include "AutosaveInfoDialog.h"
#include "QtUtilities.h"

#include "ApplicationStorage.h"
#include "ApplicationFrame.h"
#include "AutosaveBox.h"
#include "IconLibrary.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

AutosaveInfoDialog::AutosaveInfoDialog( QWidget *parent)
            : QDialog(parent,"AutosaveInfoDialog",false)

{
   QVBoxLayout * l_TopLayout;
   QPushButton * w_CloseButton;
   QPushButton * w_ShowMoreButton;
   QColorGroup   qcg_ColorGroup;
   QBoxLayout  * l_DialogButtons;
   QBoxLayout  * l_InfoLayout;
   QPixmap     * qp_InfoPixmap;
   QLabel      * w_TextLabel;
   QLabel      * w_InfoLabel;
   QColor        qc_Background;
   Pixel         p_Background;

   
   qcg_ColorGroup = colorGroup();
   
   qc_Background = qcg_ColorGroup.background();
   p_Background = PIXEL_MAKE(qc_Background.red(),
                             qc_Background.green(),
                             qc_Background.blue());

   
   qp_InfoPixmap = g_Application->m_pc_IconLibrary->GetPixmap("Info",
                                            p_Background);

   // create the box
   l_TopLayout = new QVBoxLayout( this,1);

   setCaption(tr("Perico:Automatic saved images found"));

   l_TopLayout->addWidget( new QLabel("",this));

   l_InfoLayout = new QHBoxLayout( l_TopLayout,5);

   w_InfoLabel = new QLabel("",this);
   w_InfoLabel->setPixmap(*qp_InfoPixmap);
   w_InfoLabel->setMargin(10);
   
   w_TextLabel = new QLabel(tr("Automatic saved images are found"),this);
   w_TextLabel->setMargin(10);

   l_InfoLayout->addWidget( w_InfoLabel);
   l_InfoLayout->addWidget( w_TextLabel);  
   l_TopLayout->addWidget( new QLabel("",this));


   // --- the close button
   l_DialogButtons = new QHBoxLayout( l_TopLayout,5);

   w_ShowMoreButton = new QPushButton( tr("Show the Images"), this );
   w_CloseButton    = new QPushButton( tr("Close"), this );
   connect( w_ShowMoreButton,
            SIGNAL(clicked()),
            SLOT( ShowMorePressed()) );
   connect( w_CloseButton,
            SIGNAL(clicked()),
            SLOT( ClosePressed()) );

   l_DialogButtons->addWidget(new QtUtilSpacer(true,false,this) );
   l_DialogButtons->addWidget(w_ShowMoreButton);
   l_DialogButtons->addWidget(w_CloseButton);

   delete qp_InfoPixmap;
} // AutosaveInfoDialog()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

AutosaveInfoDialog::~AutosaveInfoDialog()
{
} // ~AutosaveInfoDialog()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void AutosaveInfoDialog::ClosePressed()
{

   this->hide();
} // ClosePressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void AutosaveInfoDialog::ShowMorePressed()
{
   // open the autosave selection box
   g_Application->m_pc_ApplicationFrame->m_pc_AutosaveBox->show();

   this->hide();
} // ShowMorePressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

