//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: BaseData.c
//---------------------------------------------------------------------------
// Purpose: load, save, and store of preferences
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include "BaseData.h"

//============================================================================
//
//============================================================================


BaseData::BaseData()
{
   m_qs_Filename = "";
   m_b_Loaded    = false;
   m_b_Modified  = false;

} // BaseData()

//============================================================================
//
//============================================================================

BaseData::BaseData(QString qs_Filename)
{
   m_qs_Filename = "";
   m_b_Loaded    = false;
   m_b_Modified  = false;

   LoadFile(qs_Filename);

} // BaseData()

//============================================================================
//
//============================================================================

bool BaseData::LoadFile(QString qs_Filename)
{
   unsigned char   ReadBuffer[500];
   FILE          * fp_Infile;

   // remove old data
   m_List.clear();

   m_b_Modified  = false;

   // save the filename
   m_qs_Filename = qs_Filename;


   // try to open the file
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile ==NULL)
   {
      return false;
   }

   while(true)
   {
      unsigned char * uc_FgetsResult;
      QString         qs_Line;

      uc_FgetsResult = (unsigned char *) fgets((char *)ReadBuffer,
                                               499,
                                               fp_Infile);
      if(uc_FgetsResult == NULL)
      {
         break;
      }

      // remove the \t \r \n characters
      while(*uc_FgetsResult != '\0')
      {
         if(*uc_FgetsResult <' ')
         {
            *uc_FgetsResult = '\0';
            break;
         }
         uc_FgetsResult++;
      }

      // make a qstring out of it
      qs_Line.sprintf("%s",ReadBuffer);
      if(qs_Line.isEmpty())
      {
         break;
      }

      // now add to the string list
      m_List.push_back(qs_Line);

   }  // while true

   // close the input file
   fclose(fp_Infile);


            

   // loaded
   return true;
} // LoadFile()

//============================================================================
//
//============================================================================

bool BaseData::SaveFile()
{
   QString   qs_Line;
   FILE    * fp_Outfile;
   int       i;
   int       i_NumberOfLines;

   // filename available ??
   if(m_qs_Filename.isEmpty() )
   {
      return false;
   }

   // if not modified
   if(!m_b_Modified)
   {
      // then nothing has to be stored !
      return true;
   }


   fp_Outfile = fopen(m_qs_Filename,"w");
   if(fp_Outfile ==NULL)
   {
      return false;
   }

   // write all lines
   i_NumberOfLines = m_List.count();
   for(i=0;i<i_NumberOfLines;i++)
   {
      qs_Line = m_List[i];
      fprintf(fp_Outfile,"%s\n",(const char *) qs_Line);
   }

   fclose(fp_Outfile);

   return true;

} // SaveFile

//============================================================================
//  in case of failure the original value is not overwritten !
//============================================================================

int BaseData::LocateLine(QString qs_Identifier,
               QString &qs_StoredValue)
{
   QString   qs_Line;
   QString   qs_Name;
   QString   qs_Value;
   int       i;
   int       i_NumberOfLines;
   int       i_FindIndex;

   // write all lines
   i_NumberOfLines = m_List.count();
   for(i=0;i<i_NumberOfLines;i++)
   {
      qs_Line = m_List[i];
         // look for an = sign
      i_FindIndex = qs_Line.find('=');
      if(i_FindIndex <1)
      {
         continue;
      }
      qs_Name  = qs_Line.left(i_FindIndex);
      qs_Value = qs_Line.mid (i_FindIndex+1);

      if(qs_Name.compare(qs_Identifier) ==0)
      {
         // found
         qs_StoredValue = qs_Value;
         // return line number
         return i;
      }
   } // for i

   // nothing found
   return -1;
} // LocateLine()

//============================================================================
//  in case of failure the original value is not overwritten !
//============================================================================

bool BaseData::GetValue(QString qs_Identifier,
                  QString &qs_StoredValue)
{
   int i_LineNumber;

   i_LineNumber = LocateLine(qs_Identifier,qs_StoredValue);

   if(i_LineNumber <0)
   {
      return false;
   }
   return true;
} // GetValue()

//============================================================================
//  in case of failure the original value is not overwritten !
//============================================================================

bool BaseData::GetValue(QString qs_Identifier,
                  int &i_StoredValue)
{
   QString qs_Value;
   bool    b_Return;

   b_Return = GetValue(qs_Identifier,qs_Value);
   if(b_Return)
   {
      i_StoredValue = atoi((const char *)qs_Value);
   }

   return b_Return;

} // GetValue()

//============================================================================
//  in case of failure the original value is not overwritten !
//============================================================================

bool BaseData::GetValue(QString qs_Identifier,
                  double &d_StoredValue)
{
   QString qs_Value;
   bool    b_Return;

   b_Return = GetValue(qs_Identifier,qs_Value);
   if(b_Return)
   {
      d_StoredValue = atof((const char *)qs_Value);
   }

   return b_Return;

} // GetValue()

//============================================================================
//  in case of failure the original value is not overwritten !
//============================================================================


bool BaseData::GetValue(QString qs_Identifier,
                  bool &b_StoredValue)
{
   QString qs_Value;
   bool    b_Return;

   b_Return = GetValue(qs_Identifier,qs_Value);
   if(b_Return)
   {
      b_StoredValue = (atoi((const char *)qs_Value))
                      ? true : false;
   }

   return b_Return;

} // GetValue()

//============================================================================
//
//============================================================================

bool BaseData::SetValue(QString qs_Identifier,
                  QString qs_Value)
{
   QString qs_Pair;
   QString qs_StoredValue;
   int     i_LineNumber;

   qs_Pair = qs_Identifier + "=" + qs_Value;
   
   i_LineNumber = LocateLine(qs_Identifier,qs_StoredValue);

   
   if(i_LineNumber <0)
   {
      // create a new
      m_List.push_back(qs_Pair);
   }
   else
   {
      QStringList::iterator it_List;

      if(qs_Value.compare(qs_StoredValue) ==0)
      {
         // it is the same value
         return true;
      }
      
      it_List = m_List.at(i_LineNumber);
      // replace the current
      m_List.remove(it_List);
      m_List.push_back(qs_Pair);
   }

   m_b_Modified = true;

   return true;
} // SetValue()

//============================================================================
//
//============================================================================

bool BaseData::SetValue(QString qs_Identifier,
                  int i_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%d",i_Value);

   return SetValue(qs_Identifier,(QString) qs_Value);
   
} // SetValue()

//============================================================================
//
//============================================================================

bool BaseData::SetValue(QString qs_Identifier,
                  double d_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%lf",d_Value);

   return SetValue(qs_Identifier,(QString) qs_Value);
 
} // SetValue()

//============================================================================
//
//============================================================================


bool BaseData::SetValue(QString qs_Identifier,
                  bool b_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%d",(b_Value? 1 : 0));

   return SetValue(qs_Identifier,(QString) qs_Value);
 
} // SetValue()

//============================================================================
//
//============================================================================

QString BaseData::CalculateFilename(
               QString qs_ApplicationName)
{
   QString  qs_Name;
   QString  qs_Prefix;
   char     *cp_EnvReturn;
   
   cp_EnvReturn =getenv("HOME");

   qs_Name ="."+qs_ApplicationName+".rc";

   if(cp_EnvReturn ==NULL)
   {
      return qs_Name;
   }
   qs_Prefix.sprintf("%s/%s",
               cp_EnvReturn,
               (const char *) qs_Name);      

   return qs_Prefix;

} // BasicDataName()

//============================================================================
//
//============================================================================





