#ifndef _BLEND_H
#define _BLEND_H

#include "PixelBuffer.h"
#include "RectRegion.h"
#include "PixelBuffer.h"
#include "D3Utilities.h"
#include "Tracer.h"

class BlendPoint
{
public:
   BlendPoint();
   int   m_i_X;
   int   m_i_Y;
   Pixel m_p_Color;
   bool  m_b_IsBorderPoint;
};

class BlendLine 
{
public:
   BlendLine(BlendPoint *pc_Point1,BlendPoint *pc_Point2);

   bool CutsLine(BlendLine *pc_SecondLine);
   
   BlendPoint *m_pc_Point1;
   BlendPoint *m_pc_Point2;
   double d_Length;
}; // class BlendLine

class BlendTriangle
{
public:
   BlendTriangle( BlendLine *pc_Line1,
                  BlendLine *pc_Line2,
                  BlendLine *pc_Line3,
                  BlendPoint *pc_Point1,
                  BlendPoint *pc_Point2,
                  BlendPoint *pc_Point3);
   BlendLine  *m_pc_Line[3];
   BlendPoint *m_pc_Point[3];

   bool CoordinateInTriangle(float f_X, float f_Y);
   void printf();
}; // class BlendTriangle


class BlendSplinePoint
{
   // coordinate
   double d_X;
   double d_Y;

   double d_IncX;
   double d_IncY;

   Pixel p_Color;
   
};; // class BlendSplinePoint


class Blend : public Tracer
{
public:
   Blend();
   ~Blend();

   void SetPixelBuffer(PixelBuffer *pc_Buffer);
   void SetBlendPointList(   BlendPoint *pc_BlendPointList, int i_Count);
   void SetOnlyInSelectionMask(bool b_OnlyInMask);
   void SetTransparency( bool b_On, int i_Factor);
   void SetSmooth(bool b_On);
   
   void DoBlend();

   RectRegion GetChangedRegion();
   
private: // functions
   void CalculateBorderPoints();
   void DoBlend2Points();
   void DoBlend3Points();

   Pixel CalculateAveragePoint(int i_X, int i_Y);
   Pixel CalculateOuterPoint(int i_X, int i_Y);

   Pixel CalculateTrianglePoint(BlendPoint c_PointA,
            BlendPoint c_PointB,
            BlendPoint c_PointC,
            int        i_X,
            int        i_Y);
   Pixel CalculateTrianglePoint(BlendTriangle *c_Triangle,
            int        i_X,
            int        i_Y);
            

   void CreateLineList();
   void DeleteLineList();
   void CreateTriangleList();
   void DeleteTriangleList();
   void RemoveSuperTriangles();
   
   Pixel GetSamplePoint(int i_X, int i_Y);
   void FillD3Matrix(D3Matrix *pc_Matrix);
   void ApplySplineResults(D3Matrix *pc_Result);

   void SetInResult(int i_X, int i_Y, Pixel i_NewValue);
   
private: // variable
   PixelBuffer *m_pc_Destination;

   BlendPoint *m_pc_BlendPointList;
   int         m_i_BlendPointListCount;
   bool        m_b_BlendOnlyInSelectionMask;

   bool        m_b_TransparencyOn;
   int         m_i_TransparencyFactor;

   RectRegion  m_c_ChangedRegion ;

   QPtrList<BlendPoint>    m_BorderList;
   PixelBuffer             *m_pc_InnerMask;

   double *m_d_FaktorList;

   QPtrList<BlendLine>      m_LineList;
   QPtrList<BlendTriangle>  m_TriangleList;

   int m_i_SampleDistance;
   bool m_b_Smooth;

   
}; // class Blend




#endif //  _BLEND_H
