#ifndef _BLENDBOX_H
#define _BLENDBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qhgroupbox.h>
#include <qvgroupbox.h>


#include "StandardDialog.h"
#include "Tracer.h"

#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"

class MainFrame;

class BlendBox :  public StandardDialog ,
                  public Tracer
{
Q_OBJECT

public:
   BlendBox(QWidget *parent=0,
            GlobalStorage *pc_Global=0,
            MainFrame *pc_MainFrame=NULL);
   ~BlendBox();

   void DoActivate();

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

protected:
   virtual void showEvent(QShowEvent *);
   virtual void hideEvent(QHideEvent *);

public slots:
   void ActiveModeToggled(bool b_Mode);
   void AddButtonPressed();
   void DeleteButtonPressed();
   void DeleteAllButtonPressed();
   void BlendButtonPressed();

   void ColorChanged(Pixel p_NewColor);

   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);

   void DoSmoothingToggled(bool b_Mode);
   
private:  // functions

   void SetSensitivity();

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);

   void *SelectionEventArrived(void *pc_Data);
   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);


private:  // variables

   bool m_b_ObjectModeActive;
   GraphicObjectLayer     *m_pc_ObjectLayer;

   QCheckBox     *m_ActivateCheck;

   QPushButton   *m_AddButton;
   QPushButton   *m_DeleteButton;
   QPushButton   *m_DeleteAllButton;
   QPushButton   *m_BlendButton;
   QHGroupBox    *m_RegionBox;
   QUColorDragLabeledWidget *m_w_ColorSelector;
   QCheckBox     *m_OnlyInSelectionMaskCheck;

   QCheckBox     *m_DoSmoothingCheck;

   QVGroupBox        *m_TransparencyBox;
   QCheckBox         *m_TransparencyEnable;
   QULabelSlider     *m_TransparencySlider;
   
   bool           m_b_InRedraw;
   bool           m_b_TransparencyEnabled;
   bool           m_b_DoSmoothing;

   int            m_i_TransparencyFactor;

};


#endif // _BLENDBOX_H
