#ifndef _BUFFERPAINTER_H
#define _BUFFERPAINTER_H

#include "PixelBuffer.h"
#include "RectRegion.h"
#include "BufferPainterEdges.h"
#include "BrushMode.h"
#include "PainterBrush.h"
#include "GraphicObjectLayer.h"
#include "PList.h"

//==================================================================

class BufferPainter
{
public:
   BufferPainter();
   ~BufferPainter();

   void DrawPoint(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   void DrawLine(PixelBuffer *pc_Buffer,int i_X1, int i_Y1,int i_X2, int i_Y2);
   void DrawRectangle(PixelBuffer *pc_Buffer,int i_X1, int i_Y1,int i_X2, int i_Y2);

   void DrawEllipse(PixelBuffer *pc_Buffer,int i_CenterX, int i_CenterY,
                                           int i_RadiusX, int i_Radiusy);

                                           
   void DrawBrush(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   void DrawAirBrush(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   
   void DrawSpeckle(PixelBuffer *pc_Buffer, int i_X, int i_Y);

   void DrawCopyBrush(PixelBuffer *pc_Buffer, int i_X, int i_Y);
   void DrawMarkedPixel(PixelBuffer *pc_Buffer,
                        PixelBuffer *pc_MarkedBuffer,
                        RectRegion  *pc_ReducedRegion);

   void DrawSmear(PixelBuffer *pc_Buffer,
                  int i_X,
                  int i_Y,
                  int i_LastX,
                  int i_LastY);

   RectRegion GetChangedRegion();

   //--------- set functions for properties

   void SetLineWidth(int i_LineWidth);
   void SetDrawInnerEnable(bool b_On);
   void SetTransparencyEnable(bool b_on);
   void SetTransparencyFactor(int i_Factor);

   void SetInnerColor(Pixel c_Color);
   void SetBorderColor(Pixel c_Color);
   void SetInnerPattern(PixelBuffer * pc_Buffer);
   void SetBorderPattern(PixelBuffer * pc_Buffer);
   void SetInnerDrawMode(BrushMode i_BrushMode);
   void SetBorderDrawMode(BrushMode i_BrushMode);
   void SetInnerMotiveLayer(GraphicObjectLayer *pc_MotivLayer);
   void SetBorderMotiveLayer(GraphicObjectLayer *pc_MotivLayer);

   void SetBrushShape(PixelBuffer *pc_BrushShape);
   void SetOneTimeDraw(bool b_On);

   void DrawEllipseSimple(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 float i_CenterX, float i_CenterY,
                 float i_RadiusX, float i_Radiusy);

   void DrawFreeFigure(PixelBuffer *pc_Buffer,
                       PMemVector<PixelCoordinate> *pc_List,
                       bool b_ClosedFigure = true);
   void DrawSpline(PixelBuffer *pc_Buffer,
                       GeoCoordinateList *pc_List,
                       bool b_ClosedFigure = true);

   void SetMaskMode(bool b_EnableMode,bool b_White);
   void SetLightnessMode(bool b_EnableMode);
   void SetLightnessFactor(int i_Factor);
   void SetSaturationMode(bool b_EnableMode);
   
   void SetSpeckleParameter(int i_SpeckleSize, int i_SpeckleDensity);
   void SetSmearParameter(int i_SmearStrength);
   void SetCopyBrushParameter(int i_DifferenceX,int i_DifferenceY);
      
   enum 
   {
      FilterAlgorithmMiddle =0,
      FilterAlgorithmMedian,
      FilterAlgorithmSharp
   };
   void SetFilterMode(bool b_EnableMode);
   void SetFilterWidth(int i_Width);
   void SetFilterNearDistanceActive(bool b_Active);
   void SetFilterNearDistance(int i_Distance);
   void SetFilterAlgorithm(int i_Algorithm);
   void SetRecolorMode(bool b_EnableMode);
   void SetRecolorParameter(Pixel p_From, Pixel p_To,int i_Distance);
   void ResetBrushModes();

   PixelBuffer *GetBrushShape();

private:
   void FilterPixelMiddle(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y);
   void FilterPixelSharp(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y);
   void FilterPixelMedian(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y);
   void DrawPixel(PixelBuffer *pc_Buffer,
                                PainterBrush *pc_Brush,
                                int i_X,
                                int i_Y);
   void DrawPixel(PixelBuffer *pc_Buffer,
                                Pixel p_NewColor,
                                int i_X,
                                int i_Y);
   void DrawHorizontalLine(PixelBuffer *pc_Buffer,
                                PainterBrush *pc_Brush,
                                int i_StartX,
                                int i_EndX,
                                int i_Y);
   void DrawLineWithWidth1(PixelBuffer *pc_Buffer,
                        PainterBrush *pc_Brush,
                        int i_X1, int i_Y1,int i_X2, int i_Y2);


   void BaseDrawLine(PixelBuffer *pc_Buffer,
                PainterBrush *pc_Brush,
                int i_StartX, int i_StartY,
                int i_EndX, int i_EndY,
                int i_LineWidth,
                RectRegion &c_ChangedRegion);

   void DrawEllipseSimple(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 int i_CenterX, int i_CenterY,
                 int i_RadiusX, int i_Radiusy);

  void CalculateStartEndLine(PMemVector<PixelCoordinate> *pc_List,
                 int &i_StartLine,
                 int &i_EndLine);
  void CalculateStartEndLine(GeoCoordinateList *pc_List,
                 int &i_StartLine,
                 int &i_EndLine);
   void CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    PixelCoordinate c_From,
                    PixelCoordinate c_To,
                    int &LastDirection);
   void CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    GeoCoordinate c_From,
                    GeoCoordinate c_To,
                    int &LastDirection);

   int CalculateInitialLastDirection(PMemVector<PixelCoordinate> *pc_List);

   int CalculateInitialLastDirection(GeoCoordinateList *pc_List);

   void DrawEdgesField(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 BufferPainterEdges * pc_Edges,
                 RectRegion *pc_ChangedRegion=NULL);

   Pixel CalculateRecolor(Pixel p_Old);

private:
    PainterBrush m_BorderBrush;
    PainterBrush m_InnerBrush;

    PainterBrush m_BlackBrush;
    PainterBrush m_WhiteBrush;

    PainterBrush *m_pc_CurrentBorderBrush;
    PainterBrush *m_pc_CurrentInnerBrush;

    bool         m_b_DrawInnerPart;

    bool         m_b_DrawTransparent;
    int          m_i_TransparencyFactor;

    int          m_i_LineWidth;

    int          m_i_FilterWidth;
    bool         m_b_FilterNearDistanceActive;
    int          m_i_FilterNearDistance;
    int          m_i_FilterAlgorithm;
    int          m_i_CopyDifferenceX;
    int          m_i_CopyDifferenceY;

   RectRegion m_ChangedRegion;

   int m_i_SpeckleSize;
   int m_i_SpeckleDensity;

   int m_i_SmearStrength;
   PixelBuffer *m_pc_SmearCopy;

   Pixel m_p_RecolorFromColor;
   Pixel m_p_RecolorToColor;
   int   m_i_RecolorDistance;
   int   m_i_RecolorFromH;
   int   m_i_RecolorFromL;
   int   m_i_RecolorFromS;
   int   m_i_RecolorToH;
   int   m_i_RecolorToL;
   int   m_i_RecolorToS;
   

//   PixelBuffer *m_pc_OneTimeDrawBuffer;
   bool        m_b_UseOneTimeDrawBuffer;
   
   bool        m_b_MaskModeEnabled;

}; // class BufferPainter


#endif // _BUFFERPAINTER_H
