// module to calculate color representations into others

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "ColorCalculation.h"
#include "Utilities.h"


//===========================================================================
//
//===========================================================================

void ColorCalculation::RGBToHLS(int i_Red  , int i_Green  , int i_Blue,
                                int &i_Hue , int &i_Lightness , int &i_Saturation)
{

   double d_Maximum;
   double d_Minimum;
   double d_Red;
   double d_Green;
   double d_Blue;
   double d_Hue;
   double d_Lightness;
   double d_Saturation;


   // adjust to 0 .. 1
   d_Red   = (double)i_Red   / 255.0;
   d_Green = (double)i_Green / 255.0;
   d_Blue  = (double)i_Blue  / 255.0;

   // calculate the maximum
   d_Maximum = d_Red;
   if(d_Green>d_Maximum)
   {
      d_Maximum = d_Green;
   }
   if(d_Blue>d_Maximum)
   {
      d_Maximum = d_Blue;
   }

   d_Minimum = d_Red;
   if(d_Green<d_Minimum)
   {
      d_Minimum = d_Green;
   }
   if(d_Blue<d_Minimum)
   {
      d_Minimum = d_Blue;
   }

   d_Lightness = 50.0 * (d_Maximum+d_Minimum);

   if ( fabs(d_Minimum-d_Maximum) < 0.0001 )
   {
      d_Hue         = 0.0;
      d_Saturation  = 0.0;
   }
   else
   {
      double d_Summ;
      double d_Difference;
      double d_HueHelp;
      
      d_Summ       = d_Maximum + d_Minimum;
      d_Difference = d_Maximum - d_Minimum;

      if ( d_Summ <= 1.0 )
      {
         d_Saturation  = 100.0 * (d_Difference) / (d_Summ) ;
      }
      else
      {
         d_Saturation  = 100.0 * (d_Difference) / (2.0 - d_Maximum - d_Minimum) ;
      }

      d_Red   = (d_Red   - d_Minimum) / (d_Difference);
      d_Green = (d_Green - d_Minimum) / (d_Difference);
      d_Blue  = (d_Blue  - d_Minimum) / (d_Difference);


      if ( d_Red >= 0.999 ) // near to 1
      {
         d_HueHelp = d_Green - d_Blue;
      }
      else if ( d_Green >= 0.999 ) // near to 1
      {
         d_HueHelp = 2.0 + d_Blue - d_Red;
      }
      else
      {
         d_HueHelp = 4.0 + d_Red - d_Green;
      }


      if ( d_HueHelp < 0.0 )
      {
         d_HueHelp += 6.0;
      }

      d_Hue = 60.0 * d_HueHelp ;
      if ( d_Hue >= 360.0 )
      {
         d_Hue -= 360.0;
      }
   } // else difference is greater

   i_Hue        = (int) d_Hue;
   i_Lightness  = (int) d_Lightness;
   i_Saturation = (int) d_Saturation;


} // RGBToHLS

//===========================================================================
//
//===========================================================================

void ColorCalculation::PixelToHLS(Pixel p_Pixel,
                        int &i_H , int &i_L , int &i_S)
{
   return RGBToHLS(PIXEL_RED(p_Pixel),
                   PIXEL_GREEN(p_Pixel),
                   PIXEL_BLUE(p_Pixel),
                   i_H,
                   i_L,
                   i_S);
} // PixelToHLS

//===========================================================================
//
//===========================================================================

static double HueValue (double d_Metric1,
                        double d_Metric2,
                        double d_Hue
                       )
{
   if ( d_Hue < 0.0 )
   {
      d_Hue += 360.0;
   }
   if ( d_Hue > 360.0 )
   {
      d_Hue -= 360.0;
   }


   if ( d_Hue < 60.0 )
   {
      return d_Metric1 + (d_Metric2-d_Metric1) * d_Hue / 60.0;
   }

   if ( d_Hue < 180.0 )
   {
      return d_Metric2;
   }

   if ( d_Hue < 240.0 )
   {
      return d_Metric1 + (d_Metric2-d_Metric1) * (240.0 - d_Hue) / 60.0;
   }

   return d_Metric1;
} // HueValue

//===========================================================================
//
//===========================================================================

void ColorCalculation::HLSToRGB(int i_H  , int i_L  , int i_S,
                                int &i_R , int &i_G , int &i_B)
{

   double d_Hue;
   double d_Lightness;
   double d_Saturation;

   double d_Metric1;
   double d_Metric2;

   double d_Red;
   double d_Green;
   double d_Blue;

   // convert into double
   d_Hue        = (double) i_H;
   d_Lightness  = (double) i_L / 100.0;
   d_Saturation = (double) i_S / 100.0;

   if ( d_Lightness <= 0.5 )
   {
      d_Metric2 = d_Lightness * ( 1.0 + d_Saturation);
   }
   else
   {
      d_Metric2 = d_Lightness + d_Saturation - d_Lightness * d_Saturation;
   }

   d_Metric1 = 2.0 * d_Lightness - d_Metric2;

   d_Red   = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue + 120.0) ;
   d_Green = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue        ) ;
   d_Blue  = 255.0 * HueValue (d_Metric1, d_Metric2, d_Hue + 240.0) ;


   // back to int

   i_R = (int) d_Red;
   i_G = (int) d_Green;
   i_B = (int) d_Blue;


} // HLStoRGB

//===========================================================================
//
//===========================================================================

Pixel ColorCalculation::HLSToPixel(int i_H  , int i_L  , int i_S)
{
   int i_R;
   int i_G;
   int i_B;
   HLSToRGB(i_H,i_L,i_S,i_R,i_G,i_B);

   return PIXEL_MAKE(i_R,i_G,i_B);
} // HLSToPixel()

//===========================================================================
//
//===========================================================================

Pixel ColorCalculation::TransparentColor(Pixel p_Old,
                                         Pixel p_New,
                                         int i_TransparencyFactor)
{
   Pixel p_Result;
   int   i_InversFactor;
   int   i_Red;
   int   i_Green;
   int   i_Blue;

   i_InversFactor = 100 - i_TransparencyFactor;

   i_Red   = PIXEL_RED(p_Old )     * i_TransparencyFactor +
             PIXEL_RED(p_New) * i_InversFactor;
   i_Green = PIXEL_GREEN(p_Old )     * i_TransparencyFactor +
             PIXEL_GREEN(p_New) * i_InversFactor;
   i_Blue  = PIXEL_BLUE(p_Old )     * i_TransparencyFactor +
             PIXEL_BLUE(p_New) * i_InversFactor;

   // add 50 for a better rounding
   i_Red   += 50;
   i_Green += 50;             
   i_Blue  += 50;
   
   i_Red   /= 100;
   i_Green /= 100;
   i_Blue  /= 100;

   p_Result =  PIXEL_MAKE(i_Red,i_Green,i_Blue);

   return p_Result;
} // TransparentColor()

//===========================================================================
//
//===========================================================================

Pixel ColorCalculation::TransparentAirBrushColor(Pixel p_Old,
                                         Pixel p_New,
                                         int i_TransparencyFactor)
{
   Pixel p_Result;
   int   i_InversFactor;
   int   i_OldRed;
   int   i_OldGreen;
   int   i_OldBlue;
   int   i_NewRed;
   int   i_NewGreen;
   int   i_NewBlue;
   int   i_Red;
   int   i_Green;
   int   i_Blue;

   i_InversFactor = 100 - i_TransparencyFactor;

   i_OldRed   = PIXEL_RED(p_Old );
   i_OldGreen = PIXEL_GREEN(p_Old );
   i_OldBlue  = PIXEL_BLUE(p_Old );
   
   i_NewRed   = PIXEL_RED(p_New );
   i_NewGreen = PIXEL_GREEN(p_New );
   i_NewBlue  = PIXEL_BLUE(p_New );
   
   i_Red   = i_OldRed   * i_TransparencyFactor +
             i_NewRed   * i_InversFactor;
   i_Green = i_OldGreen * i_TransparencyFactor +
             i_NewGreen * i_InversFactor;
   i_Blue  = i_OldBlue  * i_TransparencyFactor +
             i_NewBlue  * i_InversFactor;

   // make exakter rounding
   i_Red   =(int)( i_Red   / 100.0 +0.5) ;
   i_Green =(int)( i_Green / 100.0 +0.5) ;
   i_Blue  =(int)( i_Blue  / 100.0 +0.5) ;

   if(    i_Red    == i_OldRed
      &&  i_NewRed != i_OldRed)
   {
      int i_Inc;
      i_Inc = 1;
      
      if(i_Red > i_NewRed)
      {
         i_Inc = -1;
      }

      if(Utilities::Random(201) > i_TransparencyFactor)
      {
         i_Red += i_Inc;
      }

   } // red correction  
         
   if(    i_Green  == i_OldGreen
      &&  i_NewGreen != i_OldGreen)
   {
      int i_Inc;
      i_Inc = 1;

      if(i_Green > i_NewGreen)
      {
         i_Inc = -1;
      }

      if(Utilities::Random(201) > i_TransparencyFactor)
      {
         i_Green += i_Inc;
      }

   } // Green correction
      
   if(    i_Blue  == i_OldBlue
      &&  i_NewBlue != i_OldBlue)
   {
      int i_Inc;
      i_Inc = 1;

      if(i_Blue > i_NewBlue)
      {
         i_Inc = -1;
      }

      if( Utilities::Random(201) > i_TransparencyFactor)
      {
         i_Blue += i_Inc;
      }

   } // Green correction

   p_Result =  PIXEL_MAKE(i_Red,i_Green,i_Blue);

   return p_Result;
} // TransparentAirBrushColor()

//===========================================================================
//
//===========================================================================

int ColorCalculation::ColorDifferenceRGB(Pixel p_Color1,Pixel p_Color2)
{
   int i_Return;
   int i_Red1;
   int i_Green1;
   int i_Blue1;
   int i_Red2;
   int i_Green2;
   int i_Blue2;

   i_Red1  = PIXEL_RED  (p_Color1);
   i_Green1= PIXEL_GREEN(p_Color1);
   i_Blue1 = PIXEL_BLUE (p_Color1);

   i_Red2  = PIXEL_RED  (p_Color2);
   i_Green2= PIXEL_GREEN(p_Color2);
   i_Blue2 = PIXEL_BLUE (p_Color2);

   i_Return =    abs(i_Red1  -i_Red2)
              +  abs(i_Green1-i_Green2)
              +  abs(i_Blue1 -i_Blue2)
               ;
   
   return i_Return;
} // ColorDifferenceRGB()

//===========================================================================
// nearly the same like transparentcolor, but based on float factor
//===========================================================================

Pixel ColorCalculation::MixColor(Pixel p_Old,
                                         Pixel p_New,
                                         float f_MixFactor)
{
   Pixel p_Result;
   float f_InversFactor;
   float f_Red;
   float f_Green;
   float f_Blue;

   f_InversFactor = 1.00 - f_MixFactor;

   f_Red   = PIXEL_RED(p_New)  * f_MixFactor +
             PIXEL_RED(p_Old)  * f_InversFactor;
   f_Green = PIXEL_GREEN(p_New)* f_MixFactor +
             PIXEL_GREEN(p_Old)* f_InversFactor;
   f_Blue  = PIXEL_BLUE(p_New) * f_MixFactor +
             PIXEL_BLUE(p_Old) * f_InversFactor;

   // add 0.5 for a better rounding
   f_Red   += 0.5;
   f_Green += 0.5;
   f_Blue  += 0.5;

   // clip at 255
   if(f_Red> 255.0)
   {
      f_Red = 255.0;
   }
   if(f_Green> 255.0)
   {
      f_Green = 255.0;
   }
   if(f_Blue> 255.0)
   {
      f_Blue = 255.0;
   }

   // back to a pixel
   p_Result =  PIXEL_MAKE(((int)f_Red),
                          ((int)f_Green),
                          ((int)f_Blue)
                          );

   return p_Result;
} // MixColor()

//===========================================================================
//
//===========================================================================




