
//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: D3Utilities
//---------------------------------------------------------------------------
// Purpose: 5-Dimensional point, with some basic operations
//  on it. Additional a vector and a matrix out of it.
//---------------------------------------------------------------------------

#include <stdio.h>

#include "D3Utilities.h"

//======================================================================
//
//======================================================================

D3Coordinate::D3Coordinate()
{
   x = 0.0;
   y = 0.0;
   z1 = 0.0;
   z2 = 0.0;
   z3 = 0.0;
} // contructor
   
//======================================================================
//
//======================================================================

D3Coordinate::D3Coordinate(int i_X,int i_Y, int i_Z1,int i_Z2,int i_Z3)
{
   x = i_X;
   y = i_Y;
   z1 = i_Z1;
   z2 = i_Z2;
   z3 = i_Z3;
} // init constructor

//======================================================================
//
//======================================================================

D3Coordinate& D3Coordinate::operator+( D3Coordinate  other)
{
   x = x+other.x;
   y = y+other.y;
   z1 = z1+other.z1;
   z2 = z2+other.z2;
   z3 = z3+other.z3;
   return *this;
} // operator+

//======================================================================
//
//======================================================================

D3Coordinate& D3Coordinate::operator-( D3Coordinate  other)
{
   x = x-other.x;
   y = y-other.y;
   z1 = z1-other.z1;
   z2 = z2-other.z2;
   z3 = z3-other.z3;
   return *this;
} // operator- 

//======================================================================
//
//======================================================================

D3Coordinate D3Coordinate::operator* (  double  d_Value)
{
   D3Coordinate result;
   result.x = x * d_Value;
   result.y = y * d_Value;
   result.z1 = z1 * d_Value;
   result.z2 = z2 * d_Value;
   result.z3 = z3 * d_Value;
   return result;

} // operator*   


//======================================================================
//
//======================================================================

D3Vector::D3Vector(int i_Dimension)
{
   m_i_Dimension = i_Dimension;
   m_pc_Coordinates = new D3Coordinate[ m_i_Dimension];

} // constructor

//======================================================================
//
//======================================================================

D3Vector::~ D3Vector()
{
   delete  m_pc_Coordinates;
   m_pc_Coordinates = NULL;
} // destructor

//======================================================================
//
//======================================================================

D3Coordinate D3Vector::Get(int i_Index)
{
   if(   i_Index <0
      || i_Index>= m_i_Dimension)
   {
      return D3Coordinate(0,0,0,0,0);
   }

   return m_pc_Coordinates[i_Index];
} // Get()


//======================================================================
//
//======================================================================

void D3Vector::Set(int i_Index, D3Coordinate d_Value)
{
   if(   i_Index <0
      || i_Index>= m_i_Dimension)
   {
      return;
   }
   m_pc_Coordinates[i_Index] = d_Value;
} // Set()


//======================================================================
//
//======================================================================

D3Matrix::D3Matrix(int i_DimensionX,int i_DimensionY)
{
   m_i_DimensionX = i_DimensionX;
   m_i_DimensionY = i_DimensionY;
   m_pc_Coordinates = new D3Coordinate[ m_i_DimensionX*m_i_DimensionY];

}  // constructor

//======================================================================
//
//======================================================================

D3Matrix::~ D3Matrix()
{
   delete  m_pc_Coordinates;
   m_pc_Coordinates = NULL;
} // destructor

//======================================================================
//
//======================================================================

D3Coordinate D3Matrix::Get(int i_IndexX, int i_IndexY)
{
   int i_Index;
   i_Index = i_IndexY * m_i_DimensionX + i_IndexX;
   if(   i_IndexX <0
      || i_IndexY <0
      || i_IndexX>= m_i_DimensionX
      || i_IndexY>= m_i_DimensionY
      )
   {
      return D3Coordinate(0,0,0,0,0);
   }
   return m_pc_Coordinates[i_Index];
} // Get()


//======================================================================
//
//======================================================================

void D3Matrix::Set(int i_IndexX,int i_IndexY, D3Coordinate d_Value)
{
   int i_Index;
   if(   i_IndexX <0
      || i_IndexY <0
      || i_IndexX>= m_i_DimensionX
      || i_IndexY>= m_i_DimensionY
      )
   {
      return ;
   }
   i_Index = i_IndexY * m_i_DimensionX + i_IndexX;
   m_pc_Coordinates[i_Index] = d_Value;
} // Set();

//======================================================================
//
//======================================================================

int D3Matrix::GetWidth()
{
   return m_i_DimensionX;
} // GetWidth
   
//======================================================================
//
//======================================================================

int D3Matrix::GetHeight()
{
   return m_i_DimensionY;
} // GetHeight

//======================================================================
//
//======================================================================


