//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: DragandDrop.c
//---------------------------------------------------------------------------
// Purpose: Utility functions to get data out of drag and
//          drop events.
//---------------------------------------------------------------------------

#include "DragAndDrop.h"
#include "PixelBuffer.h"

//============================================================================
//
//============================================================================

DragAndDrop::DragAndDrop()
{
   // default accept nothing
   m_i_AcceptType = 0;
} // DragAndDrop()

//============================================================================
//
//============================================================================

DragAndDrop::~DragAndDrop()
{

} // ~DragAndDrop()

//============================================================================
//
//============================================================================

bool DragAndDrop::IsEventAcceptable(QDragEnterEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;

   // block for checking external types
   {
      char *pc_Result;
      int  i_FormatNumber ;

      for(i_FormatNumber =0;;i_FormatNumber ++)
      {
         pc_Result = (char *) event->format(i_FormatNumber);
         if(pc_Result ==NULL)
         {
            break;
         }

         // accept plain text
         if(strcmp(pc_Result,"text/plain")==0)
         {
            if(m_i_AcceptType & AcceptPlain)
            {
               return true;
            }
            return false;
         }
      }
   } // block for external types
   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);


   qs_Source =  GetDragSourceName(&ba_Data);

   // cheack all possible types
   if(qs_Source.compare( DRAGANDDROP_TYPE_PIXELBUFFER) ==0)
   {
      if(m_i_AcceptType & AcceptPixelbuffer)
      {
         return true;
      }
      return false;
   }
   // cheack all possible types
   if(qs_Source.compare( DRAGANDDROP_TYPE_COLOR) ==0)
   {
      if(m_i_AcceptType & AcceptColor)
      {
         return true;
      }
      return false;
   }

   // default 
   return false;
   
} // IsEventAcceptable()

//============================================================================
//
//============================================================================


void DragAndDrop::SetAcceptType(int i_AcceptType)
{
   m_i_AcceptType = i_AcceptType;
} // SetAcceptType()

//============================================================================
//
//============================================================================

QString DragAndDrop::GetDragSourceName(QByteArray *qba_Data,
                                    QString *pqs_Value)
{
   QString qs_Return;

   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if(    i_Size > 1
       && i_Size < 100 )
   {
      for(i=0; i< i_Size ; i++)
      {
         a_Character = qba_Data->at(i);
         if(a_Character== '=')
         {
            if(pqs_Value == NULL)
            {
               return qs_Return;
            }
            // skip the equal sign
            i++ ;
            *pqs_Value ="";
            while(i< i_Size)
            {
               a_Character = qba_Data->at(i);
               *pqs_Value += a_Character;
               i++;
            }
            return qs_Return;
         }
         qs_Return += a_Character;
      }

   } // id size long enough

   return qs_Return;
   
} // GetDragSourceName()
//============================================================================
//
//============================================================================

QString DragAndDrop::GetDragSourceText(QByteArray *qba_Data)
{
   QString qs_Return;
   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if( i_Size <= 0 )
   {
      return qs_Return;
   }

   for(i=0; i< i_Size ; i++)
   {
      a_Character = qba_Data->at(i);
      if( ((unsigned char) a_Character) >=' ')
      {
         // don't take control chars
         qs_Return  += a_Character;

      }
   } // for i

   return qs_Return;
} // GetDrawSourceText()


//============================================================================
//
//============================================================================

int DragAndDrop::GetEventType(QDropEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;

   // block for checking external types
   {
      char *pc_Result;
      int  i_FormatNumber ;

      for(i_FormatNumber =0;;i_FormatNumber ++)
      {
         pc_Result = (char *) event->format(i_FormatNumber);
         if(pc_Result ==NULL)
         {
            break;
         }

         // accept plain text
         if(strcmp(pc_Result,"text/plain")==0)
         {
            return  AcceptPlain;
         }
      }
   } // block for external types

   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);

    qs_Source =  GetDragSourceName(&ba_Data);

   // cheack all possible types
   if(qs_Source.compare( DRAGANDDROP_TYPE_PIXELBUFFER) ==0)
   {
      return  AcceptPixelbuffer;
   }
   // cheack all possible types
   if(qs_Source.compare( DRAGANDDROP_TYPE_COLOR) ==0)
   {
      return  AcceptColor;
   }

   // default nothing
   return 0;

} // GetEventType()

//============================================================================
//
//============================================================================

PixelBuffer *DragAndDrop::GetPixelBuffer(QDropEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;
   QString    qs_Data;

   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);
   qs_Source  = GetDragSourceName(&ba_Data,&qs_Data);

   if(qs_Source.compare( DRAGANDDROP_TYPE_PIXELBUFFER) ==0)
   {
      PixelBuffer *pc_ReceivedImage;
      long         l_Void;
      int          i_ScanResult;

      i_ScanResult = sscanf((const char *)qs_Data,"%lx",&l_Void );

      if(i_ScanResult ==1)
      {
         PixelBuffer *pc_NewImage;
         QPoint       p_DropPosition;

         pc_ReceivedImage = (PixelBuffer *) l_Void;

         // make an own copy
         pc_NewImage = new PixelBuffer(pc_ReceivedImage);

         return pc_NewImage;
      }

   }

   // not a pixel buffer
   return NULL;
      
} // GetPixelBuffer()

//============================================================================
//
//============================================================================

Pixel DragAndDrop::GetColor(QDropEvent* event)
{
   QByteArray ba_Data;
   QString    qs_Source;
   QString    qs_Data;

   ba_Data    = event->encodedData(DRAGANDDROP_APPLICATION);
   qs_Source  = GetDragSourceName(&ba_Data,&qs_Data);

   if(qs_Source.compare( DRAGANDDROP_TYPE_COLOR) ==0)
   {
      Pixel c_NewColor;
      int   i_ScanResult;

      i_ScanResult = sscanf((const char *)qs_Data,"%d",&c_NewColor );

      if(i_ScanResult ==1)
      {
         return c_NewColor;
      }
   }

   // no valid color
   return 0xffffffff;

} // GetPixel()

//============================================================================
//
//============================================================================

QString DragAndDrop::GetFilename(QDropEvent* event)
{
   QByteArray  ba_Data;
   QString     qs_PlainText;
   QString     qs_Start;
   QString     qs_Filename;
   char       *pc_Result;
   int         i_FormatNumber ;


   // initial empty
   qs_Filename="";
   for(i_FormatNumber =0;;i_FormatNumber ++)
   {
   
      pc_Result = (char *) event->format(i_FormatNumber);
      if(pc_Result ==NULL)
      {
         break;
      }
   
       ba_Data      = event->encodedData(pc_Result);
       qs_PlainText = GetDragSourceText(&ba_Data);

      // filename starts with "file:"
       qs_Start     = qs_PlainText.left(5);

      if(qs_Start.compare("file:") ==0)
      {
         // extract filename

         // remove the file: from the filename
         qs_Filename = qs_PlainText.mid(5);
         return qs_Filename;
      }

   } // for i_FormatNumber
   
   return qs_Filename;

} // GetFilename()

//============================================================================
//
//============================================================================




