#ifndef _EFFECTBOX_H
#define _EFFECTBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

#include "StandardDialog.h"
#include "BufferPainter.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "Tracer.h"
#include "QUColorOrPattern.h"
#include "QUColorDragWidget.h"
#include "QUIconButton.h"

class MainFrame;

class EffectBox :  public StandardDialog ,
                  public Tracer
{
Q_OBJECT

public:
   EffectBox(QWidget *parent=0,
            GlobalStorage *pc_Global=0,
            MainFrame *pc_MainFrame=NULL
            );
   ~EffectBox();

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

public slots:

   void ActivateComic();
   void ActivateMosaic();
   void ActivateReduce();
   void ActivateBlackWhite();

   void ComicPreCalculationBoostCheckModeToggled(bool b_On);
   void ComicAlgorithmusChanged(int i_Mode);
   void ComicLineDetectionChanged(int i_Value);
   void ComicLineDetectionToggled(bool b_On);

   void ComicDoEffect();
   
   void MosaicFormChanged(int i_Mode);
   void MosaicSizeChanged(int i_Value);
   void MosaicVarySizeToggled(bool b_On);
   void MosaicShineFactorChanged(int i_Value);

   void MosaicDoEffect();

   void ReduceColorNumberChanged(int i_Value);
   void ReduceLeastDifferenceChanged(int i_Value);

   void ReduceDoEffect();

   void BlackWhiteFromColorChanged(Pixel c_Color);
   void BlackWhiteToColorChanged(Pixel c_Color);
   void BlackWhiteCorrectionChecked( bool b_On);
   void BlackWhitePureChecked( bool b_On);

   void BlackWhiteDoEffect();

private:

   void ComicSetEffectAlgorithmusWidget(int i_Mode);
   enum
   {
      EffectComic=0,
      EffectMosaic,
      EffectColorReduce,
      EffectBlackWhite
   };

   enum
   {
      MosaicFormRound=0,
      MosaicFormSquare
   };

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   //void *SelectionEventArrived(void *pc_Data);

   //static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void SetSensitivity();
   void SetWidgetToMode(int i_Mode);


   // variables
   // widgets
   QHGroupBox        *m_ActivationBox;
   QVGroupBox        *m_ComicModeBox;
   QUIconButton      *m_ComicButton;
   QUIconButton      *m_MosaicButton;
   QUIconButton      *m_ReduceButton;
   QUIconButton      *m_BlackWhiteButton;
   
   QULabelSlider     *m_ComicLineDectionLevel;
   QCheckBox         *m_ComicLineDetectionEnableCheck;

   QButtonGroup      *m_ComicColorAlgorithmusBox;

   QRadioButton      *m_ComicAlgCopy;
   QRadioButton      *m_ComicAlgFlatten;
   QRadioButton      *m_ComicAlgBoost;
   QRadioButton      *m_ComicAlgFlattenAndBoost;
   QCheckBox         *m_ComicPreCalculationBoostCheck;
   QPushButton       *m_ComicExecuteButton;

   QVGroupBox        *m_MosaicModeBox;
   QRadioButton      *m_MosaicFormRoundButton;
   QRadioButton      *m_MosaicFormSquareButton;
   QULabelSlider     *m_MosaicElementSizeSlider;
   QCheckBox         *m_MosaicVarySizeCheck;
   QULabelSlider     *m_MosaicShineSlider;
   QPushButton       *m_MosaicExecuteButton;

   QVGroupBox        *m_ReduceModeBox;
   QULabelSlider     *m_ReduceColorNumberSlider;
   QULabelSlider     *m_ReduceLeastDifferenceSlider;
   QPushButton       *m_ReduceExecuteButton;
   
   QVGroupBox        *m_BlackWhiteModeBox;
   QUColorDragLabeledWidget *m_BlackWhiteFromColorWidget;
   QUColorDragLabeledWidget *m_BlackWhiteToColorWidget;
   QCheckBox         *m_BlackWhiteOpticalCheck;
   QCheckBox         *m_BlackWhitePureCheck;
   QPushButton       *m_BlackWhiteExecuteButton;


   BufferPainter *m_pc_BufferPainter;

   MainFrame     *m_pc_MainFrame;
   int            m_i_CurrentGraphikMode;

   bool           m_b_ComicPreBoost;
   bool           m_b_ComicLineEnabled;
   int            m_i_ComicAlgorithmus;
   int            m_i_ComicLineDetection;

   int            m_i_MosaicElementSize;
   int            m_i_MosaicForm;
   bool           m_b_MosaicVarySize;
   int            m_i_MosaicShineFactor;

   int            m_i_ReduceNumberOfColors;
   int            m_i_ReduceLeastDifference;

   Pixel          m_p_BlackWhiteFromColor;
   Pixel          m_p_BlackWhiteToColor;
   bool           m_b_BlackWhiteOpticalCorrection;
   bool           m_b_BlackWhitePure;
};


#endif // _EFFECTBOX_H
