//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: Effects
//---------------------------------------------------------------------------
// Purpose:
//---------------------------------------------------------------------------

#include <stdlib.h>

#include "Effects.h"
#include "Utilities.h"
#include "ColorCalculation.h"

//============================================================================
//
//============================================================================


Effects::Effects()
{
   m_i_ComicDiffenceSpace    = 1;
   m_i_ComicBlackLineLevel   = 2050;
   m_b_ComicBlackLineEnabled = true;

   m_i_ComicColorActionMode = ComicColorActionFlattenAndBoost ;
   m_i_ComicPrepixelMode    = ComicPrePixelActionCopy;

   m_i_ReduceMinimumDistance = 20;

   m_b_BlackWhiteOpticalCorrection = false;
   m_b_BlackWhitePureMode          = false;


} // Effects()

//============================================================================
//
//============================================================================

Effects::~Effects()
{

} // ~Effects()

//============================================================================
//
//============================================================================


void Effects::ComicColorCopy(Pixel &p_Value)
{
} // ComicColorCopy()

//============================================================================
//
//============================================================================

void Effects::ComicColorWhite(Pixel &p_Value)
{
   p_Value = PIXEL_MAKE(0xff,0xff,0xff);
} // ComicColorWhite()

//============================================================================
//
//============================================================================

void Effects::ComicFlattenColor(Pixel &p_Value)
{
   int i_Red ;
   int i_Green ;
   int i_Blue ;

   i_Red   = PIXEL_RED  (p_Value);
   i_Green = PIXEL_GREEN(p_Value);
   i_Blue  = PIXEL_BLUE (p_Value);


   i_Red   = 32*(i_Red  /32);
   i_Green = 32*(i_Green/32);
   i_Blue  = 32*(i_Blue /32);

   p_Value = PIXEL_MAKE(i_Red,i_Green,i_Blue);
   
} // ComicFlattenColor

//============================================================================
//
//============================================================================

void Effects::ComicColorBoost(Pixel &p_Value)
{
   int p_Red ;
   int p_Green ;
   int p_Blue ;
   int i_Maximum;

   i_Maximum =0;

   // look for largest color value
   p_Red   = PIXEL_RED  (p_Value);
   p_Green = PIXEL_GREEN(p_Value);
   p_Blue  = PIXEL_BLUE (p_Value);


   if (p_Red > i_Maximum)
   {
      i_Maximum = p_Red;
   }
   if (p_Green > i_Maximum)
   {
      i_Maximum = p_Green;
   }
   if (p_Blue > i_Maximum)
   {
      i_Maximum = p_Blue;
   }


   //  make biggest component 100% (255)
   if (i_Maximum!=0)
   {
       p_Red   = (p_Red  *255)/i_Maximum;
       p_Green = (p_Green*255)/i_Maximum;
       p_Blue  = (p_Blue *255)/i_Maximum;
   }

   p_Value = PIXEL_MAKE(p_Red,p_Green,p_Blue);
   
} // ComicColorBoost

//============================================================================
//
//============================================================================

void Effects::ComicFlattenAndBoost(Pixel &p_Value)
{
   ComicFlattenColor(p_Value);
   ComicColorBoost  (p_Value);
} // ComicFlattenAndBoost()

//============================================================================
//
//============================================================================

long Effects::ComicGetError(Pixel p1,Pixel p2)
{
   long l_Return;
   int  i_1Red ;
   int  i_1Green ;
   int  i_1Blue ;
   int  i_2Red ;
   int  i_2Green ;
   int  i_2Blue ;

   /*
    *   Find biggest of r,g and b
    */
   i_1Red   = PIXEL_RED  (p1);
   i_1Green = PIXEL_GREEN(p1);
   i_1Blue  = PIXEL_BLUE (p1);
   i_2Red   = PIXEL_RED  (p2);
   i_2Green = PIXEL_GREEN(p2);
   i_2Blue  = PIXEL_BLUE (p2);


   l_Return =  (i_1Red   - i_2Red)   * (i_1Red   - i_2Red);
   l_Return += (i_1Green - i_2Green) * (i_1Green - i_2Green);
   l_Return += (i_1Blue  - i_2Blue)  * (i_1Blue  - i_2Blue);

   return l_Return ;
} // ComicGetError()

//============================================================================
//
//============================================================================

long Effects::ComicGetMaximumContrast(PixelBuffer *pc_Image,int x,int y)
{
   Pixel p_Color1;
   Pixel p_Color2;
   long l_Error;
   long i_Maximum;

   i_Maximum=0;

   //  Calculate 1. error (horizontal)

   p_Color1 = pc_Image->Get(x-m_i_ComicDiffenceSpace,y);
   p_Color2 = pc_Image->Get(x+m_i_ComicDiffenceSpace,y);
   ComicPrePixelModify(p_Color1);
   ComicPrePixelModify(p_Color2);

   l_Error = ComicGetError(p_Color1,p_Color2);
   if (l_Error>i_Maximum)
   {
      i_Maximum=l_Error;
   }

   //  calculate 2. error (vertical)

   p_Color1 = pc_Image->Get(x,y-m_i_ComicDiffenceSpace);
   p_Color2 = pc_Image->Get(x,y+m_i_ComicDiffenceSpace);
   ComicPrePixelModify(p_Color1);
   ComicPrePixelModify(p_Color2);
   l_Error = ComicGetError(p_Color1,p_Color2);
   if (l_Error>i_Maximum)
   {
      i_Maximum=l_Error;
   }

   
   // calculate 3. error (upperleft-lowerright)
   
   p_Color1 = pc_Image->Get(x-m_i_ComicDiffenceSpace,y-m_i_ComicDiffenceSpace);
   p_Color2 = pc_Image->Get(x+m_i_ComicDiffenceSpace,y+m_i_ComicDiffenceSpace);
   ComicPrePixelModify(p_Color1);
   ComicPrePixelModify(p_Color2);
   l_Error = ComicGetError(p_Color1,p_Color2);
   if (l_Error>i_Maximum)
   {
      i_Maximum=l_Error;
   }

   //  calculate 4. error (lowerleft-upperright)
   
   p_Color1 = pc_Image->Get(x+m_i_ComicDiffenceSpace,y-m_i_ComicDiffenceSpace);
   p_Color2 = pc_Image->Get(x-m_i_ComicDiffenceSpace,y+m_i_ComicDiffenceSpace);
   ComicPrePixelModify(p_Color1);
   ComicPrePixelModify(p_Color2);
   l_Error = ComicGetError(p_Color1,p_Color2);
   if (l_Error>i_Maximum)
   {
      i_Maximum=l_Error;
   }

   return i_Maximum;
} // ComicGetMaximumContrast()

//============================================================================
//
//============================================================================

void Effects::ComicPrePixelModify (Pixel &p_Pixel)
{
   switch(m_i_ComicPrepixelMode)
   {
   case ComicPrePixelActionCopy:
      ComicColorCopy(p_Pixel);
      break;
   case ComicPrePixelActionColorBoost:
      ComicColorBoost(p_Pixel);
   }
   
}  // ComicPrePixelModify

//============================================================================
//
//============================================================================

void Effects::ComicColorAction(Pixel &p_Pixel)
{
   switch( m_i_ComicColorActionMode)
   {  
   case ComicColorActionFlatten:
      ComicFlattenColor(p_Pixel);
      break;
   case ComicColorActionCopy:
      ComicColorCopy(p_Pixel);
      break;
   case ComicColorActionColorBoost:
      ComicColorBoost(p_Pixel);
      break;
   case ComicColorActionFlattenAndBoost:
      ComicFlattenAndBoost(p_Pixel);
      break;
   }
   

}  // ComicColorAction();


//============================================================================
//
//============================================================================

void Effects::DoComicEffect(PixelBuffer *pc_Image)
{
   PixelBuffer *pc_SelectionMask;
   RectRegion   c_MaskRegion;
   Pixel        p_Original;
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   int i_FromY;
   int i_FromX;
   int i_ToX;
   int i_ToY;

   // create a copy of the buffer
   m_pc_Copy = new PixelBuffer(pc_Image);

   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();

   i_FromX = 0;
   i_FromY = 0;
   i_ToX   = i_Width  -1;
   i_ToY   = i_Height -1;

   pc_SelectionMask = NULL;

   if(pc_Image->GetSelectionMaskActive() )
   {

      pc_SelectionMask = m_pc_Copy->GetSelectionMask();
      c_MaskRegion = pc_SelectionMask->GetUsedRegion(1);

      if(c_MaskRegion.IsEmpty())
      {
         // empty mask - do nothing
         // clean up the processing
         delete m_pc_Copy;
         return;
      }

      i_FromX = c_MaskRegion.GetStartX();
      i_FromY = c_MaskRegion.GetStartY();
      i_ToX   = c_MaskRegion.GetEndX();
      i_ToY   = c_MaskRegion.GetEndY();
   }


   for(i_RunY= i_FromY+m_i_ComicDiffenceSpace;
       i_RunY <= i_ToY-m_i_ComicDiffenceSpace ;
       i_RunY ++)
   {

      for(i_RunX= i_FromX+m_i_ComicDiffenceSpace;
          i_RunX <= i_ToX-m_i_ComicDiffenceSpace;
          i_RunX ++)
      {
         // if selection mask aktive
         if(pc_SelectionMask != NULL)
         {
            if(! pc_SelectionMask->Get(i_RunX,i_RunY) )
            {
               // and point not selected, then do no calculation
               continue;
            }
         }

         p_Original = m_pc_Copy->Get(i_RunX,i_RunY);

         if(m_b_ComicBlackLineEnabled)
         {
            if (ComicGetMaximumContrast(m_pc_Copy,i_RunX,i_RunY)>m_i_ComicBlackLineLevel)
            {
               // Make a border pixel // in black
                
               pc_Image->Set(i_RunX,i_RunY,0);
            }
            else
            {
               //   Copy original color
               
               ComicColorAction(p_Original);

               // set the new pixel into the image
               pc_Image->Set(i_RunX,i_RunY,p_Original);
            }
         }
         else // no black line
         {
            //   Copy original color

            ComicColorAction(p_Original);

            // set the new pixel into the image
            pc_Image->Set(i_RunX,i_RunY,p_Original);
         }
      } // for i_Run_X
   } // for i_Run_Y

   // clean up the processing
   delete m_pc_Copy;

} // DoComicEffect()

//============================================================================
//
//============================================================================

void Effects::SetComicAlgorithmus(int  i_ColorActionMode,int i_ComicPrepixelMode)
{
   m_i_ComicColorActionMode = i_ColorActionMode;
   m_i_ComicPrepixelMode    = i_ComicPrepixelMode;

}  // SetComicAlgorithmus()

//============================================================================
//
//============================================================================

void Effects::SetComicBlackLineLevel(bool b_Enabled,
                                     int i_ComicBlackLineLevel)
{
   m_i_ComicBlackLineLevel = i_ComicBlackLineLevel;
   m_b_ComicBlackLineEnabled = b_Enabled;
} // SetComicBlackLineLevel()

//============================================================================
//
//============================================================================

void Effects::MosaicSetPixel(int i_RandX,int i_RandY,
                             int &i_SetPixel)
{
   Pixel p_NewColor;
   Pixel p_TransColor;
   Pixel p_OldColor;
   float f_Transparency;
   int   i_Transparency;
   int   i_Radius;
   int   i_FromX;
   int   i_FromY;
   int   i_ToX;
   int   i_ToY;
   int   i_X;
   int   i_Y;
   int   i_MaxDistance ;
   int   i_DiffX;
   int   i_DiffY;
   int   i_Position;

   
   if(m_pc_MosaicMask->Get(i_RandX,i_RandY)!= 0)

   {
      // already used
      return;
   }

   if(m_b_MosaicVarySize)
   {
      i_Radius = 1 +Utilities::Random(m_i_MosaicSize);
   }
   else
   {
      i_Radius = m_i_MosaicSize;
   }

   // get the middle color
   p_NewColor = m_pc_Copy->Get(i_RandX,i_RandY);

   // calculate the maximal possible range and clip at image border
   i_FromX = i_RandX -  i_Radius;
   if( i_FromX<0 )
   {
      i_FromX =0;
   }
   i_FromY = i_RandY -  i_Radius;
   if( i_FromY<0 )
   {
      i_FromY =0;
   }

   i_ToX = i_RandX +  i_Radius;
   if( i_ToX >= m_pc_Original->GetWidth() )
   {
      i_ToX = m_pc_Original->GetWidth()-1;
   }

   i_ToY = i_RandY +  i_Radius;
   if( i_ToY >= m_pc_Original->GetHeight() )
   {
      i_ToY = m_pc_Original->GetHeight()-1;
   }


   if( m_i_MosaicForm == MosaicFormSquare)
   { 

      // max distance times sqrt(2);
      i_MaxDistance = (int)((float)(i_Radius*i_Radius) *1.4142) ;

      for(i_Y=i_FromY; i_Y <= i_ToY; i_Y ++)
      {
         i_DiffY = i_Y - i_RandY;
         i_DiffY *= i_DiffY;
         for(i_X=i_FromX; i_X <= i_ToX; i_X ++)
         {
            i_DiffX = i_X - i_RandX;

            i_Position = i_DiffX*i_DiffX + i_DiffY;

            // if shine through 
            if( m_i_MosaicShineThrough)
            {
               // calculate the transparency. More transparent at outer
               // region.
               f_Transparency = (float) i_Position / (float) i_MaxDistance;
               if(f_Transparency>1.0)
               {
                  f_Transparency = 1.0;
               }
               f_Transparency *= m_i_MosaicShineThrough*10.0;
               i_Transparency = (int) f_Transparency;

               p_OldColor   = m_pc_Copy->Get(i_X,i_Y);
               p_TransColor = ColorCalculation::TransparentColor(p_OldColor,
                                                           p_NewColor,
                                                           i_Transparency);

               m_pc_Original->Set(i_X,i_Y,p_TransColor);
               if(m_pc_MosaicMask->Get(i_X,i_Y) ==0)
               {
                  m_pc_MosaicMask->Set(i_X,i_Y,1) ;
                  i_SetPixel += 1;
               }
            }
            else // not shine through
            {

               m_pc_Original->Set(i_X,i_Y,p_NewColor);
               if(m_pc_MosaicMask->Get(i_X,i_Y) ==0)
               {
                  m_pc_MosaicMask->Set(i_X,i_Y,1) ;
                  i_SetPixel += 1;
            
               }
            } // else not shine through
         } // for i_X

      } // for i_Y
   }
   else if( m_i_MosaicForm == MosaicFormRound)
   {
      i_MaxDistance = (i_Radius*i_Radius);

      for(i_Y=i_FromY; i_Y <= i_ToY; i_Y ++)
      {
         i_DiffY = i_Y - i_RandY;
         i_DiffY *= i_DiffY;
      
         for(i_X=i_FromX; i_X <= i_ToX; i_X ++)
         {
         
            i_DiffX = i_X - i_RandX;
            i_Position = i_DiffX*i_DiffX + i_DiffY;
                  
            if( i_Position >i_MaxDistance)
            {
               continue;
            }
            if( m_i_MosaicShineThrough)
            {
               // calculate the transparency. More transparent at outer
               // region.
               f_Transparency = (float) i_Position / (float) i_MaxDistance;
               f_Transparency *= m_i_MosaicShineThrough*10.0;
               i_Transparency = (int) f_Transparency;

               p_OldColor   = m_pc_Copy->Get(i_X,i_Y);
               p_TransColor = ColorCalculation::TransparentColor(p_OldColor,
                                                           p_NewColor,
                                                           i_Transparency);
         
               m_pc_Original->Set(i_X,i_Y,p_TransColor);
               if(m_pc_MosaicMask->Get(i_X,i_Y) ==0)
               {
                  m_pc_MosaicMask->Set(i_X,i_Y,1) ;
                  i_SetPixel += 1;
               }
            }
            else // not shine through
            {
               m_pc_Original->Set(i_X,i_Y,p_NewColor);
               if(m_pc_MosaicMask->Get(i_X,i_Y) ==0)
               {
                  m_pc_MosaicMask->Set(i_X,i_Y,1) ;
                  i_SetPixel += 1;
               }

            }
         } // for i_X

      } // for i_Y
   } // form round

} // MosaicSetPixel()

//============================================================================
//
//============================================================================

void Effects::DoMosaicEffect(PixelBuffer *pc_Image)
{

   int i_TotalPixel;
   int i_SetBorder ;
   int i_SetPixel;
   int i_Width;
   int i_Height;

   int i_X;
   int i_Y;
   int i_Difference;
   
   // create a copy of the buffer
   m_pc_Original = pc_Image;
   m_pc_Copy     = new PixelBuffer(pc_Image);

   // get the selection mask and clear it
   m_pc_MosaicMask= m_pc_Copy->GetSelectionMask();
   m_pc_MosaicMask->Clean(0);

   m_pc_Copy->GetWidthHeight(i_Width,i_Height);
   
   i_TotalPixel = i_Width*i_Height;
   i_SetBorder  = i_TotalPixel*80/100;
   i_SetPixel   = 0;
   
   while(i_SetPixel< i_SetBorder)
   {
      int i_RandX;
      int i_RandY;

      i_RandX = Utilities::Random(i_Width);
      i_RandY = Utilities::Random(i_Height);

      MosaicSetPixel(i_RandX,
                     i_RandY,
                     i_SetPixel);
      
   }

   // second loop to get all the remaining pixel;
   
   for(i_Difference = 16;
       i_Difference > 0;
       i_Difference >>=1 )
   {
      for(i_Y=0;i_Y < i_Height;i_Y += i_Difference)
      {
         for(i_X=0;i_X < i_Width;i_X += i_Difference)
         {
            if( m_pc_MosaicMask->Get(i_X,i_Y) ==0)
            {
               MosaicSetPixel(i_X,
                              i_Y,
                              i_SetPixel);

            }
         } // for i_Y
      } // for i_Y
   } // for Difference

   // mask is deleted together with the copy
   delete m_pc_Copy;
   
} // DoMosaicEffect()

//============================================================================
//
//============================================================================

void Effects::SetMosaicSize(int i_Size,bool b_VarySize)
{
   m_i_MosaicSize     = i_Size;
   m_b_MosaicVarySize = b_VarySize;
} // SetMosaicSize()

//============================================================================
//
//============================================================================

void Effects::SetMosaicForm(int i_Form)
{
   m_i_MosaicForm = i_Form;
} // SetMosaicForm()

//============================================================================
//
//============================================================================

void Effects::SetMosaicShineThrough(int i_Shinethrough)
{
   m_i_MosaicShineThrough = i_Shinethrough;
} // SetMosaicShineThrough()

//============================================================================
//
//============================================================================

void Effects::DoColorReduce(PixelBuffer *pc_Image,
                            int i_NumberOfColors)
{
   // create a copy of the buffer
   m_pc_Original        = pc_Image;
   m_pc_ReduceHistogram = NULL;
   m_pc_ReduceSort      = NULL;

 
   ReduceMakeHistogram();

   ReduceSortColors();

   ReduceEleminateNearColors(i_NumberOfColors);

   ReduceConvertImage();

   // clean up
   if(  m_pc_ReduceHistogram != NULL)
   {
      delete[] m_pc_ReduceHistogram;
      m_pc_ReduceHistogram = NULL;
   }
   if(  m_pc_ReduceSort != NULL)
   {
      delete[] m_pc_ReduceSort;
      m_pc_ReduceSort = NULL;
   }
   
} // DoColorReduce()

//============================================================================
//
//============================================================================


Pixel Effects::ReducePixelBits(Pixel p_Full)
{
   Pixel p_Return;

   p_Return =  (PIXEL_RED  (p_Full) &0xfc) >>2;
   p_Return |= (PIXEL_GREEN(p_Full) &0xfc) <<4;
   p_Return |= (PIXEL_BLUE (p_Full) &0xfc) <<10;

   return p_Return;
} // ReducePixelBits

//============================================================================
//
//============================================================================

Pixel Effects::ReduceBlowUpPixelBits(Pixel p_Reduced)
{
   Pixel p_Return;
   unsigned int ui_Red;
   unsigned int ui_Green;
   unsigned int ui_Blue;

   ui_Red   = ((p_Reduced & 0x0003f) <<2 ) | 0x03;

   ui_Green = ((p_Reduced & 0x00fc0) >>4 ) | 0x03;

   ui_Blue  = ((p_Reduced & 0x3f000) >>10) | 0x03;

   p_Return = PIXEL_MAKE(ui_Red,ui_Green,ui_Blue);

   return p_Return;

} // ReduceBlowUpPixelBits()

//============================================================================
//
//============================================================================

void Effects::ReduceMakeHistogram()
{
   Pixel p_Pixel;
   int i_Width;
   int i_Height;
   int i_X;
   int i_Y;
   
   
   m_i_ReduceHistogramCount = 0x40000;  // 18 bits
   //allocate histogram-buffer
   m_pc_ReduceHistogram
            = new unsigned int[m_i_ReduceHistogramCount];
   memset(m_pc_ReduceHistogram,
          0,
          m_i_ReduceHistogramCount*sizeof(unsigned int));

   // go over all pixels
   i_Width  = m_pc_Original->GetWidth();
   i_Height = m_pc_Original->GetHeight();
          
   for(i_Y=0;i_Y<i_Height;i_Y++)
   {
      for(i_X=0;i_X<i_Width;i_X++)
      {
         // get pixel
         p_Pixel = m_pc_Original->Get(i_X,i_Y);

         // reduce it to 18 bits
         p_Pixel = ReducePixelBits(p_Pixel);

         if(   p_Pixel >=0
            && p_Pixel < m_i_ReduceHistogramCount)
         {
            // increment count in Histogram
            m_pc_ReduceHistogram[p_Pixel] += 1;   
         }
               
      }

   }
         
} // ReduceMakeHistogram()

//============================================================================
//
//============================================================================

void Effects::ReduceSortColors()
{
   int i;
   
   m_pc_ReduceSort 
            = new unsigned int[m_i_ReduceHistogramCount];
   for(i=0; i<(int)m_i_ReduceHistogramCount; i++)
   {
      m_pc_ReduceSort[i]=i;
   } 

   ReduceQSort(0,m_i_ReduceHistogramCount-1);

} // ReduceSortColors

//============================================================================
//
//============================================================================

void Effects::ReduceQSort(int i_Left, int i_Right)
{
   unsigned int ui_Element;
   unsigned int ui_Temp;
   int           i;
   int           j;

   i = i_Left;
   j = i_Right;

   ui_Element = m_pc_ReduceHistogram[m_pc_ReduceSort[ (i_Left + i_Right) / 2]];

   do
   {
      while (m_pc_ReduceHistogram[m_pc_ReduceSort[i]]  > ui_Element)
      {
         i++;
      }

      while (ui_Element > m_pc_ReduceHistogram[m_pc_ReduceSort[j]] )
      {
         j--;
      }

      if (i <= j)
      {

         ui_Temp            = m_pc_ReduceSort[i];
         m_pc_ReduceSort[i] = m_pc_ReduceSort[j];
         m_pc_ReduceSort[j] = ui_Temp;
         i++;
         j--;
      }

   }
   while (i <= j);

   if (i_Left < j)
   {
      ReduceQSort ( i_Left, j);
   }
   if (i < i_Right)
   {
      ReduceQSort ( i, i_Right);
   }


} // ReduceQSort()

//============================================================================
//
//============================================================================

int  Effects::ReduceColorDistance(unsigned int ui_Color1,
                         unsigned int ui_Color2)
{
   int i_Result;
   int i_Base1;
   int i_Base2;

   i_Base1 = ui_Color1 & 0x3f;
   i_Base2 = ui_Color2 & 0x3f;

   i_Result = abs(i_Base1 - i_Base2);

   i_Base1 = (ui_Color1>>6) & 0x3f;
   i_Base2 = (ui_Color2>>6) & 0x3f;

   i_Result += abs(i_Base1 - i_Base2);
   
   i_Base1 = (ui_Color1>>12) & 0x3f;
   i_Base2 = (ui_Color2>>12) & 0x3f;

   i_Result += abs(i_Base1 - i_Base2);

   return i_Result;
   
} // ReduceColorDistance()

//============================================================================
//
//============================================================================

void Effects::ReduceEleminateNearColors(int i_NumberOfColors)
{
   unsigned int *pc_DestinationColor;
   unsigned int i_CurrentColor;
   int          i;
   int          j;
   int          i_DifferentNext;

   i_DifferentNext=0;

   pc_DestinationColor = new unsigned int[i_NumberOfColors];
   for(i=0;i<i_NumberOfColors;i++)
   {
      bool b_Different;
      do
      {
         b_Different=true;
         i_CurrentColor= m_pc_ReduceSort[i_DifferentNext];
      
         for(j=0;j<i;j++)
         {
            int i_CurrentDifference;

            i_CurrentDifference = ReduceColorDistance(
                        i_CurrentColor,pc_DestinationColor[j]);
                        
            if(i_CurrentDifference < m_i_ReduceMinimumDistance)
            {
               // too small
               b_Different = false;
               break;
            }
         } // for j
         if( ! b_Different)
         {
            i_DifferentNext ++;  
         }
         
      } while(! b_Different
               && i_DifferentNext < (int)(m_i_ReduceHistogramCount-i) );
               
      if( i_DifferentNext >= (int)(m_i_ReduceHistogramCount-i))
      {
         break;
      }
      pc_DestinationColor[i] = i_CurrentColor;
   }

   while(i< i_NumberOfColors)
   {
      pc_DestinationColor[i] = m_pc_ReduceSort[
                   m_i_ReduceHistogramCount-i_NumberOfColors-1+i];
      i++;
   }


   // now build up the conversation table.
   // use the histogram, which is not used any longer

   for(i=0;i<(int)m_i_ReduceHistogramCount;i++)
   {
      unsigned int ui_Nearest;
      int          i_MinimumDifference;

      if(m_pc_ReduceHistogram[i] >0)
      {
         // this color exists
         i_MinimumDifference = ReduceColorDistance(i,
                                  pc_DestinationColor[0]);
         ui_Nearest =0;
         for(j =1; j< i_NumberOfColors;j++)
         {
            int i_CurrentDifference;
            i_CurrentDifference = ReduceColorDistance(i,
                                  pc_DestinationColor[j]);
            if(i_CurrentDifference < i_MinimumDifference)
            {
               ui_Nearest=j;
               i_MinimumDifference = i_CurrentDifference;
                 
            }
         } // for j;

         m_pc_ReduceHistogram[i] = pc_DestinationColor[ui_Nearest];
         m_pc_ReduceHistogram[i] = ReduceBlowUpPixelBits(m_pc_ReduceHistogram[i]);
         
      } // if color is needed
   } // for i;

   delete[] pc_DestinationColor;
} // ReduceEleminateNearColors()

//============================================================================
//
//============================================================================

void Effects::ReduceConvertImage()
{
   Pixel p_Pixel;
   int   i_Width;
   int   i_Height;
   int   i_X;
   int   i_Y;

   // go over all pixels
   i_Width  = m_pc_Original->GetWidth();
   i_Height = m_pc_Original->GetHeight();
   for(i_Y=0;i_Y<i_Height;i_Y++)
   {
      for(i_X=0;i_X<i_Width;i_X++)
      {
         // get pixel
         p_Pixel = m_pc_Original->Get(i_X,i_Y);

         // reduce it to 18 bits
         p_Pixel = ReducePixelBits(p_Pixel);

         p_Pixel = m_pc_ReduceHistogram[p_Pixel];
         m_pc_Original->Set(i_X,i_Y,p_Pixel);

      } // for i_X

   } // for i_Y


} // ReduceConvertImage()

//============================================================================
//
//============================================================================

void Effects::SetColorReduceLeastDifference(int i_Difference)
{
   m_i_ReduceMinimumDistance = i_Difference;
} // SetColorReduceLeastDifference()

//============================================================================
//
//============================================================================

void Effects::SetBlackWhiteColors(Pixel p_FromColor,Pixel p_ToColor)
{
   m_p_BlackWhiteFromColor = p_FromColor;
   m_p_BlackWhiteToColor   = p_ToColor;

} // SetBlackWhiteColors();

//============================================================================
//
//============================================================================

void Effects::DoBlackWhiteEffect(PixelBuffer *pc_Image)
{
   int i_Width;
   int i_Height;
   int i_X;
   int i_Y;

   if(m_b_BlackWhitePureMode)
   {
      DoBlackWhitePurEffect(pc_Image);
      return;
   }

   pc_Image->GetWidthHeight(i_Width,i_Height);
   for(i_Y=0; i_Y < i_Height;i_Y++)
   {
      for(i_X=0; i_X < i_Width;i_X++)
      {
         float f_RedValue;
         float f_GreenValue;
         float f_BlueValue;
         float f_TotalValue;
         
         Pixel p_ThePixel;
         Pixel p_NewPixel;

         p_ThePixel = pc_Image->Get(i_X,i_Y);

         // make a float
         f_RedValue   = (float )(PIXEL_RED  (p_ThePixel));
         f_GreenValue = (float )(PIXEL_GREEN(p_ThePixel));
         f_BlueValue  = (float )(PIXEL_BLUE (p_ThePixel));

         // adust to 0 .. 1
         f_RedValue   /= 255.0;
         f_GreenValue /= 255.0;
         f_BlueValue  /= 255.0;

         // now make a total value

         if(m_b_BlackWhiteOpticalCorrection)
         {
            f_TotalValue =   f_RedValue   * 0.299
                           + f_GreenValue * 0.587
                           + f_BlueValue  * 0.114;
         }
         else // plain method, all colors are the same
         {
            f_TotalValue = (f_RedValue+f_GreenValue+f_BlueValue)/ 3.0;
         }

         // 1 is upper border
         if(f_TotalValue>1.0)
         {
            f_TotalValue = 1.0;
         }


         p_NewPixel = ColorCalculation::MixColor(m_p_BlackWhiteFromColor,
                                                 m_p_BlackWhiteToColor,
                                                 f_TotalValue);
         // put new value back into the image
         pc_Image->Set(i_X,i_Y,p_NewPixel);
      } // for x

   } // for y
   
} // DoBlackWhiteEffect()

//============================================================================
//
//============================================================================

void Effects::SetBlackWhiteOpticalCorrection(bool b_Correction)
{
   m_b_BlackWhiteOpticalCorrection = b_Correction;
} // SetBlackWhiteOpticalCorrection()

//============================================================================
//
//============================================================================

void Effects::DoBlackWhitePurEffect(PixelBuffer *pc_Image)
{
   int i_Width;
   int i_Height;
   int i_X;
   int i_Y;

   pc_Image->GetWidthHeight(i_Width,i_Height);
   for(i_Y=0; i_Y < i_Height;i_Y++)
   {
      for(i_X=0; i_X < i_Width;i_X++)
      {
         float f_RedValue;
         float f_GreenValue;
         float f_BlueValue;
         float f_TotalValue;
         float f_CompareValue;

         Pixel p_ThePixel;
         Pixel p_NewPixel;

         p_ThePixel = pc_Image->Get(i_X,i_Y);

         // make a float
         f_RedValue   = (float )(PIXEL_RED  (p_ThePixel));
         f_GreenValue = (float )(PIXEL_GREEN(p_ThePixel));
         f_BlueValue  = (float )(PIXEL_BLUE (p_ThePixel));

         // adust to 0 .. 1
         f_RedValue   /= 255.0;
         f_GreenValue /= 255.0;
         f_BlueValue  /= 255.0;

         // now make a total value

         if(m_b_BlackWhiteOpticalCorrection)
         {
            f_TotalValue =   f_RedValue   * 0.299
                           + f_GreenValue * 0.587
                           + f_BlueValue  * 0.114;
         }
         else // plain method, all colors are the same
         {
            f_TotalValue = (f_RedValue+f_GreenValue+f_BlueValue)/ 3.0;
         }

         // 1 is upper border
         if(f_TotalValue>1.0)
         {
            f_TotalValue = 1.0;
         }

         f_CompareValue = ((float) rand()) / ((float) RAND_MAX);

         if(f_CompareValue < f_TotalValue)
         {
            p_NewPixel = m_p_BlackWhiteToColor;
         }
         else
         {
            p_NewPixel = m_p_BlackWhiteFromColor;
         }
         
         // put new value back into the image
         pc_Image->Set(i_X,i_Y,p_NewPixel);
      } // for x

   } // for y

} // DoBlackWhitePurEffect()

//============================================================================
//
//============================================================================

void Effects::SetBlackWhitePureMode(bool b_On)
{
   m_b_BlackWhitePureMode = b_On;
} // SetBlackWhitePureMode()

//============================================================================
//
//============================================================================

