#ifndef _EFFECTS_H
#define _EFFECTS_H

#include "PixelBuffer.h"

class Effects
{
public:
   Effects();
   ~Effects();

   void SetComicBlackLineLevel(bool b_Enabled,
                               int i_ComicBlackLineLevel);
   void SetComicAlgorithmus(int  i_ColorActionMode,int i_ComicPrepixelMode);
   void DoComicEffect(PixelBuffer *pc_Image);

   void SetMosaicSize(int i_Size,bool b_VarySize);
   void SetMosaicForm(int i_Form);
   void SetMosaicShineThrough(int i_Shinethrough);
   void DoMosaicEffect(PixelBuffer *pc_Image);

   void SetBlackWhiteColors(Pixel p_FromColor,Pixel p_ToColor);
   void DoBlackWhiteEffect(PixelBuffer *pc_Image);
   void SetBlackWhiteOpticalCorrection(bool b_Correction);
   void SetBlackWhitePureMode(bool b_On);

   void DoBlackWhitePurEffect(PixelBuffer *pc_Image);
   
   enum
   {
      MosaicFormRound,
      MosaicFormSquare
   };

   
   enum
   {
      ComicColorActionFlatten,
      ComicColorActionCopy,
      ComicColorActionColorBoost,
      ComicColorActionFlattenAndBoost
   };

   enum
   {
      ComicPrePixelActionCopy ,
      ComicPrePixelActionColorBoost
   };


   void DoColorReduce(PixelBuffer *pc_Image,int i_NumberOfColors);
   void SetColorReduceLeastDifference(int i_Difference);
   
private:

   int m_i_ComicDiffenceSpace ;
   int m_i_ComicBlackLineLevel;
   bool m_b_ComicBlackLineEnabled;

   void ComicColorCopy(Pixel &p_Value);
   void ComicColorWhite(Pixel &p_Value);
   void ComicFlattenColor(Pixel &p_Value);
   void ComicColorBoost(Pixel &p_Value);
   void ComicFlattenAndBoost(Pixel &p_Value) ;
   long ComicGetMaximumContrast(PixelBuffer *pc_Image,int x,int y);
   long ComicGetError(Pixel p1,Pixel p2);


   void ComicPrePixelModify (Pixel &p_Pixel);
   void ComicColorAction    (Pixel &p_Pixel);
   

   int m_i_ComicColorActionMode;
   int m_i_ComicPrepixelMode;  

   int  m_i_MosaicSize;
   bool m_b_MosaicVarySize;
   int  m_i_MosaicForm;
   int  m_i_MosaicShineThrough;

   void MosaicSetPixel(int i_RandX,int i_RandY,
                          int &i_SetPixel);

   PixelBuffer *m_pc_MosaicMask;
   PixelBuffer *m_pc_Copy;
   PixelBuffer *m_pc_Original;

   // part for the color Reduce
   Pixel ReducePixelBits(Pixel p_Full);
   Pixel ReduceBlowUpPixelBits(Pixel p_Reduced);
   void ReduceMakeHistogram();
   void ReduceSortColors();
   void ReduceEleminateNearColors(int i_NumberOfColors);
   void ReduceConvertImage();
   void ReduceQSort(int i_Left, int i_Right);
   int  ReduceColorDistance(unsigned int ui_Color1,
                            unsigned int ui_Color2);
   unsigned int m_i_ReduceHistogramCount;
   unsigned int *m_pc_ReduceHistogram;
   unsigned int *m_pc_ReduceSort;

   int m_i_ReduceMinimumDistance;

   //--- part for the blackwhite effect
   Pixel m_p_BlackWhiteFromColor;
   Pixel m_p_BlackWhiteToColor;
   bool  m_b_BlackWhiteOpticalCorrection;
   bool  m_b_BlackWhitePureMode;

}; // Effects

#endif // _EFFECTS_H
