#ifndef _FILELOADER_H
#define _FILELOADER_H

#include <qstring.h>
#include <qwidget.h>

#include "PixelBuffer.h"

extern "C"
{
#include <jpeglib.h>
#include <gif_lib.h>
#include <libexif/exif-data.h>

}

class MainFrame;

class FileLoaderOptions
{
public:
   FileLoaderOptions();

   int GetNumberOfExifLines();
   void ClearExif();
   QString GetExifLine(unsigned int i_Index);
   void AddExifLine(QString qs_Line);
   bool IsExifInfoValid();
   QStringList *GetExifInfo();
   
   bool b_HasSelectionMask;
   bool b_HasMainFramePointer;
   MainFrame *m_pc_MainFrame;

private:
   bool m_b_ExifInfoValid;
   QStringList m_sl_ExifInfo;
};


class FileLoader
{
public:
   FileLoader(QWidget *MainWidget);
   ~FileLoader();

   // later with options
   QString AskForFilename(bool b_OpenMode,QString qs_DefaultName="");

   PixelBuffer * LoadFile(QString qs_Filename,FileLoaderOptions *pc_Options = NULL);
   int SaveFile(QString qs_Filename,
                PixelBuffer *pc_Image,
                FileLoaderOptions *pc_Options = NULL);

   int SaveFileTiff(QString qs_Filename,PixelBuffer *pc_Image);
   int LoadFileTiff(QString qs_Filename,PixelBuffer *pc_Destination);

   PixelBuffer *GetIconforFile(QString qs_Filename);
   static QString GetPureName(QString qs_Filename);
   static int MakeDirectory(QString qs_Name);

private:

   int SaveFileJpeg(QString qs_Filename,PixelBuffer *pc_Image);
   int SaveFilePng(QString qs_Filename,PixelBuffer *pc_Image);
   int SaveFilePerico(QString qs_Filename,
                     PixelBuffer *pc_Image,
                     FileLoaderOptions *pc_Options = NULL);

   int SaveIconFile(PericoFileWriter *pc_FileWriter,
                 PixelBuffer *pc_Image);
   

   PixelBuffer * LoadFileJpeg(QString qs_Filename);
   PixelBuffer * LoadFileGif(QString qs_Filename);
   PixelBuffer * LoadFileTif(QString qs_Filename);
   PixelBuffer * LoadFilePng(QString qs_Filename);
   PixelBuffer * LoadFileBmp(QString qs_Filename);
   PixelBuffer * LoadFilePerico(QString qs_Filename,FileLoaderOptions *pc_Options = NULL);


   int JpegSetFilenameForLoad(QString qs_Filename,
                              int &i_Width,
                              int &i_Height);
   unsigned char *JpegReadLine (  );

   void JpegClose();

   QString GetExtension(QString qs_Name);
   
   void SetGifLine(int i_Line,
                    unsigned char * &Source,
                    PixelBuffer* pc_Destination,
                    GifColorType *Colors);
   QString GetImageFileDirectory();
   int LoadExifData(QString qs_Filename);
   void AddExifData(QString qs_Name,QString qs_Value);
   static void ExifContentForeachEntryFunction (ExifEntry *pc_Entry, void *vp_UserData);
   static void ExifDataForeachContentFunction (ExifContent *pc_Content, void *vp_UserData);

   PixelBuffer *GetIconforJpegFile(QString qs_Filename);

   PixelBuffer *GetIconforTifFile(QString qs_Filename);
   PixelBuffer *GetIconforGifFile(QString qs_Filename);
   PixelBuffer *GetIconforPngFile(QString qs_Filename);

   QWidget *m_MainWidget;
   FileLoaderOptions *m_pc_CurrentLoadOptions;


   // structure for reading a jpeg file
   FILE *m_JpegFile;
   struct jpeg_decompress_struct jpegInfo;
   struct jpeg_error_mgr         jpegError;
   JSAMPARRAY m_JpegOutputLine;

   int m_i_MaximumIconWidth;
   int m_i_MaximumIconHeight;

};

#endif // _FILELOADER_H
