//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: FilePreview.c
//---------------------------------------------------------------------------
// Purpose: deliver icon preview in load file box.
//---------------------------------------------------------------------------

#include <qimage.h>

#include "FilePreview.h"

#include "FileLoader.h"


//============================================================================
//
//============================================================================


FilePreview::FilePreview( QWidget *parent,
            FileLoader *pc_Loader ) : QLabel( parent )
{
   m_pc_FileLoader = pc_Loader;
   m_Pixmap        = new QPixmap();

} // FilePreview

//============================================================================
//
//============================================================================


void FilePreview::previewUrl( const QUrl &u )
{
   PixelBuffer *pc_Icon;
   QString      path ;
   QImage      *pc_Image;
   float        f_OriginalX;
   float        f_OriginalY;
   float        f_SmallerFactorX;
   float        f_SmallerFactorY;
   float        f_SmallerFactor;
   int          i_ImageWidth;
   int          i_ImageHeight;
   int          i_WidgetWidth;
   int          i_WidgetHeight;
   int          i_RunX;
   int          i_RunY;
   int          i_OriginalX;
   int          i_OriginalY;
   int          i_NewWidth;
   int          i_NewHeight;

   path = u.path();
   i_WidgetWidth  =   width();
   i_WidgetHeight =   height();
   

   
   pc_Icon = m_pc_FileLoader->GetIconforFile(path);

   if(pc_Icon == NULL)
   {
       setText( tr("No Preview available") );
   }
   else
   {

      pc_Icon->GetWidthHeight(i_ImageWidth,i_ImageHeight);

      f_SmallerFactorX = (i_ImageWidth  / (float)i_WidgetWidth);
      f_SmallerFactorY = (i_ImageHeight / (float)i_WidgetHeight);

      // take the larger of the both
      if(f_SmallerFactorX > f_SmallerFactorY)
      {
         f_SmallerFactor = f_SmallerFactorX;
      }
      else
      {
         f_SmallerFactor = f_SmallerFactorY;
      }
      i_NewWidth  = (int) (i_ImageWidth  / f_SmallerFactor);
      i_NewHeight = (int) (i_ImageHeight / f_SmallerFactor);


      
      pc_Image = new QImage(i_NewWidth,
                         i_NewHeight,
                         32);
      // first clear the image
      for(i_RunY=0 , f_OriginalY=0;
          i_RunY < i_NewHeight;
          i_RunY ++, f_OriginalY += f_SmallerFactor)
      {
         for(i_RunX=0 , f_OriginalX=0;
             i_RunX < i_NewWidth;
             i_RunX ++, f_OriginalX += f_SmallerFactor)
         {
            Pixel p;

            i_OriginalX = (int) f_OriginalX;
            i_OriginalY = (int) f_OriginalY;
         
            p = pc_Icon->Get(i_OriginalX,i_OriginalY);
            pc_Image->setPixel(i_RunX ,
                            i_RunY ,
                            p);
         } // for X
      } // for Y

      m_Pixmap->convertFromImage(*pc_Image,QPixmap::Color);
      setPixmap(*m_Pixmap);
      delete pc_Image;
      delete pc_Icon;

   }  // else icon available


} // previewUrl()

//============================================================================
//
//============================================================================

