#ifndef _FILTERBOX_H
#define _FILTERBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>


#include "StandardDialog.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "QUTreePath.h"
#include "QUIconButton.h"

#include "Tracer.h"
#include "Filter.h"
#include "PericoFileWriter.h"
#include "XenoHandler.h"

class MainFrame;


class FilterBox :  public StandardDialog ,
                  public Tracer
{
Q_OBJECT

public:
   FilterBox(QWidget *parent=0,
            GlobalStorage *pc_Global=0,
              MainFrame *pc_MainFrame=NULL);
   ~FilterBox();
   

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

              
public slots:

   void ActivateRegular();
   void ActivateXeno();
   
   void FilterWidthChanged(int i_Width);

   void NearDistanceChanged(int i_Distance);
   void FilterOnlyNearClicked(bool b_On);

   void OnlyOneDistanceChanged(int i_Distance);
   void FilterOnlyOneClicked(bool b_On);
   void OnlyOneColorChanged(Pixel p_NewColor);

   void ActivateMiddle();
   void ActivateMedian();
   void ActivateSharp();

   void ActivateXenoFilter();

   void XenoSelectionHasChanged(QString qs_Name);
   

private:
   void SetWidgetToMode(int i_Mode);

   void PrepareFiltering(int i_FilterType);
   void EndFiltering();

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);


   QHGroupBox     *m_ActivationBox;
   QUIconButton   *m_RegularModeButton;
   QUIconButton   *m_XenoModeButton;


   QVGroupBox   *m_FilterStyleBox;
   QHGroupBox   *m_RegularActivationBox;
   

   QULabelSlider * m_FilterWidthSlider;
   QPushButton   * m_MiddleButton;
   QPushButton   * m_MedianButton;
   QPushButton   * m_SharpButton;
   
   
   QVGroupBox    * m_NearBox;
   QCheckBox     * m_OnlyFilterNearColorsCheck;
   QULabelSlider * m_NearDistanceSlider;

   QVGroupBox    * m_OnlyOneColorBox;
   QCheckBox     * m_OnlyOneColorCheck;
   QULabelSlider * m_OnlyOneColorDistanceSlider;
   QUColorDragLabeledWidget * m_OnlyOneColorButton;

   QVGroupBox    * m_XenoParameterBox;
   // QComboBox     * m_XenoKernelSelectionCombo;
   QPushButton   * m_XenoStartButton;

   Filter        * m_pc_Filter;
   XenoHandler   * m_pc_Xenohandler;
   QStringList   * m_pqsl_XenoFilterNames;
   QString         m_qs_XenoFilterName;
   QUTreePath    * m_FilterTreePath;

   int             m_i_FilterWidth;
   int             m_i_NearDistance;
   bool            m_b_NearEnabled;

   bool            m_b_OnlyOneColorActive;
   int             m_i_OnlyOneColorDistance;
   Pixel           m_p_OnlyOneColor;

   enum
   {
      ModeRegular,
      ModeXeno
   };

   int m_i_Mode;
   

};


#endif // _FILTERBOX_H
