#ifndef _GEOMETRY_H
#define _GEOMETRY_H

#include <stdio.h>

// utility class to work with geometrical coordinates and list
// of coordinates

#include "PList.h"
#include "RectRegion.h"

class Geometry
{
public:
   static double AngleRadianToDegree(double d_RadianAngle);
   static double AngleDegreeToRadian(double d_DegreeAngle);
   static double AngleNormalize(double d_RadianAngle);
};


class GeoCoordinate
{
   // allow RectRegion to access my private members
   friend class RectRegion;

public:
   GeoCoordinate();
   GeoCoordinate(float f_X, float f_Y);
   ~GeoCoordinate();

   void SetXY(float f_X, float f_Y);
   void SetX( float f_X);
   void SetY( float f_Y);
   
   float GetX();
   float GetY();

   void Transform(float f_DifferenceX, float f_DifferenceY);
   void Multiply(float f_FaktorX, float f_FaktorY);

   float DifferenceTo(GeoCoordinate c_Other);
   float DifferenceTo(float f_X, float f_Y);
   float AngleTo(float f_X, float f_Y);
   float AngleTo(GeoCoordinate c_Other);

   void printf();
private:
   float m_f_X;
   float m_f_Y;

}; // GeoCoordinate



class GeoCoordinateList
{

public:
   GeoCoordinateList();

   GeoCoordinateList(int i_InitialSize);

   int GetCount();
   int AddElement(GeoCoordinate c_Element);
   int InsertElement(GeoCoordinate c_Element,int i_Position);

   GeoCoordinate Get(int i_Index);
   void Set(int i_Index,GeoCoordinate c_Element);

   void Transform(float f_DifferenceX, float f_DifferenceY);

   void Clean();
   void RemoveElement(int i_Index);



private:
   PMemArray<GeoCoordinate> m_CoordinateList;

   int m_i_NumberOfElements;
   int m_i_NumberInArray;

}; // GeoCoordinateList


class GeoLine
{
public:
   GeoLine();
   GeoLine(float f_StartX,float f_StartY,float f_EndX,float f_EndY);
   GeoLine(GeoCoordinate c_Start,GeoCoordinate c_End);

   void Set(float f_StartX,float f_StartY,float f_EndX,float f_EndY);
   void Set(GeoCoordinate c_Start,GeoCoordinate c_End);


   GeoCoordinate GetStartPoint();
   GeoCoordinate GetEndPoint();
   void Get( float &f_StartX, float &f_StartY, float &f_EndX,float &f_EndY);

   void Transform(float f_DifferenceX, float f_DifferenceY);

   bool CutsLine(GeoLine c_SecondLine, GeoCoordinate &c_CutPoint);
   bool CutsLineExtended(GeoLine c_SecondLine, GeoCoordinate &c_CutPoint,
                              float *pf_CuttingFactorFirst  = NULL,
                              float *pf_CuttingFactorSecond = NULL
                              );
   
   bool ClipInRectRegion(RectRegion c_Region,GeoLine &c_ResultLine);

   double PerpendicularOfPointCutsAt(GeoCoordinate c_Point,
                                     GeoCoordinate &c_CutPoint);

   bool IsOnSameSideOfLine(GeoCoordinate c_Point1, GeoCoordinate c_Point2);

private:
   GeoCoordinate m_Start;
   GeoCoordinate m_End;

   float CalcDeterminante(float x1,float y1,float x2,float y2);
   
   bool ClipInRectRegionHorizontal(RectRegion c_Region,GeoLine &c_ResultLine);

   bool ClipInRectRegionVertical(RectRegion c_Region,GeoLine &c_ResultLine);

} ; // GeoLine


#endif  // _GEOMETRY_H

