
#include <stdio.h>
#include <stdlib.h>

#include "GraphicObject.h"
#include "GraphicObjectList.h"
#include "GraphicObjectAttributeList.h"

//===========================================================================
//
//===========================================================================

GraphicObject::GraphicObject()
{

   m_pc_Attributes = new GraphicObjectAttributeList;
   // no group
   m_b_IsGroup = false;
   // and no childs
   m_ChildObjects= NULL;

   // default : not selected
   m_b_IsSelected = false;

   // no type so far
   m_i_ObjectType = TypeInvalid;
   
   // initital no chached size
   m_b_CachedSizeValid = false;
   
   // set some default object attributes
   m_b_IsResizeable  = false;
   m_b_IsSelectable = true;
   m_b_IsVisible     = true;


} // GraphicObject()

//===========================================================================
//
//===========================================================================

GraphicObject::GraphicObject(PericoFileWriter *pc_FileWriter)
{
   m_pc_Attributes = new GraphicObjectAttributeList;

   // no group
   m_b_IsGroup = false;
   // and no childs
   m_ChildObjects= NULL;

   // default : not selected
   m_b_IsSelected = false;

   // no type so far
   m_i_ObjectType = TypeInvalid;

   // initital no chached size
   m_b_CachedSizeValid = false;

   // set some default object attributes
   m_b_IsResizeable  = false;
   m_b_IsSelectable  = true;
   m_b_IsVisible     = true;

   LoadFile(pc_FileWriter);
   
} // GraphicObject()

//===========================================================================
//
//===========================================================================

GraphicObject::~GraphicObject()
{
   if(m_ChildObjects != NULL)
   {
      delete m_ChildObjects;
   }
   delete m_pc_Attributes;
} // ~GraphicObject()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObject::GetAttribute(int e_AttributeType)
{
   return m_pc_Attributes->GetAttribute( e_AttributeType);

} // GetAttribute()

//===========================================================================
//
//===========================================================================

int GraphicObject::GetObjectType()
{
   return m_i_ObjectType;

} // GetObjectType()

//===========================================================================
//
//===========================================================================

void GraphicObject::MoveTo( float f_OriginX, float f_OriginY)
{
} // MoveTo

//===========================================================================
//
//===========================================================================

void GraphicObject::MoveOffset( float f_OffsetX, float f_OffsetY)
{
} // MoveOffset()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetColor(Pixel p_Color)
{
   GraphicObjectAttribute *pc_ColorAttribute;
   // look whether available
   pc_ColorAttribute = GetAttribute(GraphicObjectAttribute::Color);
   if(pc_ColorAttribute == NULL)
   {
      // create new
      pc_ColorAttribute = new GraphicObjectAttribute;
      pc_ColorAttribute->m_e_ObjectAttributeType = GraphicObjectAttribute::Color;
      pc_ColorAttribute->m_p_Color = p_Color;
      // add to list
      m_pc_Attributes->AddAttribute(pc_ColorAttribute);
   }
   else
   {
      // modify existing
      pc_ColorAttribute->m_p_Color = p_Color;
   }
} // SetColor()

//===========================================================================
//
//===========================================================================

Pixel GraphicObject::GetColor()
{
   GraphicObjectAttribute *pc_ColorAttribute;
   // look whether available
   pc_ColorAttribute = GetAttribute(GraphicObjectAttribute::Color);

   if(pc_ColorAttribute == NULL)
   {
      return 0;
   }

   return pc_ColorAttribute->m_p_Color;

} // GetColor()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObject::GetSize()
{
   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();
   m_b_CachedSizeValid = true;
   return m_CachedSize;

} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetSize(RectRegion c_NewSize)
{
   // do nothing on a empty object.
   // has to be overloaded
} // SetSize()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsSelected()
{
   return m_b_IsSelected;
} // IsSelected()

//===========================================================================
//
//===========================================================================

void GraphicObject::SelectObject(bool b_State)
{
   m_b_IsSelected = b_State;
} // SelectObject()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsResizeable()
{
   return m_b_IsResizeable;
} // IsResizeable()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsSelectable()
{
   return m_b_IsSelectable;
} // IsSelectable()

//===========================================================================
//
//===========================================================================


void GraphicObject::SetSelectable(bool b_Selectable)
{
   m_b_IsSelectable = b_Selectable;
} // SetSelectable()

//===========================================================================
//
//===========================================================================

bool GraphicObject::IsVisible()
{
   return m_b_IsVisible;
} // IsVisible()

//===========================================================================
//
//===========================================================================

void GraphicObject::SetVisible(bool b_Visible)
{
   m_b_IsVisible = b_Visible;
} // SetVisible()

//===========================================================================
//
//===========================================================================


Pixel GraphicObject::GetPixelAt(int i_X, int i_Y, bool &b_Found)
{
   b_Found = false;

   return 0;

} // GetPixelAt()

//===========================================================================
//
//===========================================================================

int GraphicObject::WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name)
{
   int i_Count;
   int i;
   
   pc_FileWriter->StartStructure(qs_Name,GetObjectTypeString());

   pc_FileWriter->AddValue("ObjectType",m_i_ObjectType);

   pc_FileWriter->AddValue("IsGroup",
                           m_b_IsGroup?1:0);
   pc_FileWriter->AddValue("IsResizeable",
                           m_b_IsResizeable?1:0);
   pc_FileWriter->AddValue("IsSelectable",
                           m_b_IsSelectable?1:0);
   pc_FileWriter->AddValue("IsVisible",
                           m_b_IsVisible?1:0);

   i_Count = m_pc_Attributes->GetCount();
   for(i=0;i<i_Count;i++)
   {
      GraphicObjectAttribute *pc_Attribute;
      QString qs_AttributeName;

      pc_Attribute = m_pc_Attributes->GetAttributeByIndex(i);
      qs_AttributeName.sprintf("Attribute%d",i);

      pc_Attribute->WriteFile(pc_FileWriter,qs_AttributeName);
      
   }
                                                      
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;


} // WriteFile()

//===========================================================================
//
//===========================================================================

int GraphicObject::LoadFile(PericoFileWriter *pc_FileWriter)
{
   PERICO_READ_STRUCTURE_START

   if( qs_Name.compare("ObjectType") == 0)
   {
      m_i_ObjectType = atoi((const char *) qs_Value);
   }
   else if( qs_Name.compare("IsGroup") == 0)
   {
     m_b_IsGroup =  atoi((const char *) qs_Value) ? true : false;
   }
   else if( qs_Name.compare("IsResizeable") == 0)
   {
     m_b_IsResizeable =  atoi((const char *) qs_Value) ? true : false;
   }
   else if( qs_Name.compare("IsSelectable") == 0)
   {
     m_b_IsSelectable =  atoi((const char *) qs_Value) ? true : false;
   }
   else if( qs_Name.compare("IsVisible") == 0)
   {
     m_b_IsVisible =  atoi((const char *) qs_Value) ? true : false;
   }
   else if(     qs_Value.compare("GraphicObjectAttribute") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObjectAttribute *pc_Attribute;
      pc_Attribute = new GraphicObjectAttribute(pc_FileWriter);
      m_pc_Attributes->AddAttribute(pc_Attribute);
   }

   PERICO_READ_STRUCTURE_END

   return 0;
} // LoadFile()

//===========================================================================
//
//===========================================================================

QString GraphicObject::GetObjectTypeString()
{
   return "GraphicObject";
} // GetObjectType()

//===========================================================================
//
//===========================================================================

