#ifndef _GRAPHIC_OBJECT_H
#define _GRAPHIC_OBJECT_H

#include <qstring.h>

#include "PTypes.h"
#include "Geometry.h"

#include "GlobalStorage.h"
#include "RectRegion.h"

#include "GraphicObjectAttribute.h"


//#########################################################################

class GraphicObjectList;
class GraphicObjectAttributeList;

class GraphicObject
{
public:
   GraphicObject();
   GraphicObject(PericoFileWriter *pc_FileWriter);
   virtual ~GraphicObject();
   GraphicObjectAttribute *GetAttribute(int e_AttributeType);

   // different types of objects
   enum
   {
       TypeInvalid =0,
       TypePoint,
       TypeLine,
       TypeRectangle,
       TypeEllipse,
       TypeImage,
       TypeGroup,
       TypePolygon,
       TypeSpline
   };


   int GetObjectType();
   virtual void SetColor(Pixel p_Color);
   virtual Pixel GetColor();

   virtual bool IsResizeable();
   virtual bool IsSelected();
   virtual void SelectObject(bool b_State);
   
   virtual bool IsSelectable();
   virtual bool IsVisible();
   virtual void SetSelectable(bool b_Selectable);
   virtual void SetVisible(bool b_Visible);

   virtual void MoveTo( float f_OriginX, float f_OriginY);
   virtual void MoveOffset( float f_OffsetX, float f_OffsetY);
   virtual RectRegion GetSize();
   virtual void SetSize(RectRegion c_NewSize);
   virtual Pixel GetPixelAt(int i_X, int i_Y, bool &b_Found);
   virtual int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   virtual int LoadFile(PericoFileWriter *pc_FileWriter);
   virtual QString GetObjectTypeString();

protected:

   int m_i_ObjectType;
   GraphicObjectAttributeList *m_pc_Attributes;
   bool m_b_IsGroup;
   bool m_b_IsSelected;
   bool m_b_IsResizeable;
   bool m_b_IsSelectable;
   bool m_b_IsVisible;


   GraphicObjectList *m_ChildObjects;

   RectRegion m_CachedSize;
   bool       m_b_CachedSizeValid;
   
};


#endif //  _GRAPHIC_OBJECT_H

