//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: GraphicObjectAttribute
//---------------------------------------------------------------------------
// Purpose: different attributes for graphic objects
//---------------------------------------------------------------------------

#include <stdlib.h>

#include "GraphicObjectAttribute.h"


//===========================================================================
//
//===========================================================================

GraphicObjectAttribute::GraphicObjectAttribute()
{
   m_e_ObjectAttributeType = Invalid;
   m_pc_ImageData          = NULL;
} // GraphicObjectAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute::GraphicObjectAttribute(PericoFileWriter *pc_FileWriter)
{
   m_e_ObjectAttributeType = Invalid;
   m_pc_ImageData          = NULL;

   LoadFile(pc_FileWriter);
   
} // GraphicObjectAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute::~GraphicObjectAttribute()
{
   if( m_pc_ImageData != NULL)
   {
      delete m_pc_ImageData;
      m_pc_ImageData = NULL;
   }
} // ~GraphicObjectAttribute()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeStartpoint(
                                 GeoCoordinate c_Coordinate)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = StartPoint;
   p_Return->m_c_Coordinate          = c_Coordinate;

   return p_Return;

} // GraphicAttributeStartpoint()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeEndpoint(
                                 GeoCoordinate c_Coordinate)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = EndPoint;
   p_Return->m_c_Coordinate          = c_Coordinate;

   return p_Return;

} // GraphicAttributeEndpoint()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeColor(
                                 Pixel p_TheColor)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = Color;
   p_Return->m_p_Color               = p_TheColor;

   return p_Return;

} // GraphicAttributeColor()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeRotationAngel(
                                 double d_Angle)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = RotationAngle;
   p_Return->m_d_Value               = d_Angle;
   
   return p_Return;

} //  GraphicAttributeRorationAngel()

//===========================================================================
//
//===========================================================================

GraphicObjectAttribute *GraphicObjectAttribute::GraphicAttributeImage(
                                                    PixelBuffer *pc_Image)
{
   GraphicObjectAttribute *p_Return;

   p_Return = new GraphicObjectAttribute();
   p_Return->m_e_ObjectAttributeType = Image;
   p_Return->m_pc_ImageData          = pc_Image;

   return p_Return;

} // GraphicAttributeImage();

//===========================================================================
//
//===========================================================================

Pixel GraphicObjectAttribute::GetColor()
{
   if( m_e_ObjectAttributeType != Color)
   {
      return 0;
   }
   return m_p_Color;

} // GetColor()

//===========================================================================
//
//===========================================================================

void  GraphicObjectAttribute::SetColor(Pixel p_Color )
{
   if( m_e_ObjectAttributeType != Color)
   {
      return ;
   }
   m_p_Color = p_Color;
} //  SetColor()

//===========================================================================
//
//===========================================================================

double GraphicObjectAttribute::GetRotationAngle()
{
   if( m_e_ObjectAttributeType != RotationAngle)
   {
      return 0.0;
   }
   return m_d_Value ;

} // GetRotationAngle()

//===========================================================================
//
//===========================================================================

void   GraphicObjectAttribute::SetRotationAngle(double d_Angle)
{
   if( m_e_ObjectAttributeType != RotationAngle)
   {
      return;
   }
   m_d_Value = d_Angle;
   
} // SetRotationAngle()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectAttribute::GetStartPoint()
{
   if(m_e_ObjectAttributeType != StartPoint)
   {
      return GeoCoordinate(0,0);
   }
   return m_c_Coordinate;

} // GetStartPoint()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectAttribute::GetEndPoint()
{
   if(m_e_ObjectAttributeType != EndPoint)
   {
      return GeoCoordinate(0,0);
   }
   return m_c_Coordinate;

} // GetEndPoint()

//===========================================================================
//
//===========================================================================

int GraphicObjectAttribute::WriteFile(PericoFileWriter *pc_FileWriter,
                                      QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"GraphicObjectAttribute");

   pc_FileWriter->AddValue("AttributeType",m_e_ObjectAttributeType);

   switch(m_e_ObjectAttributeType)
   {
   case Color:
      pc_FileWriter->AddValue("Color",(int)m_p_Color);
      break;
   case StartPoint:
      pc_FileWriter->AddValue("Coordinate.X",m_c_Coordinate.GetX());
      pc_FileWriter->AddValue("Coordinate.Y",m_c_Coordinate.GetY());
      break;
   case EndPoint:
      pc_FileWriter->AddValue("Coordinate.X",m_c_Coordinate.GetX());
      pc_FileWriter->AddValue("Coordinate.Y",m_c_Coordinate.GetY());
      break;
   case Image:
      m_pc_ImageData->WriteFile(pc_FileWriter,"Image");
      break;
   case RotationAngle:
      pc_FileWriter->AddValue("RotationAngle",m_d_Value);
      break;
   case AllowsLineEndMovement:
      pc_FileWriter->AddValue("AllowsLineEndMovement",m_i_Value);
      break;
   case PointList:
      // here a more complex attribute
      // first the count, then the list of points
      WritePointList(pc_FileWriter);
      break;
   case ClosedFigure:
      pc_FileWriter->AddValue("ClosedFigure",m_i_Value);
      break;
   default:
      break;
   }

   
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;


} // :WriteFile()

//===========================================================================
//
//===========================================================================

int GraphicObjectAttribute::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if( qs_Name.compare("AttributeType") == 0)
   {
      m_e_ObjectAttributeType = atoi((const char *) qs_Value);
   }
   else if( qs_Name.compare("Color") == 0)
   {
      m_p_Color = (Pixel) atoi((const char *) qs_Value);
   }
   else if( qs_Name.compare("Coordinate.X") == 0)
   {
      m_c_Coordinate.SetX( atof((const char *) qs_Value));
   }
   else if( qs_Name.compare("Coordinate.Y") == 0)
   {
      m_c_Coordinate.SetY( atof((const char *) qs_Value));
   }
   else if( qs_Name.compare("RotationAngle") == 0)
   {
      m_d_Value = atof((const char *) qs_Value);
   }
   else if( qs_Name.compare("AllowsLineEndMovement") == 0)
   {
      m_i_Value = atoi((const char *) qs_Value);
   }
   else if(    qs_Name.compare("Image") == 0
            && qs_Value.compare("PixelBuffer") == 0
            && b_IsStartStructure)
   {
      m_pc_ImageData = new PixelBuffer(pc_FileWriter);
   }

   else if(    qs_Name.compare("PointList") == 0
            && qs_Value.compare("GeoCoordinateList") == 0
            && b_IsStartStructure)
   {
      LoadPointList(pc_FileWriter);
   }
   else if( qs_Name.compare("ClosedFigure") == 0)
   {
      m_i_Value = atoi((const char *) qs_Value);
   }
   
   PERICO_READ_STRUCTURE_END

   // no problem
   return 0;
} // LoadFile()

//===========================================================================
//
//===========================================================================

GeoCoordinateList *GraphicObjectAttribute::GetCoordinateList()
{
   return &m_l_Coordinates;
} // GetCoordinateList()

//===========================================================================
//
//===========================================================================

int GraphicObjectAttribute::WritePointList(PericoFileWriter *pc_FileWriter)
{
   int i_NumberOfPoints;
   int i;
   
   i_NumberOfPoints = m_l_Coordinates.GetCount();
   pc_FileWriter->StartStructure("PointList","GeoCoordinateList");

   pc_FileWriter->AddValue("NumberOfPoints",i_NumberOfPoints);

   for(i=0;i<i_NumberOfPoints;i++)
   
   {
      GeoCoordinate c_CurrentCoordinate;
      QString       qs_Value;

      c_CurrentCoordinate = m_l_Coordinates.Get(i);
      qs_Value.sprintf("%f,%f",
                       c_CurrentCoordinate.GetX(),
                       c_CurrentCoordinate.GetY());
      pc_FileWriter->AddValue("Point",qs_Value);

   }   

   pc_FileWriter->FinishStructure();

   // all ok
   return 0;
} // WritePointList()

//===========================================================================
//
//===========================================================================


int GraphicObjectAttribute::LoadPointList(PericoFileWriter *pc_FileWriter)
{
   GeoCoordinate c_Coordinate;
   float         f_X;
   float         f_Y;
   
   m_l_Coordinates.Clean();
   
   PERICO_READ_STRUCTURE_START

   if( qs_Name.compare("NumberOfPoints") == 0)
   {
      // currently ignore it
   }
   else if( qs_Name.compare("Point") == 0)
   {
      // extract a point
      f_X=0;
      f_Y=0;
      sscanf((const char *)qs_Value,"%f,%f",& f_X,&f_Y);
      c_Coordinate.SetXY(f_X,f_Y);
      m_l_Coordinates.AddElement(c_Coordinate);
      
      // currently ignore it
   }

   PERICO_READ_STRUCTURE_END

   // no problem
   return 0;
} // LoadPointList()

//===========================================================================
//
//===========================================================================

