#ifndef _GRAPHIC_OBJECT_LAYER_H
#define _GRAPHIC_OBJECT_LAYER_H


#include "GraphicObjectAttribute.h"
#include "GraphicObject.h"
#include "GraphicObjectEllipse.h"
#include "GraphicObjectImage.h"
#include "GraphicObjectLine.h"
#include "GraphicObjectPoint.h"
#include "GraphicObjectRectangle.h"
#include "GraphicObjectPolygon.h"
#include "GraphicObjectSpline.h"
#include "PericoFileWriter.h"

//#########################################################################

class GraphicObjectLayer
{
public:
   GraphicObjectLayer(QString qs_LayerName);
   GraphicObjectLayer(PericoFileWriter *pc_FileWriter);
   ~GraphicObjectLayer();

   void AddObject(GraphicObject *pc_NewObject);
   int  GetObjectCount();
   GraphicObject *GetObject(int i_Index);
   bool IsVisible();
   void SetVisible(bool b_Visible);
   
   GraphicObject *SearchObjectAt(float f_PositionX,float f_PositionY,
                                 GraphicObject *pc_StartObject=NULL);


   GraphicObject *GetLastSelectedObject();                                                                  

   QString GetName();
   
   int GetNumberOfSelectedObjects(GraphicObject * & pc_FirstObject);
   GraphicObject *GetNextSelectedObject(GraphicObject *pc_PreviousObject);
   bool CursorOverSelectedObjects(int i_CursorX, int i_CursorY);
   
   void MoveSelectedObjects(int i_DifferenceX, int i_DifferenceY,
                            RectRegion &c_ChangedRegion);

   void DeleteObject( GraphicObject *pc_ObjectToDelete);
   void DeleteAllObjects( );
   void RemoveObject( GraphicObject *pc_ObjectToDelete);
   void RemoveAllObjects( );

   void SetObjectOrderBottom(GraphicObject *pc_Object);
   void SetObjectOrderTop(GraphicObject *pc_Object);

   Pixel GetPixelAt(int i_X, int i_Y, bool &b_Found);

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);
   

private: // variables

   bool m_b_IsActive;
   bool m_b_IsVisible;

   GraphicObjectList *m_Objects;
   GraphicObjectList *m_SelectedObjects;

   // every layer gets a name
   QString m_qs_LayerName;


}; // GraphicObjectLayer

#endif //  _GRAPHIC_OBJECT_LAYER_H
