
#include <stdio.h>

#include "GraphicObjectLine.h"
#include "GraphicObjectAttributeList.h"

//===========================================================================
//
//===========================================================================

GraphicObjectLine :: GraphicObjectLine(float f_StartX, float f_StartY,
                                             float f_EndX, float f_EndY)
                                      :GraphicObject()
{
   Initialize(f_StartX, f_StartY, f_EndX,  f_EndY);
} // GraphicObjectLine()

//===========================================================================
//
//===========================================================================

GraphicObjectLine :: GraphicObjectLine(GeoCoordinate c_Start,GeoCoordinate c_End)
{
   Initialize(c_Start.GetX(),c_Start.GetY(),
              c_End.GetX()  ,c_End.GetY() );
   
} // GraphicObjectLine()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::Initialize(float f_StartX, float f_StartY,
                     float f_EndX,   float f_EndY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   // store the object type
   m_i_ObjectType = TypeLine;

   m_StartPoint = new GraphicObjectAttribute();
   m_EndPoint   = new GraphicObjectAttribute();

   if(    m_StartPoint == NULL
       || m_EndPoint   == NULL
     )
   {
      return ;
   }

   // create a startpoint
   m_StartPoint->m_e_ObjectAttributeType = GraphicObjectAttribute::StartPoint;

   m_StartPoint->m_c_Coordinate.SetXY(f_StartX, f_StartY);

   m_pc_Attributes->AddAttribute(m_StartPoint);

   // create a endpoint
   m_EndPoint->m_e_ObjectAttributeType = GraphicObjectAttribute::EndPoint;

   m_EndPoint->m_c_Coordinate.SetXY(f_EndX, f_EndY);

   m_pc_Attributes->AddAttribute(m_EndPoint);

   m_b_IsResizeable = true;

} // Initialize()

//===========================================================================
//
//===========================================================================

GraphicObjectLine :: GraphicObjectLine(PericoFileWriter *pc_FileWriter)
{
   LoadFile(pc_FileWriter);
} // GraphicObjectImage()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObjectLine::GetSize()
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return m_CachedSize;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   m_CachedSize.SetXYXY(c_StartPosition.GetX(),
                    c_StartPosition.GetY(),
                    c_EndPosition.GetX(),
                    c_EndPosition.GetY()
                    );

   m_b_CachedSizeValid = true;

   return m_CachedSize;
} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetSize(RectRegion c_NewSize)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   

   // disable the cache size
   m_b_CachedSizeValid = false;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return ;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return ;
   }

   if(m_StartPoint->m_c_Coordinate.GetX() < m_EndPoint->m_c_Coordinate.GetX())
   {
      if(m_StartPoint->m_c_Coordinate.GetY() < m_EndPoint->m_c_Coordinate.GetY())
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetStartYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetEndYFloat());
      }
      else //startY > endY
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetEndYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetStartYFloat());
      }
   }
   else // startX > endX
   {
      if(m_StartPoint->m_c_Coordinate.GetY() < m_EndPoint->m_c_Coordinate.GetY())
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetStartYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetEndYFloat());
      }
      else //startY > endY
      {
         m_StartPoint->m_c_Coordinate.SetXY(c_NewSize.GetEndXFloat(),
                                      c_NewSize.GetEndYFloat());
         m_EndPoint  ->m_c_Coordinate.SetXY(c_NewSize.GetStartXFloat(),
                                      c_NewSize.GetStartYFloat());
      }
   }

} // SetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::MoveTo(float f_OriginX, float f_OriginY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   double                   d_OffsetX;
   double                   d_OffsetY;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   d_OffsetX = c_EndPosition.GetX() - c_StartPosition.GetX();
   d_OffsetY = c_EndPosition.GetY() - c_StartPosition.GetY();

   m_StartPoint->m_c_Coordinate.SetXY(f_OriginX, f_OriginY);
   m_EndPoint  ->m_c_Coordinate.SetXY(f_OriginX+d_OffsetX,
                                    f_OriginY+d_OffsetY);

   m_b_CachedSizeValid = false;
} // MoveTo()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::MoveOffset( float f_OffsetX, float f_OffsetY)
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }

   m_StartPoint->m_c_Coordinate.Transform(f_OffsetX,f_OffsetY);
   m_EndPoint  ->m_c_Coordinate.Transform(f_OffsetX,f_OffsetY);


   m_b_CachedSizeValid = false;

} // MoveOffset()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetPosition(float f_StartX, float f_StartY,
                                    float f_EndX,   float f_EndY   )
{
   GraphicObjectAttribute * m_StartPoint;
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_StartPosition;
   GeoCoordinate            c_EndPosition;

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return;
   }
   c_StartPosition= m_StartPoint->m_c_Coordinate;

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint ==NULL)
   {
      return;
   }
   c_EndPosition= m_EndPoint->m_c_Coordinate;

   m_StartPoint->m_c_Coordinate.SetXY(f_StartX, f_StartY);
   m_EndPoint  ->m_c_Coordinate.SetXY(f_EndX,f_EndY);

   m_b_CachedSizeValid = false;

} // SetPosition()

//===========================================================================
//
//===========================================================================

bool GraphicObjectLine::AllowsLineEndMovement()
{
   GraphicObjectAttribute * m_AllowsMovement;

   m_AllowsMovement = GetAttribute(GraphicObjectAttribute::AllowsLineEndMovement);
   if(m_AllowsMovement ==NULL)
   {
      return false;
   }
   if(m_AllowsMovement->m_i_Value)
   {
      return true;
   }
   return false;

} // AllowsLineEndMovement()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetAllowsLineEndMovement(bool b_Allow)
{
   GraphicObjectAttribute * m_AllowsMovement;

   m_AllowsMovement = GetAttribute(GraphicObjectAttribute::AllowsLineEndMovement);

   if(m_AllowsMovement == NULL)
   {
      m_AllowsMovement = new GraphicObjectAttribute();

      // error in creation ?
      if(m_AllowsMovement ==NULL)
      {
         return;
      }

      // set attribute type
      m_AllowsMovement->m_e_ObjectAttributeType
                       = GraphicObjectAttribute::AllowsLineEndMovement;

      // add it to the list                       
      m_pc_Attributes->AddAttribute(m_AllowsMovement);
   }

   // set i_value depending on the flag
   if(b_Allow)
   {
      m_AllowsMovement->m_i_Value =1;
   }
   else
   {
      m_AllowsMovement->m_i_Value =0;
   }
   

} // SetAllowsLineEndMovement()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectLine::GetStartPoint()
{
   GraphicObjectAttribute * m_StartPoint;
   GeoCoordinate            c_Return;

   c_Return.SetXY(0,0);

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint !=NULL)
   {
      c_Return= m_StartPoint->m_c_Coordinate;
   }

   return c_Return;
   
} // GetStartPoint()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectLine::GetEndPoint()
{
   GraphicObjectAttribute * m_EndPoint;
   GeoCoordinate            c_Return;

   c_Return.SetXY(0,0);

   m_EndPoint = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_EndPoint !=NULL)
   {
      c_Return= m_EndPoint->m_c_Coordinate;
   }

   return c_Return;
} // GetEndPoint()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetStartPoint(GeoCoordinate c_Start)
{
   GraphicObjectAttribute * m_Point;

   m_Point = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_Point ==NULL)
   {
      return;
   }
   m_Point->m_c_Coordinate = c_Start;

   m_b_CachedSizeValid = false;
} // SetStartPoint()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetEndPoint  (GeoCoordinate c_End)
{
   GraphicObjectAttribute * m_Point;

   m_Point = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_Point ==NULL)
   {
      return;
   }
   m_Point->m_c_Coordinate= c_End;

   m_b_CachedSizeValid = false;

} // SetEndPoint()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetStartPoint(float f_StartX, float f_StartY)
{
   GraphicObjectAttribute * m_Point;

   m_Point = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_Point ==NULL)
   {
      return;
   }
   m_Point->m_c_Coordinate.SetXY(f_StartX,f_StartY);

   m_b_CachedSizeValid = false;
} // SetStartPoint()

//===========================================================================
//
//===========================================================================

void GraphicObjectLine::SetEndPoint  (float f_EndX,   float f_EndY)
{
   GraphicObjectAttribute * m_Point;

   m_Point = GetAttribute(GraphicObjectAttribute::EndPoint);
   if(m_Point ==NULL)
   {
      return;
   }
   m_Point->m_c_Coordinate.SetXY(f_EndX,f_EndY);

   m_b_CachedSizeValid = false;

} // SetEndPoint()

//===========================================================================
//
//===========================================================================

GeoLine GraphicObjectLine::GetLine()
{
   GeoCoordinate c_Start;
   GeoCoordinate c_End;
   GeoLine       c_Return;

   c_Start = GetStartPoint();
   c_End   = GetEndPoint();

   c_Return.Set(c_Start,c_End);

   return c_Return;

} // GetLine()

//===========================================================================
//
//===========================================================================

QString GraphicObjectLine::GetObjectTypeString()
{
   return "GraphicObjectLine";
} // GetObjectType()

//===========================================================================
//
//===========================================================================

