//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: GraphicObjectMultipoint
//---------------------------------------------------------------------------
// Purpose: functions for the multipoint class  objects
//---------------------------------------------------------------------------

#include <stdio.h>

#include "GraphicObjectMultipoint.h"
#include "GraphicObjectAttributeList.h"

//===========================================================================
//
//===========================================================================

GraphicObjectMultipoint :: GraphicObjectMultipoint()
                       : GraphicObject()
{
   GraphicObjectAttribute   *pc_ListAttribute;
   GraphicObjectAttribute   *pc_ClosedFigureAttribute;

   pc_ListAttribute = new GraphicObjectAttribute();

   // create a pointlist
   pc_ListAttribute->m_e_ObjectAttributeType = GraphicObjectAttribute::PointList;
   m_pc_Attributes->AddAttribute(pc_ListAttribute);


   pc_ClosedFigureAttribute = new GraphicObjectAttribute();

   // create a pointlist
   pc_ClosedFigureAttribute->m_e_ObjectAttributeType = GraphicObjectAttribute::ClosedFigure;
   pc_ClosedFigureAttribute->m_i_Value = 0; // Open
   m_pc_Attributes->AddAttribute(pc_ClosedFigureAttribute);
   m_b_IsResizeable = true;

} // GraphicObjectMultipoint()

//===========================================================================
//
//===========================================================================

GraphicObjectMultipoint :: GraphicObjectMultipoint(PericoFileWriter *pc_FileWriter)
{
   LoadFile(pc_FileWriter);
} // GraphicObjectMultipoint()

//===========================================================================
//
//===========================================================================

GraphicObjectMultipoint :: ~GraphicObjectMultipoint()
{

}

//===========================================================================
//
//===========================================================================

GeoCoordinateList *GraphicObjectMultipoint ::GetCoordinateList()
{
   GeoCoordinateList *pc_CoordinateList;
   GraphicObjectAttribute   *pc_ListAttribute;

   pc_ListAttribute = m_pc_Attributes->GetAttribute(GraphicObjectAttribute::PointList);
   pc_CoordinateList = pc_ListAttribute->GetCoordinateList();

   return pc_CoordinateList;
} // GetCoordinateList()

//===========================================================================
//
//===========================================================================


RectRegion GraphicObjectMultipoint::GetSize()
{
   GeoCoordinateList *pc_CoordinateList;
   int                i;
   int                i_Count;
   
   if( m_b_CachedSizeValid)
   {
      return m_CachedSize;
   }

   m_CachedSize.SetEmpty();
   pc_CoordinateList = GetCoordinateList();
   i_Count = pc_CoordinateList->GetCount();
   for(i=0;i<i_Count;i++)
   {
      GeoCoordinate c_Coordinate;
      c_Coordinate = pc_CoordinateList->Get(i);
      m_CachedSize.AddCoordinate(&c_Coordinate);
   }
   
   return m_CachedSize;
} // GetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::SetSize(RectRegion c_NewSize)
{
   GeoCoordinateList *pc_CoordinateList;
   int                i;
   int                i_Count;

   float f_FactorX;
   float f_FactorY;
   float f_StartOldX;
   float f_StartOldY;
   float f_StartNewX;
   float f_StartNewY;
   
   RectRegion c_OldSize;
   c_OldSize = GetSize();

   f_StartOldX = - c_OldSize.GetStartXFloat();
   f_StartOldY = - c_OldSize.GetStartYFloat();
   
   f_StartNewX = c_NewSize.GetStartXFloat();
   f_StartNewY = c_NewSize.GetStartYFloat();

   // calculate the resizing factor in each dimension
   f_FactorX = c_NewSize.GetWidthFloat()  / c_OldSize.GetWidthFloat();
   f_FactorY = c_NewSize.GetHeightFloat() / c_OldSize.GetHeightFloat();

   // go through all the point   
   pc_CoordinateList = GetCoordinateList();
   i_Count = pc_CoordinateList->GetCount();
   for(i=0;i<i_Count;i++)
   {
      GeoCoordinate c_Coordinate;

      c_Coordinate = pc_CoordinateList->Get(i);

      // move in the old system to the origin (0,0)
      c_Coordinate.Transform(f_StartOldX,f_StartOldY);
      // multiply with sizing faktor
      c_Coordinate.Multiply( f_FactorX,f_FactorY);
      // move to startpoint in new system
      c_Coordinate.Transform(f_StartNewX,f_StartNewY);

      // write new value back into the list
      pc_CoordinateList->Set(i,c_Coordinate);
   }

   // the NewSize is also the new Region
   m_CachedSize        = c_NewSize;      
   m_b_CachedSizeValid = true;
} // SetSize()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::MoveTo(float f_OriginX, float f_OriginY)
{
   RectRegion c_CurrentSize;
   float f_OffsetX;
   float f_OffsetY;

   c_CurrentSize = GetSize();

   f_OffsetX = f_OriginX - c_CurrentSize.GetStartXFloat();
   f_OffsetY = f_OriginY - c_CurrentSize.GetStartYFloat();

   MoveOffset(f_OffsetX,f_OffsetY);
   
   m_b_CachedSizeValid = false;
} // MoveTo()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::MoveOffset( float f_OffsetX, float f_OffsetY)
{

   GeoCoordinateList *pc_CoordinateList;
   int                i;
   int                i_Count;

   pc_CoordinateList = GetCoordinateList();
   i_Count = pc_CoordinateList->GetCount();

   m_CachedSize.SetEmpty();
   // transform every coordinate point
   
   for(i=0;i<i_Count;i++)
   {
      GeoCoordinate c_Coordinate;
      c_Coordinate = pc_CoordinateList->Get(i);
      c_Coordinate.Transform(f_OffsetX,f_OffsetY);
      pc_CoordinateList->Set(i,c_Coordinate);

      // also recalculate the object size
      m_CachedSize.AddCoordinate(&c_Coordinate);
   }

   m_b_CachedSizeValid = true;

} // MoveOffset()

//===========================================================================
//
//===========================================================================


QString GraphicObjectMultipoint::GetObjectTypeString()
{
   return "GraphicObjectMultipoint";
} // GetObjectType()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::AddCoordinate(GeoCoordinate c_Coordinate)
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   pc_CoordinateList->AddElement(c_Coordinate);
   m_b_CachedSizeValid = false;

} // AddCoordinate()

//===========================================================================
//
//===========================================================================

int GraphicObjectMultipoint::GetCoordinateCount()
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   return pc_CoordinateList->GetCount();
   
} // GetCoordinateCount()

//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectMultipoint::GetCoordinate(int i_Index)
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   return pc_CoordinateList->Get(i_Index);

} // GetCoordinate()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::RemoveAllCoordinates()
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   pc_CoordinateList->Clean();

   m_CachedSize.SetEmpty();
   m_b_CachedSizeValid = true;

} // RemoveAllCoordinates()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::SetCoordinate(int i_Index,GeoCoordinate c_Coordinate)
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();


   pc_CoordinateList->Set(i_Index,c_Coordinate);
   m_b_CachedSizeValid = false;

} // SetCoordinate()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::InsertCoordinate(int i_Index,GeoCoordinate c_Coordinate)
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   pc_CoordinateList->InsertElement(c_Coordinate,i_Index);
   m_b_CachedSizeValid = false;

} // InsertCoordinate()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::RemoveCoordinate(int i_Index)
{
   GeoCoordinateList *pc_CoordinateList;
   pc_CoordinateList = GetCoordinateList();

   pc_CoordinateList->RemoveElement(i_Index);
   m_b_CachedSizeValid = false;

} // RemoveCoordinate()

//===========================================================================
//
//===========================================================================

void GraphicObjectMultipoint::SetClosedFigure(bool b_Closed)
{
   GraphicObjectAttribute   *pc_ClosedFigureAttribute;

   pc_ClosedFigureAttribute = m_pc_Attributes->GetAttribute(GraphicObjectAttribute::ClosedFigure);
   pc_ClosedFigureAttribute->m_i_Value = b_Closed ? 1 :0 ;

} // SetClosedFigure()

//===========================================================================
//
//===========================================================================

bool GraphicObjectMultipoint::IsClosedFigure()
{
   GraphicObjectAttribute   *pc_ClosedFigureAttribute;

   pc_ClosedFigureAttribute = m_pc_Attributes->GetAttribute(GraphicObjectAttribute::ClosedFigure);
   return pc_ClosedFigureAttribute->m_i_Value != 0;
   
} // IsClosedFigure()

//===========================================================================
//
//===========================================================================

