//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: GraphicObjectPoint.c
//---------------------------------------------------------------------------
// Purpose: Overloaded GraphicObject with special attributes
//          for points
//---------------------------------------------------------------------------

#include <stdio.h>

#include "GraphicObjectPoint.h"
#include "GraphicObjectAttributeList.h"


GraphicObjectPoint :: GraphicObjectPoint(float f_X, float f_Y)
                     :GraphicObject()
{
   Initialize(f_X,f_Y);
   
} // GraphicObjectPoint()

//===========================================================================
//
//===========================================================================

GraphicObjectPoint :: GraphicObjectPoint(GeoCoordinate c_Coordinate)
                     :GraphicObject()
{
   Initialize(c_Coordinate.GetX(),c_Coordinate.GetY());

} // GraphicObjectPoint()

//===========================================================================
//
//===========================================================================

void GraphicObjectPoint::Initialize(float f_X,float f_Y)
{
   GraphicObjectAttribute * m_StartPoint;

   // store the object type
   m_i_ObjectType = TypePoint;

   m_StartPoint = new GraphicObjectAttribute();

   if( m_StartPoint == NULL)
   {
      return ;
   }

   // create a startpoint
   m_StartPoint->m_e_ObjectAttributeType = GraphicObjectAttribute::StartPoint;

   m_StartPoint->m_c_Coordinate.SetXY(f_X, f_Y);

   m_pc_Attributes->AddAttribute(m_StartPoint);

   // currently no other attributes

} // Initialize()


//===========================================================================
//
//===========================================================================

GraphicObjectPoint :: GraphicObjectPoint(PericoFileWriter *pc_FileWriter)
{
   LoadFile(pc_FileWriter);
} // GraphicObjectImage()

//===========================================================================
//
//===========================================================================

void GraphicObjectPoint::GetPosition(float &f_X,float &f_Y)
{
   GeoCoordinate            c_Position;

   c_Position= GetPosition();
   f_X = c_Position.GetX();
   f_Y = c_Position.GetY();

} // GetPosition()


//===========================================================================
//
//===========================================================================

GeoCoordinate GraphicObjectPoint::GetPosition()
{
   GraphicObjectAttribute * m_StartPoint;

   // get the startpoint
   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return GeoCoordinate(0.0,0.0);
   }

   return m_StartPoint->m_c_Coordinate;

} // GetPosition()

//===========================================================================
//
//===========================================================================

void GraphicObjectPoint::MoveTo( float f_OriginX, float f_OriginY)
{
   GraphicObjectAttribute * m_StartPoint;
   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);

   // set a new coordinate
   m_StartPoint->m_c_Coordinate.SetXY(f_OriginX,f_OriginY);

   m_b_CachedSizeValid = false;
} // MoveTo

//===========================================================================
//
//===========================================================================

void GraphicObjectPoint::MoveOffset( float f_OffsetX, float f_OffsetY)
{
   GraphicObjectAttribute * m_StartPoint;
   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);

   // add offset to coordinate
   m_StartPoint->m_c_Coordinate.Transform(f_OffsetX,f_OffsetY);

   m_b_CachedSizeValid = false;
} // MoveOffset()

//===========================================================================
//
//===========================================================================

RectRegion GraphicObjectPoint::GetSize()
{
   GraphicObjectAttribute * m_StartPoint;
   GeoCoordinate            c_Position;

   // if cached ?
   if( m_b_CachedSizeValid)
   {
      // simply return
      return m_CachedSize;
   }
   m_CachedSize.SetEmpty();

   m_StartPoint = GetAttribute(GraphicObjectAttribute::StartPoint);
   if(m_StartPoint ==NULL)
   {
      return m_CachedSize;
   }

   // calculate position
   c_Position= m_StartPoint->m_c_Coordinate;

   // put something around into the coordiante
   m_CachedSize.SetXYWH(c_Position.GetX()-5.0,
                       c_Position.GetY()-5.0,
                       (float)10.0,
                       10.0);

   m_b_CachedSizeValid = true;
   return m_CachedSize;
} // GetSize()

//===========================================================================
//
//===========================================================================

QString GraphicObjectPoint::GetObjectTypeString()
{
   return "GraphicObjectPoint";
} // GetObjectType()

//===========================================================================
//
//===========================================================================

