#ifndef _GRAPHIKBOX_H
#define _GRAPHIKBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qvgroupbox.h>

#include "BrushMode.h"
#include "StandardDialog.h"
#include "BufferPainter.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "Tracer.h"
#include "QUColorOrPattern.h"
#include "GraphicObjectLine.h"
#include "GraphicObjectPoint.h"
#include "GraphicObjectMain.h"
#include "GraphicObjectMultipoint.h"
#include "QUIconButton.h"


class MainFrame;

class GraphikBox :  public StandardDialog,
                    public Tracer
{
Q_OBJECT

public:
   GraphikBox(QWidget *parent=0,
              GlobalStorage *pc_Global=0,
              MainFrame *pc_MainFrame=NULL);
   ~GraphikBox();

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);


public slots:

   void ActivatePoint();
   void ActivateLine();
   void ActivateRectangle();
   void ActivateQuadrat();
   void ActivateEllipse();
   void ActivateCircle();
   void ActivateFreeDraw();
   void ActivateConstruct();

   void LineThicknessChanged(int i_Width);
   void DrawInnerChanged(bool b_on);
   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);

   void LineColorChanged(Pixel c_Color);
   void LinePatternChanged(PixelBuffer *pc_Buffer);
   void LineModeChanged(BrushMode i_BrushMode);
   void InnerColorChanged(Pixel c_Color);
   void InnerPatternChanged(PixelBuffer *pc_Buffer);
   void InnerModeChanged(BrushMode i_BrushMode);

   void ConstructNewPolygon();
   void ConstructDrawPolygon();
   void ConstructAddPoint();
   void ConstructDeletePoint();
   void ConstructEditPoint();
   void ConstructClosedFigureChanged(bool b_On);
   void ConstructSplineChanged(bool b_On);
   

   
private:

   virtual void showEvent(QShowEvent *);

   enum
   {
      GraphikNone,
      GraphikPoint,
      GraphikLine,
      GraphikRectangle,
      GraphikQuadrat,
      GraphikEllipse,
      GraphikCircle,
      GraphikFreeDraw,
      GraphikConstruct


   };

    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);
   void SetModeTo(int i_Mode);

   void DrawPoint();
   void DrawLine();
   void DrawRectangle();
   void DrawEllipse();
   void DrawCircle();
   void DrawQuadrat();
   void DrawFreeDraw();
   void DrawConstruct();

   PixelBuffer *GetDestinationBuffer();

   void ConstructDeleteCurvePoints();
   void ConstructSwitchToMode(int i_NewMode);
   void ConstructSetSensitivity();

   void GraphicObjektCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);
   

   void ConstructPointsToPolygon();
   void ContructPolygonToPoints(GraphicObjectMultipoint *pc_Polygon);
   void ConstructDeleteAllPolygons();
   void ConstructSetLastLinesVisible();
   
   int ConstructWriteFile(PericoFileWriter *pc_FileWriter);
   int ConstructLoadFile(PericoFileWriter *pc_FileWriter);
   RectRegion ConstructReCalcConstructSpline();
   void ConstructRemoveObject(GraphicObject *pc_Object);
   
   //-----------------------------------
   // variables
   QUIconButton *m_PointButton;
   QUIconButton *m_LineButton;
   QUIconButton *m_RectangleButton;
   QUIconButton *m_QuadratButton;
   QUIconButton *m_EllipseButton;
   QUIconButton *m_CircleButton;
   QUIconButton *m_FreeDrawButton;
   QUIconButton *m_ConstructButton;

   QUColorOrPattern *m_LineColorWidget;
   QUColorOrPattern *m_InnerColorWidget;

   QULabelSlider *m_LineWidthSlider;

   QCheckBox *m_DrawInnerEnable;

   QULabelSlider *m_TransparencySlider;
   QCheckBox *m_TransparencyEnable;


   int m_i_CurrentGraphikMode;
   int m_i_LineWidth;
   bool m_b_DrawInner;

   bool m_b_TransparencyOn;
   int  m_i_TransparencyFactor;

   BufferPainter *m_pc_BufferPainter;
   
   QVGroupBox   *m_InnerBox;
   MainFrame    *m_pc_MainFrame;
   GraphicObjectLayer     *m_pc_ObjectLayer;
   
   QVGroupBox   *m_ConstructBox;
   QPushButton  *m_ConstructNewPolygonButton;
   QPushButton  *m_ConstructDrawIntoButton;
   QPushButton  *m_ConstructAddPointButton;
   QPushButton  *m_ConstructDeletePointButton;
   QPushButton  *m_ConstructEditPointButton;
   QCheckBox    *m_ConstructClosedFigureCheck;
   QCheckBox    *m_ConstructSplineCheck;

   QPtrList<GraphicObjectPoint > m_pc_CurvePoints;
   QPtrList<GraphicObjectLine  > m_pc_CurveLines;
   QPtrList<GraphicObjectMultipoint > m_pc_CurvePolygons;
   


   GraphicObjectPoint      * m_pc_RubberStartPoint;
   GraphicObjectLine       * m_pc_RubberLine;
   GraphicObjectMultipoint * m_pc_ContructEditedObject;
   GraphicObjectSpline     * m_pc_ContructEditedSpline;
   enum
   {
      ConstructModeOff=0,
      ConstructModeAdd,
      ConstructModeModify,
      ConstructModeAddMiddle
   };
   int m_i_ConstructMode;
   bool m_b_ContructIgnoreRedraw;
   int m_i_ConstructAddPointPosition;
};


#endif // _GRAPHIKBOX_H
