//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:
//---------------------------------------------------------------------------
// Purpose:
//---------------------------------------------------------------------------

#include "Hardcopy.h"

#include "GlobalStorage.h"
#include "RectRegion.h"
#include "PixelBuffer.h"
#include "ImageFrame.h"
#include "DrawingArea.h"
#include "MessageServer.h"

#include <qapplication.h>

#ifndef NO_X11
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>
#endif


class X11Info
{
public:
#ifndef NO_X11
   Display *theDisplay;
   Window   RootWindow;
#endif
   RectRegion m_SelectedRegion;

};


//============================================================================
//
//============================================================================

Hardcopy::Hardcopy( GlobalStorage *pc_Global)
{
   m_pc_GlobalStorage = pc_Global;
} // Hardcopy()

//============================================================================
//
//============================================================================

Hardcopy::~Hardcopy()
{

} // ~Hardcopy()

//============================================================================
//
//============================================================================

void Hardcopy::DoHardcopy()
{
#ifndef NO_X11
   XWindowAttributes  s_WindowAttributes;
   unsigned char     *pc_Data;
   unsigned char     *pc_Line;
   PixelBuffer       *pc_Image;
   RectRegion         c_ClipRegion;
   X11Info            X11Direct;
   QWidget           *w_MainWidget;
   Display           *theDisplay;
   XImage            *TheXImage;
   Window             RootWindow;
   Pixmap             p_Pixmap;
   int                i_RootDepth;
   int                i_StartX;
   int                i_StartY;
   int                i_Width;
   int                i_Height;
   int                i_X;
   int                i_Y;
   int                i_Error;
   GC                 gc;

   w_MainWidget = m_pc_GlobalStorage->m_pc_ImageFrame;

   // get the main X11 access variables
   theDisplay = w_MainWidget->x11Display();
   
   RootWindow = RootWindow(theDisplay, DefaultScreen(theDisplay));

   X11Direct.theDisplay = theDisplay;
   X11Direct.RootWindow = RootWindow;

   // get dimension of the root window
   XGetWindowAttributes(theDisplay,
                        RootWindow,
                        &s_WindowAttributes);

   // wait for a window to get clicked in
   WaitForButtonKlick(&X11Direct);

   // get widht and depth of root   
   i_RootDepth = DefaultDepth(theDisplay, DefaultScreen(theDisplay));
   i_Width  = s_WindowAttributes.width;
   i_Height = s_WindowAttributes.height;


   // copy the entire window into a pixmap
   p_Pixmap = XCreatePixmap(theDisplay,
                            RootWindow,
                            i_Width,
                            i_Height,
                            i_RootDepth);

   gc = XCreateGC(theDisplay, RootWindow, 0, 0);

   XSetSubwindowMode(theDisplay, gc, IncludeInferiors);

   i_Error = XCopyArea(theDisplay,
             RootWindow,
             p_Pixmap,
             gc,
             0,0,
             i_Width,
             i_Height,
             0,0);
   XSetSubwindowMode(theDisplay, gc, ClipByChildren);
   c_ClipRegion.SetXYWH(0,0,i_Width,i_Height);
   X11Direct.m_SelectedRegion.ClipAt(c_ClipRegion);
   
   // now get rectangle of the selected window
   i_StartX = X11Direct.m_SelectedRegion.GetStartX();
   i_StartY = X11Direct.m_SelectedRegion.GetStartY() ;
   i_Width  = X11Direct.m_SelectedRegion.GetWidth();
   i_Height = X11Direct.m_SelectedRegion.GetHeight();

   
   
   // now copy part of the pixmap into a ximage
   TheXImage= XGetImage (theDisplay,
                          p_Pixmap,
                          i_StartX,i_StartY,
                          i_Width,i_Height,
                          0xffffff,
                          ZPixmap);

   // get data pointer
   pc_Data = (unsigned char *) TheXImage->data;

   // create a new pixelbuffer as final destination
   pc_Image = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   // now access the single pixels
   for(i_Y=0; i_Y<i_Height; i_Y++)
   {
      pc_Line = pc_Data;
      for(i_X=0; i_X<i_Width; i_X++)
      {
         unsigned int i_Red;
         unsigned int i_Green;
         unsigned int i_Blue;
         Pixel        p_Pixel;

         i_Red   = 0;
         i_Green = 0;
         i_Blue  = 0;

         // attention: byte order is only a guess. nut in my X11
         // the XImage->*mask had not properly set
         // it might be, that it does not work on your machine !!!
         
         if(TheXImage->bits_per_pixel ==24)
         {
            i_Blue  = (unsigned int)  (*pc_Line++);
            i_Green = ((unsigned int) (*pc_Line++));
            i_Red   = ((unsigned int) (*pc_Line++));
         }
         else if (TheXImage->bits_per_pixel ==32)
         {
            i_Blue  = (unsigned int)  (*pc_Line++);
            i_Green = ((unsigned int) (*pc_Line++));
            i_Red   = ((unsigned int) (*pc_Line++));
            // skip empty byte
            pc_Line++;
         }

         p_Pixel = PIXEL_MAKE(i_Red,i_Green,i_Blue);
         pc_Image->Set(i_X,i_Y,p_Pixel);
      }

      // go to the next line
      pc_Data += TheXImage->bytes_per_line;
   }

   // now replace the current image by the just taken hardcopy
   m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_Image);
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // release the structures
   XFreePixmap(theDisplay,p_Pixmap);
   XDestroyImage(TheXImage);
   XFreeGC(theDisplay,gc);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageNewbuffer");
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                 this,
                                                 "ImageChanged");

#endif                                                 
} // DoHardcopy()

//============================================================================
//
//============================================================================

void Hardcopy::WaitForButtonKlick(X11Info *X11Direct)
{
#ifndef NO_X11
   XWindowAttributes s_WindowAttributes;
   Window            SelectedWindow;
   XEvent            event;
   Cursor            cu_CursorGrab;
   int               i_GrabResult;
   int               i_EventMask =ButtonPressMask;


   // wait for a button press
   i_EventMask =ButtonPressMask;

   // TODO: paint a better cursor
   cu_CursorGrab = XCreateFontCursor(X11Direct->theDisplay,
                                    XC_coffee_mug);

   // grap the pointer for my program
   i_GrabResult = XGrabPointer(X11Direct->theDisplay,
                      X11Direct->RootWindow,
                      False,
                      i_EventMask,
                      GrabModeSync,
                      GrabModeAsync,
                      None,
                      cu_CursorGrab,
                      CurrentTime);

   if(i_GrabResult == GrabSuccess)
   {
      // local event loop, wait for the button event
      while (true)
      {
         XAllowEvents(X11Direct->theDisplay,
                   SyncPointer,
                   CurrentTime);
         XWindowEvent(X11Direct->theDisplay,
                      X11Direct->RootWindow,
                      i_EventMask,
                      &event);

         if (event.type == ButtonPress)
         {
            SelectedWindow = event.xbutton.subwindow;
            break;
         }
      }

      // release the pointer
      XUngrabPointer(X11Direct->theDisplay, CurrentTime);
   }
   else
   {
      SelectedWindow = X11Direct->RootWindow;
   }

   if(SelectedWindow != 0)
   {
      // get the selected window
      XGetWindowAttributes(X11Direct->theDisplay,
                        SelectedWindow,
                        &s_WindowAttributes);

      // fill in region
      X11Direct->m_SelectedRegion.SetXYWH(s_WindowAttributes.x,
               s_WindowAttributes.y,
               s_WindowAttributes.width,
               s_WindowAttributes.height);
   }
   else
   {
      // select the root window
      XGetWindowAttributes(X11Direct->theDisplay,
                        X11Direct->RootWindow,
                        &s_WindowAttributes);
      X11Direct->m_SelectedRegion.SetXYWH(0,
               0,
               s_WindowAttributes.width,
               s_WindowAttributes.height);
   }

#endif
} // WaitForButtonKlick()

//============================================================================
//
//============================================================================






