//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: IconBox.c
//---------------------------------------------------------------------------
// Purpose: display a box with icons
//---------------------------------------------------------------------------

#include <qimage.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qptrlist.h>

#include <stdio.h>

#include "IconBox.h"



//===========================================================================
//
//===========================================================================

Thumbnail::Thumbnail()
{
   m_pc_IconImage = NULL;
   m_b_Selected   = false;
   m_qs_Text      = "";

} // Thumbnail()

//===========================================================================
//
//===========================================================================


Thumbnail::~Thumbnail()
{
   if(m_pc_IconImage != NULL)
   {
      delete m_pc_IconImage;
      m_pc_IconImage = NULL;
   }

} // ~Thumbnail()


//===========================================================================
//
//===========================================================================

IconBox::IconBox(QWidget *parent) :
               QWidget(parent,"IconBox")
{
   int i_FontHeight;
  
    
   // maximum width of an icon
   m_i_ThumbnailWidth      = 60;
   m_i_ThumbnailHeight     = 70;
   
   m_i_ImageFontPercentage = 80;
   
   m_i_MaximumImageWidth   = (m_i_ThumbnailWidth *94)/100;
   m_i_MaximumImageHeight  = (m_i_ThumbnailHeight*m_i_ImageFontPercentage)/100;
   
   m_p_SelectedColor = PIXEL_MAKE(209,45,70);
   
   // load a suitable font   
   m_fo_TheFont = new QFont("courier");
   i_FontHeight = (m_i_ThumbnailHeight
                   - (m_i_ThumbnailHeight*m_i_ImageFontPercentage/100)) *75/100;
   m_fo_TheFont->setPixelSize(i_FontHeight);
   

   // initial size
   setMinimumSize(2*m_i_ThumbnailWidth,2*m_i_ThumbnailHeight);
   
   // recalculate geometry parameters
   RecalculateGeometry();

   setMouseTracking(true);

}  // IconBox()


//===========================================================================
//
//===========================================================================

void IconBox::SetIconSize(int i_Width, int i_Height)
{
   int i_FontHeight;

   // maximum width of an icon
   m_i_ThumbnailWidth   = i_Width;
   m_i_ThumbnailHeight  = i_Height;

   m_i_MaximumImageWidth  = (m_i_ThumbnailWidth *94)/100;
   m_i_MaximumImageHeight = (m_i_ThumbnailHeight*m_i_ImageFontPercentage)/100;

   i_FontHeight = (m_i_ThumbnailHeight - (m_i_ThumbnailHeight*m_i_ImageFontPercentage/100)) *75/100;
   m_fo_TheFont->setPixelSize(i_FontHeight);


   // initial size
   setMinimumSize(2*m_i_ThumbnailWidth,2*m_i_ThumbnailHeight);

   // recalculate geometry parameters
   RecalculateGeometry();

} // SetIconSize()

//===========================================================================
//
//===========================================================================

void IconBox::DeletaAllIcons()
{
   Thumbnail *pc_Image;
   
   pc_Image = m_IconList.getFirst();
   while(pc_Image != NULL)
   {
      delete pc_Image;
      m_IconList.removeFirst();
      
      pc_Image = m_IconList.getFirst();
   }
      

   // neu anzeigen 
   RecalculateGeometry();
   RepaintEverythingNew();

} // DeletaAllIcons()


//===========================================================================
//
//===========================================================================


void IconBox::AddImage(PixelBuffer *pc_Image, QString qs_Text)
{
   Thumbnail *pc_NewThumbnail;
   QImage    *image;
   int       i_Width;
   int       i_Height;
   int       i_NewWidth;
   int       i_NewHeight;
   
   int       i_RunX;
   int       i_RunY;
   int       i_OldLineCount;


   pc_NewThumbnail = new Thumbnail();
   
   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();

   {
      // calculate size of qimage
      float f_FactorX;
      float f_FactorY;
      float f_SmallerFactor;

      f_FactorX =  (float) m_i_MaximumImageWidth / (float) i_Width ;
      f_FactorY =  (float) m_i_MaximumImageHeight / (float) i_Height ;
      
      if(f_FactorX < f_FactorY)
      {
         f_SmallerFactor = f_FactorX;
      }
      else
      {
         f_SmallerFactor = f_FactorY;
      }
         
      i_NewWidth  =(int)( i_Width  * f_SmallerFactor);
      i_NewHeight =(int)( i_Height * f_SmallerFactor);
      
   }
   
   //  allocate a qimage
   image = new QImage(i_NewWidth,i_NewHeight,32);
   
   {
      int i_OldY ;
      int i_OldX ;
      int i_DifferenceX;
      int i_DifferenceY;

      i_OldY =0;
      i_DifferenceY=i_NewHeight /2;

      // bilddaten uebertragen
      for( i_RunY=0 ; i_RunY < i_NewHeight; i_RunY++)
      {
         uint *imImage;

         i_OldX =0;
                  
         i_DifferenceX = i_NewWidth /2;

         imImage = (uint *) image->scanLine(i_RunY);
         for( i_RunX=0; i_RunX < i_NewWidth; i_RunX++)
         {

            Pixel p;
            p = pc_Image->Get(i_OldX,i_OldY);
            *imImage++ = qRgb(PIXEL_RED(p),PIXEL_GREEN(p),PIXEL_BLUE(p));      

            // new column in the original image     
            i_DifferenceX -= i_Width;
            while(i_DifferenceX <0)
            {
               i_DifferenceX += i_NewWidth;
               i_OldX++;
            }
         
         }
    
         // new row in the original image     
         i_DifferenceY -= i_Height;
         while(i_DifferenceY <0)
         {
            i_DifferenceY += i_NewHeight;
            i_OldY++;
         }
      }
   }
   
   // fill thumbnail with information
   pc_NewThumbnail->m_pc_IconImage = image;
   pc_NewThumbnail->m_qs_Text      = qs_Text;
   // add thumbnail to list
   m_IconList.append(pc_NewThumbnail);
   
   // recalculate geometry
   
   i_OldLineCount =  m_i_NumberOfLines;
   m_i_NumberOfLines     = m_IconList.count() / m_i_NumberIconsPerLine;

   // a line with nut fully filled
   if( (m_IconList.count() % m_i_NumberIconsPerLine) >0)
   {
      m_i_NumberOfLines += 1;
   }
   
   if( m_i_NumberOfLines != i_OldLineCount )
   {
      // scroll down
      if (m_i_NumberOfLines > m_i_VisibleLines)
      {
         m_i_TopLine ++;
      }
      RepaintEverythingNew();
   }
   else
   {
      // only draw last icon
      DrawTumbnail(m_IconList.count() -1 );
   }

   GeometryChanged();


} // AddImage()

//===========================================================================
//
//===========================================================================

void IconBox::mousePressEvent(QMouseEvent *theEvent)
{
   // Thumbnail *pc_SelectedIcon;
   int       i_MouseX;
   int       i_MouseY;
   int       i_ImageNumber;

   i_MouseX = theEvent->x();
   i_MouseY = theEvent->y();

   i_ImageNumber = GetIconForPosition(i_MouseX,i_MouseY);

   if(i_ImageNumber >= 0)
   {
      IconClicked(i_ImageNumber);
   }

} // mousePressEvent()

//===========================================================================
//
//===========================================================================

#ifdef ALT
void  IconBox::enterEvent(QEvent *theEvent)
{
fprintf(stderr,"enter Event\n");
}
void  IconBox::leaveEvent(QEvent *theEvent)
{
fprintf(stderr,"leave Event\n");
}

void  IconBox::mouseMoveEvent(QMouseEvent *theEvent)
{
fprintf(stderr,"mouse move Event\n");

}
#endif

//===========================================================================
//
//===========================================================================


void IconBox::mouseDoubleClickEvent(QMouseEvent *theEvent)
{
   // Thumbnail *pc_SelectedIcon;
   int       i_MouseX;
   int       i_MouseY;
   int       i_ImageNumber;
   
   i_MouseX = theEvent->x();
   i_MouseY = theEvent->y();

   i_ImageNumber = GetIconForPosition(i_MouseX,i_MouseY);
   
   if(i_ImageNumber >= 0)
   {
      IconSelected(i_ImageNumber);
   }
   
 
} // mousePressEvent()

//===========================================================================
//
//===========================================================================

int IconBox::GetIconForPosition(int i_X, int i_Y)
{
   int i_ImageNumber;

   
   i_X /= m_i_ThumbnailWidth;
   i_Y /= m_i_ThumbnailHeight;

   if( i_X >= m_i_NumberIconsPerLine)
   {
      return -1;
   }
   if( i_Y >= (m_i_VisibleLines+1))
   {
      return -1;
   }

   i_ImageNumber =  m_i_TopLine*m_i_NumberIconsPerLine
                + i_Y * m_i_NumberIconsPerLine
                + i_X;
   if(i_ImageNumber >= (int) m_IconList.count())
   {
      return -1;
   }

   return i_ImageNumber;

} // GetIconForPosition();

//===========================================================================
//
//===========================================================================


void IconBox::paintEvent(QPaintEvent *theEvent)
{
   QRect rect;
   
   rect = theEvent->rect();

   // redraw all icons   
   Repaint();
   
} // paintEvent()

//===========================================================================
//
//===========================================================================


void IconBox::resizeEvent(QResizeEvent *theEvent)
{
   QRect rect;
   
   
   // recalculate the geometry
   RecalculateGeometry();
   
} // resizeEvent()

//===========================================================================
//
//===========================================================================


void IconBox::RepaintEverythingNew()
{
   // delete the background
   erase();

   // and paint everything
   Repaint();
} // RepaintEverythingNew()

//===========================================================================
// draw all visible thumbnails, but don't delete the background
//===========================================================================

void IconBox::Repaint()
{
   int i_FirstImage;
   int i_X;
   int i_Y;
   
   QPainter painter(this);
   i_FirstImage = m_i_TopLine * m_i_NumberIconsPerLine;
   
   for(i_Y=0;i_Y<(m_i_VisibleLines+1);i_Y++)
   {
      for(i_X=0; i_X<m_i_NumberIconsPerLine; i_X++)
      {
         Thumbnail *pc_Icon ;
         int       i_ActualIcon;
         int       i_PositionX;
         int       i_PositionY;
         
         i_ActualIcon = i_FirstImage + i_Y*m_i_NumberIconsPerLine+i_X;

         pc_Icon = (Thumbnail *) m_IconList.at((uint) i_ActualIcon);
         if(pc_Icon == NULL)
            break;

         i_PositionX = i_X* m_i_ThumbnailWidth;
         i_PositionY = i_Y* m_i_ThumbnailHeight;
         DrawTumbnail(&painter,pc_Icon,i_PositionX,i_PositionY);
            
      } // for i_X ;
   } // for y
   painter.flush();
   
} // Repaint()

//===========================================================================
//
//===========================================================================


void IconBox::DrawTumbnail(int i_Index)
{
   QPainter painter(this);
   int      i_FirstImage;
   int      i_X;
   int      i_Y;

   i_FirstImage = m_i_TopLine * m_i_NumberIconsPerLine;
   
   // above the visible area
   if (i_Index < i_FirstImage )
   {
      return;
   }
   
   // below the visible area
   if(i_Index >= i_FirstImage + (m_i_VisibleLines+1)*m_i_NumberIconsPerLine)
   {
      return;
   }
   
   i_X = i_Index % m_i_NumberIconsPerLine;
   i_Y = ((i_Index-i_FirstImage) / m_i_NumberIconsPerLine );
   
   i_X *= m_i_ThumbnailWidth;
   i_Y *= m_i_ThumbnailHeight;

   DrawTumbnail(&painter, 
                 m_IconList.at(i_Index),
                 i_X,
                 i_Y);
   painter.flush();
} // DrawTumbnail()


//===========================================================================
// paint the image and the text below
//===========================================================================

void IconBox::DrawTumbnail(QPainter *painter,
                           Thumbnail *pc_Icon,
                           int i_X,
                           int i_Y)
{
   QFontMetrics fontmetrics(*m_fo_TheFont);
   QColor       White =QColor(255,255,255);
   QColor       Selected =QColor(0,0,0);
   QRect        r_TextArea;
   int          i_ImageHeight;
   int          i_ImageX;
   int          i_ImageY;
   int          i_TextX;
   int          i_TextY;

   Selected = QColor(PIXEL_RED(m_p_SelectedColor),
                     PIXEL_GREEN(m_p_SelectedColor),
                     PIXEL_BLUE(m_p_SelectedColor)
                     );
   
   if(pc_Icon->m_b_Selected)
   {
      painter->setBrush(Selected);
      painter->setPen(White);
      painter->fillRect(i_X,i_Y,m_i_ThumbnailWidth,m_i_ThumbnailHeight,Selected);
   }
   else
   {
      painter->setBrush(White);
      painter->setPen(Selected);
      painter->fillRect(i_X,i_Y,m_i_ThumbnailWidth,m_i_ThumbnailHeight,White);
   }
   i_ImageHeight = m_i_ThumbnailHeight *m_i_ImageFontPercentage/100;
   
   // center the image
   i_ImageY = (i_ImageHeight - pc_Icon->m_pc_IconImage->height() ) /2;
   if (i_ImageY <0)
   {
      i_ImageY = 0;
   }

   i_ImageX = (m_i_ThumbnailWidth - pc_Icon->m_pc_IconImage->width() ) /2;
   if (i_ImageX <0)
   {
      i_ImageX = 0;
   }

   painter->drawImage(i_X+i_ImageX,
                      i_Y+i_ImageY,
                      *(pc_Icon->m_pc_IconImage));
   
   painter->setFont(*m_fo_TheFont);
   fontmetrics = painter->fontMetrics();
   
   r_TextArea = fontmetrics.boundingRect(pc_Icon->m_qs_Text);

   i_TextX = (m_i_ThumbnailWidth - r_TextArea.width() ) /2;
   if (i_TextX <0)
   {
      i_TextX = 0;
   }
   i_TextY = ((m_i_ThumbnailHeight -i_ImageHeight) -r_TextArea.height() ) /2;
   if (i_TextY <0)
   {
      i_TextY = 0;
   }

   painter->drawText(i_X+i_TextX,
                     i_Y,
                     m_i_ThumbnailWidth,
                     (m_i_ThumbnailHeight), 
                     Qt::AlignBottom,                     
                     pc_Icon->m_qs_Text);
}

//===========================================================================
// after a geometry change recalculate all affected variables
//===========================================================================


void IconBox::RecalculateGeometry()
{

   m_i_ActualBoxWidth   = width();
   m_i_ActualBoxHeight  = height();
   
   m_i_NumberIconsPerLine  = m_i_ActualBoxWidth / m_i_ThumbnailWidth;
   m_i_NumberOfLines       = m_IconList.count() / m_i_NumberIconsPerLine;

   // line not completly filled ??
   if( (m_IconList.count() % m_i_NumberIconsPerLine) >0)
   {
      m_i_NumberOfLines += 1;
   }
   
   // start display at the top
   m_i_TopLine      = 0; 
   m_i_VisibleLines = m_i_ActualBoxHeight / m_i_ThumbnailHeight;

   GeometryChanged();

} // RecalculateGeometry()

//===========================================================================
//
//===========================================================================

void IconBox::SetSelected(int i_Index)
{
   Thumbnail *pc_Thumbnail;
   int        i;

   // direct to the child

   for(i=0; i< (int) m_IconList.count();i++)
   {
      pc_Thumbnail = m_IconList.at(i);
      if(pc_Thumbnail->m_b_Selected)
      {
         if(i == i_Index)
         {
         }
         else
         {
            pc_Thumbnail->m_b_Selected = false;
            DrawTumbnail(i);
         }
      }
      else // was not selected
      {
         if(i == i_Index)
         {
            // select it
            pc_Thumbnail->m_b_Selected = true;
            DrawTumbnail(i);
         }
         else
         {
         }
      }
   }   

} // SetSelected();

//===========================================================================
//
//===========================================================================

int  IconBox::GetSelected()
{
   Thumbnail *pc_Thumbnail;
   int        i;

   // direct to the child

   for(i=0; i< (int) m_IconList.count();i++)
   {
      pc_Thumbnail = m_IconList.at(i);
      if(pc_Thumbnail->m_b_Selected)
      {
         return i;
      }
   }

   // nothing selected
   return -1;
} // GetSelected()

//===========================================================================
//
//===========================================================================

void IconBox::DeleteImage(int i_Index)
{
   Thumbnail *pc_Thumbnail;
   int        i_OldLineCount;

   // wrong index
   if(   i_Index <0
      || i_Index >=  (int) m_IconList.count())
   {
      return;
   }

   // remove the thumbnail from the list
   pc_Thumbnail = m_IconList.at(i_Index);
   m_IconList.remove(i_Index);

   // delete the thumbnail
   if(pc_Thumbnail != NULL)
   {
      delete pc_Thumbnail;
   }

   // recalculate geometry

   i_OldLineCount    = m_i_NumberOfLines;
   m_i_NumberOfLines = m_IconList.count() / m_i_NumberIconsPerLine;

   // a line with nut fully filled
   if( (m_IconList.count() % m_i_NumberIconsPerLine) >0)
   {
      m_i_NumberOfLines += 1;
   }

   if( m_i_NumberOfLines < i_OldLineCount )
   {
      if( m_i_TopLine >0)
      {
         // scroll up
         m_i_TopLine --;
      }
      RepaintEverythingNew();
   }
   RepaintEverythingNew();

   GeometryChanged();

         
} // DeleteImage()

//===========================================================================
//
//===========================================================================


//===========================================================================
//===========================================================================
//===========================================================================

ScrollIconBox::ScrollIconBox(QWidget *parent,int i_Style) : 
                                         QWidget(parent,NULL)
{
   QBoxLayout   *l_Layout;

   m_i_Style = i_Style;

   
   l_Layout     = new  QHBoxLayout( this,1);

   m_pc_VerticalScrollbar = NULL;
   m_pc_IconBox           = new IconBox(this);
   l_Layout->addWidget(m_pc_IconBox);

   if(m_i_Style & UseScrollbar)
   {
      m_pc_VerticalScrollbar = new QScrollBar(0,
                                              1,
                                              1,
                                              1,
                                              0,
                                              QScrollBar::Vertical,
                                              this);
      l_Layout->addWidget(m_pc_VerticalScrollbar);
   }

   // signal auf den scrollbar umsetzen
   if(m_pc_VerticalScrollbar != NULL)
   {
      connect(m_pc_VerticalScrollbar,
           SIGNAL(valueChanged(int)),
           this,
           SLOT(ScrollbarMoved(int)) );
   }
   connect(m_pc_IconBox,
           SIGNAL(IconSelected(int)),
           this,
           SLOT(BoxIconSelected(int)) );

   connect(m_pc_IconBox,
           SIGNAL(IconClicked(int)),
           this,
           SLOT(BoxIconClicked(int)) );

   connect(m_pc_IconBox,
           SIGNAL(GeometryChanged()),
           this,
           SLOT(GeometryHasChanged()) );

    

} // ScrollIconBox()

//===========================================================================
//  adjust the crollbar to new given values
//===========================================================================


void ScrollIconBox::SetScrollbar(int i_Total,int i_Visible,int i_Position)
{

   if(m_pc_VerticalScrollbar == NULL)
   {
      return;
   }

   if(i_Total <= i_Visible)
   {
      // scrollbar is full
      m_pc_VerticalScrollbar->setValue(0);
      m_pc_VerticalScrollbar->setRange(0,0);
      m_pc_VerticalScrollbar->setSteps(1,1);
   }  
   else
   {
      // scrollbar is visible only partial
      m_pc_VerticalScrollbar->setValue(0);
      m_pc_VerticalScrollbar->setRange(0,i_Total-i_Visible);
      m_pc_VerticalScrollbar->setSteps(1,i_Visible);
      m_pc_VerticalScrollbar->setValue(i_Position);
   }
}  // SetScrollbar()

//===========================================================================
//
//===========================================================================


void ScrollIconBox::ScrollbarMoved(int i_Value)
{
   
   m_pc_IconBox->m_i_TopLine = i_Value;
   m_pc_IconBox->RepaintEverythingNew();

} // ScrollbarMoved()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::BoxIconSelected(int i_Value)
{
   // direct to the icon box
   IconSelected(i_Value);

} // BoxIconSelected()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::BoxIconClicked(int i_Value)
{
   // direct to the icon box
   IconClicked(i_Value);

} // BoxIconClicked()

//===========================================================================
//
//===========================================================================


void  ScrollIconBox::AddImage(PixelBuffer *pc_Image, QString qs_Text)
{
   // direct to the icon box
   m_pc_IconBox->AddImage(pc_Image,qs_Text);
} // AddImage()

//===========================================================================
//
//===========================================================================

void ScrollIconBox:: GeometryHasChanged()
{
   // set scrollbar according to the values in the child
   SetScrollbar(m_pc_IconBox->m_i_NumberOfLines,
                m_pc_IconBox->m_i_VisibleLines,
                m_pc_IconBox->m_i_TopLine) ;

} // GeometryHasChanged()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::DeleteAllImages()
{
   // direct to the icon box
   m_pc_IconBox->DeletaAllIcons();
} // DeleteAllImages()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::DeleteImage(int i_Index)
{
   // direct to the icon box
   m_pc_IconBox->DeleteImage(i_Index);
} // DeleteImage()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::SetSelected(int i_Index)
{
   // direct to the icon box
   m_pc_IconBox->SetSelected(i_Index);
} // SetSelected()

//===========================================================================
//
//===========================================================================

void ScrollIconBox::SetIconSize(int i_Width, int i_Height)
{
   m_pc_IconBox->SetIconSize(i_Width, i_Height);
} // SetIconSize()

//===========================================================================
//
//===========================================================================

int  ScrollIconBox::GetSelected()
{
   // direct to the icon box
   return m_pc_IconBox->GetSelected();
} // GetSelected()

//===========================================================================
//
//===========================================================================

