// malbox.h
// Box um irgendwas rein zu malen

#ifndef _ICONBOX_H
#define _ICONBOX_H

#include <qapplication.h>
#include <qptrlist.h>

#include <qwidget.h>
#include <qimage.h>
#include <qscrollbar.h>
#include <qpainter.h>
#include <qfont.h>

#include "PixelBuffer.h"
#include "StandardDialog.h"

class Thumbnail
{
public:
   Thumbnail();
   ~Thumbnail();

   QImage     *m_pc_IconImage;
   bool        m_b_Selected;
   QString     m_qs_Text;
};

class IconBox : public QWidget
{
   Q_OBJECT

   
   public:

   IconBox(QWidget *parent);

   signals:
   void	IconSelected(int);
   void	IconClicked(int);
   void  GeometryChanged();

public:
   void DeletaAllIcons();
   void AddImage(PixelBuffer *bild, QString qs_Text);
   void DeleteImage(int i_Index);

   void RecalculateGeometry();
   void Repaint();
   void RepaintEverythingNew();
   void DrawTumbnail(int index);
   void SetSelected(int i_Index);
   void SetIconSize(int i_Width, int i_Height);
   int  GetSelected();
      
   
   
   
   // current dimension
   int m_i_ThumbnailWidth;
   int m_i_ThumbnailHeight;
   
   // maximal dimension of an image within the thumbnail
   int m_i_MaximumImageWidth;
   int m_i_MaximumImageHeight; 

   // area of the box
   int m_i_ActualBoxWidth;
   int m_i_ActualBoxHeight;
   
   int m_i_NumberIconsPerLine;
   int m_i_NumberOfLines;
   
  
   // scrolling top position
   int m_i_TopLine;

   //  visible lines in the box
   int m_i_VisibleLines;
   
   QPtrList<Thumbnail> m_IconList;
   
   

   protected:
      void  paintEvent(QPaintEvent *theEvent);
      void  resizeEvent(QResizeEvent *theEvent);
      void  mouseDoubleClickEvent(QMouseEvent *theEvent);
      void  mousePressEvent(QMouseEvent *theEvent);
#ifdef ALT      
      void  enterEvent(QEvent *theEvent);
      void  leaveEvent(QEvent *theEvent);
      void  mouseMoveEvent(QMouseEvent *theEvent);
#endif
   
   void DrawTumbnail(QPainter *painter,Thumbnail *dasBild,int x,int y);

   
   
   private:
      int    GetIconForPosition(int i_X, int i_Y);

   
      QFont *m_fo_TheFont;
      int    m_i_ImageFontPercentage ;
      Pixel  m_p_SelectedColor;

//    void ScrollbarMoved(int i_Value);
};


class ScrollIconBox : public QWidget
{
   Q_OBJECT


   public:

   enum
   {
      UseScrollbar=1
   };
   
   ScrollIconBox(QWidget *parent,int i_Style);
   void SetScrollbar(int i_Total,int i_Visible,int i_Position);
   void DeleteAllImages();
   void DeleteImage(int i_Index);
   void AddImage(PixelBuffer *pc_Image, QString qs_Text);
   void SetSelected(int i_Index);
   void SetIconSize(int i_Width, int i_Height);
   int  GetSelected();

   signals:
   void	IconSelected(int);
   void	IconClicked(int);

public slots:
   void BoxIconSelected(int i_Value);
   void BoxIconClicked(int i_Value);
   void GeometryHasChanged();
   void ScrollbarMoved(int i_Value);

public:

private:
   QScrollBar * m_pc_VerticalScrollbar;
   IconBox    * m_pc_IconBox;

   int          m_i_Style;
};




#endif // _ICONBOX_H
