//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: ImageInfo.c
//---------------------------------------------------------------------------
// Purpose: additional image information
//---------------------------------------------------------------------------

#include <unistd.h>

#include "ImageInfo.h"
#include "PericoFileWriter.h"

//============================================================================
//
//============================================================================

ImageInfo::ImageInfo()
{
   Clear();
} // ImageInfo()

//============================================================================
//
//============================================================================

ImageInfo::~ImageInfo()
{

} // ~ImageInfo()

//============================================================================
//
//============================================================================

void ImageInfo::SetImageName(QString qs_Name)
{
   m_qs_ImageName = qs_Name;
   
} // SetImageName()

//============================================================================
//
//============================================================================

void ImageInfo::SetExifInfo(QStringList *p_qsl_ExifInfo)
{
   m_qsl_ExifInfo = *p_qsl_ExifInfo;
} // SetExifInfo()

//============================================================================
//
//============================================================================

void ImageInfo::Clear()
{
   char *cp_Login;
   m_qs_ImageName = "";
   m_qs_Author    = "";
   m_qs_Comment   = "";

   m_qsl_ExifInfo.clear();
   
   cp_Login = getlogin();
   if(cp_Login != NULL)
   {
      m_qs_Author.sprintf("%s",cp_Login);
   }
   
} // SetImageName()

//============================================================================
//
//============================================================================

int ImageInfo::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   // start an own structure
   pc_FileWriter->StartStructure(qs_Name,"ImageInfo");

   // write values into file
   pc_FileWriter->AddValue("ImageName",m_qs_ImageName);
   pc_FileWriter->AddValue("Author",m_qs_Author);
   pc_FileWriter->AddValue("Comment",m_qs_Comment);

   if( m_qsl_ExifInfo.count() >0)
   {
      QString     qs_Filename;
      QString     qs_FilenameWithoutPath;

      qs_Filename = pc_FileWriter->GetNextFilename(qs_FilenameWithoutPath);

      pc_FileWriter->AddValue("ExifInfo",qs_FilenameWithoutPath);
      
      pc_FileWriter->WriteStringList(qs_Filename,m_qsl_ExifInfo);
      
   }

   // ready with structure
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int ImageInfo::LoadFile(PericoFileWriter *pc_FileWriter)
{
   QString qs_Filename;
   
   PERICO_READ_STRUCTURE_START

   if( qs_Name.compare("ImageName") == 0)
   {
      m_qs_ImageName = qs_Value;
   }
   else if( qs_Name.compare("Author") == 0)
   {
      m_qs_Author = qs_Value;
   }
   else if( qs_Name.compare("Comment") == 0)
   {
      m_qs_Comment = qs_Value;
   }
   else if( qs_Name.compare("ExifInfo") == 0)
   {
      qs_Filename =  pc_FileWriter->GetPathFilename(qs_Value);
      pc_FileWriter->ReadStringList(qs_Filename,&m_qsl_ExifInfo);
   }


   PERICO_READ_STRUCTURE_END

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

