//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: ImageInfoBox.c
//---------------------------------------------------------------------------
// Purpose: dialog to show Image additional information.
//---------------------------------------------------------------------------

#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include "ImageInfoBox.h"
#include "QtUtilities.h"
#include "ApplicationStorage.h"

//============================================================================
//
//============================================================================

ImageInfoBox::ImageInfoBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
                         StandardDialog(parent,
                              pc_Global,
                              StandardDialog::DialogHasNoCloseButton)
{
   QGridLayout *l_Grid;
   QHBoxLayout *l_DialogButtonLayout;
   QBoxLayout  *l_Layout;
   QWidget     *w_GridWidget;
   int          i_CharWidth;

   LoadPosition("ImageInfoBox");
   setCaption(tr("Image Information"));
  
   i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();

   l_Layout     = new  QVBoxLayout( m_w_TopWidget,1);
   w_GridWidget = new  QWidget( m_w_TopWidget);
   l_Layout->addWidget(w_GridWidget);

   l_Grid = new QGridLayout( w_GridWidget, 4, 2,5);
   l_Grid->addWidget(new QLabel(tr("Image Name"),w_GridWidget),0,0);
   l_Grid->addWidget(new QLabel(tr("Size")      ,w_GridWidget),1,0);
   l_Grid->addWidget(new QLabel(tr("Author"),    w_GridWidget),2,0);
   l_Grid->addWidget(new QLabel(tr("Comment"),   w_GridWidget),3,0);
   l_Grid->addWidget(new QLabel(tr("Exif Data"), w_GridWidget),4,0);
   
   m_ImageNameEdit = new QLineEdit(w_GridWidget);
   m_SizeEdit      = new QtUtilReadOnlyLineEdit(w_GridWidget);
   m_AuthorEdit    = new QLineEdit(w_GridWidget);
   m_CommentEdit   = new QLineEdit(w_GridWidget);
   m_ExifHeader    = new QTextEdit(w_GridWidget);
   m_ExifHeader->setReadOnly(true);
   m_ExifHeader->setMinimumWidth(i_CharWidth*29);
   QtUtility::SetWidgetReadOnlyColor(m_ExifHeader);

   l_Grid->addWidget(m_ImageNameEdit,0,1);
   l_Grid->addWidget(m_SizeEdit     ,1,1);
   l_Grid->addWidget(m_AuthorEdit   ,2,1);
   l_Grid->addWidget(m_CommentEdit  ,3,1);
   l_Grid->addWidget(m_ExifHeader   ,4,1);

   // --- the close button
   l_DialogButtonLayout = new QHBoxLayout( l_Layout,5);

   m_ApplyButton = new QPushButton( tr("Apply"), m_w_TopWidget );
   l_DialogButtonLayout->addWidget(new QtUtilSpacer(true,false,m_w_TopWidget) );
   l_DialogButtonLayout->addWidget(m_ApplyButton);
   connect( m_ApplyButton,
            SIGNAL(clicked()),
            SLOT( ApplyPressed()) );
   


} // ImageInfoBox()

//============================================================================
//
//============================================================================

ImageInfoBox::~ImageInfoBox()
{
} // ~ImageInfoBox()

//============================================================================
//
//============================================================================

void ImageInfoBox::ApplyPressed()
{
   InformationChanged();

} // ApplyPressed()

//============================================================================
//
//============================================================================

void ImageInfoBox::SetSize(int i_Width,int i_Height)
{
   QString qs_Out;

   qs_Out.sprintf("%d x %d",i_Width,i_Height);
   m_SizeEdit->setText(qs_Out);

} // SetSize()

//============================================================================
//
//============================================================================

void ImageInfoBox::SetImageInfo(ImageInfo *pc_Info)
{
   unsigned int ui_TextLines;
   unsigned int ui_Index;
   m_ImageNameEdit ->setText(pc_Info->m_qs_ImageName);
   m_AuthorEdit    ->setText(pc_Info->m_qs_Author);
   m_CommentEdit   ->setText(pc_Info->m_qs_Comment);

   m_ExifHeader->clear();

   ui_TextLines = pc_Info->m_qsl_ExifInfo.count();
   for(ui_Index=0; ui_Index<ui_TextLines;ui_Index++)
   {
      QString qs_Line;
      qs_Line = pc_Info->m_qsl_ExifInfo[ui_Index];
      m_ExifHeader->append(qs_Line);
   }

} // SetImageInfo

//============================================================================
//
//============================================================================

void ImageInfoBox::GetImageInfo(ImageInfo *pc_Info)
{
   pc_Info->m_qs_ImageName = m_ImageNameEdit->text();
   pc_Info->m_qs_Author    = m_AuthorEdit->text();
   pc_Info->m_qs_Comment   = m_CommentEdit->text();

} // GetImageInfo

//============================================================================
//
//============================================================================



