//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: LineIterator
//---------------------------------------------------------------------------
// Purpose: calculate all points on a line between two coordinates.
//---------------------------------------------------------------------------

#include <stdlib.h>
#include "LineIterator.h"

//============================================================================
//
//============================================================================

LineIterator::LineIterator()
{
   m_i_FromX   = 0;
   m_i_FromY   = 0;
   m_i_ToX     = 0;
   m_i_ToY     = 0;
   m_b_DoXRun    = false;
   m_b_ReverseRun= false;
   m_i_DiffX   = 0;
   m_i_DiffY   = 0;
   m_i_AbsDiffX= 0;
   m_i_AbsDiffY= 0;
   m_i_RunX    = 0;
   m_i_RunY    = 0;
} // LineIterator()

//============================================================================
//
//============================================================================

LineIterator::LineIterator(int i_FromX, int i_FromY,int i_ToX,int i_ToY)
{
   Set(i_FromX,i_FromY,i_ToX,i_ToY);
 
} // LineIterator()

//============================================================================
// set start and endpoint and initialize the iterator
//============================================================================

void LineIterator::Set(int i_FromX, int i_FromY,int i_ToX,int i_ToY)
{
   m_i_FromX = i_FromX;
   m_i_FromY = i_FromY;
   m_i_ToX   = i_ToX;
   m_i_ToY   = i_ToY;


   m_i_DiffX = m_i_FromX - m_i_ToX;
   m_i_DiffY = m_i_FromY - m_i_ToY;

   m_i_AbsDiffX = abs(m_i_DiffX);
   m_i_AbsDiffY = abs(m_i_DiffY);

   m_i_RunX=0;
   m_i_RunY=0;

   m_b_ReverseRun = true;
   
   if(m_i_AbsDiffX > m_i_AbsDiffY )
   {
      // diffX is larger
      m_b_DoXRun = true;
      if( m_i_DiffX <0)
      {
         m_b_ReverseRun =false;
      }

   }
   else 
   {
      // diffY is larger
      m_b_DoXRun = false;
      if( m_i_DiffY <0)
      {
         m_b_ReverseRun =false;
      }
   }


} // Set()

//============================================================================
// Is the operator ready, meaning all points have been calculated
//============================================================================

bool LineIterator::Ready()
{
   if(m_b_DoXRun)
   {
      if(m_i_RunX < m_i_AbsDiffX)
      {
         return false;
      }
   }
   else // yrun
   {
      if(m_i_RunY < m_i_AbsDiffY)
      {
         return false;
      }
   }
      
   return true;
} // Ready()

//============================================================================
// is operator running? opposite of Ready()
//============================================================================

bool LineIterator::Running()
{
   if(m_b_DoXRun)
   {
      if(m_i_RunX < m_i_AbsDiffX)
      {
         return true;
      }
   }
   else // yrun
   {
      if(m_i_RunY < m_i_AbsDiffY)
      {
         return true;
      }
   }

   return false;
} // Ready()

//============================================================================
// calculate the next points
//============================================================================

bool LineIterator::Next(int &i_X,int &i_Y)
{
   if(m_b_DoXRun)
   {
      m_i_RunX++;
      // end reached?
      if(m_i_RunX > m_i_AbsDiffX)
      {
         return false;
      }
      
      if( m_b_ReverseRun)
      {
         // when reverse then substract
         i_X = m_i_FromX - m_i_RunX;
      }
      else
      {
         i_X = m_i_FromX + m_i_RunX;
      }
      // calculate y
      i_Y =  m_i_FromY - m_i_DiffY *m_i_RunX / m_i_AbsDiffX;
   }
   else // yrun
   {
      m_i_RunY++;
      // end reached?
      if(m_i_RunY > m_i_AbsDiffY)
      {
         return false;
      }

      if( m_b_ReverseRun)
      {
         // when reverse then substract
         i_Y = m_i_FromY - m_i_RunY;
      }
      else
      {
         i_Y = m_i_FromY + m_i_RunY;
      }
      // calculate y
      i_X = m_i_FromX - m_i_DiffX *m_i_RunY / m_i_AbsDiffY;
   }


   return true;

} // Next()

//============================================================================
// gets the startpoint. does not reset the iterator.
//============================================================================

void LineIterator::Start(int &i_X,int &i_Y)
{
   i_X = m_i_FromX;
   i_Y = m_i_FromY;
   
} // Start()

//============================================================================
//
//============================================================================

