//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: MainFrame
//---------------------------------------------------------------------------
// Purpose: Main structure for an single image editor.
// the root of all image dependend widget
//---------------------------------------------------------------------------

#include <stdlib.h>

#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qsizepolicy.h>
#include <qtoolbar.h>

#include "MainFrame.h"
#include "FileLoader.h"
#include "GlobalStorage.h"
#include "ColorSelectBox.h"
#include "QtUtilities.h"
#include "StatusBar.h"
#include "Undo.h"
#include "ApplicationFrame.h"
#include "ApplicationStorage.h"
#include "NewImageBox.h"
#include "Selection.h"
#include "QtUtilities.h"
#include "DragAndDropConstants.h"

#include "ManipulationBox.h"
#include "GraphikBox.h"
#include "PaintBox.h"
#include "AboutBox.h"
#include "CorrectionBox.h"
#include "RotationBox.h"
#include "MotiveBox.h"
#include "RedEyeBox.h"
#include "BlendBox.h"
#include "EffectBox.h"
#include "Autosave.h"
#include "AutosaveBox.h"
#include "SnapshotBox.h"
#include "IconLibrary.h"
#include "ImageInfoBox.h"
#include "Zoom.h"
#include "TestObjectBox.h"
#include "Hardcopy.h"
#include "QUToolButton.h"
#include "FilterBox.h"

//===========================================================================
//
//===========================================================================

MainFrame::MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame)
   : QMainWindow( parent,
                 "MainFrameFrame",
                 WType_TopLevel )

{
   CreateSimple(mainApplication,pc_ApplicationFrame);

   // remember the changes for undo
   DoChangedProcessing();

} // MainFrame constructor

//===========================================================================
//
//===========================================================================

MainFrame::MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame,
                  int i_Width, int i_Height)
   : QMainWindow( parent,
                 "MainFrameFrame",
                 WType_TopLevel )

{
   PixelBuffer *pc_EmptyImage;

   CreateSimple(mainApplication,pc_ApplicationFrame);

   pc_EmptyImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   // replace the old by the new image
   m_pc_ImageFrame->ReplacePixelbuffer(pc_EmptyImage);

   // remember the changes for undo
   DoChangedProcessing();

} // MainFrame constructor

//===========================================================================
//
//===========================================================================

MainFrame::~MainFrame()
{
   if(m_pc_Autosave != NULL)
   {
      delete m_pc_Autosave;
      m_pc_Autosave =NULL;
   }

   if(m_pc_ParentMessageHandler != NULL)
   {
      m_pc_ParentMessageHandler->TellChildTerminate();
      m_pc_ParentMessageHandler->RemoveChildCallback();
      
   }

   if(m_pc_GlobalStorage->m_pc_Undo != NULL)
   {
      delete m_pc_GlobalStorage->m_pc_Undo;
      m_pc_GlobalStorage->m_pc_Undo = NULL;
   }

   if(m_pc_ImageFrame != NULL)
   {
      delete m_pc_ImageFrame;
      m_pc_ImageFrame = NULL;
   }


   // should be first, because if might get accessed
   delete m_pc_ColorSelectBox;

   delete m_pc_ManipulationBox;

   delete m_pc_RotationBox ;
   
   delete m_pc_ZoomBox ;        

   delete m_pc_GraphikBox ;    

   delete m_pc_PaintBox ;       

   delete m_pc_TracerBox;       

   delete m_pc_FilterBox;     

   delete m_pc_MaskBox;        

   delete m_pc_AboutBox;       

   delete m_pc_GridBox ;       

   delete m_pc_TestObjectBox;  

   delete m_pc_CorrectionBox ;  

   delete m_pc_RedEyeBox  ;   

   delete m_pc_MotiveBox ;      

   delete m_pc_BlendBox ;

   delete m_pc_EffectBox ;

   delete m_pc_SnapshotBox;

   delete m_pc_ImageInfoBox;

   
} // ~MainFrame()

//===========================================================================
//
//===========================================================================

void MainFrame::CreateSimple(QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame)
{
   // set Titel
   setCaption("Perico");

   // load the Perico icon as the main application Icon
   {
      QPixmap *pc_Icon;

      pc_Icon = g_Application->m_pc_IconLibrary->GetPixmap("Perico");
      if(pc_Icon != NULL)
      {
         setIcon(*pc_Icon);

         delete pc_Icon;
      }
      
   } // block for icon load
   setUsesBigPixmaps(true);

  
   // i dont know why, but events must be created here to have
   // a correct drawing area
   //mainApplication->processEvents();

   // clear the filename
   m_qs_Filename ="";
   m_c_ImageInfo.Clear();
   m_c_ImageInfo.SetImageName(IMAGEINFO_DEFAULT);

   // default there is no parent handler
   m_pc_ParentMessageHandler = NULL;
   m_i_ParentMenuId = -1;

   // save the application frame
   m_pc_ApplicationFrame = pc_ApplicationFrame;
   // create substructures

   m_w_Toolbar = new QToolBar(this,"MainFrameToolbar");
   m_w_Toolbar -> setLabel("System");

//   m_pc_ImageFrame = new ImageFrame(this,mainApplication,NULL,ImageFrame::UndoButton);
   m_pc_ImageFrame = new ImageFrame(this,mainApplication,NULL,0);
   m_pc_GlobalStorage = m_pc_ImageFrame->GetGlobalStorage();
   connect(m_pc_ImageFrame,
           SIGNAL(UndoClicked()),
           SLOT(UndoButtonPressed())
           );

   m_DragAndDrop.SetAcceptType(  DragAndDrop::AcceptPlain
                               | DragAndDrop::AcceptPixelbuffer);
   setAcceptDrops(true);
   
   m_pc_GlobalStorage->m_pc_Undo = new Undo(this);

   // create the boxes

   // should be first, because if might get accessed
   m_pc_ColorSelectBox  = new ColorSelectBox(this,m_pc_GlobalStorage);

   m_pc_ManipulationBox = new ManipulationBox(this,m_pc_GlobalStorage);

   m_pc_RotationBox     = new RotationBox(this,m_pc_GlobalStorage);

   m_pc_ZoomBox         = new ZoomBox(this,m_pc_GlobalStorage);

   m_pc_GraphikBox      = new GraphikBox(this,m_pc_GlobalStorage,this);

   m_pc_PaintBox        = new PaintBox(this,m_pc_GlobalStorage,this);

   m_pc_TracerBox       = new TracerBox(this);

   m_pc_FilterBox       = new FilterBox(this,m_pc_GlobalStorage,this);

   m_pc_MaskBox         = new MaskBox(this,m_pc_GlobalStorage,this);

   m_pc_AboutBox        = new AboutBox(this);

   m_pc_GridBox         = new GridBox(this,m_pc_GlobalStorage,this);

   m_pc_TestObjectBox   = new TestObjectBox(this,m_pc_GlobalStorage);
   
   m_pc_CorrectionBox   = new CorrectionBox(this,m_pc_GlobalStorage);

   m_pc_RedEyeBox       = new RedEyeBox(this,m_pc_GlobalStorage);

   m_pc_MotiveBox       = new MotiveBox(this,m_pc_GlobalStorage);

   m_pc_BlendBox        = new BlendBox(this,m_pc_GlobalStorage,this);

   m_pc_EffectBox       = new EffectBox(this,m_pc_GlobalStorage,this);

   m_pc_SnapshotBox     = new SnapshotBox(this);

   m_pc_ImageInfoBox    = new ImageInfoBox(this,m_pc_GlobalStorage);
   setCentralWidget(m_pc_ImageFrame);

   // create the menus
   CreateMenu();

   m_pc_GlobalStorage->m_pc_StatusBar = new StatusBar(statusBar());
   m_pc_GlobalStorage->mainApplication = mainApplication;

    // add toolbar buttons
   m_w_UndoButton = new QUToolButton(m_w_Toolbar,"Undo",tr("Undo"));
   m_w_RedoButton = new QUToolButton(m_w_Toolbar,"Redo",tr("Redo"));
   m_w_ColorSelectButton = new QUToolButton(m_w_Toolbar,"ColorSelect",tr("Color Selection"));
   m_w_ZoomInButton  = new QUToolButton(m_w_Toolbar,"ZoomIn",tr("Zoom In"));
   m_w_ZoomOutButton = new QUToolButton(m_w_Toolbar,"ZoomOut",tr("Zoom Out"));
   m_w_SnapshotButton = new QUToolButton(m_w_Toolbar,"Snapshot",tr("Snapshot"));
   m_w_HardcopyButton = new QUToolButton(m_w_Toolbar,"Hardcopy",tr("Grab Image"));

   connect(m_w_UndoButton,
           SIGNAL(pressed()),
           SLOT(UndoButtonPressed())
           );
   connect(m_w_RedoButton,
           SIGNAL(pressed()),
           SLOT(RedoButtonPressed())
           );
   SetUndoMenu();
   connect(m_w_ColorSelectButton,
           SIGNAL(pressed()),
           SLOT(ColorSelectMenu())
           );
   connect(m_w_ZoomInButton,
           SIGNAL(pressed()),
           SLOT(ZoomInAccelPressed())
           );
   connect(m_w_ZoomOutButton,
           SIGNAL(pressed()),
           SLOT(ZoomOutAccelPressed())
           );

   connect(m_pc_GlobalStorage->m_pc_Undo,
           SIGNAL(IconSelected(int)),
           SLOT(UndoIconSelected(int))
           );

   connect(m_w_SnapshotButton,
           SIGNAL(pressed()),
           SLOT(SnapshotPressed())
           );
   connect(m_w_HardcopyButton,
           SIGNAL(pressed()),
           SLOT(HardcopyPressed())
           );


   // connect special subdialog signals
   connect(m_pc_ImageInfoBox,
           SIGNAL(InformationChanged()),
           SLOT(ImageInformationChanged())
           );


   // add a message receiver
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageChanged");

   // this is the default size
   this->resize(400,500 );

   // create the accelerators
   
   m_Accelerator = new QAccel(this);
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Less),
                                 this,
                                 SLOT(ZoomOutAccelPressed())
                                 );
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Greater),
                                 this,
                                 SLOT(ZoomInAccelPressed())
                                 );

   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_7),
                                 this,
                                 SLOT(RotateLeftPressed())
                                 );

   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_8),
                                 this,
                                 SLOT(Rotate180Pressed())
                                 );

   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_9),
                                 this,
                                 SLOT(RotateRightPressed())
                                 );

   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Up),
                                 this,
                                 SLOT(KeyUpPressed())
                                 );
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Down),
                                 this,
                                 SLOT(KeyDownPressed())
                                 );
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Left),
                                 this,
                                 SLOT(KeyLeftPressed())
                                 );
   m_Accelerator->connectItem(m_Accelerator->insertItem(Key_Right),
                                 this,
                                 SLOT(KeyRightPressed())
                                 );
   m_pc_Autosave = new Autosave(this);
   startTimer( 1000 );
   UpdateImageSize();

   m_pc_ImageInfoBox->SetImageInfo(&m_c_ImageInfo);

   
} // CreateSimple()

//===========================================================================
//
//===========================================================================

void MainFrame::CreateMenu()
{

   m_NewMenu = new QPopupMenu(this);
   m_NewMenu ->insertItem(tr("Load File"),   this,SLOT(NewFileLoad() )  );
   m_NewMenu ->insertItem(tr("Create Empty"),this,SLOT(NewFileBlank() )  );

   // items for the file menu
   m_FileMenu = new QPopupMenu( this );
   m_FileMenu->insertItem(tr("New Image"), m_NewMenu );
   m_FileMenu->insertSeparator();
   m_FileMenu->insertItem(tr("Load Image"), this, SLOT( LoadFile() ) );
   m_FileMenu->insertItem(tr("Save Image"), this, SLOT( SaveFile() ) );
   m_FileMenu->insertSeparator();
   m_FileMenu->insertItem(tr("Snapshots"), this, SLOT( SnapshotBoxMenu() ) );
   if(g_Application->m_pc_ApplicationFrame->m_pc_AutosaveBox->GetNumberOfFiles()>0)
   {
      m_FileMenu->insertItem(tr("Automatic saved Images"), this, SLOT( AutosaveBoxMenu() ) );
   }
   m_FileMenu->insertSeparator();
   m_FileMenu->insertItem(tr("Close Image"), this, SLOT( CloseFile() ) );
   m_FileMenu->insertSeparator();
   m_FileMenu->insertItem(tr("Exit All"),this,SLOT(ExitFile() ));

   // Items for the work menu
   m_WorkMenu = new QPopupMenu( this );
   m_WorkMenu->insertItem(tr("Manipulate"), this,SLOT( ManipulationBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Rotate"), this,SLOT( RotationBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Graphic"), this,SLOT( GraphikBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Paint"), this,SLOT( PaintBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Filter"), this,SLOT( FilterBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Correction"), this,SLOT( CorrectionBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Red Eye Reduction"), this,SLOT( RedEyeBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Motive"), this,SLOT( MotiveBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Blend"), this,SLOT( BlendBoxMenu() ) );
   m_WorkMenu->insertItem(tr("Effects"), this,SLOT( EffectBoxMenu() ) );

   // items for the view menu
   m_ViewMenu = new QPopupMenu( this );
   m_ViewMenu->insertItem(tr("Zoom"), this,SLOT( ZoomBoxMenu() ) );
   m_ViewMenu->insertItem(tr("Mask"), this,SLOT( MaskBoxMenu() ) );
   m_ViewMenu->insertItem(tr("Color Selection"), this,SLOT( ColorSelectMenu() ) );
   m_ViewMenu->insertItem(tr("Grid"), this,SLOT( GridBoxMenu() ) );
   m_ViewMenu->insertSeparator();
   m_ViewMenu->insertItem(tr("Image Details"), this,SLOT( ImageInfoBoxMenu()  ) );

   // items for the options menu
   m_OptionsMenu = new QPopupMenu( this );
   m_OptionsMenu->insertItem(tr("Settings"), this,SLOT( SettingsMenu() ) );

   // items for the undo menu
   m_UndoMenu = new QPopupMenu( this );
   m_i_UndoId = m_UndoMenu->insertItem(tr("Undo"), this,SLOT( UndoMenu() ) );
   m_UndoMenu->insertItem(tr("Show Undo Buffers"), this,SLOT( UndoSelectionMenu() ) );

   // items for the help menu
   m_HelpMenu = new QPopupMenu( this );
   m_HelpMenu->insertItem(tr("About Perico"), this, SLOT( About() )  );

   // items for the TRACE menu
#ifdef _DEBUG
   m_TraceMenu = new QPopupMenu( this );
   m_TraceMenu->insertItem(tr("Trace-Messages"), this,SLOT( TracerBoxMenu() ) );
   m_TraceMenu->insertItem(tr("Test Object"), this,SLOT( TestObjectBoxMenu() ) );
#endif

   // add menus to menubars
   menuBar()->insertItem(tr("File") , m_FileMenu );
   menuBar()->insertItem(tr("Work") , m_WorkMenu );
   menuBar()->insertItem(tr("View") , m_ViewMenu );
   menuBar()->insertItem(tr("Options") , m_OptionsMenu );
   menuBar()->insertItem(tr("Undo") , m_UndoMenu );
   menuBar()->insertItem(tr("Help") , m_HelpMenu );
#ifdef _DEBUG
   menuBar()->insertItem(tr("Debug"), m_TraceMenu );
#endif

} // Createmenu

//===========================================================================
//
//===========================================================================

void MainFrame::About()
{
   // open the about box
   m_pc_AboutBox->show();

} // About()

//===========================================================================
//
//===========================================================================


void MainFrame::LoadFile()
{
   FileLoaderOptions  c_Options;

   PixelBuffer        *pc_NewImage;
   FileLoader         *mp_FileLoader;
   QString             qs_Filename;

   // create a file loader
   mp_FileLoader = new FileLoader(this);

   // open a dialog and ask for the filename
   qs_Filename = mp_FileLoader->AskForFilename(true);

   if(qs_Filename.isEmpty())
   {
      // empty name, then return
      delete mp_FileLoader;
      return;
   }

   // deactivate all selections first, to clean up selection
   // private storage and modes
   m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection();

   m_c_ImageInfo.Clear();
      
   // load the file
   c_Options.b_HasMainFramePointer=true;
   c_Options.m_pc_MainFrame       = this;

   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename,
                            &c_Options);
   delete mp_FileLoader;

   // success?
   if(pc_NewImage == NULL)
   {
      return ;
   }

   ReplaceNewImage(pc_NewImage,qs_Filename);
   TakeImageName(qs_Filename,&c_Options);

} // LoadFile

//===========================================================================
//
//===========================================================================

void MainFrame::SaveFile()
{
   FileLoaderOptions  c_Options;
   FileLoader        *mp_FileLoader;
   QString            qs_Filename;

   // create the file loader
   mp_FileLoader = new FileLoader(this);

   // ask for a filename
   qs_Filename = mp_FileLoader->AskForFilename(false,m_qs_Filename);

   // empty file, then finsh
   if(qs_Filename.isEmpty())
   {
      delete mp_FileLoader;
      return;
   }


   c_Options.b_HasMainFramePointer=true;
   c_Options.m_pc_MainFrame       = this;
   // save image as new file
   mp_FileLoader->SaveFile(qs_Filename,
                            m_pc_ImageFrame->GetActualPixelBuffer(),
                            &c_Options);

   delete mp_FileLoader;

} // SaveFile

//===========================================================================
// overloaded function without asking for a filename
//===========================================================================

int MainFrame::SaveFile(QString qs_Filename)
{
   FileLoaderOptions  c_Options;
   FileLoader        *mp_FileLoader;
   int                i_Result;

   // create the file loader
   mp_FileLoader = new FileLoader(this);

   // empty file, then finsh
   if(qs_Filename.isEmpty())
   {
      delete mp_FileLoader;
      return -1;
   }


   c_Options.b_HasMainFramePointer=true;
   c_Options.m_pc_MainFrame       = this;
   // save image as new file
   i_Result = mp_FileLoader->SaveFile(qs_Filename,
                            m_pc_ImageFrame->GetActualPixelBuffer(),
                            &c_Options);

   delete mp_FileLoader;

   return i_Result;

} // SaveFile

//===========================================================================
//
//===========================================================================

void MainFrame::CloseFile()
{
   // tell the MainFram about it
   m_pc_ApplicationFrame->RemoveMainFrame(this);

   // delete myself
   delete this;
} // CloseFile()

//===========================================================================
//
//===========================================================================

void MainFrame::ExitFile()
{
   // tell the MainFram about it
   m_pc_ApplicationFrame->ExitCall(this);

} // ExitFile()

//===========================================================================
//
//===========================================================================

void MainFrame::GridBoxMenu()
{
   m_pc_GridBox->show();

} // GridBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ManipulationBoxMenu()
{
   m_pc_ManipulationBox->show();

} // ManipulationBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::RotationBoxMenu()
{
   m_pc_RotationBox->show();

} // RotationBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomBoxMenu()
{
   m_pc_ZoomBox->show();

} // ZoomBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::GraphikBoxMenu()
{
   m_pc_GraphikBox->show();

} // GraphikBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::PaintBoxMenu()
{
   m_pc_PaintBox->show();

} // PaintBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::FilterBoxMenu()
{
   m_pc_FilterBox->show();

} // FilterBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::MaskBoxMenu()
{
   m_pc_MaskBox->show();

} // FilterBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::TracerBoxMenu()
{
   m_pc_TracerBox->Reload();
   m_pc_TracerBox->show();

} // TracerBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::TestObjectBoxMenu()
{
   m_pc_TestObjectBox->show();

} // TestObjectBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ColorSelectMenu()
{
   m_pc_ColorSelectBox->show();

} // ColorSelectMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::CorrectionBoxMenu()
{
   m_pc_CorrectionBox->show();

} // CorrectionBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::RedEyeBoxMenu()
{
   m_pc_RedEyeBox->show();
   m_pc_RedEyeBox->DoActivate();

} // RedEyeBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::MotiveBoxMenu()
{
   m_pc_MotiveBox->show();

} // MotiveBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::BlendBoxMenu()
{
   m_pc_BlendBox->show();
   m_pc_BlendBox->DoActivate();

} // BlendBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::EffectBoxMenu()
{
   m_pc_EffectBox->show();
   // m_pc_EffectBox->DoActivate();

} // EffectBoxMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::SnapshotBoxMenu()
{
   m_pc_SnapshotBox->show();

} // EffectBoxMenu()

//===========================================================================
//
//===========================================================================
void MainFrame::SettingsMenu()
{
   m_pc_ApplicationFrame->OpenSettingsBox();

} // SettingsMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::ImageInfoBoxMenu()
{
   m_pc_ImageInfoBox->show();

} // ImageInfoMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::closeEvent(QCloseEvent *)
{
   CloseFile();
} // closeEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoSelectionMenu()
{
   m_pc_GlobalStorage->m_pc_Undo->ShowSelectionBox();
} // UndoSelectionMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoMenu()
{
   bool b_SizeChanged;

   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoUndo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }
   else if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoRedo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }

   // if size changed
   if(b_SizeChanged)
   {
      // tell it to the image frame
      m_pc_ImageFrame->ChangeImageDimensions();

      // and send a message to all the other modules
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");
   }

   // set the new state of the menu
   SetUndoMenu();

} // UndoMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoIconSelected(int i_Value)
{
   bool b_SizeChanged;

   m_pc_GlobalStorage->m_pc_Undo->DoUndo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged,
                              i_Value);

   // repaint area
   m_pc_ImageFrame->RedrawAll();

   // if size changed
   if(b_SizeChanged)
   {
      // tell it to the image frame
      m_pc_ImageFrame->ChangeImageDimensions();

      // and send a message to all the other modules
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");
   }


} // UndoIconSelected()

//===========================================================================
//
//===========================================================================

void MainFrame::UndoButtonPressed()
{
   bool b_SizeChanged;

   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoUndo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }
      
   // if size changed
   if(b_SizeChanged)
   {
      // tell it to the image frame
      m_pc_ImageFrame->ChangeImageDimensions();

      // and send a message to all the other modules
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");
   }

   // set the new state of the menu
   SetUndoMenu();
}  // UndoButtonPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::RedoButtonPressed()
{
   bool b_SizeChanged;

   if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      m_pc_GlobalStorage->m_pc_Undo->DoRedo(
                              m_pc_GlobalStorage->m_pc_MainImage,
                              b_SizeChanged);

      // repaint area
      m_pc_ImageFrame->RedrawAll();

   }

   // if size changed
   if(b_SizeChanged)
   {
      // tell it to the image frame
      m_pc_ImageFrame->ChangeImageDimensions();

      // and send a message to all the other modules
      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageResize");
   }

   // set the new state of the menu
   SetUndoMenu();

}  // RedoButtonPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::AutosaveBoxMenu()
{
   g_Application->m_pc_ApplicationFrame->m_pc_AutosaveBox->show();
} // UndoSelectionMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::OnMessageReceiveCB(void *p_ThisData,
                                   QString qs_MessageType,
                                   void *p_Data)
{
   MainFrame *p_This;

   p_This =(MainFrame *) p_ThisData;
   p_This->OnMessageReceive(qs_MessageType,p_Data);

} // OnMessageReceiveCB()

//----------------------------------------------------------------------

void MainFrame::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   if(qs_MessageType.compare("ImageChanged") == 0)
   {
      DoChangedProcessing();
      UpdateImageSize();
   }
   else if(qs_MessageType.compare("ImageResize") == 0)
   {
      UpdateImageSize();
   }
   else if(qs_MessageType.compare("ImageNewBuffer") == 0)
   {
      UpdateImageSize();
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void MainFrame::SetUndoMenu()
{
   QString qs_Text;
   // set the menu depending on the state of the
   // undo state machine
   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      qs_Text = tr("Undo");
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,true);
      
   }
   else if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      qs_Text = tr("Redo");
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,true);
   }
   else
   {
      // nothing is possible
      qs_Text ="----";
      m_pc_ImageFrame->SetUndoText(qs_Text);
      m_UndoMenu->changeItem(m_i_UndoId,qs_Text);
      m_UndoMenu->setItemEnabled(m_i_UndoId,false);
   }

   if(m_pc_GlobalStorage->m_pc_Undo->IsUndoPossible())
   {
      m_w_UndoButton-> setEnabled(true);
   }
   else
   {
      m_w_UndoButton-> setEnabled(false);
   }

   if(m_pc_GlobalStorage->m_pc_Undo->IsRedoPossible())
   {
      m_w_RedoButton-> setEnabled(true);
   }
   else
   {
      m_w_RedoButton-> setEnabled(false);
   }

   
} // SetUndoMenu()

//===========================================================================
//
//===========================================================================

void MainFrame::DoChangedProcessing()
{
   // save the image for Undo
   m_pc_GlobalStorage->m_pc_Undo->SaveBufferToUndo(
                              m_pc_GlobalStorage->m_pc_MainImage);


   SetUndoMenu();
} // ChangedProcessing()

//===========================================================================
//
//===========================================================================

int MainFrame::LoadImage(QString qs_Filename)
{
   FileLoaderOptions c_Options;
   PixelBuffer       *pc_NewImage;
   FileLoader        *mp_FileLoader;

   // filename must not be ampty
   if(qs_Filename.isEmpty())
   {
      return -1;
   }

   // load the files
   mp_FileLoader = new FileLoader(this);
   if(mp_FileLoader == NULL)
   {
      return -1;
   }

   m_c_ImageInfo.Clear();
   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename,&c_Options);

   // if loaded, replace the old
   if(pc_NewImage != NULL)
   {
      m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);
   }
   
   TakeImageName(qs_Filename,&c_Options);

   // set zoom to default values
   m_pc_ImageFrame->SetZoomFactor(1);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ZoomChanged");

   // remember the changes for undo
   DoChangedProcessing();

   delete mp_FileLoader;

   // set caption the the new image name
   setCaption(qs_Filename);

   return 0;
} // LoadImage()

//===========================================================================
//
//===========================================================================

void MainFrame::NewFileLoad()
{
   FileLoaderOptions  c_Options;
   PixelBuffer       *pc_NewImage;
   FileLoader        *mp_FileLoader;
   MainFrame         *pc_Image;
   QString           qs_Filename;

   // create a file loader
   mp_FileLoader = new FileLoader(this);
   if(mp_FileLoader == NULL)
   {
      return;
   }

   // open a dialog and ask for the filename
   qs_Filename = mp_FileLoader->AskForFilename(true);

   if(qs_Filename.isEmpty())
   {
      // empty name, then return
      delete mp_FileLoader;
      return;
   }

   // create a new blank image
   pc_Image = new MainFrame(0,
                               m_pc_GlobalStorage->mainApplication,
                               m_pc_ApplicationFrame,100,100);


   c_Options.b_HasMainFramePointer = true;
   c_Options.m_pc_MainFrame        = pc_Image;                               
   // load the file
   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename,&c_Options);
   delete mp_FileLoader;

   // success?
   m_pc_ApplicationFrame ->AddMainFrame(pc_Image);

   // replace the old by the new image
   if(pc_NewImage != NULL)
   {
      pc_Image->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);
   }
   pc_Image->TakeImageName(qs_Filename,&c_Options);

   // remember the changes for undo
   pc_Image->DoChangedProcessing();

   // set caption the the new image name
   pc_Image->setCaption(qs_Filename);
   pc_Image->m_qs_Filename = qs_Filename;

   // show the window
   pc_Image->show();

} // NewFileLoad()

//===========================================================================
//
//===========================================================================

void MainFrame::NewFileBlank()
{
   NewImageBox *pc_NewBox;
   int          i_Result;

   // create a box to ask for dimensions
   pc_NewBox = new NewImageBox(this,m_pc_GlobalStorage);
   if(pc_NewBox == NULL)
   {
      return;
   }

   // show the box and wait until finished
   pc_NewBox->show();
   i_Result = pc_NewBox->exec();

   // if dialog was finished with OK button
   if(i_Result == QDialog::Accepted)
   {
      PixelBuffer *pc_Buffer;
      MainFrame   *pc_Image;
      int          i_NewWidth;
      int          i_NewHeight;
      int          i_RunX;
      int          i_RunY;

      // ask for new values
      i_NewWidth  = pc_NewBox->GetWidth();
      i_NewHeight = pc_NewBox->GetHeight();
      
      // create a new blank image
      pc_Image = new MainFrame(0,
                               m_pc_GlobalStorage->mainApplication,
                               m_pc_ApplicationFrame,
                               i_NewWidth,
                               i_NewHeight);
      m_pc_ApplicationFrame ->AddMainFrame(pc_Image);
      
      // set a white default image into it
      pc_Buffer = pc_Image->m_pc_GlobalStorage->m_pc_MainImage;
      for(i_RunY=0;i_RunY<i_NewHeight;i_RunY++)
      {
         for(i_RunX=0;i_RunX<i_NewWidth;i_RunX++)
         {
            pc_Buffer->Set(i_RunX,i_RunY,0xffffff);
         }
      }

      // show the window
      pc_Image->show();

      m_c_ImageInfo.SetImageName(IMAGEINFO_DEFAULT);
   }
   delete pc_NewBox;

} // NewFileBlank()

//===========================================================================
//
//===========================================================================

ImageFrame  *MainFrame::GetImageFrame()
{
   return m_pc_ImageFrame;

} // GetImageFrame()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomInAccelPressed()
{
   m_pc_ZoomBox->ZoomInOneStep();

} // ZoomInAccelPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::ZoomOutAccelPressed()
{
   m_pc_ZoomBox->ZoomOutOneStep();

} // ZoomOutAccelPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::RotateLeftPressed()
{
   if(m_pc_RotationBox != NULL)
   {
      m_pc_RotationBox->Rotate270();
   }
}  // RotateLeftPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::RotateRightPressed()
{
   if(m_pc_RotationBox != NULL)
   {
      m_pc_RotationBox->Rotate90();
   }

} // RotateRightPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::Rotate180Pressed()
{
   if(m_pc_RotationBox != NULL)
   {
      m_pc_RotationBox->Rotate180();
   }

} // Rotate180Pressed()

//===========================================================================
//
//===========================================================================

void MainFrame::dragEnterEvent(QDragEnterEvent* event)
{
   bool b_Accept;
   b_Accept = m_DragAndDrop.IsEventAcceptable(event);
   event->accept(b_Accept);

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::dragMoveEvent(QDragMoveEvent* event)
{
}

//===========================================================================
//
//===========================================================================

void MainFrame::dropEvent(QDropEvent* event)
{

   int i_DropType;

   i_DropType = m_DragAndDrop.GetEventType( event);
   if(i_DropType == DragAndDrop::AcceptPlain)
   {
      FileLoaderOptions  c_Options;
      PixelBuffer       *pc_Buffer;
      FileLoader        *pc_Loader;
      QString            qs_Filename;

      qs_Filename = m_DragAndDrop.GetFilename(event);
      if(qs_Filename.isEmpty())
      {
         return;
      }
      // create a file loader
      pc_Loader = new FileLoader(this);
      if(pc_Loader == NULL)
      {
         return;
      }

      m_c_ImageInfo.Clear();
      // load the file
      c_Options.b_HasMainFramePointer = true;
      c_Options.m_pc_MainFrame        = this;
      pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_Options);

      // success?
      if(pc_Buffer != NULL)
      {
         // then replace the current
         ReplaceNewImage( pc_Buffer, qs_Filename);
         TakeImageName(qs_Filename,&c_Options);
      }
      delete pc_Loader;
   }
   else if(i_DropType == DragAndDrop::AcceptPixelbuffer)
   {
      PixelBuffer *pc_NewImage;
      QPoint       p_DropPosition;
      int          i_BufferX;
      int          i_BufferY;

      pc_NewImage = m_DragAndDrop.GetPixelBuffer(event);
      if(pc_NewImage ==NULL)
      {
         return;
      }

      // get drop position in MainFram coordiantes
      p_DropPosition = event->pos();
      // map to global coordinates
      p_DropPosition = mapToGlobal(p_DropPosition);
      // map into drawing area coordinates
      p_DropPosition = m_pc_GlobalStorage->m_pc_DrawingArea->mapFromGlobal(p_DropPosition);

      // make buffer coordinates out out it
      m_pc_GlobalStorage->m_pc_Zoom->GetBufferCoordinate(p_DropPosition.x(),
                                                            p_DropPosition.y(),
                                                      i_BufferX,
                                                      i_BufferY);
      // now tell the motivbox, that there is
      // a new image
      m_pc_MotiveBox->show();

      // add motiv to the buffer coordinate
      m_pc_MotiveBox->AddImageToWorkArea(pc_NewImage,
                                                i_BufferX,
                                                i_BufferY);
         // ImageDropped(pc_ReceivedImage);

   }
} // dropEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::ReplaceNewImage(PixelBuffer *pc_NewImage, QString qs_Filename)
{
   // replace the old by the new image
   m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   // reset the zoom factor to 1
   m_pc_ImageFrame->SetZoomFactor(1);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                     this,
                                                     "ZoomChanged");

   // remember the changes for undo
   DoChangedProcessing();

   // set caption the the new image name
   setCaption(qs_Filename);

   m_qs_Filename = qs_Filename;


} // ReplaceNewImage()

//===========================================================================
//
//===========================================================================

void MainFrame::SetParentPicture(PictureUpdateConnection *pc_ParentMessageHandler)
{
   m_pc_ParentMessageHandler = pc_ParentMessageHandler;

   if(m_pc_ParentMessageHandler != NULL)
   {
      // attach my own callback
      m_pc_ParentMessageHandler->SetChildCallback(
                                       Callback(ParentUpdateMessageCB,this));

      // add a menu point
      m_FileMenu->insertSeparator();
      m_i_ParentMenuId = m_FileMenu->insertItem(tr("Update Parent"),
                                                 this,
                                                 SLOT(UpdateParentPressed() ));
   }

} // SetParentPicture()

//===========================================================================
//
//===========================================================================

void *MainFrame::ParentUpdateMessage(void *p_Data)
{
   PictureUpdateConnectionMessage *pc_UpdateMessage;

   pc_UpdateMessage = (PictureUpdateConnectionMessage *) p_Data;
   if(pc_UpdateMessage->m_i_MessageType
                     ==  PictureUpdateConnectionMessage::ParentTermination)
   {
      // make update parent invisible
      if(m_i_ParentMenuId != -1)
      {
         m_FileMenu->setItemVisible(m_i_ParentMenuId,false);
      }
      m_pc_ParentMessageHandler = NULL;
   }
      

   return NULL;
} // ParentUpdateMessage()

void *MainFrame::ParentUpdateMessageCB(void *p_ThisData,
                                  void *p_Data)
{
   MainFrame * pc_This;
   pc_This = (MainFrame *) p_ThisData;
   return pc_This->ParentUpdateMessage(p_Data);
} // ParentUpdateMessageCB()

//===========================================================================
//
//===========================================================================

MainFrame * MainFrame::NewPicture(PixelBuffer *pc_Example)
{
   PixelBuffer *pc_NewImage;
   MainFrame   *pc_Image;
   QString      qs_Filename;

   // create an own duplicate
   pc_NewImage = new PixelBuffer(pc_Example);
   if(pc_NewImage == NULL)
   {
      return NULL;
   }

   // create a new blank image
   pc_Image = new MainFrame(0,
                            g_Application->m_pc_QApplication,
                            g_Application->m_pc_ApplicationFrame,
                            100,100);

   g_Application->m_pc_ApplicationFrame ->AddMainFrame(pc_Image);

   // replace the old by the new image
   pc_Image->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);


   // remember the changes for undo
   pc_Image->DoChangedProcessing();

   // set caption the the new image name
   pc_Image->setCaption(qs_Filename);
   pc_Image->m_qs_Filename = qs_Filename;

   // show the window
   pc_Image->show();

   return pc_Image;
} // NewPicture()

//===========================================================================
//
//===========================================================================

MainFrame * MainFrame::NewPicture(QString qs_Filename)
{

   FileLoaderOptions  c_Options;
   FileLoader        *mp_FileLoader;
   PixelBuffer       *pc_NewImage;
   MainFrame         *pc_Image;

   // create a new blank image
   pc_Image = new MainFrame(0,
                            g_Application->m_pc_QApplication,
                            g_Application->m_pc_ApplicationFrame,
                            100,100);

   mp_FileLoader = new FileLoader(NULL);

   if(mp_FileLoader == NULL)
   {
      return NULL;
   }
   
   // create an own duplicate
   c_Options.b_HasMainFramePointer = true;
   c_Options.m_pc_MainFrame        = pc_Image;

   // load the file
   pc_NewImage = mp_FileLoader->LoadFile(qs_Filename,&c_Options);
   delete mp_FileLoader;

   if(pc_NewImage == NULL)
   {
      return NULL;
   }


   g_Application->m_pc_ApplicationFrame ->AddMainFrame(pc_Image);

   // replace the old by the new image
   pc_Image->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);
   pc_Image->TakeImageName(qs_Filename,&c_Options);


   // remember the changes for undo
   pc_Image->DoChangedProcessing();

   // set caption the the new image name
   pc_Image->setCaption(qs_Filename);
   pc_Image->m_qs_Filename = qs_Filename;

   // show the window
   pc_Image->show();

   return pc_Image;
} // NewPicture()

//===========================================================================
//
//===========================================================================

void MainFrame::UpdateParentPressed()
{
   if(m_pc_ParentMessageHandler != NULL)
   {
      m_pc_ParentMessageHandler->TellChildHasUpdatedImage(
                                     m_pc_GlobalStorage->m_pc_MainImage);
   }
   
} // UpdateParentPressed()

//===========================================================================
//
//===========================================================================

ColorSelectBox  *MainFrame::GetColorSelectBox()
{
   return m_pc_ColorSelectBox;
}

//===========================================================================
//
//===========================================================================

int MainFrame::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"MainFrame");

   m_pc_MotiveBox->WriteFile(pc_FileWriter,"MotiveBox");

   m_pc_GridBox->WriteFile(pc_FileWriter,"GridBox");
   
   m_pc_ColorSelectBox->WriteFile(pc_FileWriter,"ColorSelectBox");

   m_pc_RedEyeBox->WriteFile(pc_FileWriter,"RedEyeBox");

   m_pc_FilterBox->WriteFile(pc_FileWriter,"FilterBox");

   m_pc_PaintBox->WriteFile(pc_FileWriter,"PaintBox");

   m_pc_GraphikBox->WriteFile(pc_FileWriter,"GraphikBox");

   m_pc_RotationBox->WriteFile(pc_FileWriter,"RotationBox");

   m_pc_CorrectionBox->WriteFile(pc_FileWriter,"CorrectionBox");

   m_pc_BlendBox->WriteFile(pc_FileWriter,"BlendBox");

   m_pc_EffectBox->WriteFile(pc_FileWriter,"EffectBox");

   m_c_ImageInfo.WriteFile(pc_FileWriter,"ImageInfo");

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int MainFrame::LoadFile(PericoFileWriter *pc_FileWriter)
{

   m_c_ImageInfo.Clear();

   PERICO_READ_STRUCTURE_START

   if(    qs_Name.compare("MotiveBox") == 0
       && qs_Value.compare("MotiveBox") == 0
       && b_IsStartStructure)
   {
      m_pc_MotiveBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("GridBox") == 0
            && qs_Value.compare("GridBox") == 0
            && b_IsStartStructure)
   {
      m_pc_GridBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("ColorSelectBox") == 0
            && qs_Value.compare("ColorSelectBox") == 0
            && b_IsStartStructure)
   {
      m_pc_ColorSelectBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("RedEyeBox") == 0
            && qs_Value.compare("RedEyeBox") == 0
            && b_IsStartStructure)
   {
      m_pc_RedEyeBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("FilterBox") == 0
            && qs_Value.compare("FilterBox") == 0
            && b_IsStartStructure)
   {
      m_pc_FilterBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("PaintBox") == 0
            && qs_Value.compare("PaintBox") == 0
            && b_IsStartStructure)
   {
      m_pc_PaintBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("GraphikBox") == 0
            && qs_Value.compare("GraphikBox") == 0
            && b_IsStartStructure)
   {
      m_pc_GraphikBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("RotationBox") == 0
            && qs_Value.compare("RotationBox") == 0
            && b_IsStartStructure)
   {
      m_pc_RotationBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("CorrectionBox") == 0
            && qs_Value.compare("CorrectionBox") == 0
            && b_IsStartStructure)
   {
      m_pc_CorrectionBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("BlendBox") == 0
            && qs_Value.compare("BlendBox") == 0
            && b_IsStartStructure)
   {
      m_pc_BlendBox->LoadFile(pc_FileWriter);
   }

   else if(    qs_Name.compare("EffectBox") == 0
            && qs_Value.compare("EffectBox") == 0
            && b_IsStartStructure)
   {
      m_pc_EffectBox->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("ImageInfo") == 0
            && qs_Value.compare("ImageInfo") == 0
               && b_IsStartStructure)
   {
      m_c_ImageInfo.LoadFile(pc_FileWriter);
   }
   PERICO_READ_STRUCTURE_END
   
   // everything is OK
   return 0;
} // LoadFile()

//===========================================================================
//
//===========================================================================

void MainFrame::KeyUpPressed()
{
   m_pc_ImageFrame->ScrollbarShortcut(true,false,false,false);
} // :KeyUpPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::KeyDownPressed()
{
   m_pc_ImageFrame->ScrollbarShortcut(false,true,false,false);
} // :KeyDownPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::KeyLeftPressed()
{
   m_pc_ImageFrame->ScrollbarShortcut(false,false,true,false);
} // :KeyLeftPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::KeyRightPressed()
{
   m_pc_ImageFrame->ScrollbarShortcut(false,false,false,true);
} // :KeyRightPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::AddToToolbar(QWidget *w_AWidget)
{
   QBoxLayout *l_Box;

   l_Box = m_w_Toolbar->boxLayout();

   l_Box->addWidget( w_AWidget );

} // AddToToolbar

//===========================================================================
//
//===========================================================================

QToolBar *MainFrame::GetSystemToolbar()
{
   return m_w_Toolbar;
} // GetSystemToolbar

//===========================================================================
//
//===========================================================================

void MainFrame::timerEvent( QTimerEvent *e )
{
   bool b_IsInSelection;

   b_IsInSelection = m_pc_GlobalStorage->m_pc_Selection->IsInSelection();

   /*
   static int i_Timer=0;
   fprintf(stderr,"TimerEvent %d %d \n",
           i_Timer++,
           (int) b_IsInSelection);
   */
   if( ! b_IsInSelection)
   {
      m_pc_Autosave->TimerEvent();
   }
   
}  // timerEvent()

//===========================================================================
//
//===========================================================================

void MainFrame::SnapshotPressed()
{
   QString   qs_Directory;
   QString   qs_Filename;
   QString   qs_ImageName;
   QString   qs_Fullname;
   time_t    tt_CurrentTime;
   struct tm tm_Time;
   
   tt_CurrentTime = time(NULL);

   memcpy(&tm_Time,localtime(&tt_CurrentTime),sizeof(struct tm));

   /*
   fprintf(stderr,"ImageName is %s\n",
         (const char *) m_c_ImageInfo.m_qs_ImageName);
   */

   qs_Directory = g_Application->m_qs_TempDirectory;
   qs_Directory +="/Snapshot";

   // create it, in case of non existent
   FileLoader::MakeDirectory( qs_Directory);

   qs_ImageName = m_c_ImageInfo.m_qs_ImageName;
   if(qs_ImageName.length() <1)
   {
      qs_ImageName=IMAGEINFO_DEFAULT;
   }
   qs_Filename.sprintf("%s-%04d%02d%02d_%02d%02d%02d.perico",
            (const char *) qs_ImageName,
            tm_Time.tm_year+1900,
            tm_Time.tm_mon+1,
            tm_Time.tm_mday,
            tm_Time.tm_hour,
            tm_Time.tm_min,
            tm_Time.tm_sec);

   qs_Fullname =qs_Directory + "/" + qs_Filename;


   SaveFile(qs_Fullname);

   m_pc_SnapshotBox->Update();
} // SnapshotPressed()

//===========================================================================
//
//===========================================================================

void MainFrame::HardcopyPressed()
{
   Hardcopy *pc_Hardcopy;

   // create hardcopy
   pc_Hardcopy = new Hardcopy(m_pc_GlobalStorage);

   // perform it
   pc_Hardcopy->DoHardcopy();

   // delete hardcopy
   delete pc_Hardcopy;
} // HardcopyPressed()

//===========================================================================
//
//===========================================================================


void MainFrame::TakeImageName(QString qs_Filename,
               FileLoaderOptions *pc_FileOptions)
{
   //fprintf(stderr,"image name is %s\n",
   //   (const char *) m_c_ImageInfo.m_qs_ImageName);
   if(m_c_ImageInfo.m_qs_ImageName.isEmpty())
   {
      
      m_c_ImageInfo.SetImageName(FileLoader::GetPureName(qs_Filename));
   }
   if(pc_FileOptions != NULL)
   {
      if(pc_FileOptions->IsExifInfoValid())
      {
         m_c_ImageInfo.SetExifInfo(pc_FileOptions->GetExifInfo());
      }
   }
   
   m_pc_SnapshotBox->SetImageName(m_c_ImageInfo.m_qs_ImageName);
   m_pc_ImageInfoBox->SetImageInfo(&m_c_ImageInfo);

   UpdateImageSize();

} // TakeImageName()

//===========================================================================
//
//===========================================================================

void MainFrame::UpdateImageSize()
{
   PixelBuffer *pc_ActualImage;
   pc_ActualImage = m_pc_ImageFrame->GetActualPixelBuffer(),
   m_pc_ImageInfoBox->SetSize(pc_ActualImage->GetWidth(),
                                   pc_ActualImage->GetHeight());

} // UpdateImageSize()

//===========================================================================
//
//===========================================================================

void MainFrame::ImageInformationChanged()
{
   m_pc_ImageInfoBox->GetImageInfo(&m_c_ImageInfo);

   if(!m_c_ImageInfo.m_qs_ImageName.isEmpty())
   {
      m_pc_SnapshotBox->SetImageName(m_c_ImageInfo.m_qs_ImageName);

   }

} // ImageInformationChanged()

//===========================================================================
//
//===========================================================================

