#ifndef _MAINFRAME_H
#define _MAINFRAME_H


#include <qpopupmenu.h>
#include <qaccel.h>


#include "ImageFrame.h"

#include "ZoomBox.h"
#include "GlobalStorage.h"
#include "ColorSelectBox.h"
#include "TracerBox.h"
#include "MaskBox.h"
#include "GridBox.h"
#include "PictureUpdateConnection.h"
#include "PericoFileWriter.h"
#include "ImageInfo.h"
#include "DragAndDrop.h"

class QUPixelButton;
class QUToolButton;
class ManipulationBox;
class PaintBox;
class GraphikBox;
class AboutBox;
class ApplicationFrame;
class RotationBox;

class CorrectionBox;
class MotiveBox;
class RedEyeBox;
class BlendBox;
class EffectBox;
class SnapshotBox;
class Autosave;
class ImageInfoBox;
class FileLoaderOptions;
class TestObjectBox;
class FilterBox;

class MainFrame : public QMainWindow
{
  Q_OBJECT

public:
   MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame);
   MainFrame(QWidget *parent,
                  QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame,
                  int i_Width,int i_Height);
   ~MainFrame();


   void CreateMenu();
   int  LoadImage(QString qs_Filename);
   ImageFrame      *GetImageFrame();
   ColorSelectBox  *GetColorSelectBox();
   

   void SetParentPicture(PictureUpdateConnection *pc_ParentPicture);
   static MainFrame * NewPicture(PixelBuffer *pc_Example);
   static MainFrame * NewPicture(QString qs_Filename);

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

   void AddToToolbar(QWidget *w_AWidget);
   QToolBar *GetSystemToolbar();

   int SaveFile(QString qs_Filename);
   
public slots:
   void About();

   void LoadFile();
   void SaveFile();
   void CloseFile();
   void ExitFile();

   void NewFileLoad();
   void NewFileBlank();

   void ManipulationBoxMenu();
   void RotationBoxMenu();
   void GraphikBoxMenu();
   void PaintBoxMenu();
   void ZoomBoxMenu();
   void ColorSelectMenu();
   void FilterBoxMenu();
   void MaskBoxMenu();
   void GridBoxMenu();
   void CorrectionBoxMenu();
   void RedEyeBoxMenu();
   void MotiveBoxMenu();
   void BlendBoxMenu();
   void EffectBoxMenu();
   void SnapshotBoxMenu();
   void ImageInfoBoxMenu();
   

   void SettingsMenu();

   void UndoMenu();
   void UndoSelectionMenu();

   void AutosaveBoxMenu();

   void TracerBoxMenu();
   void TestObjectBoxMenu();

   void DoChangedProcessing();

   void UndoIconSelected(int i_Value);
   void UndoButtonPressed();
   void RedoButtonPressed();
   void ZoomInAccelPressed();
   void ZoomOutAccelPressed();
   void UpdateParentPressed();
   void SnapshotPressed();
   void HardcopyPressed();

   void RotateLeftPressed();
   void RotateRightPressed();
   void Rotate180Pressed();
   void KeyUpPressed();
   void KeyDownPressed();
   void KeyLeftPressed();
   void KeyRightPressed();

   void ImageInformationChanged();


protected:
   void closeEvent(QCloseEvent *);

   void dragEnterEvent(QDragEnterEvent* event);
   void dragMoveEvent(QDragMoveEvent* event);
   void dropEvent(QDropEvent* event);


private: // functions
   void CreateSimple(QApplication *mainApplication,
                  ApplicationFrame *pc_ApplicationFrame);

     virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   static void OnMessageReceiveCB(void *p_ThisData,
                                  QString qs_MessageType,
                                  void *p_Data);

   void SetUndoMenu();
   void ReplaceNewImage(PixelBuffer *pc_NewImage, QString qs_Filename);

   void *ParentUpdateMessage(void *p_Data);

   static void *ParentUpdateMessageCB(void *p_ThisData,
                                  void *p_Data);
   virtual void timerEvent( QTimerEvent *e ) ;

   void TakeImageName(QString qs_Filename,
               FileLoaderOptions *pc_FileOptions);
   void UpdateImageSize();

private: // variables

   ApplicationFrame *m_pc_ApplicationFrame;

   ImageFrame      *m_pc_ImageFrame;

   ColorSelectBox  *m_pc_ColorSelectBox;
   ManipulationBox *m_pc_ManipulationBox;
   RotationBox     *m_pc_RotationBox;
   ZoomBox         *m_pc_ZoomBox;
   GraphikBox      *m_pc_GraphikBox;
   PaintBox        *m_pc_PaintBox;
   GlobalStorage   *m_pc_GlobalStorage;
   TracerBox       *m_pc_TracerBox;
   FilterBox       *m_pc_FilterBox;
   MaskBox         *m_pc_MaskBox;
   AboutBox        *m_pc_AboutBox;
   GridBox         *m_pc_GridBox;
   TestObjectBox   *m_pc_TestObjectBox;
   CorrectionBox   *m_pc_CorrectionBox;
   RedEyeBox       *m_pc_RedEyeBox;
   BlendBox        *m_pc_BlendBox;
   EffectBox       *m_pc_EffectBox;
   SnapshotBox     *m_pc_SnapshotBox;
   ImageInfoBox     *m_pc_ImageInfoBox;

   QToolBar        *m_w_Toolbar;

   QUToolButton *m_w_UndoButton;
   QUToolButton *m_w_RedoButton;
   QUToolButton *m_w_ColorSelectButton;
   QUToolButton *m_w_ZoomInButton;
   QUToolButton *m_w_ZoomOutButton;
   QUToolButton *m_w_SnapshotButton;
   QUToolButton *m_w_HardcopyButton;

   DragAndDrop    m_DragAndDrop;
   
public:
   MotiveBox       *m_pc_MotiveBox;
private:
   QPopupMenu      *m_UndoMenu;
   QAccel          *m_Accelerator ;

   int              m_i_UndoId;
   int              m_i_ParentMenuId;

   QString          m_qs_Filename;
   ImageInfo       m_c_ImageInfo;

   PictureUpdateConnection *m_pc_ParentMessageHandler;

   Autosave * m_pc_Autosave;


   QPopupMenu  *m_FileMenu;
   QPopupMenu  *m_WorkMenu;
   QPopupMenu  *m_ViewMenu;
   QPopupMenu  *m_TraceMenu;
   QPopupMenu  *m_HelpMenu;
   QPopupMenu  *m_NewMenu;
   QPopupMenu  *m_OptionsMenu;


}; // class MainFrame

#endif //  _MAINFRAME_H
