#ifndef _MANIPULATIONBOX_H
#define _MANIPULATIONBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qradiobutton.h>

#include <qvgroupbox.h>

#include "StandardDialog.h"
#include "QUPixelButton.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "QUColorDragWidget.h"


class ManipulationBox :  public StandardDialog 
{
Q_OBJECT

public:
   ManipulationBox(QWidget *parent=0,
      GlobalStorage *pc_Global=0);

   ~ManipulationBox();
    
public slots:
   void DoSliderValueChanged(int);
 
   void DoScale();
   
   void CutSelection();
   void CutRectangle();
   void CutDestinationChanged(int i_Destination);
   void FrameSelectionModeToggled(bool b_Mode);
   void FrameDoCut();
   void FrameReset();

   void FrameXTextChanged (const QString &);
   void FrameYTextChanged (const QString &);
   void FrameWidthTextChanged (const QString &);
   void FrameHeightTextChanged (const QString &);

   void ExtendDoExtend();

protected:
   virtual void hideEvent(QHideEvent *);
        
private:

   void SetImageSize();
   void ReceivedNewImageSize();
    
   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);
   void HandleCuttedImage(PixelBuffer *pc_NewImage);

   void SetFrameModeSensitivity();
   void DisplayFrameRectangle();
   void SetFrameIntoValidImage();
   void SetFrameToDefaultValue();


   void GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);

    int GetIntFromText(QLineEdit *pc_Edit);
private:
    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);


   QPushButton *m_ExecuteButton;
   QUPixelButton *m_MirrorXButton;
   QUPixelButton *m_MirrorYButton;
   QLineEdit   *m_OriginalWidth;
   QLineEdit   *m_OriginalHeight;
   QLineEdit   *m_NewWidth;
   QLineEdit   *m_NewHeight;
   QSlider     *m_ResizeSlider;
   QLabel      *m_ResizeSliderLabel;
   
   QPushButton *m_CutSelectionButton;
   QPushButton *m_CutRectangleButton;
   QRadioButton *m_CutReplaceCurrent;
   QRadioButton *m_CutCreateNew;


   enum
   {
      CutDestinationReplace=0,
      CutDestinationNew
   };
   int          m_i_CutDestination;


   int m_i_LowerScaleEnd;
   int m_i_UpperScaleEnd;
   int m_i_CurrentSlider;


   QVGroupBox * m_FrameBox;
   QCheckBox  *m_FrameActiveCheck;
   QWidget     *m_FrameGridWidget;
   QGridLayout *m_FrameGridLayout;
   QLineEdit   *m_FrameXEdit;
   QLineEdit   *m_FrameYEdit;
   QLineEdit   *m_FrameWidthEdit;
   QLineEdit   *m_FrameHeightEdit;
   QPushButton *m_FrameCutExecuteButton;
   QPushButton *m_FrameResetButton;
   QHBoxLayout  *FrameCutExecuteLayout;
   QWidget      *FrameCutExecuteBox;

   bool m_b_FrameModeActive;   
   GraphicObjectLayer     *m_pc_FrameLayer;
   GraphicObjectRectangle *m_pc_FrameRectangle;
   bool m_b_InUpdate;


   QVGroupBox  *m_ExtendBox;
   QGridLayout *m_ExtendGridLayout;
   QLineEdit   *m_ExtendTopEdit;
   QLineEdit   *m_ExtendBottomEdit;
   QLineEdit   *m_ExtendLeftEdit;
   QLineEdit   *m_ExtendRightEdit;
   QPushButton *m_ExtendExecuteButton;
   QUColorDragWidget *m_ExtendColor;
   
   
};


#endif // _MANIPULATIONBOX_H
