//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: MotiveBox.c
//---------------------------------------------------------------------------
// Purpose: dialog box for all motive operations.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <float.h>
#include <stdlib.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>


#include "PixelBuffer.h"
#include "MotiveBox.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "ApplicationFrame.h"
#include "ApplicationStorage.h"
#include "Selection.h"
#include "PixelBufferObject.h"
#include "QUClickLabel.h"
#include "BufferPainter.h"
#include "GraphicObjectList.h"
#include "Zoom.h"

//===========================================================================

#define MOTIVELAYER "MotiveLayer"

//===========================================================================
//
//===========================================================================

MotiveBox::MotiveBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
                        StandardDialog(parent,pc_Global)
{
   QBoxLayout   *l_Layout;
   QHGroupBox   *CutBox;
   QHGroupBox   *DeleteBox;
   QHGroupBox   *OrderBox;
   QVGroupBox   *InsertBox;
   QWidget      *PreviewWidget;

   LoadPosition("MotiveBox");
   
   setCaption(tr("Motive"));

   m_i_PixmapDimension           = 64;
   m_b_ObjectModeActive          = false;
   m_i_SelectionMode             = MotiveSelectionNone;
   m_b_TransparencyEnabled       = false;
   m_i_TransparencyFactor        = 50;
   m_b_TransparencyBorderEnabled = false;
   m_i_TransparencyBorderFactor  = 10;
   m_b_ObjectViewTransparent     = false;
   m_b_InTextUpdate              = false;
   m_b_FactorDisplayed           = true;

   if(m_pc_GlobalStorage->m_pc_Zoom != NULL)
   {
      m_pc_GlobalStorage->m_pc_Zoom
                        ->DrawObjectImagesTransparent(m_b_ObjectViewTransparent);
   }

   // create the layer for the motive
   m_pc_ObjectLayer= m_pc_GlobalStorage->m_pc_ObjectMain
                                       ->CreateLayer(MOTIVELAYER);
   m_pc_ObjectLayer->SetVisible(false);

   // create the box
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);
   // create tha active checkbox
   m_ActivateCheck = new QCheckBox(tr("Active"),m_w_TopWidget);
   l_Layout->addWidget(m_ActivateCheck);

   // create tha view transparent checkbox
   m_ObjectViewTransparentCheck = new QCheckBox(tr("View Transparent"),m_w_TopWidget);
   l_Layout->addWidget(m_ObjectViewTransparentCheck);

   
   CutBox = new QHGroupBox(tr("Cut"),m_w_TopWidget);
   l_Layout->addWidget(CutBox);
   {
       m_CutSelectionMask = new QPushButton(tr("Selection Mask"),CutBox);
       m_CutRectangle     = new QPushButton(tr("Rectangle"),CutBox);
       m_CutFreeHand      = new QPushButton(tr("Free Hand"),CutBox);

   } // CutBox

   DeleteBox = new QHGroupBox(tr("Delete"),m_w_TopWidget);
   l_Layout->addWidget(DeleteBox);
   {
       m_DeleteButton = new QPushButton(tr("Selected"),DeleteBox);
       m_DeleteAllButton = new QPushButton(tr("All"),DeleteBox);

   } // InsertBox

   InsertBox = new QVGroupBox(tr("Insert"),m_w_TopWidget);
   l_Layout->addWidget(InsertBox);
   {
      QWidget *w_ButtonWidget;
      QHBoxLayout *l_ButtonLayout;
      w_ButtonWidget = new QWidget(InsertBox);
      l_ButtonLayout = new QHBoxLayout(w_ButtonWidget,2);
      
      m_InsertButton = new QPushButton(tr("Selected"),w_ButtonWidget);
      m_InsertAllButton = new QPushButton(tr("All"),w_ButtonWidget);
      l_ButtonLayout->addWidget(m_InsertButton);  
      l_ButtonLayout->addWidget(m_InsertAllButton);


      m_TransparencyEnableCheck = new QCheckBox(tr("Transparency"),
                                                InsertBox);
      m_TransparencyEnableCheck->setChecked(m_b_TransparencyEnabled);

      m_TransparencySlider = new QULabelSlider(InsertBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_TransparencyFactor,
                                       tr("Factor"),
                                       6);

      m_TransparencyBorderEnableCheck = new QCheckBox(tr("Transparency at Border"),
                                                InsertBox);
      m_TransparencyBorderEnableCheck->setChecked(m_b_TransparencyBorderEnabled);

      m_TransparencyBorderSlider = new QULabelSlider(InsertBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_TransparencyBorderFactor,
                                       tr("Border Width"),
                                       12);

   } // InsertBox

   OrderBox = new QHGroupBox(tr("Object Order"),m_w_TopWidget);
   l_Layout->addWidget(OrderBox);
   {
       m_OrderUpButton   = new QPushButton(tr("At Top"),OrderBox);
       m_OrderDownButton = new QPushButton(tr("At Bottom"),OrderBox);
   } // OrderBox

   PreviewWidget = new QWidget(m_w_TopWidget);
   l_Layout->addWidget(PreviewWidget);
   {
      QHBoxLayout *l_PreviewLayout;
      QGridLayout  *grid;
      int          i_CharWidth;
      
      i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();

      l_PreviewLayout = new QHBoxLayout(PreviewWidget,2);

      // create the drag area
      m_SelectedImageLabel = new QUImageDragLabel(PreviewWidget,
                                              QUImageDragLabel::ModeUseWidthHeight
                                            | QUImageDragLabel::ModeEnableEdit,
                                      m_pc_GlobalStorage,
                                      m_i_PixmapDimension,
                                      m_i_PixmapDimension);
      l_PreviewLayout->addWidget(m_SelectedImageLabel);

      m_SelectedInfoWidget= new QWidget(PreviewWidget);
      l_PreviewLayout->addWidget(m_SelectedInfoWidget);

      m_OriginalWidthEdit = new QtUtilReadOnlyLineEdit(m_SelectedInfoWidget);
      m_OriginalHeightEdit= new QtUtilReadOnlyLineEdit(m_SelectedInfoWidget);
      m_ScaleXEdit        = new QLineEdit(m_SelectedInfoWidget);
      m_ScaleYEdit        = new QLineEdit(m_SelectedInfoWidget);

      m_OriginalWidthEdit   ->setMaximumWidth(i_CharWidth*4);
      m_OriginalHeightEdit  ->setMaximumWidth(i_CharWidth*4);
      m_ScaleXEdit          ->setMaximumWidth(i_CharWidth*6);
      m_ScaleYEdit          ->setMaximumWidth(i_CharWidth*6);

      grid = new QGridLayout( m_SelectedInfoWidget, 2, 4, 5 );

      m_ScaleXLabel = new QUClickLabel(QString(tr("Scale")),m_SelectedInfoWidget);
      m_ScaleYLabel = new QUClickLabel(QString(tr("Scale")),m_SelectedInfoWidget);
      
      // grid with all labels and edit boxes
      grid->addWidget(new QLabel(tr("Original Width"),m_SelectedInfoWidget),0,0);
      grid->addWidget(m_OriginalWidthEdit,0,1);
      grid->addWidget(m_ScaleXLabel,0,2);
      grid->addWidget(m_ScaleXEdit,0,3);
      grid->addWidget(new QLabel(tr("Original Height"),m_SelectedInfoWidget),1,0);
      grid->addWidget(m_OriginalHeightEdit,1,1);
      grid->addWidget(m_ScaleYLabel,1,2);
      grid->addWidget(m_ScaleYEdit,1,3);

                
   }  // block for previewWidget

         
   // connect all elements with slots
   connect( m_ActivateCheck,
            SIGNAL(toggled(bool)),
            SLOT( ActiveModeToggled(bool )));

   connect( m_ObjectViewTransparentCheck,
            SIGNAL(toggled(bool)),
            SLOT( ViewTransparenToggled(bool )));

   connect( m_CutSelectionMask,
            SIGNAL(pressed()),
            SLOT( CutSelectionMaskPressed()));

   connect( m_CutRectangle,
            SIGNAL(pressed()),
            SLOT( CutRectanglePressed()));

   connect( m_CutFreeHand,
            SIGNAL(pressed()),
            SLOT( CutFreehandPressed()));

   connect( m_DeleteButton,
            SIGNAL(pressed()),
            SLOT( DeleteButtonPressed()));

   connect( m_DeleteAllButton,
            SIGNAL(pressed()),
            SLOT( DeleteAllButtonPressed()));

   connect( m_InsertButton,
            SIGNAL(pressed()),
            SLOT( InsertButtonPressed()));

   connect( m_InsertAllButton,
            SIGNAL(pressed()),
            SLOT( InsertAllButtonPressed()));

   connect( m_TransparencyEnableCheck,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
   connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

   connect( m_TransparencyBorderEnableCheck,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyBorderEnableChanged(bool) )
             );
   connect( m_TransparencyBorderSlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyBorderFactorChanged(int )) );

   connect( m_OrderUpButton,
            SIGNAL(pressed()),
            SLOT( OrderUpClicked()));
   connect( m_OrderDownButton,
            SIGNAL(pressed()),
            SLOT( OrderDownClicked()));


   connect(m_ScaleXEdit,
            SIGNAL(textChanged(const QString &)),
            SLOT(SizeXTextChanged (const QString &)) );
   connect(m_ScaleYEdit,
            SIGNAL(textChanged(const QString &)),
            SLOT(SizeYTextChanged (const QString &)) );

   connect(m_ScaleXLabel,
            SIGNAL(clicked()),
            SLOT(SizeLabelClicked(void)) );
   connect(m_ScaleYLabel,
            SIGNAL(clicked()),
            SLOT(SizeLabelClicked(void)) );

   connect(m_SelectedImageLabel,
           SIGNAL(ImageDropped(PixelBuffer*)),
           SLOT(MotiveDropped(PixelBuffer *)) );
               
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");



   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");

   // clear text                                                       
   SetObjectInfo(NULL);
} // MotiveBox()

//===========================================================================
//
//===========================================================================

MotiveBox::~MotiveBox()
{
    
} // destructor

//===========================================================================
//
//===========================================================================

void MotiveBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   if(qs_MessageType.compare("ImageResize") ==0)
   {
      PutMotivesOverTheImage();

   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
      PutMotivesOverTheImage();

   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void MotiveBox::showEvent(QShowEvent *)
{
   m_ActivateCheck->setChecked(true);
   SetSensitivity();   
}  // showEvent()

//===========================================================================
//
//===========================================================================

void MotiveBox::hideEvent(QHideEvent *)
{
   // make the layer invisible
   m_ActivateCheck->setChecked(false);
   
}  // hideEvent()

//===========================================================================
//
//===========================================================================

void MotiveBox::ActiveModeToggled(bool b_Mode)
{
   if(b_Mode == m_b_ObjectModeActive)
   {
      // no change
      return;
   }

   m_b_ObjectModeActive = b_Mode;

   if( m_b_ObjectModeActive)
   {
      // make the layer visible
      m_pc_ObjectLayer->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      // set this layer as input layer
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(MOTIVELAYER);

      // enable active action
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(true);

      // set the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
   }
   else // not active
   {
      // make the layer invisible
      // remove the event callback
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));

      // disable input
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);

      m_pc_ObjectLayer->SetVisible(false);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   } // else not active


} // ActiveModeToggled()

//===========================================================================
//
//===========================================================================

void MotiveBox::ViewTransparenToggled(bool b_Mode)
{
   m_b_ObjectViewTransparent = b_Mode;

   m_pc_GlobalStorage->m_pc_Zoom->DrawObjectImagesTransparent(b_Mode);
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   
} // ViewTransparenToggled()

//===========================================================================
//
//===========================================================================

void MotiveBox::CutSelectionMaskPressed()
{
   PixelBuffer * pc_NewImage;
   RectRegion    c_CopyRegion;
   
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage
                ->CreateNewImageFromSelected( &c_CopyRegion);

   if(pc_NewImage == NULL)
   {
      return;
   }

   AddImageToWorkArea(pc_NewImage,
                      c_CopyRegion.GetStartX(),
                      c_CopyRegion.GetStartY()
                      );
   
} // CutSelectionMaskPressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::CutRectanglePressed()
{
   // active a rectangle selection

   // set the selection mode
   m_i_SelectionMode = MotiveSelectionRectangle;

   if(  m_pc_GlobalStorage != NULL)
   {
      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionRectangle,
                                 tr("Cut Rectangle Motive"),
                                 Callback(SelectionEventArrivedCB,this)
                                 );

   }
   
} // CutRectanglePressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::CutFreehandPressed()
{
   // active a freehand selection

   // set the selection mode
   m_i_SelectionMode = MotiveSelectionFreehand;


   if(  m_pc_GlobalStorage != NULL)
   {
      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionFreeDraw,
                                 tr("Cut free Hand Motive"),
                                 Callback(SelectionEventArrivedCB,this)
                                 );

   }
   
} //  CutFreehandPressed()

//===========================================================================
//
//===========================================================================

void *MotiveBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      // fprintf(stderr,"MotiveBox: SelectionEventArrived\n");

      switch(m_i_SelectionMode)
      {
      case MotiveSelectionRectangle:
         CutRectangleImage();         
         break;
      case MotiveSelectionFreehand:
         CutFreeHandImage();
         break;
      default:
        break;
      }
   }
   
   // reset the SelectionMode
   m_i_SelectionMode = MotiveSelectionNone;
   return NULL;
}

void *MotiveBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   MotiveBox * pc_This;
   pc_This = (MotiveBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);
} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void MotiveBox::AddImageToWorkArea(PixelBuffer *pc_Image,
                     int i_PositionX,
                     int i_PositionY)
{
   GraphicObjectImage  *pc_ImageObject;
   RectRegion           c_RedrawRegion;

   if(pc_Image == NULL)
   {
      return;
   }

   // enable the mask to display only selected pixel
   pc_Image->SetSelectionMaskActive(true);

   // create the object
   pc_ImageObject = new GraphicObjectImage(i_PositionX,
                                               i_PositionY,
                                               pc_Image);
   if(pc_ImageObject == NULL)
   {
      return;
   }

   // add to layer
   m_pc_ObjectLayer->AddObject(pc_ImageObject);

   // calculate the object size
   c_RedrawRegion = pc_ImageObject->GetSize();

   // now redraw the changed region
   m_pc_GlobalStorage->m_pc_ImageFrame
                     ->RedrawBufferRegion(c_RedrawRegion);


} // AddImageToWorkArea()

//===========================================================================
//
//===========================================================================


void MotiveBox::GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // now redisplay the size of the graphic object

   switch(pc_CallbackData->i_CallbackType)
   {
   case GraphicObjectMainEvent::SelectionDisabled:
      // disable the frame mode
      if( m_b_ObjectModeActive )
      {
         if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
            == m_pc_ObjectLayer )
         {
            // reset the callback for graphic object changes
            m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
            // disnable active action
            m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
         }
         m_pc_ObjectLayer->SetVisible(false);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
         m_ActivateCheck->setChecked(false);
         
      }
      SetSensitivity();
      break;
   case GraphicObjectMainEvent::Moved:         // object is moved
      break;
    
   case GraphicObjectMainEvent::DragEnd:       // end of moving operation
      break;
   case GraphicObjectMainEvent::Selected:      // object switch to selected
      SetSensitivity();
      break;
   case GraphicObjectMainEvent::Deselected:    // object switched to deselected
      SetSensitivity();
      break;
   case GraphicObjectMainEvent::Resized:       // object size has changed
      SetSensitivity();
      break;
   default:
      break;      
   } // switch

} // GraphicObjectCallback()

//===========================================================================
//
//===========================================================================

void *MotiveBox::GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData)
{
   MotiveBox *pc_This;
   pc_This = (MotiveBox *) thisData;

   pc_This-> GraphicObjectCallback( (GraphicObjectMainEvent *)GraphicObjectCallbackData );

   return NULL;
} // GraphicObjectCallbackCB()

//===========================================================================
//
//===========================================================================

void MotiveBox::CutRectangleImage()
{
   PixelBuffer *pc_NewImage;
   PixelBuffer *pc_SelectionMask;
   RectRegion   c_ClipRegion;
   RectRegion   c_CopyRegion;
   int          i_X1;
   int          i_X2;
   int          i_Y1;
   int          i_Y2;

   // get the Rectangle
   m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,
                                                             i_X2,i_Y2);

   c_CopyRegion.SetXYXY( i_X1,i_Y1,i_X2,i_Y2);

   // get region of the image
   c_ClipRegion.SetXYWH(0,0,
                        m_pc_GlobalStorage->m_pc_MainImage->GetWidth(),
                        m_pc_GlobalStorage->m_pc_MainImage->GetHeight());

   // clip copyregion at the image
   c_CopyRegion.ClipAt(c_ClipRegion);

   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromRegion(c_CopyRegion);

   // fill the selection mask in the entire image
   pc_SelectionMask = pc_NewImage->GetSelectionMask();
   pc_SelectionMask->Clean(1);
   
   AddImageToWorkArea(pc_NewImage,
                      c_CopyRegion.GetStartX(),
                      c_CopyRegion.GetStartY());
} // CutRectangleImage()

//===========================================================================
//
//===========================================================================

void MotiveBox::CutFreeHandImage()
{
   PMemVector<PixelCoordinate> *pc_List;
   BufferPainter                  c_BufferPainter;
   PixelBuffer                   *pc_SelectionMask;
   PixelBuffer                   *pc_NewImage;
   RectRegion                     c_CopyRegion;
   int                            i_Size;

   // get the list of coordinates from the free hand selection
   pc_List = m_pc_GlobalStorage->m_pc_Selection->GetSelectionPointList();
   if(pc_List == NULL)
   {
      return;
   }

   // list should not be empty
   i_Size = pc_List->GetCount();
   if(i_Size ==0)
   {
      return;
   }


   // create a mask, same size as the original image,
   // only 1 bit depth
   pc_SelectionMask = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                        1,
                        false,false,false);
   if(pc_SelectionMask ==NULL)
   {
      return;
   }

   // prepare the buffer painter to draw in the mask
   c_BufferPainter.SetBorderColor(1);
   c_BufferPainter.SetInnerColor(1);
   c_BufferPainter.SetDrawInnerEnable(true);  

   // finaly draw it
   c_BufferPainter.DrawFreeFigure(pc_SelectionMask,
                       pc_List);

   
   // create image after the mask in the selection mask
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromSelected(
                                  &c_CopyRegion,
                                  pc_SelectionMask);

   // delete the selection mask   
   delete pc_SelectionMask;

   // add the new image to the work area
   if(pc_NewImage != NULL)
   {
      AddImageToWorkArea(pc_NewImage,
                      c_CopyRegion.GetStartX(),
                      c_CopyRegion.GetStartY());
   }                     
} // CutFreeHandImage()

//===========================================================================
//
//===========================================================================

void MotiveBox::SetSensitivity()
{
   GraphicObject *pc_Selected;
   int            i_NumberOfSelectedObjects;
   int            i_NumberOfObjects;

   i_NumberOfSelectedObjects = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Selected);
   i_NumberOfObjects = m_pc_ObjectLayer->GetObjectCount();

   if(i_NumberOfSelectedObjects == 1)
   {
      if(pc_Selected != NULL)
      {
         GraphicObjectAttribute * pc_Attribute;
         PixelBuffer            * pc_ImageBuffer;
         pc_Attribute = pc_Selected->GetAttribute(GraphicObjectAttribute::Image);
         if(pc_Attribute !=NULL)
         {

            pc_ImageBuffer = pc_Attribute->m_pc_ImageData;

            //m_SelectedInfoWidget->setEnabled(true);
            m_SelectedImageLabel->SetPixelBuffer(pc_ImageBuffer);
            SetObjectInfo(pc_Selected);
         }
      }
   }
   else
   {
      m_SelectedImageLabel->SetPixelBuffer(NULL);
      SetObjectInfo(NULL);
      //m_SelectedInfoWidget->setEnabled(false);
   }
   
   if(i_NumberOfSelectedObjects > 0)
   {
      m_DeleteButton->setEnabled(true);
      m_InsertButton->setEnabled(true);
   }
   else
   {
      m_DeleteButton->setEnabled(false);
      m_InsertButton->setEnabled(false);
      
   }

   if(i_NumberOfObjects>0)
   {
      m_InsertAllButton->setEnabled(true);
   }
   else
   {
      m_InsertAllButton->setEnabled(false);
   }
   

   if(m_b_TransparencyEnabled)
   {
      m_TransparencySlider->setEnabled(true);

   }
   else
   {
      // transparency is disabled
      m_TransparencySlider->setEnabled(false);

   }

   if(m_b_TransparencyBorderEnabled)
   {
      m_TransparencyBorderSlider->setEnabled(true);

   }
   else
   {
      // transparency is disabled
      m_TransparencyBorderSlider->setEnabled(false);

   }
   
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void MotiveBox::DeleteButtonPressed()
{
   GraphicObject *pc_Object;
   int            i_Number;

   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   while(i_Number >0)
   {
      m_pc_ObjectLayer->DeleteObject(pc_Object);

      // get new number;
      i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   }


   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();


} // DeleteButtonPressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::DeleteAllButtonPressed()
{
   // delete all objects from the layer
   m_pc_ObjectLayer->DeleteAllObjects();

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();

} // DeleteAllButtonPressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::InsertButtonPressed()
{
   PixelBufferObject  c_BufferObject;
   GraphicObject     *pc_Object;
   RectRegion         c_ChangedRegion;
   int                i_Number;


   c_ChangedRegion.SetEmpty();
   
   c_BufferObject.SetDestinationBuffer(m_pc_GlobalStorage->m_pc_MainImage);
   
   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   while(pc_Object != NULL)
   {
      RectRegion c_ObjectSize;
      RectRegion c_InsertSize;

      // insert the object

      c_ObjectSize = pc_Object->GetSize();

      // set object attributes into the bufferobject structure
      c_BufferObject.SetObject(pc_Object);
      c_BufferObject.SetRegion(c_ObjectSize);
      c_BufferObject.SetTransparency(m_b_TransparencyEnabled,
                                     m_i_TransparencyFactor);
      c_BufferObject.SetMergeBorder(m_b_TransparencyBorderEnabled,
                                     m_i_TransparencyBorderFactor);
      // insert object into te buffer
      c_InsertSize = c_BufferObject.InsertObject();

      // combine the change regions
      c_ChangedRegion.AddRegion(c_InsertSize);

      
      // get the next object;
      pc_Object = m_pc_ObjectLayer->GetNextSelectedObject(pc_Object);
   }

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   // send image changed for Undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

   SetSensitivity();

} // InsertButtonPressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::InsertAllButtonPressed()
{

   PixelBufferObject  c_BufferObject;
   GraphicObject     *pc_Object;
   RectRegion         c_ChangedRegion;
   int                i_Number;
   int                i;

   c_ChangedRegion.SetEmpty();

   c_BufferObject.SetDestinationBuffer(m_pc_GlobalStorage->m_pc_MainImage);

   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetObjectCount();

   for(i=0;i<i_Number;i++)
   {
      RectRegion c_ObjectSize;
      RectRegion c_InsertSize;

      // insert the object

      pc_Object = m_pc_ObjectLayer->GetObject(i);
      c_ObjectSize = pc_Object->GetSize();

      // set object attributes into the bufferobject structure
      c_BufferObject.SetObject(pc_Object);
      c_BufferObject.SetRegion(c_ObjectSize);
      c_BufferObject.SetTransparency(m_b_TransparencyEnabled,
                                     m_i_TransparencyFactor);
      c_BufferObject.SetMergeBorder(m_b_TransparencyBorderEnabled,
                                     m_i_TransparencyBorderFactor);

      // insert object into te buffer
      c_InsertSize = c_BufferObject.InsertObject();

      // combine the change regions
      c_ChangedRegion.AddRegion(c_InsertSize);


      // get the next object;
      pc_Object = m_pc_ObjectLayer->GetNextSelectedObject(pc_Object);
   } // for all objects

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   // send image changed for Undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   SetSensitivity();

} // InsertAllButtonPressed()

//===========================================================================
//
//===========================================================================

void MotiveBox::TransparencyBorderEnableChanged(bool b_on)
{
   m_b_TransparencyBorderEnabled = b_on;

   SetSensitivity();
   
} // TransparencyBorderEnableChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::TransparencyBorderFactorChanged(int i_Factor)
{
   m_i_TransparencyBorderFactor = i_Factor;
   
} // TransparencyBorderFactorChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::TransparencyEnableChanged(bool b_on)
{
   m_b_TransparencyEnabled = b_on;

   SetSensitivity();

} // TransparencyEnableChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::TransparencyFactorChanged(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;

} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::PutMotivesOverTheImage()
{
   GraphicObject  *pc_Object;
   RectRegion      c_ImageRegion;
   int             i;
   int             i_Number;

   c_ImageRegion.Set( m_pc_GlobalStorage->m_pc_MainImage);
   i_Number = m_pc_ObjectLayer->GetObjectCount();

   for(i=0;i<i_Number;i++)
   {
      RectRegion c_ObjectSize;
      RectRegion c_InsertSize;

      // insert the object

      pc_Object = m_pc_ObjectLayer->GetObject(i);
      c_ObjectSize = pc_Object->GetSize();

      if( ! c_ObjectSize.OverlapsRegion(c_ImageRegion) )
      {
         // object is not in the image.
         // move if to (0/0)
         pc_Object->MoveTo(0,0);
      } // if not in
      
   } // for all images
   

} // PutMotivesOverTheImage()

//===========================================================================
//
//===========================================================================

void MotiveBox::SetObjectInfo(GraphicObject *pc_Object)
{
   GraphicObjectAttribute * pc_Attribute;
   PixelBuffer            * pc_ImageBuffer;
   RectRegion              c_ObjectSize;
   QString                 qs_Out;
   float                   f_Factor;
   int                     i_OriginalWidth;
   int                     i_OriginalHeight;
   int                     i_ScaledWidth;
   int                     i_ScaledHeight;

   if(pc_Object == NULL)
   {
      m_b_InTextUpdate = true;
      m_OriginalWidthEdit ->setText(" ");
      m_OriginalHeightEdit->setText(" ");
      m_ScaleXEdit->setText(" ");
      m_ScaleYEdit->setText(" ");
      m_b_InTextUpdate = false;
      return;
   }
      
   pc_Attribute = pc_Object->GetAttribute(GraphicObjectAttribute::Image);
   if(pc_Attribute ==NULL)
   {
      return;
   }
   pc_ImageBuffer = pc_Attribute->m_pc_ImageData;
   c_ObjectSize = pc_Object->GetSize();

   i_OriginalWidth = pc_ImageBuffer->GetWidth(); 
   i_OriginalHeight = pc_ImageBuffer->GetHeight();

   i_ScaledWidth  = c_ObjectSize.GetWidth();   
   i_ScaledHeight = c_ObjectSize.GetHeight();

   m_b_InTextUpdate = true;

   qs_Out.sprintf("%d",i_OriginalWidth);
   m_OriginalWidthEdit->setText(qs_Out);

   qs_Out.sprintf("%d",i_OriginalHeight);
   m_OriginalHeightEdit->setText(qs_Out);

   if(m_b_FactorDisplayed)
   {
      f_Factor = (float)i_ScaledWidth / i_OriginalWidth;
      qs_Out.sprintf("%7f", f_Factor);
      m_ScaleXEdit->setText(qs_Out);


      f_Factor =  (float)i_ScaledHeight / i_OriginalHeight;
      qs_Out.sprintf("%7f", f_Factor);
      m_ScaleYEdit->setText(qs_Out);
   }
   else
   {
      qs_Out.sprintf("%d", i_ScaledWidth);
      m_ScaleXEdit->setText(qs_Out);

      qs_Out.sprintf("%d", i_ScaledHeight);
      m_ScaleYEdit->setText(qs_Out);
   }

   m_b_InTextUpdate = false;
    
} // SetObjectInfo()

//===========================================================================
//
//===========================================================================

void MotiveBox::SizeXTextChanged (const QString & qs_Value)
{
   GraphicObject *pc_Selected;
   PixelBuffer   *pc_ImageBuffer;
   RectRegion     c_ObjectSize;
   RectRegion     c_UpdateSize;
   RectRegion     c_ImageSize;
   float          f_Value;
   float          f_ScaledWidth;
   int            i_NumberOfSelectedObjects;
   int            i_OriginalWidth;

   if(m_b_InTextUpdate)
   {
      return;
   }

   i_NumberOfSelectedObjects = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Selected);

   if( i_NumberOfSelectedObjects != 1)
   {
      return;
   }

   if(pc_Selected == NULL)
   {
      return;
   }

   pc_ImageBuffer = ((GraphicObjectImage *)pc_Selected)->GetImageData();
   if( pc_ImageBuffer == NULL)
   {
      return;
   }

   // get the current size
   c_ObjectSize = pc_Selected->GetSize();

   // get a number out of the string
   f_Value = atof( (const char *) qs_Value);
   if(f_Value <= 0.0)
   {
      return ;
   }

   // get original size
   i_OriginalWidth = pc_ImageBuffer->GetWidth();

   if(m_b_FactorDisplayed)
   {
      // multiply by entered factor
      f_ScaledWidth  = i_OriginalWidth * f_Value;
   }
   else
   {
      f_ScaledWidth  =  f_Value;
   }

   c_UpdateSize = c_ObjectSize;
   c_ObjectSize.SetWidth(f_ScaledWidth);

   // increase the update size by new region
   c_UpdateSize.AddRegion(c_ObjectSize);

   pc_Selected->SetSize(c_ObjectSize);

   c_ImageSize.Set(m_pc_GlobalStorage->m_pc_MainImage);
   if(!c_ImageSize.OverlapsRegion(c_ObjectSize))
   {
      // object is out of the main image
      // move it to 0/0
      pc_Selected->MoveTo(0,0);
      
      // redraw all
      c_UpdateSize = c_ImageSize;
   }
   
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_UpdateSize);
     
   
} // SizeXTextChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::SizeYTextChanged (const QString &qs_Value)
{
   GraphicObject *pc_Selected;
   PixelBuffer   *pc_ImageBuffer;
   RectRegion     c_ObjectSize;
   RectRegion     c_UpdateSize;
   RectRegion     c_ImageSize;
   float          f_Value;
   float          f_ScaledHeight;
   int            i_NumberOfSelectedObjects;
   int            i_OriginalHeight;

   if(m_b_InTextUpdate)
   {
      return;
   }

   i_NumberOfSelectedObjects = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Selected);

   if( i_NumberOfSelectedObjects != 1)
   {
      return;
   }

   if(pc_Selected == NULL)
   {
      return;
   }

   pc_ImageBuffer = ((GraphicObjectImage *)pc_Selected)->GetImageData();
   if( pc_ImageBuffer == NULL)
   {
      return;
   }

   // get the current size
   c_ObjectSize = pc_Selected->GetSize();

   // get a number out of the string
   f_Value = atof( (const char *) qs_Value);
   if(f_Value <= 0.0)
   {
      return ;
   }

   // get original size
   i_OriginalHeight = pc_ImageBuffer->GetHeight();

   if(m_b_FactorDisplayed)
   {
      // multiply by entered factor
      f_ScaledHeight  = i_OriginalHeight * f_Value;
   }
   else
   {
      // use the value
      f_ScaledHeight  =  f_Value;
   }

   c_UpdateSize = c_ObjectSize;
   c_ObjectSize.SetHeight(f_ScaledHeight);

   // increase the update size by new region
   c_UpdateSize.AddRegion(c_ObjectSize);

   pc_Selected->SetSize(c_ObjectSize);

   c_ImageSize.Set(m_pc_GlobalStorage->m_pc_MainImage);
   if(!c_ImageSize.OverlapsRegion(c_ObjectSize))
   {
      // object is out of the main image
      // move it to 0/0
      pc_Selected->MoveTo(0,0);

      // redraw all
      c_UpdateSize = c_ImageSize;
   }
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_UpdateSize);

}  // SizeYTextChanged()

//===========================================================================
//
//===========================================================================

void MotiveBox::SizeLabelClicked(void)
{
   GraphicObject *pc_Selected;
   int            i_NumberOfSelectedObjects;

   // reverse the current mode
   m_b_FactorDisplayed = ! m_b_FactorDisplayed;

   // get selected objects
   i_NumberOfSelectedObjects = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Selected);

   // set the label text depending on the current mode
   if(m_b_FactorDisplayed)
   {
      m_ScaleXLabel->setText(tr("Scale"));
      m_ScaleYLabel->setText(tr("Scale"));
   }
   else
   {
      m_ScaleXLabel->setText(tr("New"));
      m_ScaleYLabel->setText(tr("New"));
   }

   // is one object active
   if( i_NumberOfSelectedObjects == 1)
   {
      // update the object info
      SetObjectInfo(pc_Selected);
   }
   
} // SizeLabelClicked()

//===========================================================================
//
//===========================================================================

void MotiveBox::MotiveDropped(PixelBuffer *pc_ImageBuffer)
{
   PixelBuffer * pc_MyOwnImage;
   if( pc_ImageBuffer == NULL)
   {
      return;
   }

   // create an own copy

   pc_MyOwnImage = new PixelBuffer(pc_ImageBuffer);
   AddImageToWorkArea(pc_MyOwnImage,
                      0,
                      0
                      );
   
} // MotiveDropped()

//===========================================================================
//
//===========================================================================

void MotiveBox::OrderChange(bool b_UpDirection)
{
   GraphicObjectList * pc_Selected;
   GraphicObject     * pc_Object;
   RectRegion          c_UpdateRegion;
   int                 i_ObjectCount;
   int                 i;
   int                 i_SelectedCount;

   // get all selected objects and add it to the list
   i_ObjectCount = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   if(i_ObjectCount == 0)
   {
      return;
   }

   // create a help list
   pc_Selected = new GraphicObjectList(false);

   if(pc_Object != NULL)
   {
      pc_Selected->AddObject(pc_Object);
   }
   for(i=1;i<i_ObjectCount;i++)
   {
      pc_Object = m_pc_ObjectLayer->GetNextSelectedObject(pc_Object);
      if(pc_Object != NULL)
      {
         pc_Selected->AddObject(pc_Object);
      }
   }

   // all selected objects are now in the list
   // now move all to the top

   i_SelectedCount = pc_Selected->GetCount();
   for(i=0;i<i_SelectedCount;i++)
   {
      RectRegion c_ObjectRegion;
      pc_Object = pc_Selected->GetObject(i);

      // add size to total size
      c_ObjectRegion = pc_Object->GetSize();
      c_UpdateRegion.AddRegion(c_ObjectRegion);

      if(b_UpDirection )
      {
         m_pc_ObjectLayer->SetObjectOrderBottom(pc_Object);
      }
      else
      {
         m_pc_ObjectLayer->SetObjectOrderTop(pc_Object);
      }

   }

   // delete the help list
   delete pc_Selected;

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_UpdateRegion);

} // OrderChange

//===========================================================================
//
//===========================================================================

void MotiveBox::OrderUpClicked()
{
   OrderChange(true);
   
} // OrderUpClicked()

//===========================================================================
//
//===========================================================================

void MotiveBox::OrderDownClicked()
{
   OrderChange(false);

} // OrderDownClicked()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer * MotiveBox::GetMotiveLayer()
{
   return m_pc_ObjectLayer;
} // GetMotiveLayer()

//===========================================================================
//
//===========================================================================

int MotiveBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"MotiveBox");

   m_pc_ObjectLayer->WriteFile(pc_FileWriter,"ObjectLayer");

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int MotiveBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(    qs_Name.compare("ObjectLayer") == 0
       && qs_Value.compare("GraphicObjectLayer") == 0
       && b_IsStartStructure)
   {
      // delete old objects
      m_pc_ObjectLayer->DeleteAllObjects();

      // load objects from a file
      m_pc_ObjectLayer->LoadFile(pc_FileWriter);

      // disable selection of an object
      SetObjectInfo(NULL);
   }

   PERICO_READ_STRUCTURE_END

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

