//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PList.c
//---------------------------------------------------------------------------
// Purpose: generic list and array
//---------------------------------------------------------------------------

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "PList.h"

//============================================================================
//
//============================================================================


PList::PList()
{
   m_ui_ListSize = 10;
   m_pv_List = new void * [m_ui_ListSize];
   m_ui_NumberOfElements =0;

   m_b_AutoDelete = false;

} // PList constructor()

//============================================================================
//
//============================================================================

PList::~PList()
{
   if(m_pv_List != NULL)
   {
      delete[]  m_pv_List;
      m_pv_List = NULL;
   }

} // ~PList()

//============================================================================
//
//============================================================================

void PList::AddTail(void *pc_Element)
{
   // enough room?
   if(m_ui_NumberOfElements+1 >= m_ui_ListSize)
   {
      // no, then make list larger
      Resize(m_ui_ListSize*2);
   }

   // add new element at the end   
   m_pv_List[m_ui_NumberOfElements] = pc_Element;
   m_ui_NumberOfElements ++;
} // AddTail

//============================================================================
//
//============================================================================

void PList::AddHead(void *pc_Element)
{
   int i;
   // enough room?
   if(m_ui_NumberOfElements+1 >= m_ui_ListSize)
   {
      // no, then make list larger
      Resize(m_ui_ListSize*2);
   }

   // increase the size
   m_ui_NumberOfElements ++;

   // move all to the back
   for(i=m_ui_NumberOfElements-1;i>0;i--)
   {
      m_pv_List[i] = m_pv_List[i-1];
   }      
   // add new element at the beginning
   m_pv_List[0] = pc_Element;
} // AddHead()

//============================================================================
//
//============================================================================

void PList::Resize(unsigned int ui_NewNumber)
{
   void **pc_NewList;

   // need at least 1 element
   if(ui_NewNumber == 0)
   {
      return;
   }

   pc_NewList = new void *[ui_NewNumber];
   if(pc_NewList != NULL)
   {
      // copy the elements
      memcpy(pc_NewList,
             m_pv_List,
             m_ui_NumberOfElements * sizeof(void *)
             );

      delete [] m_pv_List;
      m_pv_List = pc_NewList;
      m_ui_ListSize = ui_NewNumber;

      // reduce the number of already stored elements,
      // if necessary
      if(m_ui_NumberOfElements >= m_ui_ListSize)
      {
         m_ui_NumberOfElements = m_ui_ListSize;
      }
   }
   
} // Resize()

//============================================================================
//
//============================================================================

unsigned int PList::GetCount()
{
   return m_ui_NumberOfElements;
} // GetCount()

//============================================================================
//
//============================================================================

bool PList::IsEmpty()
{
   return m_ui_NumberOfElements ==0;
} // IsEmpty()

//============================================================================
//
//============================================================================

void *PList::GetIndex(unsigned int ui_Index)
{
   // element in the current range
   if(ui_Index < m_ui_NumberOfElements)
   {
      return m_pv_List[ui_Index];
   }
   return NULL;
} // GetIndex()

//============================================================================
//
//============================================================================

void PList::SetAutodelete(bool b_Delete)
{
   m_b_AutoDelete = b_Delete;
} // SetAutodelete()

//============================================================================
//
//============================================================================

bool PList::GetAutodelete()
{
   return m_b_AutoDelete;
} // GetAutodelete()

//============================================================================
//
//============================================================================

void PList::DeleteElement(void *p_Item)
{
} // DeleteElement

//============================================================================
//
//============================================================================

void PList::Remove()
{
   if(m_b_AutoDelete)
   {
      // delete elements first;
      unsigned int i;
      for(i=0;i < m_ui_NumberOfElements; i++)
      {
         if(m_pv_List[i] != NULL)
         {
            DeleteElement( m_pv_List[i]);
            m_pv_List[i] = NULL;
         }
      }
   }
   m_ui_NumberOfElements = 0;
   
} // Remove

//============================================================================
//
//============================================================================

void PList::Remove(void *pc_Item)
{
   unsigned int i_Index;
   unsigned int i;
   
   if(   pc_Item ==NULL
      || m_ui_NumberOfElements==0)
   {
      return;
   }

   i_Index =0xffffffff;
   for(i=0;i< m_ui_NumberOfElements;i++)
   {
      if(m_pv_List[i] == pc_Item)
      {
         i_Index= i;
         break;
      }
   }
   // found ??
   if(i_Index == 0xffffffff)
   {
      // no, then exit
      return;
   }

   // delete the item (if autodelete)
   if(m_b_AutoDelete)
   {
      DeleteElement( m_pv_List[i_Index]);
   }

   // move the rest up to the front
   for(i=i_Index; i<m_ui_NumberOfElements-1;i++)
   {
      m_pv_List[i] = m_pv_List[i+1];
   }
   
   m_ui_NumberOfElements -= 1;

} // Remove

//============================================================================
//
//============================================================================

void PList::Remove(unsigned int i_Index)
{
   unsigned int i;

   if( i_Index >= m_ui_NumberOfElements)
   {
      return;
   }

   // delete the item (if autodelete)
   if(m_b_AutoDelete)
   {
      DeleteElement( m_pv_List[i_Index]);
   }

   // move the rest up to the front
   for(i=i_Index; i<m_ui_NumberOfElements-1;i++)
   {
      m_pv_List[i] = m_pv_List[i+1];
   }

   m_ui_NumberOfElements -= 1;

} // Remove

//============================================================================
//############################################################################
//############################################################################
//============================================================================

PArray::PArray(unsigned int i_MemberSize,
          unsigned int i_InitialSize)
{
   m_ui_NumberOfElements = i_InitialSize;
   m_ui_ElementSize      = i_MemberSize;

   m_Memory = (void *) new unsigned char[m_ui_NumberOfElements* m_ui_ElementSize];
   
} // PArray constructor

//============================================================================
//
//============================================================================

PArray::~PArray()
{
   delete (unsigned char*) m_Memory;
}

//============================================================================
//
//============================================================================

void PArray::Resize(unsigned int i_NewNumber)
{
   unsigned char * puc_NewMemory;
   unsigned int    ui_TogetherSize;
   puc_NewMemory = new unsigned char[i_NewNumber* m_ui_ElementSize];

   ui_TogetherSize = i_NewNumber;
   if(ui_TogetherSize > m_ui_NumberOfElements)
   {
      ui_TogetherSize = m_ui_NumberOfElements;
   }

   // now copy the common area into the new array

   memcpy(puc_NewMemory,
          m_Memory,
          m_ui_ElementSize*ui_TogetherSize);

   // exchange pointer, and delete the old
   delete (unsigned char *) m_Memory;
   m_Memory = (void *) puc_NewMemory;

   m_ui_NumberOfElements = i_NewNumber;
            
} // Resize

//============================================================================
//
//============================================================================

void *PArray::ElementAddress(unsigned int i_Index)
{
   if( i_Index>= m_ui_NumberOfElements)
   {
      return NULL;
   }

   return (void *) (&( (unsigned char *)m_Memory)[i_Index*m_ui_ElementSize]) ;
   
} // ElementAddress()

//============================================================================
//
//============================================================================

PVector::PVector(unsigned int i_MemberSize,
          unsigned int i_InitialSize) :
          PArray(i_MemberSize,i_InitialSize)
{
   m_ui_UsedElements = 0;
} // PVector() 

//============================================================================
//
//============================================================================

PVector::~PVector()
{
} // ~PVector()

//============================================================================
//
//============================================================================


unsigned int PVector::GetCount()
{
   return m_ui_UsedElements;
}

//============================================================================
//
//============================================================================

void PVector::AddTail(void *pc_Element)
{
   void *pc_Destination;

   // increase list, if necessaray
   if(m_ui_UsedElements +1 > m_ui_NumberOfElements)
   {
      Resize(m_ui_NumberOfElements*2);
   }

   // get destination address
   pc_Destination = ElementAddress( m_ui_UsedElements);
   // copy element
   memcpy(pc_Destination,pc_Element,m_ui_ElementSize);

   // increase size
   m_ui_UsedElements ++;

} // AddTail()

//============================================================================
//
//============================================================================

void PVector::Clear()
{
   // clean up list
   m_ui_UsedElements = 0;
} // Clear()

//============================================================================
//
//============================================================================

void PVector::RemoveTail()
{
   // remove last element
   if(m_ui_UsedElements>0)
   {
      m_ui_UsedElements --;

   }
} // RemoveTail()

//============================================================================
//
//============================================================================



