#ifndef _PLIST_H
#define _PLIST_H

class PList
{
public:
   PList();
   virtual ~PList();

   void AddTail(void *pc_Element);
   void AddHead(void *pc_Element);
   unsigned int  GetCount();
   bool IsEmpty();
   void *GetIndex(unsigned int ui_Index);
   void SetAutodelete(bool b_Delete);
   bool GetAutodelete();
   void Remove();
   void Remove(void *pc_Item);
   void Remove(unsigned int i_Index);
   
protected:
   void **m_pv_List;
   unsigned int  m_ui_ListSize;
   unsigned int  m_ui_NumberOfElements;
   void Resize(unsigned int i_NewNumber);
   bool m_b_AutoDelete;

   virtual void DeleteElement(void *p_Item); 
}; // PList

//*******************************************************


template<class Type>
class PPtrList : public PList
{
public:
   PPtrList() {}
   virtual ~PPtrList()
   {
      if(m_b_AutoDelete)
      {
         // delete elements first;
         PList::Remove();
      }

   }

   void AddTail(Type *pc_Element)
      { PList::AddTail((void *) pc_Element); }
   void AddHead(Type *pc_Element)
      { PList::AddHead((void *) pc_Element); }

      
   Type *GetIndex(unsigned int ui_Index)
      { return (Type *)(PList::GetIndex(ui_Index)) ;}

   virtual void DeleteElement(void *p_Item)
   {
      delete (Type*) p_Item;   
   }
   void Remove(Type *pc_Item)
   {
      PList::Remove((void *) pc_Item);
   }
   void Remove()
   {
      PList::Remove();
   }
   void Remove(unsigned int i_Index)
   {
      PList::Remove(i_Index);
   }

};

//*******************************************************

class PArray
{
public:
   PArray(unsigned int i_MemberSize,
          unsigned int i_InitialSize =10);
   virtual ~PArray();

   void Resize(unsigned int i_NewNumber);
   void *ElementAddress(unsigned int i_Index);


protected:
   void *m_Memory;
   unsigned int  m_ui_NumberOfElements;
   unsigned int  m_ui_ElementSize;

} ; // PArray

//*******************************************************

template<class Type>
   class PMemArray : public PArray
{
public:
   PMemArray() : PArray(sizeof(Type)) {}
   
   virtual ~PMemArray() {}
   
   void Resize(unsigned int i_NewNumber)
   {
      PArray::Resize(i_NewNumber);
   }
   void *ElementAddress(unsigned int i)
   {
      return PArray::ElementAddress(i);
   }

   Type  Get(int i_Index)
   {
      return (Type )(*(Type*)(ElementAddress(i_Index)));
   }
   void Set(int i_Index, Type Element)
   {
      *( (Type*)(ElementAddress(i_Index))) = Element;
   }

}; // PMemArray

//*******************************************************

class PVector : public PArray
{
public:
   PVector(unsigned int i_MemberSize,
          unsigned int i_InitialSize =10);
   virtual ~PVector();

   unsigned int  m_ui_UsedElements;
   unsigned int GetCount();

   void AddTail(void *pc_Element);
   void Clear();
   void RemoveTail();
   
} ; // PVector

//*******************************************************

template<class Type>
   class PMemVector : public PVector
{
public:
   PMemVector() : PVector(sizeof( Type),10)
            {};
   virtual ~PMemVector()
            {};
   void AddTail(Type c_Element)
   {
      PVector::AddTail(&c_Element);
   };

   Type  Get(int i_Index)
   {
      return (Type )(*(Type*)(ElementAddress(i_Index)));
   }
};


#endif // _PLIST_H
