#ifndef _PAINTBOX_H
#define _PAINTBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

#include "StandardDialog.h"
#include "BufferPainter.h"
#include "QULabelSlider.h"
#include "QUColorDragWidget.h"
#include "Tracer.h"
#include "QUColorOrPattern.h"
#include "PictureUpdateConnection.h"
#include "QUIconButton.h"
#include "QUSelectIconButton.h"


class MainFrame;

class PaintBox :  public StandardDialog ,
                  public Tracer
{
Q_OBJECT

public:
   PaintBox(QWidget *parent=0,
            GlobalStorage *pc_Global=0,
            MainFrame *pc_MainFrame=NULL 
            );
   ~PaintBox();

   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

public slots:

   void ActivateBrush();
   void ActivateFill();
   void ActivateLightness();
   void ActivateSaturation();
   void ActivateFilter();
   void ActivateSpeckle();
   void ActivateAirBrush();
   void ActivateSmear();
   void ActivateCopy();
   void ActivateRecolor();

   void ThicknessChanged(int i_Width);
   void ContinuousEnableChanged(bool b_on);


   void BrushColorChanged(Pixel c_Color);
   void BrushPatternChanged(PixelBuffer *pc_Buffer);
   void BrushModeChanged(BrushMode i_BrushMode);

   void CreateBrushShape(int i_Thickness);

   void TransparencyEnableChanged(bool b_on);
   void TransparencyFactorChanged(int i_Factor);
   void TransparencyMaskUseCheckChanged(bool b_On);
   
   void TransparencyAutomaticChanged(int i_NewValue);
   void TransparencyMaskClear();

   void FillColorChanged(Pixel c_Color);
   void FillDistanceChanged(int i_Factor);
   void FillDistanceFromStartPoint(bool b_On);
   void FillDistanceFromColor(bool b_On);

   void LightnessFactorChanged(int i_Factor);

   void FilterWidthChanged(int i_Factor);
   void FilterDistanceEnableChanged(bool b_on);
   void FilterDistanceChanged(int i_Distance);
   void FilterAlgorithmChanged(int i_NewMode);

   void SpeckleSizeChanged(int i_Size);
   void SpeckleDensityChanged(int i_Density);

   void SmearStrengthChanged(int i_Density);

   void CopyBrushNewOrigin();

   void RecolorFromColorChanged(Pixel c_Color);
   void RecolorToColorChanged(Pixel c_Color);
   void RecolorDistanceChanged(int i_Factor);

   void BrushShapeCircleSelected();
   void BrushShapeSquareSelected();
   void BrushShapeVerticalSelected();
   void BrushShapeHorizontalSelected();
   void BrushShapeSWNESelected();
   void BrushShapeNWSESelected();
   void BrushShapeFreeSelected();

   void BrushShapeFreeImageDropped(PixelBuffer *pc_Buffer);
   void BrushShapeFreeMenuSelected(int i_Index);
   
   
private:


   enum
   {
      PaintNone=0,
      PaintBrush,
      PaintFill,
      PaintLightness,
      PaintSaturation,
      PaintFilter,
      PaintSpeckle,
      PaintAirBrush,
      PaintSmear,
      PaintCopy,
      PaintRecolor
   };


   enum
   {
      BrushShapeCircle,
      BrushShapeSquare,
      BrushShapeVerticalLine,
      BrushShapeHorizontalLine,
      BrushShapeNWSELine,
      BrushShapeSWNELine,
      BrushShapeFreeFigure

   };
    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void DrawBrush();
   void DrawLightness();
   void DrawSaturation();
   void DrawFilter();
   void FillAtPoint();
   void DrawSpeckle();
   void DrawAirBrush();
   void DrawSmear();
   void DrawCopy();
   void DrawRecolor();

   void DrawSpecial(int i_Mode);
   void SetSensitivity();
   void SetWidgetToMode(int i_Mode);
   void PaintCopyEvent(int i_EventType);
   void DisableCopyBrushPoint();

   void SetBrushShapeSensitivity();
   void CenterCoordinateToBrush(int &i_X,int &i_Y);  
   void InitBrushShapeFree();
   void SetFreeBrushShapeIcon(PixelBuffer *pc_Buffer);

   void *FreeBrushPictureConnectionEventArrived(void *pc_Data);
   static void *FreeBrushPictureConnectionEventArrivedCB(void *Thisdata,void *pc_Data);

   // variables
   // widgets
   QULabelSlider     *m_BrushThicknessSlider;
   QCheckBox         *m_BrushContinuousEnable;
   
   QCheckBox         *m_TransparencyEnable;
   QULabelSlider     *m_TransparencySlider;
   QCheckBox         *m_TransparencyMaskUseCheck;

   QUColorOrPattern  *m_BrushColorWidget;
   
   QUIconButton     *m_BrushButton;
   QUIconButton     *m_FillButton;
   QUIconButton     *m_LightnessButton;
   QUIconButton     *m_SaturationButton;
   QUIconButton     *m_FilterButton;
   QUIconButton     *m_SpeckleButton;
   QUIconButton     *m_AirBrushButton;
   QUIconButton     *m_SmearButton;
   QUIconButton     *m_CopyButton;
   QUIconButton     *m_RecolorButton;
   
   QPushButton       *m_ClearTransparencyMaskButton;
   QComboBox         *m_TransparencyClearCombo;

   QULabelSlider     *m_FillDistanceSlider;
   QUColorDragWidget *m_FillDistanceColor;
   QButtonGroup      *m_FillModeWidget;
   QRadioButton      *m_FillModeFromOrigin;
   QRadioButton      *m_FillModeFromColor;

   QULabelSlider     *m_FilterWidthSlider;
   QCheckBox         *m_FilterDistanceActiveCheck;
   QULabelSlider     *m_FilterDistanceSlider;
   QRadioButton      *m_FilterMiddleButton;
   QRadioButton      *m_FilterMedianButton;
   QRadioButton      *m_FilterSharpButton;

   QVGroupBox        *m_BrushSizeModeBox;
   QVGroupBox        *m_BrushColorModeBox;
   QVGroupBox        *m_TransparencyBox;
   QHGroupBox        *m_MaskClearModeBox;
   QVGroupBox        *m_FillModeBox;
   QVGroupBox        *m_LightnessModeBox;
   QVGroupBox        *m_FilterModeBox;
   QVGroupBox        *m_SpeckleModeBox;
   QVGroupBox        *m_SmearModeBox;
   QVGroupBox        *m_CopyBrushModeBox;
   QVGroupBox        *m_RecolorModeBox;
   QHGroupBox        *m_BrushShapeModeBox;

   QULabelSlider     *m_LightnessSlider;

   QULabelSlider     *m_SpeckleSizeSlider;
   QULabelSlider     *m_SpeckleDensitySlider;

   QULabelSlider     *m_SmearStrengthSlider;

   QPushButton       *m_CopyBrushResetButton;

   QUColorDragWidget *m_RecolorFromColor;
   QUColorDragWidget *m_RecolorToColor;
   QULabelSlider     *m_RecolorDistanceSlider;
   
   QUIconButton     *m_BrushShapeCircleButton;
   QUIconButton     *m_BrushShapeSquareButton;
   QUIconButton     *m_BrushShapeVerticalButton;
   QUIconButton     *m_BrushShapeHorizontalButton;
   QUIconButton     *m_BrushShapeNWSEButton;
   QUIconButton     *m_BrushShapeSWNEButton;
   QUSelectIconButton     *m_BrushShapeFreeButton;

   BufferPainter *m_pc_BufferPainter;
   bool           m_b_BrushContinuous;
   bool           m_b_TransparencyEnabled;
   bool           m_b_LastPoint;
   bool           m_b_UseTransparencyMask;
   int            m_i_TransparencyFactor;
   int            m_i_LastX;
   int            m_i_LastY;
   int            m_i_MouseEventType;
   int            m_i_CurrentThickness;
   int            m_i_TransparencyClearMode;
   int            m_i_CurrentGraphikMode;
   int            m_i_LightnessFactor;
   int            m_i_SpeckleSize;
   int            m_i_SpeckleDensity;
   int            m_i_SpeckleDensityMaximum;
   int            m_i_SmearStrength;

   int            m_i_CopyDifferenceX;
   int            m_i_CopyDifferenceY;
   bool           m_b_SelectOriginalPoint;
   bool           m_b_FirstPoint;
   
   int            m_i_BrushShapeStyle;
   PixelBuffer *  m_pc_BrushShapeFreeBuffer;
   PictureUpdateConnection m_BrushShapeUpdateConnection;

   
   // mode for transparecy single draw mask clearing
   enum
   {
      ClearModeManual=0,
      ClearModeAutomatic=1,
   };

   Pixel m_p_FillDistanceColor;
   int   m_i_FillDistance;
   bool  m_b_FillFromStartPoint;
   bool  m_b_FillFromColor;

   // variables for the filter settings
    int          m_i_FilterWidth;
    bool         m_b_FilterNearDistanceActive;
    int          m_i_FilterNearDistance;
    int          m_i_FilterAlgorithm;

   MainFrame     *m_pc_MainFrame;
   GraphicObjectLayer     *m_pc_ObjectLayer;
   GraphicObjectPoint     *m_pc_CopyBrushOriginPoint;

   Pixel m_p_RecolorFromColor;
   Pixel m_p_RecolorToColor;
   int   m_i_RecolorDistance;
   

};


#endif // _PAINTBOX_H
