//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PainterBrush.c
//---------------------------------------------------------------------------
// Purpose: Brushes to paint into a buffer
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "PainterBrush.h"

#include "Utilities.h"
#include "PixelBuffer.h"
#include "ColorCalculation.h"

//===========================================================================
//
//===========================================================================

PainterBrush::PainterBrush()
{
   m_Color             = 0;
   m_i_BrushMode       = SelectColor;

   m_Pattern           = NULL;

   m_pc_MotiveLayer    = NULL;

   m_pc_BrushShape     = NULL;

   m_b_Lightness       = false;
   m_b_Saturation      = false;
   m_b_Filter          = false;
   m_b_Recolor         = false;
   m_i_LightnessFactor = 0;

} // PainterBrush

//===========================================================================
//
//===========================================================================

PainterBrush::~PainterBrush()
{
   if(m_Pattern != NULL)
   {
      delete m_Pattern;
      m_Pattern = NULL;
   }

   if(m_pc_BrushShape != NULL)
   {
      delete m_pc_BrushShape;
      m_pc_BrushShape = NULL;
   }

   
} // ~PainterBrush()

//===========================================================================
//
//===========================================================================

void PainterBrush::SetMotiveLayer(GraphicObjectLayer *pc_MotiveLayer)
{
   m_pc_MotiveLayer = pc_MotiveLayer;
} // SetMotiveLayer()

//===========================================================================
//
//===========================================================================

void PainterBrush::SetPattern(PixelBuffer *pc_Pattern)
{
   if(m_Pattern != NULL)
   {
      delete m_Pattern;
      m_Pattern = NULL;
   }

   if(pc_Pattern == NULL)
   {
      return;
   }
   
   // create a copy !!
   m_Pattern = new PixelBuffer(pc_Pattern);

   // calculate for speed
   m_i_PatternWidth = m_Pattern->GetWidth();
   m_i_PatternHeight= m_Pattern->GetHeight();

} // SetPattern()

//===========================================================================
//
//===========================================================================

Pixel PainterBrush::GetBrushPixel(int i_X, int i_Y,bool &b_Valid)
{

   if(    m_i_BrushMode == SelectPattern
       && m_Pattern != NULL)
   {
      PixelBuffer *pc_SelectionMask;
      // reduce coordinate into the pattern
      if(i_X >= m_i_PatternWidth)
      {
         i_X %= m_i_PatternWidth;
      }
      if(i_Y >= m_i_PatternHeight)
      {
         i_Y %= m_i_PatternHeight;
      }

      // calculate later b_Valid, if a pattern mask is set
      pc_SelectionMask = m_Pattern->GetSelectionMask();
      if(    pc_SelectionMask != NULL )
      {
         if(pc_SelectionMask->Get(i_X,i_Y) == 0)
         {
            b_Valid = false;
            return 0;
         }
      }

      b_Valid = true;

      // get pixel out of the pattern
      return m_Pattern->Get(i_X,i_Y);
   }
   else if(    m_i_BrushMode == SelectMotive)
   {
      if( m_pc_MotiveLayer == NULL)
      {
         b_Valid = false;
         return 0;
      }
      return m_pc_MotiveLayer->GetPixelAt(i_X,i_Y,b_Valid);
      
   }
   else // use only the color
   {
      b_Valid = true;
      return m_Color;
   }

} // GetBrushPixel()

//===========================================================================
//
//===========================================================================




