//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PericoControlFile.c
//---------------------------------------------------------------------------
// Purpose: Load and save of the controlfile. It is needed for the
// *.perico format.
//---------------------------------------------------------------------------

#include <stdio.h>

#include "PericoControlFile.h"

// a mximum indent of 40, not to get the controlfile to wide

#define LEVEL_MAX_INDENT 40

//============================================================================
//
//============================================================================

PericoControlFile::PericoControlFile()
{

   m_i_CurrentLevel=0;
} // PericoControlFile - constructor

//============================================================================
//
//============================================================================

PericoControlFile::~PericoControlFile()
{

} // ~PericoControlFile - destructor

//============================================================================
//
//============================================================================


int PericoControlFile::LoadFile(QString qs_Filename)
{
   unsigned char   ReadBuffer[500];
   FILE          * fp_Infile;

   m_i_CurrentReadLine =0;
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile ==NULL)
   {
      return -1;
   }

   while(true)
   {
      unsigned char * uc_FgetsResult;
      QString         qs_Line;

      uc_FgetsResult = (unsigned char *) fgets((char *)ReadBuffer,
                                               499,
                                               fp_Infile);
      if(uc_FgetsResult == NULL)
      {
         break;
      }
      while(*uc_FgetsResult != '\0')
      {
         if(*uc_FgetsResult <' ')
         {
            *uc_FgetsResult = '\0';
            break;
         }
         uc_FgetsResult++;
      }

      // make a qstring out of it
      qs_Line.sprintf("%s",ReadBuffer);
      if(qs_Line.isEmpty())
      {
         break;
      }

      // now add to the string list
      m_List.push_back(qs_Line);
       
   }  // while true

   // close the input file
   fclose(fp_Infile);

   // everything is ok
   return 0;

} // LoadFile()

//============================================================================
//
//============================================================================

int PericoControlFile::SaveFile(QString qs_Filename)
{
   QString   qs_Line;
   FILE    * fp_Outfile;
   int       i;
   int       i_NumberOfLines;

   fp_Outfile = fopen(qs_Filename,"w");
   if(fp_Outfile ==NULL)
   {
      return -1;
   }

   // write all lines
   i_NumberOfLines = m_List.count();
   for(i=0;i<i_NumberOfLines;i++)
   {
      qs_Line = m_List[i];
      fprintf(fp_Outfile,"%s\n",(const char *) qs_Line);
   }   

   fclose(fp_Outfile);

   // everything is ok
   return 0;

} // SaveFile()

//============================================================================
//
//============================================================================

int PericoControlFile::AddValue(QString qs_Name,QString qs_Value)
{
   QString qs_Line;
   QString qs_Level;

   // first make the level blanks
   if(m_i_CurrentLevel> LEVEL_MAX_INDENT)
   {
      qs_Level.fill(' ',LEVEL_MAX_INDENT);
   }
   else
   {
      qs_Level.fill(' ',m_i_CurrentLevel);
   }
   
   qs_Line.sprintf("%s%s=%s",
            (const char *)qs_Level,
            (const char *)qs_Name,
            (const char *)qs_Value);
   // now add to the string list
   m_List.push_back(qs_Line);

   return 0;
} // AddValue()

//============================================================================
//
//============================================================================

int PericoControlFile::AddValue(QString qs_Value)
{
   QString qs_Line;
   QString qs_Level;

   // first make the level blanks
   if(m_i_CurrentLevel> LEVEL_MAX_INDENT)
   {
      qs_Level.fill(' ',LEVEL_MAX_INDENT);
   }
   else
   {
      qs_Level.fill(' ',m_i_CurrentLevel);
   }

   qs_Line.sprintf("%s%s",
            (const char *)qs_Level,
            (const char *)qs_Value);
   // now add to the string list
   m_List.push_back(qs_Line);

   return 0;
} // AddValue()

//============================================================================
//
//============================================================================

void PericoControlFile::LevelIncrement()
{
   m_i_CurrentLevel += 1;
} // LevelIncrement()

//============================================================================
//
//============================================================================

void PericoControlFile::LevelDecrement()
{
   m_i_CurrentLevel -= 1;
   if(m_i_CurrentLevel <0)
   {
      m_i_CurrentLevel = 0;
   }

} // LevelDecrement()

//============================================================================
//
//============================================================================

int PericoControlFile::GetValue(QString &qs_Name,
                QString &qs_Value,
                bool &b_IsStartStructure,
                bool &b_IsEndStructure)
{
   QString qs_Line;
   int     i_NumberOfLines;
   int     i_FindIndex;
   int     i_ValueLength;
   
   i_NumberOfLines = m_List.count();

   if(m_i_CurrentReadLine>=i_NumberOfLines)
   {
      return -1;
   }
   
   qs_Line = m_List[m_i_CurrentReadLine];
   m_i_CurrentReadLine += 1;

   // now check, what kind of line we have
   
   qs_Line = qs_Line.stripWhiteSpace();

   if(qs_Line.compare("}") ==0)
   {
      // got an end of block
      qs_Name           ="";
      qs_Value          ="";
      b_IsStartStructure=false;
      b_IsEndStructure  =true;
      
      return 0;
   }

   // look for an = sign
   i_FindIndex = qs_Line.find('=');
   if(i_FindIndex <1)
   {
      return -1;
   }

   qs_Name  = qs_Line.left(i_FindIndex);
   qs_Value = qs_Line.mid (i_FindIndex+1);
   b_IsStartStructure =false;
   b_IsEndStructure   =false;

   
   i_ValueLength = qs_Value.length();
   if(i_ValueLength>0)
   {
      if(qs_Value[i_ValueLength-1] == '{')
      {
         b_IsStartStructure   = true;
         qs_Value = qs_Value.left(i_ValueLength-1);
      }
         
   }
   
   // got valid value
   return 0;
} // GetValue

//============================================================================
//
//============================================================================

void PericoControlFile::SkipStructure()
{
   QString qs_Name;
   QString qs_Value;
   bool    b_IsStartStructure;
   bool    b_IsEndStructure;
   int     i_Level;
   int     i_GetValueStatus;

   i_Level =0;
   while(true)
   {
      i_GetValueStatus = GetValue(qs_Name,
                                  qs_Value,
                                  b_IsStartStructure,
                                  b_IsEndStructure);
      // error during read
      if(i_GetValueStatus != 0)
      {
         return ;
      }
      if(b_IsStartStructure)
      {
         i_Level++;
      }
      else if(b_IsEndStructure)
      {
         i_Level --;
         if( i_Level < 0)
         {
            // we are at the end
            return;
         }
      }
   } // read forever
   
} // SkipStructure()

//============================================================================
//
//============================================================================


