//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PericoFileWriter.c
//---------------------------------------------------------------------------
// Purpose: Functions to create a file from type perico
//---------------------------------------------------------------------------

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

#include "PericoFileWriter.h"
#include "Tracer.h"

//============================================================================
//
//============================================================================

PericoFileWriter::PericoFileWriter() :Tracer("PericoFileWriter")
{
   m_i_OpenMode     = OpenModeUnknown;
   m_i_FileIndex    = 0;
   m_pc_ControlFile = NULL;

} // PericoFileWriter() - constructor

//============================================================================
//
//============================================================================

PericoFileWriter::~PericoFileWriter()
{
   if(m_pc_ControlFile != NULL)
   {
      delete m_pc_ControlFile;
      m_pc_ControlFile = NULL;
   }
} // PericoFileWriter() - destructor

//============================================================================
//
//============================================================================

int PericoFileWriter::OpenForWrite(QString qs_Filename)
{
   char ca_DirectoryBuffer[80];
   char *cp_Result;
   
   m_qs_RealFilename = qs_Filename;

   // create a tempory directory
   strcpy(ca_DirectoryBuffer,"/tmp/pericoXXXXXX");

   cp_Result = mkdtemp(ca_DirectoryBuffer);
   if(cp_Result ==NULL)
   {
      // could not create directory
      return -1;
   }
   
   m_qs_TempDirectoryName = cp_Result;


   TRACE << "mkdtemp = "
         << m_qs_TempDirectoryName
         << "\n";
      
   // have we got a valid directory?
   if(m_qs_TempDirectoryName.isEmpty())
   {
      return -1;
   }

   m_i_OpenMode      = OpenModeWrite;
   // everything is ok

   m_pc_ControlFile = new PericoControlFile();
   
   return 0;
   
} // OpenForWrite()

//============================================================================
//
//============================================================================

int PericoFileWriter::OpenForRead(QString qs_Filename)
{
   QString qs_ControlFilename;
   QString qs_Command;
   char    ca_DirectoryBuffer[80];
   char   *cp_Result;
   int     i_Result;
 
   m_qs_RealFilename = qs_Filename;

   // create a tempory directory
   strcpy(ca_DirectoryBuffer,"/tmp/pericoXXXXXX");

   cp_Result = mkdtemp(ca_DirectoryBuffer);
   if(cp_Result ==NULL)
   {
      // could not create directory
      return -1;
   }
   m_qs_TempDirectoryName = cp_Result;


   // make a command
   qs_Command.sprintf("(cd %s ;  tar -xzf %s)",
                      (const char *)  m_qs_TempDirectoryName,
                      (const char *) m_qs_RealFilename);

   TRACE <<"qs_Command ="
         << qs_Command
         << "\n";
   // execute the tar command
   i_Result = system(qs_Command);



   // create controlfile stucture
   m_pc_ControlFile = new PericoControlFile();
      
   qs_ControlFilename = m_qs_TempDirectoryName + "/ControlFile";
   m_pc_ControlFile->LoadFile(qs_ControlFilename);


   // everything is ok
   m_qs_RealFilename = qs_Filename;
   m_i_OpenMode      = OpenModeRead;
   return 0;

} // OpenForRead()

//============================================================================
//
//============================================================================

int PericoFileWriter::Close()
{
   QString qs_Command;
   int     i_Result;

   if(m_i_OpenMode == OpenModeWrite)
   {
      QString qs_ControlFileName;
      
      // first save the controllfile
      if(m_pc_ControlFile != NULL)
      {
         qs_ControlFileName =m_qs_TempDirectoryName + "/ControlFile";
         m_pc_ControlFile->SaveFile(qs_ControlFileName);
         delete m_pc_ControlFile;
         m_pc_ControlFile = NULL;
      }

      // now tar all the files

      // make a command
      qs_Command.sprintf("(cd %s ;  tar -czf %s *)",
                      (const char *)  m_qs_TempDirectoryName,
                      (const char *) m_qs_RealFilename);

      TRACE <<"qs_Command ="
            << qs_Command
            << "\n";
      // execute the tar command
      i_Result = system(qs_Command);



      // now remove the temporary directory with all files
      qs_Command.sprintf("rm -rf %s ",
                        (const char *) m_qs_TempDirectoryName);
      TRACE <<"qs_Command ="
            << qs_Command
            << "\n";
      // execute the rm command
      i_Result = system(qs_Command);
     
      
   }
   else if(m_i_OpenMode == OpenModeRead)
   {
      // now remove the temporary directory with all files
      qs_Command.sprintf("rm -rf %s ",
                        (const char *) m_qs_TempDirectoryName);
      TRACE <<"qs_Command ="
            << qs_Command
            << "\n";
      // execute the rm command
      i_Result = system(qs_Command);

   }

   // at the end delete the control file
   if(m_pc_ControlFile != NULL)
   {
      delete m_pc_ControlFile;
      m_pc_ControlFile = NULL;
   }

   // set mode to unknown
   m_i_OpenMode = OpenModeUnknown;
   
   // everything is ok
   return 0;

} // Close()

//============================================================================
//
//============================================================================

QString PericoFileWriter::GetNextFilename(QString &qs_NameWithoutPath)
{
   QString qs_Return;


   if(m_i_OpenMode != OpenModeWrite)
   {
      return "";
   }
   
   qs_NameWithoutPath.sprintf("FILE%08d",
                      m_i_FileIndex);
   qs_Return.sprintf("%s/%s",
                      (const char *) m_qs_TempDirectoryName,
                      (const char *) qs_NameWithoutPath);
   m_i_FileIndex += 1;

   return qs_Return;

} // GetNextFilename()

//============================================================================
//
//============================================================================

int PericoFileWriter::AddValue(QString qs_Name,QString qs_Value)
{
   if(m_pc_ControlFile == NULL)
   {
      return -1;
   }
   return m_pc_ControlFile->AddValue(qs_Name,qs_Value);
} // AddValue()

//============================================================================
//
//============================================================================

int PericoFileWriter::AddValue(QString qs_Name,int i_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%d",i_Value);
   return AddValue(qs_Name,qs_Value);
} // AddValue();

//============================================================================
//
//============================================================================

int PericoFileWriter::AddValue(QString qs_Name,double d_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%lf",d_Value);
   return AddValue(qs_Name,qs_Value);
} // AddValue();

//============================================================================
//
//============================================================================

int PericoFileWriter::AddValue(QString qs_Name, bool b_Value)
{
   QString qs_Value;
   qs_Value.sprintf("%d",b_Value? 1 : 0);
   return AddValue(qs_Name,qs_Value);
} // AddValue();

//============================================================================
//
//============================================================================

int PericoFileWriter::StartStructure(QString qs_Name,QString qs_Type)
{
   QString qs_Structure;
   int     i_Return;
   
   if(m_pc_ControlFile == NULL)
   {
      return -1;
   }

   qs_Structure = qs_Type + "{";

   i_Return = m_pc_ControlFile->AddValue(qs_Name,qs_Structure);
   if(i_Return ==0)
   {
      m_pc_ControlFile->LevelIncrement();
   }
      
   return i_Return;   

} // StartStructure()

//============================================================================
//
//============================================================================

int PericoFileWriter::FinishStructure()
{
   int     i_Return;

   if(m_pc_ControlFile == NULL)
   {
      return -1;
   }

   i_Return = m_pc_ControlFile->AddValue("}");
   if(i_Return ==0)
   {
      m_pc_ControlFile->LevelDecrement();
   }

   return i_Return;

} // FinishStructure()

//============================================================================
//
//============================================================================

int PericoFileWriter::GetValue(QString &qs_Name,
                QString &qs_Value,
                bool &b_IsStartStructure,
                bool &b_IsEndStructure)
{
   if(m_pc_ControlFile == NULL)
   {
      return -1;
   }

   return m_pc_ControlFile->GetValue(qs_Name,
                     qs_Value,
                     b_IsStartStructure,
                     b_IsEndStructure);
                     
} // GetValue()

//============================================================================
//
//============================================================================

QString PericoFileWriter::GetPathFilename(QString qs_Filename)
{
   QString qs_Return;

   qs_Return.sprintf("%s/%s",
                      (const char *) m_qs_TempDirectoryName,
                      (const char *) qs_Filename);

   return qs_Return;
} // GetPathFilename()

//============================================================================
//
//============================================================================

void PericoFileWriter::SkipStructure()
{
   if(m_pc_ControlFile == NULL)
   {
      return ;
   }

   m_pc_ControlFile->SkipStructure();
} // SkipStructure()

//============================================================================
//
//============================================================================


bool PericoFileWriter::atob(QString qs_Value)
{
   if(qs_Value[0]=='0')
   {
      return false;
   }
   return true;
} // atob

//============================================================================
//
//============================================================================

QString  PericoFileWriter::GetIconFileForFile(QString qs_Filename)
{
   struct stat s_StatBuffer;
   QString qs_Command;
   QString qs_Path;
   int     i_Result;

   char ca_DirectoryBuffer[80];
   char *cp_Result;

   m_i_OpenMode = OpenModeIconRead;
   m_qs_TempDirectoryName = "";
   
   // create a tempory directory
   strcpy(ca_DirectoryBuffer,"/tmp/pericoXXXXXX");

   cp_Result = mkdtemp(ca_DirectoryBuffer);
   if(cp_Result ==NULL)
   {
      // could not create directory
      return m_qs_TempDirectoryName;
   }
   m_qs_TempDirectoryName = cp_Result ;

   // remove a possible previous existing one
   qs_Command.sprintf("rm -rf %s/Icon.tif ",
                        (const char *) m_qs_TempDirectoryName);

   // fprintf(stderr,"qs_Command = %s\n",(const char *) qs_Command);
   // execute the rm command
   i_Result = system(qs_Command);

   // create the tar command
   qs_Command.sprintf("tar -xzf %s -C %s Icon.tif",
                        (const char *) qs_Filename,
                        (const char *) m_qs_TempDirectoryName);

   // fprintf(stderr,"qs_Command = %s\n",(const char *) qs_Command);
   // execute the tar command
   i_Result = system(qs_Command);

   qs_Path = m_qs_TempDirectoryName + "/Icon.tif";

   // lookup, whether the file really exists
   i_Result = stat(qs_Path,&s_StatBuffer);
   if(i_Result != 0)
   {
      return QString("");
   }

   
   return qs_Path;
} // GetIconFileForFile()

//============================================================================
//
//============================================================================

void  PericoFileWriter::DeleteTempIconFile()
{
   QString qs_Command;
   int i_Result;
   
   if(    m_i_OpenMode != OpenModeIconRead
       || m_qs_TempDirectoryName.isEmpty())
   {
      return;
   }

   // now remove the temporary directory with all files
   qs_Command.sprintf("rm -rf %s ",
                        (const char *) m_qs_TempDirectoryName);
   //fprintf(stderr,"qs_Command = %s\n",(const char *) qs_Command);
   TRACE <<"qs_Command ="
            << qs_Command
            << "\n";
   // execute the rm command
   i_Result = system(qs_Command);

   m_i_OpenMode = OpenModeUnknown;
   m_qs_TempDirectoryName="";
   

} // DeleteIconFile()

//============================================================================
//
//============================================================================

int PericoFileWriter::WriteStringList(QString qs_Filename,
                                      QStringList &pqsl_List)
{
   unsigned int ui_Index;
   unsigned int ui_Count;
   FILE         *fp_Out;
   fp_Out = fopen (qs_Filename,"w");
   if(fp_Out == NULL)
   {
      return -1;
   }

   ui_Count = pqsl_List.count();
   for( ui_Index=0;ui_Index <ui_Count;ui_Index++)
   {
      QString qs_Line;

      qs_Line = pqsl_List[ui_Index];
      fprintf(fp_Out,"%s\n",(const char *) qs_Line);
   }

   fclose(fp_Out);
   return 0;
} // WriteStringList()

//============================================================================
//
//============================================================================


int PericoFileWriter::ReadStringList(QString qs_Filename,
                                     QStringList *pqsl_List)
{
   FILE          *fp_In;
   unsigned char  c_InBuffer[1000];
   unsigned char  *cp_Read;
   QString         qs_Line;

   pqsl_List->clear();
   
   fp_In = fopen(qs_Filename,"r");
   if(fp_In==NULL)
   {
      return -1;
   }

   while(!feof(fp_In))
   {
      cp_Read = (unsigned char *) fgets((char *)c_InBuffer,
                                        sizeof(c_InBuffer)-1,
                                        fp_In) ;
      if(cp_Read ==NULL)
      {
         break;
      }
      c_InBuffer[ sizeof(c_InBuffer)-1] ='\0';
      while(*cp_Read != '\0')
      {
         if(*cp_Read< ' ')
         {
            *cp_Read ='\0';
            break;
         }
         cp_Read++;
      }
      qs_Line.sprintf("%s",c_InBuffer);
      pqsl_List->append(qs_Line);
      
      
   } // while not feof


   fclose(fp_In);
   return 0;
} // ()

//============================================================================
//
//============================================================================

