#ifndef _PericoFileWriter_H
#define _PericoFileWriter_H

#include <qstring.h>
#include "PericoControlFile.h"
#include "Tracer.h"


class PericoFileWriter : Tracer
{
public:

   PericoFileWriter();
   ~PericoFileWriter();

   enum
   {
      OpenModeUnknown=0,
      OpenModeWrite,
      OpenModeRead,
      OpenModeIconRead
   };

   int OpenForWrite(QString qs_Filename);
   int OpenForRead(QString qs_Filename);

   int Close();

   int AddValue(QString qs_Name,QString qs_Value);
   int AddValue(QString qs_Name,int i_Value);
   int AddValue(QString qs_Name,double d_Value);
   int AddValue(QString qs_Name,bool b_Value);
   int StartStructure(QString qs_Name,QString qs_Type);
   int FinishStructure();
   
   QString GetNextFilename(QString &qs_NameWithoutPath);

   int GetValue(QString &qsName,
                QString &qs_Value,
                bool &b_IsStartStructure,
                bool &b_IsEndStructure);
   QString GetPathFilename(QString qs_Filename);
   void SkipStructure();

   static bool atob(QString qs_Value);
   QString GetIconFileForFile(QString qs_Filenname);
   void DeleteTempIconFile();

   int WriteStringList(QString qs_Filename,QStringList &pqsl_List);
   int ReadStringList(QString qs_Filename,QStringList *pqsl_List);
      
private:

   int m_i_OpenMode;
   QString m_qs_RealFilename;
   QString m_qs_TempDirectoryName;

   PericoControlFile *m_pc_ControlFile;

   int m_i_FileIndex;
};

// define macro for easy reading of structures
#define PERICO_READ_STRUCTURE_START                          \
   {                                                         \
      QString qs_Name;                                       \
      QString qs_Value;                                      \
      bool    b_IsStartStructure;                            \
      bool    b_IsEndStructure;                              \
      int     i_GetValueStatus;                              \
      while(true)                                            \
      {                                                      \
         i_GetValueStatus = pc_FileWriter->GetValue(qs_Name, \
                                  qs_Value,                  \
                                  b_IsStartStructure,        \
                                  b_IsEndStructure);         \
         if(i_GetValueStatus !=0)                            \
         {                                                   \
            break;                                           \
         }


#define PERICO_READ_STRUCTURE_END                            \
         else if(b_IsEndStructure)                           \
         {                                                   \
            break;                                           \
         }                                                   \
         else if(b_IsStartStructure)                         \
         {                                                   \
            pc_FileWriter->SkipStructure();                  \
         }                                                   \
      }                                                      \
   }                                                         
      

#endif // _PericoFileWriter_H
