//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PictureUpdateConnection.c
//---------------------------------------------------------------------------
// Purpose: make a loose coupling between two pictures, that modifications
// in one picture are mentioned in the other.
//---------------------------------------------------------------------------

#include "PictureUpdateConnection.h"

//============================================================================
//
//============================================================================

PictureUpdateConnection::PictureUpdateConnection()
{
   m_c_ParentCallback.Clear();
   m_c_ChildCallback.Clear();

} // contructor

//============================================================================
//
//============================================================================

PictureUpdateConnection::PictureUpdateConnection(Callback c_ParentCallback)
{
   m_c_ParentCallback = c_ParentCallback;
   m_c_ChildCallback.Clear();

} // contructor

//============================================================================
//
//============================================================================

void PictureUpdateConnection::CleanChild()
{
   m_c_ChildCallback.Clear();
} // CleanChild()

//============================================================================
//
//============================================================================

PictureUpdateConnection::~PictureUpdateConnection()
{
   if(! m_c_ChildCallback.IsEmpty())
   {
      TellParentTerminate();
   }
}

//============================================================================
//
//============================================================================

void PictureUpdateConnection::SetParentCallback(Callback c_ParentCallback)
{
   m_c_ParentCallback = c_ParentCallback;
} // SetParentCallback()

//============================================================================
//
//============================================================================

void PictureUpdateConnection::SetChildCallback(Callback c_ChildCallback)
{
   m_c_ChildCallback = c_ChildCallback;
} // SetChildCallback()

//============================================================================
//
//============================================================================

void  PictureUpdateConnection::TellParentTerminate()
{
   // send a message to the child, that its parent
   // has terminated
   m_Message.m_i_MessageType =
              PictureUpdateConnectionMessage::ParentTermination;

   m_c_ChildCallback.Execute(&m_Message);
   
} // TellParentTerminate()

//============================================================================
//
//============================================================================

void  PictureUpdateConnection::TellChildTerminate()
{
   // tell a message to the parent, that its child
   // has terminated
   m_Message.m_i_MessageType =
              PictureUpdateConnectionMessage::ChildTermination;

   m_c_ParentCallback.Execute(&m_Message);

} // TellParentTerminate()

//============================================================================
//
//============================================================================

void PictureUpdateConnection::RemoveChildCallback()
{
   // the parent wants to attach a new child.
   // tell the current client, that it is not longer
   // valid
   m_Message.m_i_MessageType =
              PictureUpdateConnectionMessage::ParentTermination;

   m_c_ChildCallback.Execute(&m_Message);
} // RemoveChildCallback()

//============================================================================
//
//============================================================================

void PictureUpdateConnection::TellChildHasUpdatedImage(PixelBuffer *pc_Buffer)
{
   // tell the parent that the child has an updated picture to offer,
   // and the parent can copy it
   m_Message.m_i_MessageType =
              PictureUpdateConnectionMessage::ImageUpdated;
   m_Message.m_pc_Buffer = pc_Buffer;

   m_c_ParentCallback.Execute(&m_Message);

}  // TellChildHasUpdatedImage()

//============================================================================
//
//============================================================================



