//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PixelBufferDistortion.c
//---------------------------------------------------------------------------
// Purpose: special distortion functions on pixel buffers
//---------------------------------------------------------------------------


#include "PixelBufferDistortion.h"
#include "ColorCalculation.h"


//============================================================================
//
//============================================================================

PixelBufferDistortion::PixelBufferDistortion()
{

} // PixelBufferDistortion()

//============================================================================
//
//============================================================================

PixelBufferDistortion::~PixelBufferDistortion()
{

} // ~PixelBufferDistortion()

//============================================================================
//
//============================================================================

PixelBuffer * PixelBufferDistortion::AdjustLinesToVertical(PixelBuffer *pc_Buffer,
                                       GeoLine c_FirstLine,
                                       GeoLine c_SecondLine)
{
   PixelBuffer   *pc_NewImage;

   GeoCoordinate c_CuttingPoint;
   GeoCoordinate c_LeftCutPoint;
   GeoCoordinate c_RightCutPoint;

   GeoLine       c_RightOpticLine;
   GeoLine       c_LeftOpticLine;
   GeoLine       c_LineLine;

   double        d_DiffOffset;
   double        d_Magnifier;
   bool          b_Cuts;
   int           i_Width;
   int           i_Height;
   int           i_RunY;

   int           i_NewImageWidth;
   int           i_MaximalOffset;

   i_Height = pc_Buffer->GetHeight();
   i_Width  = pc_Buffer->GetWidth();

   // get the both lines

   // calculate the cutting-point. it describes the X-axis center
   b_Cuts = c_FirstLine.CutsLineExtended(c_SecondLine,c_CuttingPoint);

   // lines must cut
   if(!b_Cuts)
   {
      // display an error message
      return NULL;
   }

   if(c_CuttingPoint.GetY() > i_Height)
   {
      // below the bottom

      c_LeftOpticLine.Set(GeoCoordinate(0,0),c_CuttingPoint);
      c_RightOpticLine.Set(GeoCoordinate(i_Width-1,0),c_CuttingPoint);

      // first determine the cutting point for the last line
      c_LineLine.Set(GeoCoordinate(0,i_Height-1),GeoCoordinate(i_Width-1,i_Height-1));

      c_LineLine.CutsLineExtended(c_LeftOpticLine,c_LeftCutPoint);
      c_LineLine.CutsLineExtended(c_RightOpticLine,c_RightCutPoint);

      d_Magnifier = (double) i_Width
                       /  (c_RightCutPoint.GetX() - c_LeftCutPoint.GetX()) ;

      // calculate the offset for the first point
      // assume left x is new 0,

      d_DiffOffset =  (0 - c_LeftCutPoint.GetX()) * d_Magnifier;

      i_NewImageWidth = (int)( d_Magnifier * i_Width);
      if(d_DiffOffset >=0)
      {
         i_MaximalOffset = 0;
      }
      else
      {
         i_MaximalOffset = (int)  - d_DiffOffset;
      }

      if(c_CuttingPoint.GetX() <0)
      {
         // cuttin point is left of border
         i_NewImageWidth += (int)   d_DiffOffset;
      }
      else if(c_CuttingPoint.GetX() >= i_Width)
      {
         if(i_NewImageWidth < (int)( -d_DiffOffset+ i_Width) )
         {         
            i_NewImageWidth = (int)( -d_DiffOffset+ i_Width);
         }
      }
      
      // allocate a new larger buffer for the image
      pc_NewImage = new PixelBuffer(i_NewImageWidth,
                                       i_Height,
                                       pc_Buffer->GetDepth()
                                       );

      // now determine the cutting point fore every line
      for(i_RunY=0;i_RunY < i_Height;i_RunY++)
      {
         double d_DiffOffset;
         int    i_RunX;
         int    i_CountX;
         
         c_LineLine.Set(GeoCoordinate(0,i_RunY),GeoCoordinate(i_Width,i_RunY));

         c_LineLine.CutsLineExtended(c_LeftOpticLine,c_LeftCutPoint);
         c_LineLine.CutsLineExtended(c_RightOpticLine,c_RightCutPoint);

          d_Magnifier = (double) i_Width
                       /  (c_RightCutPoint.GetX() - c_LeftCutPoint.GetX()) ;
 
         // calculate the offset for the first point
         // assume left x is new 0,
 
         d_DiffOffset =  (0 - c_LeftCutPoint.GetX()) * d_Magnifier;

         i_CountX     = (int)(i_Width * d_Magnifier);

         for(i_RunX= 0;i_RunX<i_CountX; i_RunX ++)
         {
            int   i_Source;
            int   i_Destination;

            i_Destination = (int) (i_RunX + d_DiffOffset + i_MaximalOffset);
            i_Source      = (int) ( i_RunX / d_Magnifier);

            pc_NewImage->SetFull(i_Destination,i_RunY,
                                 pc_Buffer,i_Source,i_RunY);


         } // for i_RunX

      }   // for i_RunY

      // return the new image
      return pc_NewImage;
   }
   else if (c_CuttingPoint.GetY() < 0)
   {
      // above the top
      c_LeftOpticLine.Set(GeoCoordinate(0,i_Height-1),c_CuttingPoint);
      c_RightOpticLine.Set(GeoCoordinate(i_Width,i_Height-1),c_CuttingPoint);

      // first determine the cutting point for the last line
      c_LineLine.Set(GeoCoordinate(0,0),GeoCoordinate(i_Width-1,0));

      c_LineLine.CutsLineExtended(c_LeftOpticLine,c_LeftCutPoint);
      c_LineLine.CutsLineExtended(c_RightOpticLine,c_RightCutPoint);

      d_Magnifier = (double) i_Width
                       /  (c_RightCutPoint.GetX() - c_LeftCutPoint.GetX()) ;

      // calculate the offset for the first point
      // assume left x is new 0,

      d_DiffOffset =  (0 - c_LeftCutPoint.GetX()) * d_Magnifier;

      i_NewImageWidth = (int)( d_Magnifier * i_Width);
      if(d_DiffOffset >=0)
      {
         i_MaximalOffset = 0;
      }
      else
      {
         i_MaximalOffset = (int)  - d_DiffOffset;
      }

      if(c_CuttingPoint.GetX() <0)
      {
         // cuttin point is left of border
         i_NewImageWidth += (int)   d_DiffOffset;
      }
      else if(c_CuttingPoint.GetX() >= i_Width)
      {
         if(i_NewImageWidth < (int)( -d_DiffOffset+ i_Width) )
         {
            i_NewImageWidth = (int)( -d_DiffOffset+ i_Width);
         }
      }

      // allocate a new larger buffer for the image
      pc_NewImage = new PixelBuffer(i_NewImageWidth,
                                       i_Height,
                                       pc_Buffer->GetDepth()
                                       );

      // now determine the cutting point fore every line
      for(i_RunY=0;i_RunY < i_Height;i_RunY++)
      {
         double d_DiffOffset;
         int    i_RunX;
         int    i_CountX;

         c_LineLine.Set(GeoCoordinate(0,i_RunY),GeoCoordinate(i_Width,i_RunY));

         c_LineLine.CutsLineExtended(c_LeftOpticLine,c_LeftCutPoint);
         c_LineLine.CutsLineExtended(c_RightOpticLine,c_RightCutPoint);

          d_Magnifier = (double) i_Width
                       /  (c_RightCutPoint.GetX() - c_LeftCutPoint.GetX()) ;

         // calculate the offset for the first point
         // assume left x is new 0,

         d_DiffOffset =  (0 - c_LeftCutPoint.GetX()) * d_Magnifier;

         i_CountX     = (int)(i_Width * d_Magnifier);

         for(i_RunX= 0;i_RunX<i_CountX; i_RunX ++)
         {
            int   i_Source;
            int   i_Destination;

            i_Destination = (int) (i_RunX + d_DiffOffset + i_MaximalOffset);
            i_Source      = (int) ( i_RunX / d_Magnifier);

            pc_NewImage->SetFull(i_Destination,i_RunY,
                                 pc_Buffer,i_Source,i_RunY);


         } // for i_RunX

      }   // for i_RunY

      // return the new image
      return pc_NewImage;
   }
   else
   {
      // display an error message
      return NULL;
   }
   return NULL;

} // AdjustLinesToVertical()

//============================================================================
//
//============================================================================

PixelBuffer * PixelBufferDistortion::AdjustLinesToHorizontal(PixelBuffer *pc_Buffer,
                                       GeoLine c_FirstLine,
                                       GeoLine c_SecondLine)
{
   PixelBuffer   *pc_NewImage;

   GeoCoordinate c_CuttingPoint;
   GeoCoordinate c_BottomCutPoint;
   GeoCoordinate c_TopCutPoint;

   GeoLine       c_BottomOpticLine;
   GeoLine       c_TopOpticLine;
   GeoLine       c_LineLine;

   double        d_DiffOffset;
   double        d_Magnifier;
   bool          b_Cuts;
   int           i_Width;
   int           i_Height;
   int           i_RunX;

   int           i_NewImageHeight;
   int           i_MaximalOffset;

   i_Height = pc_Buffer->GetHeight();
   i_Width  = pc_Buffer->GetWidth();

   // get the both lines

   // calculate the cutting-point. it describes the X-axis center
   b_Cuts = c_FirstLine.CutsLineExtended(c_SecondLine,c_CuttingPoint);

   // lines must cut
   if(!b_Cuts)
   {
      // display an error message
      return NULL;
   }

   if(c_CuttingPoint.GetX() > i_Width)
   {
      // right of the image

      c_TopOpticLine.Set(GeoCoordinate(0,0),c_CuttingPoint);
      c_BottomOpticLine.Set(GeoCoordinate(0,i_Height -1),c_CuttingPoint);

      // first determine the cutting point for the rightmost line
      c_LineLine.Set(GeoCoordinate(i_Width-1,0),GeoCoordinate(i_Width-1,i_Height-1));

      c_LineLine.CutsLineExtended(c_TopOpticLine,c_TopCutPoint);
      c_LineLine.CutsLineExtended(c_BottomOpticLine,c_BottomCutPoint);

      d_Magnifier = (double) i_Height
                       /  (c_BottomCutPoint.GetY() - c_TopCutPoint.GetY()) ;

      // calculate the offset for the first point
      // assume top Y is new 0,

      d_DiffOffset =  (0 - c_TopCutPoint.GetY()) * d_Magnifier;

      i_NewImageHeight = (int)( d_Magnifier * i_Height);
      if(d_DiffOffset >=0)
      {
         i_MaximalOffset = 0;
      }
      else
      {
         i_MaximalOffset = (int)  - d_DiffOffset;
      }

      if(c_CuttingPoint.GetY() <0)
      {
         // cuttin point is left of border
         i_NewImageHeight += (int)   d_DiffOffset;
      }
      else if(c_CuttingPoint.GetY() >= i_Height)
      {
         if(i_NewImageHeight < (int)( -d_DiffOffset+ i_Height) )
         {
            i_NewImageHeight = (int)( -d_DiffOffset+ i_Height);
         }
      }

      // allocate a new larger buffer for the image
      pc_NewImage = new PixelBuffer(i_Width,
                                     i_NewImageHeight,
                                       pc_Buffer->GetDepth()
                                       );

      // now determine the cutting point fore every line
      for(i_RunX=0;i_RunX < i_Width;i_RunX++)
      {
         double d_DiffOffset;
         int    i_RunY;
         int    i_CountY;

         c_LineLine.Set(GeoCoordinate(i_RunX,0),GeoCoordinate(i_RunX,i_Height));

         c_LineLine.CutsLineExtended(c_TopOpticLine,c_TopCutPoint);
         c_LineLine.CutsLineExtended(c_BottomOpticLine,c_BottomCutPoint);

          d_Magnifier = (double) i_Height
                       /  (c_BottomCutPoint.GetY() - c_TopCutPoint.GetY()) ;

         // calculate the offset for the first point
         // assume left x is new 0,

         d_DiffOffset =  (0 - c_TopCutPoint.GetY()) * d_Magnifier;

         i_CountY     = (int)(i_Height * d_Magnifier);

         for(i_RunY= 0;i_RunY<i_CountY; i_RunY ++)
         {
            int   i_Source;
            int   i_Destination;

            i_Destination = (int) (i_RunY + d_DiffOffset + i_MaximalOffset);
            i_Source      = (int) ( i_RunY / d_Magnifier);

            pc_NewImage->SetFull(i_RunX,i_Destination,
                                 pc_Buffer,i_RunX,i_Source);


         } // for i_RunY

      }   // for i_RunX

      // return the new image
      return pc_NewImage;
   }
   else if (c_CuttingPoint.GetX() < 0)
   {
      // left of the image

      c_TopOpticLine.Set(GeoCoordinate(i_Width-1,0),c_CuttingPoint);
      c_BottomOpticLine.Set(GeoCoordinate(i_Width-1,i_Height -1),c_CuttingPoint);

      // first determine the cutting point for the leftmost line
      c_LineLine.Set(GeoCoordinate(0,0),GeoCoordinate(0,i_Height-1));

      c_LineLine.CutsLineExtended(c_TopOpticLine,c_TopCutPoint);
      c_LineLine.CutsLineExtended(c_BottomOpticLine,c_BottomCutPoint);

      d_Magnifier = (double) i_Height
                       /  (c_BottomCutPoint.GetY() - c_TopCutPoint.GetY()) ;

      // calculate the offset for the first point
      // assume top Y is new 0,

      d_DiffOffset =  (0 - c_TopCutPoint.GetY()) * d_Magnifier;

      i_NewImageHeight = (int)( d_Magnifier * i_Height);
      if(d_DiffOffset >=0)
      {
         i_MaximalOffset = 0;
      }
      else
      {
         i_MaximalOffset = (int)  - d_DiffOffset;
      }

      if(c_CuttingPoint.GetY() <0)
      {
         // cuttin point is left of border
         i_NewImageHeight += (int)   d_DiffOffset;
      }
      else if(c_CuttingPoint.GetY() >= i_Height)
      {
         if(i_NewImageHeight < (int)( -d_DiffOffset+ i_Height) )
         {
            i_NewImageHeight = (int)( -d_DiffOffset+ i_Height);
         }
      }

      // allocate a new larger buffer for the image
      pc_NewImage = new PixelBuffer(i_Width,
                                     i_NewImageHeight,
                                       pc_Buffer->GetDepth()
                                       );

      // now determine the cutting point fore every line
      for(i_RunX=0;i_RunX < i_Width;i_RunX++)
      {
         double d_DiffOffset;
         int    i_RunY;
         int    i_CountY;

         c_LineLine.Set(GeoCoordinate(i_RunX,0),GeoCoordinate(i_RunX,i_Height));

         c_LineLine.CutsLineExtended(c_TopOpticLine,c_TopCutPoint);
         c_LineLine.CutsLineExtended(c_BottomOpticLine,c_BottomCutPoint);

          d_Magnifier = (double) i_Height
                       /  (c_BottomCutPoint.GetY() - c_TopCutPoint.GetY()) ;

         // calculate the offset for the first point
         // assume left x is new 0,

         d_DiffOffset =  (0 - c_TopCutPoint.GetY()) * d_Magnifier;

         i_CountY     = (int)(i_Height * d_Magnifier);

         for(i_RunY= 0;i_RunY<i_CountY; i_RunY ++)
         {
            int   i_Source;
            int   i_Destination;

            i_Destination = (int) (i_RunY + d_DiffOffset + i_MaximalOffset);
            i_Source      = (int) ( i_RunY / d_Magnifier);

            pc_NewImage->SetFull(i_RunX,i_Destination,
                                 pc_Buffer,i_RunX,i_Source);


         } // for i_RunY

      }   // for i_RunX

      // return the new image
      return pc_NewImage;
   }
   else
   {
      // display an error message
      return NULL;
   }
   return NULL;
} // AdjustLinesToHorizontal()

//============================================================================
//
//============================================================================
