//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PixelVector.c
//---------------------------------------------------------------------------
// Purpose: 1 dimensional pixel vector
//---------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "PixelVector.h"
#include "Utilities.h"

//===========================================================================
//
//===========================================================================

PixelVector::PixelVector(int i_Width,
               int i_Length)
{
   // make wisth a multiple of 8 bit
   // and calculate bytes

   i_Width = (i_Width+7) >>3;
   // maximum is 4 bytes 
   if(i_Width>4)
   {
      i_Width =4;
   }
   m_i_Length =0;

   m_i_BufferSize = i_Width*i_Length;

   // allocate a buffer
   m_p_Buffer = new unsigned char[m_i_BufferSize];
   if(m_p_Buffer == NULL)
   {
      return;
   }

   m_i_Length       = i_Length;
   m_i_BufferDepth  = i_Width*8;

   // initial clean up the buffer
   memset(m_p_Buffer,0,m_i_BufferSize);
   
} // PixelVector()

//===========================================================================
//
//===========================================================================
       
PixelVector::~PixelVector()
{
   if(m_p_Buffer != NULL)
   {
      delete m_p_Buffer;
      m_p_Buffer = NULL;
   }

} // ~PixelVector()

//===========================================================================
//
//===========================================================================

void PixelVector::Set( int i_Index,  Pixel p_Value)
{
   // check index in range
   if(   i_Index <0
      || i_Index >= m_i_Length)
   {
      return;
   }

   // store, depending on buffer depth
   switch(m_i_BufferDepth)
   {
   case 8:
      PBYTE *pc_Address;
      pc_Address =(PBYTE *) m_p_Buffer;
      pc_Address[i_Index] = p_Value;
      break;
   case 16:
      PWORD *pc_WAddress;
      pc_WAddress =(PWORD *) m_p_Buffer;
      pc_WAddress[i_Index] = p_Value;
      break;
   case 24:
      PBYTE *pc_TAddress;
      pc_TAddress =(PBYTE *) m_p_Buffer;
      pc_TAddress = &pc_TAddress[i_Index*3] ;
      pc_TAddress[0] = (p_Value    ) & 0xff;
      pc_TAddress[1] = (p_Value>>8 ) & 0xff;
      pc_TAddress[2] = (p_Value>>16) & 0xff;
      break;
   case 32:
      PLONG *pc_LAddress;
      pc_LAddress =(PLONG *) m_p_Buffer;
      pc_LAddress[i_Index] = p_Value;
      break;
   } // switch
} // set()

//===========================================================================
//
//===========================================================================

Pixel PixelVector::Get(int i_Index)
{
   Pixel  p_Return =0;

   // check index in range
   if(   i_Index <0
      || i_Index >= m_i_Length)
   {
      return 0;
   }

   // get value depending on the buffer depth
   switch(m_i_BufferDepth)
   {
   case 8:
      PBYTE *pc_Address;
      pc_Address =(PBYTE *) m_p_Buffer;
      return (Pixel) (pc_Address[i_Index]);
      break;
   case 16:
      PWORD *pc_WAddress;
      pc_WAddress =(PWORD *) m_p_Buffer;
      return (Pixel)( pc_WAddress[i_Index]);
      break;
   case 24:
      PBYTE *pc_TAddress;
      pc_TAddress =(PBYTE *) m_p_Buffer;
      pc_TAddress = &pc_TAddress[i_Index*3] ;
      p_Return  =  (Pixel) pc_TAddress[0];
      p_Return |= ((Pixel) pc_TAddress[1]) << 8;
      p_Return |= ((Pixel) pc_TAddress[2]) << 16;
      return p_Return;
      break;
   case 32:
      PLONG *pc_LAddress;
      pc_LAddress =(PLONG *) m_p_Buffer;
      return (Pixel)(pc_LAddress[i_Index] );
      break;
   } // switch

   // should not come to here !!
   return 0;
   
} // Get()

//===========================================================================
//
//===========================================================================

int PixelVector::GetDepth()
{
   return m_i_BufferDepth;
} // GetDepth()

//===========================================================================
//
//===========================================================================

int PixelVector::GetLength()
{
   return m_i_Length;
} // GetLength()

//===========================================================================
//
//===========================================================================

void  PixelVector::Inc(int i_Index)
{
   // check index in range
   if(   i_Index <0
      || i_Index >= m_i_Length)
   {
      return;
   }

   if(m_i_BufferDepth == BITNUMBER_LONG)
   {
      PLONG * pc_Address;
      pc_Address = (PLONG *) m_p_Buffer;
      pc_Address[i_Index] += 1;
   }
   else
   {
      Pixel p_Old;
      p_Old = Get(i_Index);
      Set(i_Index,p_Old +1);
   }
} // Inc()

//===========================================================================
//
//===========================================================================

void  PixelVector::Clean(Pixel i_Value )
{
   if(i_Value == 0)
   {
      // clean up the buffer
      memset(m_p_Buffer,0,m_i_BufferSize);

   }
   else
   {
      int i_Run;
      for(i_Run =0;i_Run < m_i_Length; i_Run ++)
      {
         Set(i_Run,i_Value);
      }
   }
} // Clean()

//===========================================================================
//
//===========================================================================

