//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QUFileIcon.c
//---------------------------------------------------------------------------
// Purpose:
//---------------------------------------------------------------------------

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qfiledialog.h>
#include <qregexp.h>
#include <qmessagebox.h>

#include "QUFileIcon.h"
#include "ApplicationStorage.h"
#include "BaseData.h"
#include "BaseDataStrings.h"
#include "PixelBuffer.h"
#include "FileLoader.h"
#include "QtUtilities.h"
#include "MainFrame.h"
#include "IconLibrary.h"

//===========================================================================
//
//===========================================================================

QUFileIcon::QUFileIcon()
{
   m_pc_Icon =NULL;

} // AutosavedFile()

//===========================================================================
//
//===========================================================================

QUFileIcon::~QUFileIcon()
{
   if(m_pc_Icon!= NULL)
   {
      delete m_pc_Icon;
      m_pc_Icon =NULL;
   }

} // ~AutosavedFile()

//===========================================================================
//
//===========================================================================

void QUFileIcon::LoadIcon()
{
   FileLoader c_FileLoader(NULL);
   QString    qs_PathAndName;

   qs_PathAndName = GetFullname();

   m_pc_Icon =c_FileLoader.GetIconforFile(qs_PathAndName);
   if(m_pc_Icon ==NULL)
   {
      // no icon available. Take the default
      m_pc_Icon = g_Application->m_pc_IconLibrary->GetIcon("DefaultIcon");
   }


} // LoadIcon()

//===========================================================================
//
//===========================================================================

QString QUFileIcon::GetFullname()
{
   QString qs_LastChar;

   qs_LastChar = m_qs_Path.right(1);

   if(qs_LastChar.compare("/")==0)
   {
      // dont add a slash in between
      return m_qs_Path  + m_qs_Name;
   }
   else
   {
      // add a slash
      return m_qs_Path +"/" + m_qs_Name;
   }
   
} // GetFullname()


//===========================================================================
//===========================================================================
//===========================================================================
//===========================================================================
//===========================================================================

QUFileIconWidget::QUFileIconWidget(QWidget *parent,unsigned int ui_Mode ):
                        QWidget(parent)
{
   QBoxLayout   *l_Layout;
   QWidget      *w_ButtonWidget;


   m_ui_Mode = ui_Mode;
   m_qs_Wildcard  ="*";
   m_qs_Directory ="./";
   
   l_Layout     = new  QVBoxLayout( this,1);
   m_pc_IconBox = new  ScrollIconBox(this,
                        ScrollIconBox::UseScrollbar);
   m_pc_IconBox->SetIconSize(100,100);
   l_Layout->addWidget(m_pc_IconBox,1);

   m_StatusLabel = new QLabel(this,"");
   l_Layout->addWidget(m_StatusLabel);
   m_StatusLabel->setMinimumHeight(g_Application->m_pc_QtUtility->GetMaximalFontHeight());
   m_StatusLabel->setMaximumHeight(g_Application->m_pc_QtUtility->GetMaximalFontHeight());

   m_InfoEdit =new QTextEdit(this);
   m_InfoEdit->setMinimumWidth(g_Application->m_pc_QtUtility->GetMaximalFontWidth()*40);
   m_InfoEdit->setMinimumHeight(g_Application->m_pc_QtUtility->GetMaximalFontHeight()*4);
   m_InfoEdit->setMaximumHeight(g_Application->m_pc_QtUtility->GetMaximalFontHeight()*4);
   l_Layout->addWidget(m_InfoEdit);
   m_InfoEdit->setReadOnly(true);


   w_ButtonWidget = new QWidget(this);
   {
      QBoxLayout   *l_ButtonLayout;
      l_ButtonLayout     = new  QHBoxLayout( w_ButtonWidget,1);
      
      m_pc_LoadButton   = new QPushButton(tr("Load"),w_ButtonWidget);
      m_pc_DeleteButton = new QPushButton(tr("Delete"),w_ButtonWidget);
      

      l_ButtonLayout->addWidget(m_pc_LoadButton);
      l_ButtonLayout->addWidget(m_pc_DeleteButton);

      if( m_ui_Mode & ModeDeleteAll)
      {
         m_pc_DeleteAllButton = new QPushButton(tr("Delete All"),w_ButtonWidget);
         l_ButtonLayout->addWidget(m_pc_DeleteAllButton);
      }
      else
      {
         m_pc_DeleteAllButton = NULL;
      }
      
   }
   l_Layout->addWidget(w_ButtonWidget);


   m_pc_IconBox->setSizePolicy(
          QSizePolicy(QSizePolicy::Ignored,
                      QSizePolicy::Expanding));

   m_StatusLabel->setSizePolicy(
          QSizePolicy(QSizePolicy::Ignored,
                      QSizePolicy::Minimum));


   connect( m_pc_IconBox,
               SIGNAL(IconClicked(int)),
               SLOT(IconClicked(int )) );

   connect( m_pc_IconBox,
               SIGNAL(IconSelected(int)),
               SLOT(IconSelected(int )) );


   connect( m_pc_LoadButton,
               SIGNAL(clicked()),
               SLOT(LoadIcon( )) );
   connect( m_pc_DeleteButton,
               SIGNAL(clicked()),
               SLOT(DeleteIcon( )) );
   if(m_pc_DeleteAllButton != NULL)
   {
      connect( m_pc_DeleteAllButton,
               SIGNAL(clicked()),
               SLOT(DeleteAllIcons( )) );
   }

               
} // QUFileIconWidget()

//===========================================================================
//
//===========================================================================


void QUFileIconWidget::ActicateSelection()
{
   // delete old contents
   DeleteAutosavedList();

   // read a new list
   ReadFileList();

   // laod icons for the list
   LoadIcons();

   // transfer into icon box
   TransferFilesIntoBox();

   // set widget sensitivity
   SetSensitivity();

} // ActicateSelection()


//===========================================================================
//
//===========================================================================

QUFileIconWidget::~QUFileIconWidget( )
{
   DeleteAutosavedList();
} // ~AutosaveBox()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::ReadFileList()
{
   struct dirent * pc_Entry;
   QRegExp         qr_Wildcard;
   QString         qs_LastChar;
   DIR           * pc_TheDirectory;


   pc_TheDirectory = opendir( m_qs_Directory);
   if(pc_TheDirectory == NULL)
   {
      return;
   }

   // wildcard available
   if(!m_qs_Wildcard.isEmpty() )
   {
      // then make a regular expression
      qr_Wildcard = QRegExp(m_qs_Wildcard,false,true);
   }

   while(true)
   {
      QUFileIcon *pc_NewFile;
      QString     qs_Name;
      QString     qs_End;
      
      pc_Entry = readdir(pc_TheDirectory);
      if(pc_Entry == NULL)
      {
         break;
      }

      qs_Name = pc_Entry->d_name;

      // not a regular file ??
      if(  pc_Entry->d_type !=  DT_REG)
      {
         continue;
      }

      // check, whether it ends with .perico
      qs_End = qs_Name.right(7);
      if(qs_End.compare(".perico") != 0)
      {
         continue;
      }

      // at this point a valid autosave found

      if(!m_qs_Wildcard.isEmpty() )
      {
         // use the regular expression to check the filename
         int i_ContainResult;
         
         i_ContainResult = qs_Name.find(qr_Wildcard);
         /*
         fprintf(stderr,"<%s> matches <%s>,%d\n",
            (const char *) qs_Name,
            (const char *) m_qs_Wildcard,
            i_ContainResult);
         */
         if(i_ContainResult == 0)
         {
         }
         else // no match
         {
            // do not add it to the list
            continue;
         }
      } // if wildcard available

      pc_NewFile = new QUFileIcon();
      pc_NewFile->m_qs_Name = qs_Name;
      pc_NewFile->m_qs_Path = m_qs_Directory;

      // get the date of the file
      {
         struct stat s_StatBuffer;
         QString     qs_Fullname;

         qs_Fullname = pc_NewFile->GetFullname();

         lstat ( (const char *) qs_Fullname,&s_StatBuffer);
         pc_NewFile->m_tt_SavedDate = s_StatBuffer.st_ctime;

         
      }      

      // put into the list, but sort by date
      {
         bool b_Inserted;
         int  i;

         b_Inserted= false;

         for(i=0;i<(int)m_l_IconFiles.count();i++)
         {
            QUFileIcon *pc_AFile;
            pc_AFile = m_l_IconFiles.at(i);
            if( pc_AFile->m_tt_SavedDate > pc_NewFile->m_tt_SavedDate)
            {
               // insert before
               m_l_IconFiles.insert(i,pc_NewFile);

               b_Inserted = true;
               break;
            }
            
         }
         
         if(!b_Inserted )
         {
            // add at the end
            m_l_IconFiles.append (pc_NewFile);
         }
      }
              
   }
   closedir(pc_TheDirectory);
   
} // ReadFileList()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::DeleteAutosavedList()
{
   while(m_l_IconFiles.count() >0)
   {
      QUFileIcon *pc_File;

      pc_File = m_l_IconFiles.getLast();

      if(pc_File != NULL)
      {
         delete pc_File;
      }
      m_l_IconFiles.removeLast();
      
   }
} // DeleteAutosavedList()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::LoadIcons()
{
   QUFileIcon *pc_File;
   
   if(m_l_IconFiles.count() <=0)
   {
      return;
   }
   pc_File = m_l_IconFiles.first();

   while(pc_File != NULL)
   {
      pc_File->LoadIcon();
      pc_File = m_l_IconFiles.next();
   }
   

} // LoadIcons()

//===========================================================================
//
//===========================================================================


void QUFileIconWidget::TransferFilesIntoBox()
{
   QUFileIcon *pc_File;
   int         i_Count;

   i_Count = m_l_IconFiles.count();


   m_pc_IconBox->DeleteAllImages();
   
   if(i_Count <=0)
   {
      m_StatusLabel->setText(tr("No Files"));
      return;
   }
   pc_File = m_l_IconFiles.first();

   while(pc_File != NULL)
   {

      QDateTime qdt_Time;
      QString   qs_Date;

      qdt_Time.setTime_t(pc_File->m_tt_SavedDate);
      

      qs_Date = qdt_Time.toString("hh:mm:ss");

      if(pc_File->m_pc_Icon != NULL)
      {
         m_pc_IconBox->AddImage(pc_File->m_pc_Icon,
                                qs_Date);
      }

      pc_File = m_l_IconFiles.next();
   }

   SetFileNumberText();
   

} // TransferFilesIntoBox()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::SetFileNumberText()
{
   int i_Count;

   i_Count = m_l_IconFiles.count();

   if(i_Count == 1)
   {

      m_StatusLabel->setText(tr("1 File"));
   }
   else
   {
      QString qs_Out;

      qs_Out = tr("Files");
      qs_Out.sprintf("%d %s",i_Count,(const char *) qs_Out);
      m_StatusLabel->setText(qs_Out);
   }


} // SetFileNumberText()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::IconSelected(int i_IconNumber)
{
   QUFileIcon *pc_File;
   QString     qs_Filename;
   // get file info
   pc_File = m_l_IconFiles.at(i_IconNumber);

   if(pc_File ==NULL)
   {
      return;
   }

   qs_Filename = pc_File->GetFullname();   
   
   IconSelected(qs_Filename);

   SetSensitivity();
} // IconSelected()

//===========================================================================
//
//===========================================================================

void  QUFileIconWidget::IconClicked(int i_IconNumber)
{
   m_InfoEdit->clear();

   if(i_IconNumber <0)
   {
      // nothing selected, blank out
   }
   else
   {
      QUFileIcon *pc_File;
      QDateTime      qdt_Time;
      QString        qs_Date;
      QString        qs_Total;

      // get file info
      pc_File = m_l_IconFiles.at(i_IconNumber);

      if(pc_File == NULL)
      {
         return;
      }

      // add date into the edit field
      m_InfoEdit->append(pc_File->GetFullname());

      // get date
      qdt_Time.setTime_t(pc_File->m_tt_SavedDate);

      // convert to date and time string
      qs_Date = qdt_Time.toString("yyyy-M-d hh:mm:ss");

      qs_Total = tr("File saved at");
      qs_Total += " ";
      qs_Total += qs_Date;
      // add date into the edit field
      m_InfoEdit->append(qs_Total);

      // activate the icon
      m_pc_IconBox->SetSelected(i_IconNumber);
   }
   SetSensitivity();
} // IconClicked()

//===========================================================================
//
//===========================================================================

void  QUFileIconWidget::LoadIcon()
{
   int i_Selected;

   i_Selected = m_pc_IconBox->GetSelected();
   
   if(i_Selected <0)
   {
      return;
   }

   LoadFile(i_Selected);
   
   SetSensitivity();
} // LoadIcon()

//===========================================================================
//
//===========================================================================

void  QUFileIconWidget::DeleteIcon()
{
   QUFileIcon *pc_File;
   QString     qs_Filename;
   int         i_Selected;

   i_Selected = m_pc_IconBox->GetSelected();


   if(i_Selected <0)
   {
      return;
   }

   // get file info
   pc_File = m_l_IconFiles.at(i_Selected);

   // remove it from the files list
   m_l_IconFiles.remove(i_Selected);

   // remove it from the icon box
   m_pc_IconBox->DeleteImage(i_Selected);

   // clean up the text info
   m_InfoEdit->clear();
   SetFileNumberText();


   // at this point physically delete the file
   unlink((const char *) (pc_File->GetFullname()));
   // delete the object
   delete pc_File;

   SetSensitivity();
} // DeleteIcon()

//===========================================================================
//
//===========================================================================

void  QUFileIconWidget::DeleteAllIcons()
{
   QUFileIcon *pc_File;
   QString     qs_Filename;


   // first display a warning message
   {
      int i_WarningResult;
      
      i_WarningResult = QMessageBox::warning( this,
                  tr("Perico: Delete all Files"),
                  tr("Really delete all images?")+"\n",
                  tr("Delete"),
                  tr("Cancel"),
                  0,
                  1,
                  1
                  );

      if(i_WarningResult == 1)
      {
         return;
      }
   }

   while(m_l_IconFiles.count() >0)
   {
      // get file info
      pc_File = m_l_IconFiles.at(0);

      // remove it from the files list
      m_l_IconFiles.remove((unsigned int)0);

      // remove it from the icon box
      m_pc_IconBox->DeleteImage(0);

      // at this point physically delete the file
      unlink((const char *) (pc_File->GetFullname()));
      // delete the object
      delete pc_File;

   }
   
   // clean up the text info
   m_InfoEdit->clear();
   SetFileNumberText();


   SetSensitivity();
} // DeleteAllIcons()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::SetSensitivity()
{
   int i_Selected;

   i_Selected = m_pc_IconBox->GetSelected();

   // if something is selected, enable the buttons
   if(i_Selected <0)
   {
      m_pc_LoadButton->setEnabled(false);
      m_pc_DeleteButton->setEnabled(false);
   }
   else
   {
      // nothing selected
      m_pc_LoadButton->setEnabled(true);
      m_pc_DeleteButton->setEnabled(true);

   }
   if(m_pc_DeleteAllButton != NULL)
   {
      // if delete all button exists and files available
      if(m_l_IconFiles.count() >0)
      {
         // then enable the delete all button
         m_pc_DeleteAllButton->setEnabled(true);
      }
      else
      {
         m_pc_DeleteAllButton->setEnabled(false);
      }
   }      
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::LoadFile(int i_Index)
{
   QUFileIcon *pc_File;
   QString        qs_Filename;

   // get file info
   pc_File = m_l_IconFiles.at(i_Index);

   qs_Filename = pc_File->GetFullname();
   
   IconSelected(qs_Filename);
   
} // LoadFile()

//===========================================================================
//
//===========================================================================

int QUFileIconWidget::GetNumberOfFiles()
{
   return m_l_IconFiles.count();
} // GetNumberOfFiles()

//===========================================================================
//
//===========================================================================

void QUFileIconWidget::SetPath(QString qs_Path)
{
   m_qs_Directory = qs_Path;
} // SetPath()

//===========================================================================
//
//===========================================================================


void QUFileIconWidget::SetWildcard(QString qs_Wildcard)
{
   m_qs_Wildcard = qs_Wildcard;
} // SetWildcard()

//===========================================================================
//
//===========================================================================



