//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QUFileSelectEdit.c
//---------------------------------------------------------------------------
// Purpose: a editbox and a select button for file or directory selection.
//---------------------------------------------------------------------------

#include <qhgroupbox.h>
#include <qlayout.h>
#include <qfiledialog.h>

#include "QUFileSelectEdit.h"

//===========================================================================
//
//===========================================================================


QUFileSelectEdit::QUFileSelectEdit(QWidget *parent,
                     QString qs_Description,
                     QString qs_DefaultFile,
                     QString qs_SelectCaption,
                     QString qs_Filter,
                     bool    b_IsFile)
                      : QWidget(parent)
{
   QHBoxLayout  *l_Layout;

   m_qs_SelectCaption = qs_SelectCaption;;
   m_qs_Filter        = qs_Filter ;
   m_b_IsFile         = b_IsFile;

 
   // create horizontal box
   l_Layout  = new  QHBoxLayout( this,2);

   m_w_Description = new QLabel(qs_Description, this);
   l_Layout->addWidget( m_w_Description);

   m_w_EditField = new QLineEdit(this);
   l_Layout->addWidget( m_w_EditField);

   m_w_SelectButton = new QPushButton(tr("Select"),this);
   l_Layout->addWidget( m_w_SelectButton);


   m_w_EditField->setText(qs_DefaultFile);



   connect( m_w_SelectButton,
               SIGNAL(clicked()),
               SLOT(SelectButtonClicked()) );
   
} // QUFileSelectEdit()

//===========================================================================
//
//===========================================================================

void QUFileSelectEdit::SelectButtonClicked()
{
   QFileDialog *w_FileDialog;
   QString      qs_CurrentEditText;
   QString      qs_NewFilename;
   int          i_ExecReturn;

   // get the current text
   qs_CurrentEditText = m_w_EditField->text();

   w_FileDialog = new QFileDialog(this);
   w_FileDialog->setCaption(m_qs_SelectCaption);

   // set mode to find only directories
   if(m_b_IsFile)
   {
      if(!m_qs_Filter.isEmpty())
      {
         w_FileDialog->addFilter(m_qs_Filter);
      }
   }
   else
   {
      w_FileDialog->setMode(QFileDialog::DirectoryOnly);
   }
   
   w_FileDialog->setDir(qs_CurrentEditText);

   // ask for a name
   i_ExecReturn = w_FileDialog->exec();

   // if abort, then do nothing
   if(i_ExecReturn == 0)
   {
      // delete the allocated dialog
      delete w_FileDialog;
      return ;
   }

   // get the filename
   qs_NewFilename = w_FileDialog->selectedFile();

   // delete the allocated dialog
   delete w_FileDialog;

   // set the new text
   m_w_EditField->setText(qs_NewFilename);

} // SelectButtonClicked()

//===========================================================================
//
//===========================================================================

void QUFileSelectEdit::SetFile(QString qs_File)
{
   m_w_EditField->setText(qs_File);

} // SetFile()

//===========================================================================
//
//===========================================================================

QString QUFileSelectEdit::GetFile()
{
   QString qs_Return;

   qs_Return = m_w_EditField->text();
   
   return qs_Return;
} // GetFile()


//===========================================================================
//
//===========================================================================



