//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:  QUSelectPixelButton.c
//---------------------------------------------------------------------------
// Purpose: Overload a button to display an icon instead
//---------------------------------------------------------------------------


#include "QUSelectPixelButton.h"

#include <qhgroupbox.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include "QtUtilities.h"
#include "ApplicationStorage.h"
#include "IconLibrary.h"
#include "PixelBuffer.h"
#include "qtooltip.h"
#include "DragAndDropConstants.h"

//===========================================================================
//
//===========================================================================

QUSelectPixelButton::QUSelectPixelButton(QWidget *parent,
                       QString qs_IconName,
                       QString qs_DefaultText,
                       unsigned int ui_Mode)
                     : QWidget(parent)
{
   QColorGroup  qcg_ColorGroup;
   QVBoxLayout *l_Layout;
   QColor       qc_Background;

   m_ui_Mode = ui_Mode;

   // create a simple box
   l_Layout = new QVBoxLayout(this);

   // and add a pixelbutton into it
   m_pc_Button = new QUPixelButton(this,
                                   qs_IconName,
                                   qs_DefaultText,
                                   m_ui_Mode);
   l_Layout->setMargin(4);
   l_Layout->addWidget(m_pc_Button);
   
   m_b_Selected = false;

   // save the original background color
   qcg_ColorGroup     = colorGroup();
   m_qc_OriginalColor = qcg_ColorGroup.background();

   // connect the image receiver slot
   connect( m_pc_Button,
                  SIGNAL(ImageDropped(PixelBuffer*)),
                  SLOT( ButtonImageDropped(PixelBuffer*)) );
   // connect the clicked slot
   connect( m_pc_Button,
                  SIGNAL(clicked()),
                  SLOT( ButtonClicked()) );
   // connect the context menu slot
   connect( m_pc_Button,
                  SIGNAL(ContextMenuSelected(int)),
                  SLOT( ButtonContextMenuSelected(int)) );

   // no resize at all, keep as small as possible
   setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed,false);

   
} // QUSelectPixelButton()

//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::SetSelected(bool b_Selected)
{
   QPalette    pal;

   // keep the same state?
   if(m_b_Selected == b_Selected)
   {
      // then do nothing
      return ;
   }
   
   m_b_Selected=b_Selected;

   if(m_b_Selected)
   {
      // make the widget red
      QColor      qc_Red;

      pal = palette();
      pal.setColor( QColorGroup::Background, QColor(255,209,236));

      setPalette( pal );
   }
   else
   {
      // make the widget normal

      pal = palette();
      pal.setColor( QColorGroup::Background, m_qc_OriginalColor );

      setPalette( pal );
   }
   
} // SetSelected()

//===========================================================================
//
//===========================================================================

bool QUSelectPixelButton::GetSelected()
{
   return m_b_Selected;
} // GetSelected()


//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::ButtonClicked()
{
   // simply pass the the external slot
   clicked();
} // ButtonClicked

//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::ButtonImageDropped(PixelBuffer *pc_Image)
{
   ImageDropped(pc_Image);
} // ButtonImageDropped()

//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::AddContextString(QString qs_String)
{
   m_pc_Button->AddContextString(qs_String);
} // AddContextString()

//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::ButtonContextMenuSelected(int i_Value)
{
   ContextMenuSelected(i_Value);
} // ButtonContextMenuSelected()

//===========================================================================
//
//===========================================================================

void QUSelectPixelButton::SetIcon(PixelBuffer *pc_Buffer)
{
   m_pc_Button->SetIcon(pc_Buffer);
} // SetIcon()

//===========================================================================
//
//===========================================================================

