//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QUTreePath.c
//---------------------------------------------------------------------------
// Purpose: 
//---------------------------------------------------------------------------


#include "QUTreePath.h"

#include <stdlib.h>
#include <qhgroupbox.h>
#include <qlayout.h>

#include "QtUtilities.h"
#include "ApplicationStorage.h"

//===========================================================================
// simple overloaded class to store an additional name
//===========================================================================

class FullnameListItem: public QListViewItem
{
   public:
   QString m_qs_Fullname;

   // constructors - just add the fullname
   FullnameListItem(QString qs_Name,QListView * pc_View,QString qs_ElementName)
         : QListViewItem(pc_View,qs_ElementName)
   {
      m_qs_Fullname = qs_Name;
   }
   
   FullnameListItem(QString qs_Name,QListViewItem * pc_Item,QString qs_ElementName)
         : QListViewItem(pc_Item,qs_ElementName)
   {
      m_qs_Fullname = qs_Name;
   }

};

//===========================================================================
//
//===========================================================================

QUTreePathEntry::QUTreePathEntry()
{
   m_b_IsDirectory  = false;

   m_qs_Elementname = "";
   m_qs_Fullname    = "";

} // QUTreePathEntry()

//===========================================================================
//
//===========================================================================


void QUTreePathEntry::AddPath(QString qs_Name,QString qs_Fullname)
{
   int i_FindIndex;

   // look for a slash
   i_FindIndex = qs_Name.find('/');
   if(i_FindIndex >0)
   {
      QString qs_DirectoryName;
      QString qs_RestName;
      int     i;
      int     i_Count;
      QUTreePathEntry *pc_CurrentEntry;
      bool b_Found;
      
      qs_DirectoryName = qs_Name.left(i_FindIndex);
      qs_RestName      = qs_Name.mid(i_FindIndex+1);
      
      // look up, whether directory already exists.
      i_Count = m_l_ChildList.GetCount();
      b_Found = false;
      for(i=0;i<i_Count;i++)
      {
         pc_CurrentEntry = m_l_ChildList.GetIndex(i);
         if(pc_CurrentEntry->m_qs_Elementname
                        .compare(qs_DirectoryName) == 0)
         {
            b_Found = true;
            pc_CurrentEntry->AddPath(qs_RestName,qs_Fullname);
            break;
         }
      }

      if(!b_Found)
      {
         // add a new directory
         QUTreePathEntry *pc_NewDirectory;

         pc_NewDirectory = new QUTreePathEntry();
         pc_NewDirectory->m_qs_Elementname =qs_DirectoryName;
         pc_NewDirectory->m_qs_Fullname    ="";
         pc_NewDirectory->m_b_IsDirectory = true;
         m_l_ChildList.AddTail(pc_NewDirectory);
         pc_NewDirectory->AddPath(qs_RestName,qs_Fullname);
      }
   }
   else // only a element
   {
      // add the element
      QUTreePathEntry *pc_NewEntry;

      pc_NewEntry = new QUTreePathEntry();
      pc_NewEntry->m_qs_Elementname    = qs_Name;
      pc_NewEntry->m_qs_Fullname       = qs_Fullname;
      pc_NewEntry->m_b_IsDirectory     = false;
      m_l_ChildList.AddTail(pc_NewEntry);

   } // else no '/'


} // AddPath()

//===========================================================================
//
//===========================================================================


void QUTreePathEntry::AddViewRecursive(QListViewItem *lv_Parent)
{
   int i;
   int i_Count;

   i_Count = m_l_ChildList.GetCount();

   for(i=0;i<i_Count; i++)
   {
      QUTreePathEntry * pc_Entry;
      FullnameListItem   * lv_Item;

      pc_Entry =  m_l_ChildList.GetIndex(i);

      lv_Item = new FullnameListItem(pc_Entry->m_qs_Fullname,
                                     lv_Parent,
                                     pc_Entry->m_qs_Elementname);
      if(pc_Entry->m_b_IsDirectory)
      {
         lv_Item->setSelectable(false);
         lv_Item->setExpandable(true);
         pc_Entry->AddViewRecursive(lv_Item);
      }
      else
      {

      }

   } // for i


} // AddViewRecursive()

//===========================================================================
//###########################################################################
//###########################################################################
//###########################################################################
//===========================================================================

QUTreePath::QUTreePath(QWidget *parent)
              : QListView(parent)
{
   addColumn("");
   setColumnWidthMode(0,QListView::Maximum);

 
   connect( this,
               SIGNAL(selectionChanged() ),
               SLOT( SelectionChanged() )
   );

} // QUTreePath() - constructor

//===========================================================================
//
//===========================================================================


QUTreePath::~QUTreePath()
{
} // ~QUTreepath()

//===========================================================================
//
//===========================================================================

void QUTreePath::AddPath(QString qs_Fullname)
{

   m_Root.AddPath(qs_Fullname,qs_Fullname);
   

} // AddPath()

//===========================================================================
//
//===========================================================================

void QUTreePath::ViewList(QString qs_RootName)
{
   FullnameListItem   * lv_Root;

   lv_Root = new FullnameListItem("",
                                  this,
                                  qs_RootName);
   lv_Root->setSelectable(false);
   m_Root.AddViewRecursive(lv_Root);
   setOpen(lv_Root,true);


} // ViewList()

//===========================================================================
//
//===========================================================================


void QUTreePath::SelectionChanged()
{

   FullnameListItem *pc_Item;

   pc_Item = (FullnameListItem * ) selectedItem();
   // fprintf(stderr,"item =  %x\n",pc_Item);
   if(pc_Item != NULL)
   {
      // fprintf(stderr,"name = %s\n",(const char *) pc_Item->m_qs_Fullname);
      
      m_qs_Selected = pc_Item->m_qs_Fullname;
   }
   else
   {
      m_qs_Selected = "";

   }

   SelectionHasChanged(m_qs_Selected);
   
} // SelectionChanged

//===========================================================================
//
//===========================================================================

QString QUTreePath::GetSelected()
{
   return m_qs_Selected;
} // GetSelected()

//===========================================================================
//
//===========================================================================



