//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QTUtilities
//---------------------------------------------------------------------------
// Purpose: small help classes for QT functions.
//---------------------------------------------------------------------------

#include "QtUtilities.h"
#include <qtooltip.h>
#include <qapplication.h>
#include <qcursor.h>


//===========================================================================
//
//===========================================================================

QtUtilSpacer::QtUtilSpacer(bool ExpandHorizontal,
                           bool ExpandVertical,
                           QWidget *Parent)
                  : QWidget(Parent)
{
   if(ExpandHorizontal || ! ExpandVertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Expanding,
                                 QSizePolicy::Minimum,
                                 false));
   }
   if(!ExpandHorizontal ||  ExpandVertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Minimum,
                                 QSizePolicy::Expanding,
                                 false));
   }

} // QtUtilSpacer()

//===========================================================================
//
//===========================================================================

QtUtilLine::QtUtilLine(bool Horizontal,
                           bool Vertical,
                           QWidget *Parent)
                  : QWidget(Parent)
{
   QPalette       p_Palette;
   QColor         aColor;
   
   if(Horizontal  )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Expanding,
                                 QSizePolicy::Minimum,
                                 false));
      setMinimumHeight(1);
      setMaximumHeight(1);
   }
   
   
   if(Vertical )
   {

      setSizePolicy(QSizePolicy( QSizePolicy::Minimum,
                                 QSizePolicy::Expanding,
                                 false));
      setMinimumWidth(1);
      setMaximumWidth(1);
   }

   // set the color to the normal foreground color
   aColor = colorGroup().foreground();
   p_Palette.setColor( QColorGroup::Background, aColor );
   p_Palette.setColor( QColorGroup::Button, aColor );

   setPalette( p_Palette );

} // QtUtilLine()

//===========================================================================
//
//===========================================================================

QtUtilReadOnlyLineEdit::QtUtilReadOnlyLineEdit(QWidget *Parent)
     :  QLineEdit(Parent)
{

   setReadOnly(true);

   QtUtility::SetWidgetReadOnlyColor(this);

} // QtUtilReadOnlyLineEdit

//===========================================================================
//
//===========================================================================

QtUtility::QtUtility()
{
   QPalette pal;
   QFont  font;
   QFontMetrics fm(font);

   m_i_MaxFontWidth  = fm.maxWidth();
   m_i_MaxFontHeight = fm.height();


   // black text
   pal.setColor( QColorGroup::Text, QColor(0,0,0) );
   // light red background
   pal.setColor( QColorGroup::Background, QColor(255,211,211) );

   // use this palette for tooltips
   QToolTip::setPalette( pal );

} // QtUtility()

//===========================================================================
//
//===========================================================================

int QtUtility::GetMaximalFontWidth()
{
   return m_i_MaxFontWidth;
} // GetMaximalFontWidth()

//===========================================================================
//
//===========================================================================

int QtUtility::GetMaximalFontHeight()
{
   return m_i_MaxFontHeight;
} // GetMaximalFontWidth()

//===========================================================================
//
//===========================================================================

void QtUtility::StartWaitingCursor()
{
   QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
} // StartWaitingCursor()

//===========================================================================
//
//===========================================================================

void QtUtility::EndWaitingCursor()
{
   QApplication::restoreOverrideCursor();

} // EndWaitingCursor()

//===========================================================================
//
//===========================================================================

QString QtUtility::GetDragSourceName(QByteArray *qba_Data,
                                    QString *pqs_Value)
{
   QString qs_Return;

   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if(    i_Size > 1
       && i_Size < 100 )
   {
      for(i=0; i< i_Size ; i++)
      {
         a_Character = qba_Data->at(i);
         if(a_Character== '=')
         {
            if(pqs_Value == NULL)
            {
               return qs_Return;
            }
            // skip the equal sign
            i++ ; 
            *pqs_Value ="";
            while(i< i_Size)
            {
               a_Character = qba_Data->at(i);
               *pqs_Value += a_Character;
               i++;
            }
            return qs_Return;
         }
         qs_Return += a_Character;
      }

   } // id size long enough

   return qs_Return;
} // GetDrawSourceName()

//===========================================================================
//
//===========================================================================

QString QtUtility::GetDragSourceText(QByteArray *qba_Data)
{
   QString qs_Return;
   char    a_Character;
   int     i;
   int     i_Size;

   qs_Return = "";

   i_Size = qba_Data->size();

   if( i_Size <= 0 )
   {
      return qs_Return;
   }

   for(i=0; i< i_Size ; i++)
   {
      a_Character = qba_Data->at(i);
      qs_Return  += a_Character;
   } // for i

   return qs_Return;
} // GetDrawSourceText()

//===========================================================================
//
//===========================================================================

void QtUtility::SetWidgetReadOnlyColor(QWidget *w_Widget)
{
   QColorGroup qcg_ColorGroup;
   QPalette    pal;
   QColor      qc_Background;
   qcg_ColorGroup = w_Widget->colorGroup();

   qc_Background = qcg_ColorGroup.background();

   pal = w_Widget->palette();
   pal.setColor( QColorGroup::Base, qc_Background );

   w_Widget->setPalette( pal );

} // SetWidgetReadOnlyColor()

//===========================================================================
//
//===========================================================================



