//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: RotationBox.c
//---------------------------------------------------------------------------
// Purpose: dialog for rotation of images
//---------------------------------------------------------------------------

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qbuttongroup.h>


#include "PixelBuffer.h"
#include "RotationBox.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "ApplicationStorage.h"
#include "ApplicationFrame.h"
#include "MainFrame.h"
#include "Selection.h"
#include "PixelBufferDistortion.h"

#define ROTATIONBOXLAYER "Rotation-Line"

//===========================================================================
//
//===========================================================================

RotationBox::RotationBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               StandardDialog(parent,pc_Global)
{
   QBoxLayout   *l_Layout;
   QHGroupBox   *RotateBox;
   QVGroupBox   *w_LineModeBox;
   QHGroupBox   *MirrorBox;
   int           i_CharWidth;

   LoadPosition("RotationBox");
   
   setCaption(tr("Rotation"));

   m_b_LineModeActive       = false;
   m_b_SecondLineModeActive = false;


   m_pc_LineLayer= m_pc_GlobalStorage->m_pc_ObjectMain
                             ->CreateLayer(ROTATIONBOXLAYER);
   m_pc_LineLayer->SetVisible(false);

   m_pc_LineObject = new GraphicObjectLine(10,10,20,20);
   m_pc_LineObject->SetColor( 0x5555ff);
   m_pc_LineObject->SetAllowsLineEndMovement(true);
   m_pc_LineLayer->AddObject(m_pc_LineObject);

   m_pc_SecondLineObject = new GraphicObjectLine(40,10,30,20);
   m_pc_SecondLineObject->SetColor( 0x5555ff);
   m_pc_SecondLineObject->SetAllowsLineEndMovement(true);
   m_pc_SecondLineObject->SetVisible(m_b_SecondLineModeActive);
   m_pc_LineLayer->AddObject(m_pc_SecondLineObject);

   i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   // rotate box
   RotateBox = new QHGroupBox(tr("Rotate"),m_w_TopWidget);
   {
      // 3 pixel buttons with the angels
      m_Rotate90Button  = new QUIconButton(RotateBox,"Rotate90","90");
      m_Rotate180Button = new QUIconButton(RotateBox,"Rotate180","180");
      m_Rotate270Button = new QUIconButton(RotateBox,"Rotate270","270");
      new QtUtilSpacer(true,false,RotateBox);

      // create the edit field
      m_RotateFreeAngleEdit = new QLineEdit(RotateBox);
      m_RotateFreeAngleEdit->setMaximumWidth(i_CharWidth*4);
      m_RotateFreeAngleButton = new QUIconButton(RotateBox,
                                                  "RotateFree",
                                                  tr("Free Angle"));

      m_RotateFreeAngleEdit->setText("0.0");

      connect( m_Rotate90Button ,
               SIGNAL(clicked()),
               SLOT( Rotate90() ) );
      connect( m_Rotate180Button,
               SIGNAL(clicked()),
               SLOT( Rotate180()) );
      connect( m_Rotate270Button,
               SIGNAL(clicked()),
               SLOT( Rotate270()) );
      connect( m_RotateFreeAngleButton,
               SIGNAL(clicked()),
               SLOT( RotateFree()) );

      
   }
   l_Layout->addWidget(RotateBox);

   // make a seperate bot for the line mode
   w_LineModeBox = new QVGroupBox(tr("Line Correction Rotation"),
                                   m_w_TopWidget);
   {
      m_LineActiveCheck = new QCheckBox(tr("Enable"),w_LineModeBox);
      m_RotateLineHorizontal = new QPushButton(tr("Rotate that line is horizontal"),
                                               w_LineModeBox);
      m_RotateLineVertical = new QPushButton(tr("Rotate that line is vertical"),
                                               w_LineModeBox);

      m_w_SecondLineModeBox=new QVGroupBox(tr("Geometric Correction"),
                                               w_LineModeBox);
      m_SecondLineActiveCheck = new QCheckBox(tr("Enable"),
                                              m_w_SecondLineModeBox);
                  
      m_AdjustLineHorizontal= new QPushButton(tr("Adjust that lines are horizontal"),
                                               m_w_SecondLineModeBox);
      m_AdjustLineVertical  = new QPushButton(tr("Adjust that lines are vertical"),
                                               m_w_SecondLineModeBox);

      connect( m_LineActiveCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( LineModeToggled(bool )) );
      connect( m_RotateLineHorizontal,
                  SIGNAL(clicked()),
                  SLOT( RotateToHorizontal()) );
      connect( m_RotateLineVertical,
                  SIGNAL(clicked()),
                  SLOT( RotateToVertical()) );

      connect( m_SecondLineActiveCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( SecondLineModeToggled(bool )) );
      connect( m_AdjustLineHorizontal,
                  SIGNAL(clicked()),
                  SLOT( AdjustToHorizontal()) );
      connect( m_AdjustLineVertical,
                  SIGNAL(clicked()),
                  SLOT( AdjustToVertical()) );

   }
   l_Layout->addWidget(w_LineModeBox);
      

   // mirror box
   MirrorBox = new QHGroupBox(tr("Mirror"),m_w_TopWidget);
   {
      // 2 buttons to mirror in 2 directions
      m_MirrorXButton  = new QUIconButton(MirrorBox,
                                           "MirrorX",
                                           tr("Left <-> Right"));
      m_MirrorYButton  = new QUIconButton(MirrorBox,
                                           "MirrorY",
                                           tr("Top <-> Buttom"));

      connect( m_MirrorXButton ,
               SIGNAL(clicked()),
               SLOT( MirrorX() ) );
      connect( m_MirrorYButton ,
               SIGNAL(clicked()),
               SLOT( MirrorY() ) );
   }
   l_Layout->addWidget(MirrorBox);

   // add the message receiver
   m_pc_GlobalStorage->m_pc_MessageServer
             ->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");

   m_pc_GlobalStorage->m_pc_MessageServer
              ->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");
   SetLineToDefaultValue();
   SetLineModeSensitivity();
   
} // RotationBox()

//===========================================================================
//
//===========================================================================

RotationBox::~RotationBox()
{

} // ~RotationBox()

//===========================================================================
//
//===========================================================================

void *RotationBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      // currently not used
   } // If ReasonSelection

   return NULL;
} // SelectionEventArrived()

//===========================================================================
//
//===========================================================================

void *RotationBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   RotationBox *pc_This = (RotationBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()


//===========================================================================
//
//===========================================================================

void RotationBox::Rotate90()
{
   RotateFixedAngle(90);

} // Rotate90()

//===========================================================================
//
//===========================================================================

void RotationBox::RotateFixedAngle(int i_Angle)
{
   QtUtility::StartWaitingCursor();
   m_pc_GlobalStorage->m_pc_MainImage->Rotate(i_Angle);
   m_pc_GlobalStorage->m_pc_ImageFrame->ChangeImageDimensions();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   QtUtility::EndWaitingCursor();

} // RotateFixedAngle()


//===========================================================================
//
//===========================================================================

void RotationBox::Rotate180()
{
   RotateFixedAngle(180);

} // Rotate180()

//===========================================================================
//
//===========================================================================

void RotationBox::Rotate270()
{
   RotateFixedAngle(270);

} // Rotate270()

//===========================================================================
//
//===========================================================================

void RotationBox::RotateFree()
{
   PixelBuffer *pc_NewImage;
   QString      qs_Text;
   double       d_Angle;
   bool         b_Ok;

   // get text out of edit field
   qs_Text = m_RotateFreeAngleEdit->text();
   if(qs_Text.length()==0)
   {
      return;
   }

   // make a double out of it
   d_Angle = qs_Text.toDouble(&b_Ok);

   if(!b_Ok)
   {
      return;
   }

   if(d_Angle ==0.0)
   {
      return;
   }
   
   // convert the angle from degree to radian
   d_Angle = ((M_PI * d_Angle)/180.0);

   // rotate image
   QtUtility::StartWaitingCursor();
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->Rotate(d_Angle);
   if(pc_NewImage ==NULL)
   {
      QtUtility::EndWaitingCursor();
      return;
   }

   // redisplay the image
   // replace old with new buffer
   m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   QtUtility::EndWaitingCursor();
} // RotateFree()

//===========================================================================
//
//===========================================================================

void RotationBox::MirrorX()
{
   QtUtility::StartWaitingCursor();
   m_pc_GlobalStorage->m_pc_MainImage->MirrorX();
   QtUtility::EndWaitingCursor();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // MirrorX()

//===========================================================================
//
//===========================================================================

void RotationBox::MirrorY()
{
   QtUtility::StartWaitingCursor();
   m_pc_GlobalStorage->m_pc_MainImage->MirrorY();
   QtUtility::EndWaitingCursor();
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // MirrorY()

//===========================================================================
//
//===========================================================================

void RotationBox::ReceivedNewImageSize()
{
   RectRegion c_FirstLineRegion;
   RectRegion c_SecondLineRegion;
   RectRegion c_ImageRegion;
   int        i_Width;
   int        i_Height;

   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   c_ImageRegion.SetXYWH(0,0,i_Width,i_Height);
   
   c_FirstLineRegion  = m_pc_LineObject->GetSize();
   c_SecondLineRegion = m_pc_SecondLineObject->GetSize();
   if(   c_ImageRegion.ContainsRegion(c_FirstLineRegion)
      && c_ImageRegion.ContainsRegion(c_FirstLineRegion))
   {
      // line are in the new image.
      // they must not get changed
      return;
   }

   SetLineToDefaultValue();
} // ReceivedNewImageSize()

//===========================================================================
//
//===========================================================================

void RotationBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
//fprintf(stderr,"Message Received member this = %x\n",this);
   if(qs_MessageType.compare("ImageResize") ==0)
   {
      ReceivedNewImageSize();
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
      ReceivedNewImageSize();
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================

void RotationBox::SetLineModeSensitivity()
{
   if( m_b_LineModeActive)
   {
      // enable the edit fields
      m_RotateLineHorizontal->setEnabled(true);
      m_RotateLineVertical  ->setEnabled(true);
      m_w_SecondLineModeBox ->setEnabled(true);
   }
   else // not active
   {
      // disable the edit fields
      m_RotateLineHorizontal->setEnabled(false);
      m_RotateLineVertical  ->setEnabled(false);
      m_w_SecondLineModeBox ->setEnabled(false);
   }
   
   if(m_b_SecondLineModeActive)
   {
      m_AdjustLineHorizontal->setEnabled(true);
      m_AdjustLineVertical  ->setEnabled(true);
   }
   else
   {
      m_AdjustLineHorizontal->setEnabled(false);
      m_AdjustLineVertical  ->setEnabled(false);
   }
} // SetLineModeSensitivity()

//===========================================================================
//
//===========================================================================

void RotationBox::LineModeToggled(bool b_Mode)
{
   // if mode has not changed
   if(m_b_LineModeActive == b_Mode)
   {
      return;
   }

   // remember the new mode
   m_b_LineModeActive = b_Mode;

   if( m_b_LineModeActive )
   {
      // make this layer visible
      m_pc_LineLayer->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      // set this layer as input layer
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(ROTATIONBOXLAYER);

      // enable active action
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput();

      // set the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
   }
   else // line mode not active
   {
      if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
         == m_pc_LineLayer )
      {
         // reset the callback for graphic object changes
         m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
         m_pc_LineLayer->SetVisible(false);

         // disable active action
         m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
      }
   }
   
   SetLineModeSensitivity();
} // LineModeToggled()

//===========================================================================
//
//===========================================================================

void RotationBox::SecondLineModeToggled(bool b_Mode)
{
   m_b_SecondLineModeActive = b_Mode;

   if(m_b_SecondLineModeActive)
   {
      m_pc_SecondLineObject->SetVisible(true);
   }
   else
   {
      m_pc_SecondLineObject->SetVisible(false);
   }   
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   SetLineModeSensitivity();
   
} // SecondLineModeToggled()

//===========================================================================
//
//===========================================================================

void RotationBox::GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // now redisplay the size of the graphic object
   if(    pc_CallbackData->i_CallbackType
       == GraphicObjectMainEvent::SelectionDisabled )
   {
      // disable the frame mode
      if( m_b_LineModeActive )
      {
         if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
            == m_pc_LineLayer )
         {
            // reset the callback for graphic object changes
            m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
            m_pc_LineLayer->SetVisible(false);

            // disable active action
            m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
         }
         m_LineActiveCheck->setChecked(false);
      }
      SetLineModeSensitivity();
   }
   else  // regular callback event
   {
      // nothing at the moment
   }
} // GraphicObjectCallback()

//===========================================================================
//
//===========================================================================

void *RotationBox::GraphicObjectCallbackCB(void *thisData,
                                           void *GraphicObjectCallbackData)
{
   RotationBox *pc_This;
   pc_This = (RotationBox *) thisData;

   pc_This-> GraphicObjectCallback(
                      (GraphicObjectMainEvent *)GraphicObjectCallbackData );

   return NULL;
} // GraphicObjectCallbackCB()

//===========================================================================
//
//===========================================================================

void RotationBox::hideEvent(QHideEvent *)
{
   // make the layer invisible
   m_LineActiveCheck->setChecked(false);

}  // hideEvent()

//===========================================================================
//  set frame to a default value, which is
// 1/4 -- 3/4 of the current image dimension
//===========================================================================

void RotationBox::SetLineToDefaultValue()
{
   RectRegion c_LineSize;
   RectRegion c_SecondLineSize;
   int        i_Width;
   int        i_Height;

   // get dimension of the new image
   i_Width = m_pc_GlobalStorage->m_pc_MainImage->GetWidth() ;
   i_Height= m_pc_GlobalStorage->m_pc_MainImage->GetHeight() ;

   c_LineSize.SetXYXY(i_Width   * 4/10,
                       i_Height * 2/10,
                       i_Width  * 5/10,
                       i_Height * 7/10);
   m_pc_LineObject->SetSize(c_LineSize);

   c_SecondLineSize.SetXYXY(i_Width   * 9/10,
                       i_Height * 2/10,
                       i_Width  * 8/10,
                       i_Height * 8/10);
   m_pc_SecondLineObject->SetSize(c_SecondLineSize);
   // if visible
   if(m_LineActiveCheck)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
   }

} // SetLineToDefaultValue()

//===========================================================================
//
//===========================================================================

void RotationBox::RotateToHorizontal()
{
   PixelBuffer *pc_NewImage;
   double       d_Angle;

   d_Angle = CalculateLineAngle();
         
   // rotate image
   QtUtility::StartWaitingCursor();
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->Rotate(d_Angle);
   QtUtility::EndWaitingCursor();
   if(pc_NewImage ==NULL)
   {
      return;
   }

   // redisplay the image
   // replace old with new buffer
   m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
}  // RotateToHorizontal()

//===========================================================================
//
//===========================================================================

void RotationBox::RotateToVertical()
{
   PixelBuffer *pc_NewImage;
   double       d_Angle;

   d_Angle = CalculateLineAngle();

   // add 90 degrees, to be vertical
   d_Angle -= 1 * M_PI_2;


   // do not turn the image more than +- 90 deg
   if(   d_Angle > M_PI_2
      && d_Angle < 3*M_PI_2)
   {
      d_Angle += M_PI;
   }
      
   // normate angle to 0..2Pi
   if(d_Angle <0 )
   {
      d_Angle += 2*M_PI;
   }
   else if(d_Angle > 2*M_PI )
   {
      d_Angle -= 2*M_PI;
   }

   // rotate image
   QtUtility::StartWaitingCursor();
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->Rotate(d_Angle);
   QtUtility::EndWaitingCursor();
   if(pc_NewImage ==NULL)
   {
      return;
   }

   // redisplay the image
   // replace old with new buffer
   m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewImage);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // RotateToVertical()

//===========================================================================
// use mathematical angle system
//===========================================================================

double RotationBox::CalculateLineAngle()
{
   GeoCoordinate c_StartPoint;
   GeoCoordinate c_EndPoint;
   double        d_Angle;

   c_StartPoint = m_pc_LineObject->GetStartPoint();
   c_EndPoint   = m_pc_LineObject->GetEndPoint();

   if(c_StartPoint.GetX() == c_EndPoint.GetX())
   {
      // vertical line -- is pi /2;
      return M_PI_2;
   }
   if(c_StartPoint.GetY() == c_EndPoint.GetY())
   {
      // horizontal line -- is 0;
      return 0.0;
   }

   // start should be more left than end
   if(c_StartPoint.GetX() > c_EndPoint.GetX())
   {
      // swap the values
      GeoCoordinate c_Temp;

      c_Temp       = c_StartPoint;
      c_StartPoint = c_EndPoint;
      c_EndPoint   = c_Temp;
   }

   d_Angle = atan2(c_StartPoint.GetY() - c_EndPoint.GetY(),
                   c_EndPoint.GetX() - c_StartPoint.GetX());

   // let angle be positive
   if(d_Angle <0)
   {
      d_Angle += 2*M_PI;
   }
                      
   return d_Angle;
   
} // CalculateLineAngle()

//===========================================================================
//
//===========================================================================

void RotationBox::AdjustToHorizontal()
{

   PixelBufferDistortion  c_Distort;
   PixelBuffer           *pc_NewBuffer;

   GeoLine                c_FirstLine;
   GeoLine                c_SecondLine;


   // get the both lines
   c_FirstLine  = m_pc_LineObject->GetLine();
   c_SecondLine = m_pc_SecondLineObject->GetLine();

   // perform the operation
   QtUtility::StartWaitingCursor();
   pc_NewBuffer = c_Distort.AdjustLinesToHorizontal(m_pc_GlobalStorage->m_pc_MainImage,
                              c_FirstLine,c_SecondLine);
   QtUtility::EndWaitingCursor();

   // do we have a new buffer
   if(pc_NewBuffer != NULL)
   {
      // redisplay the image
      // replace old with new buffer
      m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewBuffer);

      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   }

} // AdjustToHorizontal()

//===========================================================================
//
//===========================================================================

void RotationBox::AdjustToVertical()
{
   PixelBufferDistortion  c_Distort;
   PixelBuffer           *pc_NewBuffer;

   GeoLine                c_FirstLine;
   GeoLine                c_SecondLine;


   // get the both lines
   c_FirstLine  = m_pc_LineObject->GetLine();
   c_SecondLine = m_pc_SecondLineObject->GetLine();

   // perform the operation
   QtUtility::StartWaitingCursor();
   pc_NewBuffer = c_Distort.AdjustLinesToVertical(m_pc_GlobalStorage->m_pc_MainImage,
                              c_FirstLine,c_SecondLine);

   QtUtility::EndWaitingCursor();
   // do we have a new buffer
   if(pc_NewBuffer != NULL)
   {
      // redisplay the image
      // replace old with new buffer
      m_pc_GlobalStorage->m_pc_ImageFrame->ReplacePixelbuffer(pc_NewBuffer);

      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   }   

}  // AdjustToVertical()

//===========================================================================
//
//===========================================================================

int RotationBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"RotationBox");

   m_pc_LineLayer->WriteFile(pc_FileWriter,"ObjectLayer");

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int RotationBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(    qs_Name.compare("ObjectLayer") == 0
       && qs_Value.compare("GraphicObjectLayer") == 0
       && b_IsStartStructure)
   {
      // delete old objects
      m_pc_LineLayer->DeleteAllObjects();
      m_pc_LineObject       = NULL;
      m_pc_SecondLineObject = NULL;

      // load objects from a file
      m_pc_LineLayer->LoadFile(pc_FileWriter);

      m_pc_LineObject       = (GraphicObjectLine *) m_pc_LineLayer->GetObject(0);
      m_pc_SecondLineObject = (GraphicObjectLine *) m_pc_LineLayer->GetObject(1);

   }

   PERICO_READ_STRUCTURE_END

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

