#ifndef _ROTATIONBOX_H
#define _ROTATIONBOX_H

#include <qdialog.h>

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qradiobutton.h>

#include <qvgroupbox.h>

#include "StandardDialog.h"
#include "QUPixelButton.h"
#include "GraphicObjectLayer.h"
#include "GraphicObjectMain.h"
#include "QUIconButton.h"


class RotationBox :  public StandardDialog
{
Q_OBJECT

public:
   RotationBox(QWidget *parent=0,
      GlobalStorage *pc_Global=0);

   ~RotationBox();
   
   int WriteFile(PericoFileWriter *pc_FileWriter,QString qs_Name);
   int LoadFile(PericoFileWriter *pc_FileWriter);

      
public slots:

   void Rotate90();
   void Rotate180();
   void Rotate270();
   void RotateFree();
   void MirrorX();
   void MirrorY();
   void LineModeToggled(bool );
   void SecondLineModeToggled(bool );

   void RotateToHorizontal();
   void RotateToVertical();
   void AdjustToHorizontal();
   void AdjustToVertical();

protected:
   virtual void hideEvent(QHideEvent *);

private:

   void ReceivedNewImageSize();
   void RotateFixedAngle(int i_Angle);

   void *SelectionEventArrived(void *pc_Data);

   static void *SelectionEventArrivedCB(void *Thisdata,void *pc_Data);

   void SetLineModeSensitivity();
//   void SetFrameIntoValidImage();
   void SetLineToDefaultValue();


   void GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData);
   static void *GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData);

   double CalculateLineAngle();
private:
    virtual void OnMessageReceive(QString qs_MessageType,
                                 void *p_Data);

   QUIconButton *m_Rotate90Button;
   QUIconButton *m_Rotate180Button;
   QUIconButton *m_Rotate270Button;

   QLineEdit    *m_RotateFreeAngleEdit;
   QUIconButton *m_RotateFreeAngleButton;

   QButton      *m_RotateLineHorizontal;
   QButton      *m_RotateLineVertical;

   QButton      *m_AdjustLineHorizontal;
   QButton      *m_AdjustLineVertical;

   QUIconButton *m_MirrorXButton;
   QUIconButton *m_MirrorYButton;

   QCheckBox  *m_LineActiveCheck;
   QCheckBox  *m_SecondLineActiveCheck;

   bool m_b_LineModeActive;
   bool m_b_SecondLineModeActive;
   
   GraphicObjectLayer     *m_pc_LineLayer;
   GraphicObjectLine      *m_pc_LineObject;
   GraphicObjectLine      *m_pc_SecondLineObject;
   bool m_b_InUpdate;

  QVGroupBox              *m_w_SecondLineModeBox;


};


#endif // _ROTATIONBOX_H
