//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: SettingsBox.c
//---------------------------------------------------------------------------
// Purpose: doalog to change base settings of the program
//---------------------------------------------------------------------------

#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qfiledialog.h>

#include "SettingsBox.h"
#include "QtUtilities.h"
#include "ApplicationStorage.h"
#include "BaseData.h"
#include "BaseDataStrings.h"

//===========================================================================
//
//===========================================================================

SettingsBox::SettingsBox(QWidget *parent ):
                        StandardDialog(parent,NULL)
{
   QBoxLayout   *l_Layout;
   QWidget      *w_ButtonWidget;

   setCaption(tr("Settings"));
   l_Layout     = new  QVBoxLayout( m_w_TopWidget,1);

   m_TabWidget = new QTabWidget(m_w_TopWidget);
   l_Layout->addWidget(m_TabWidget);

   // widget for NLS settings
   m_NLSSettings = new QWidget(this);
   {
      QHBoxLayout  *l_NLSLayout;
      l_NLSLayout  = new  QHBoxLayout( m_NLSSettings,2);
      
      m_w_NLSFile = new QUFileSelectEdit(m_NLSSettings,
               QObject::tr("Language File"),
               "",
               QObject::tr("Perico: Get Filename for Language File"),
               "QM (*.qm )" ,
               true
               );

      l_NLSLayout->addWidget( m_w_NLSFile);
      
   } // nls block

   // widget for image file settings
   m_FileSettings = new QWidget(this);
   {
      QVBoxLayout  *l_FileLayout;

      l_FileLayout  = new  QVBoxLayout( m_FileSettings,2);

      m_w_DefaultDirectory  = new QUFileSelectEdit(m_FileSettings,
               QObject::tr("Image File Directory"),
               "",
               QObject::tr("Perico: Get Directory for Images"),
               "",
               false
               );

      
      l_FileLayout->addWidget( m_w_DefaultDirectory);

      m_w_JpegQualitySlider = new QULabelSlider(m_FileSettings,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    50,99,
                                    90,
                                    tr("Jpeg Quality")
                                    );
      l_FileLayout->addWidget( m_w_JpegQualitySlider);
   } // jpeg file settings block
   
   // widget for Temp file settings
   m_w_TempSettings = new QWidget(this);
   {
      QVBoxLayout  *l_TempLayout;
      l_TempLayout  = new  QVBoxLayout( m_w_TempSettings,2);

      m_w_TempDirectory = new QUFileSelectEdit(m_w_TempSettings,
               QObject::tr("Temporary Files Directory"),
               "",
               QObject::tr("Perico: Get Directory for temporary Files"),
               "",
               false
               );
      l_TempLayout->addWidget( m_w_TempDirectory);
      
   } // tempdir block

   // widget for m_w_AutosaveSettings file settings
   m_w_AutosaveSettings = new QWidget(this);
   {
      QVBoxLayout  *AutosaveLayout;

      AutosaveLayout  = new  QVBoxLayout( m_w_AutosaveSettings,2);

      m_w_AutosaveEnabledCheck = new QCheckBox(tr("Enabled") ,
                                       m_w_AutosaveSettings);
      AutosaveLayout->addWidget( m_w_AutosaveEnabledCheck);

      m_w_AutosaveNumberOfImagesSlider = new QULabelSlider(m_w_AutosaveSettings,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    1,5,
                                    2,
                                    tr("Number of Images"),
                                    20
                                    );
      AutosaveLayout->addWidget( m_w_AutosaveNumberOfImagesSlider);

      m_w_AutosaveIntervalSlider = new QULabelSlider(m_w_AutosaveSettings,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    1,30,
                                    5,
                                    tr("Interval between Saving (minutes)"),
                                    20
                                    );
      AutosaveLayout->addWidget( m_w_AutosaveIntervalSlider);

      m_w_AutosaveTellAtStartup = new QCheckBox(tr("Tell at statup about autosaved images") ,
                                       m_w_AutosaveSettings);
      AutosaveLayout->addWidget( m_w_AutosaveTellAtStartup);

   }  // autosave block

   
   m_TabWidget->addTab(m_NLSSettings,tr("Language"));
   m_TabWidget->addTab(m_FileSettings,tr("File"));
   m_TabWidget->addTab(m_w_TempSettings,tr("Temporary Files"));
   m_TabWidget->addTab(m_w_AutosaveSettings,tr("Automatic Save"));


   // now the apply and the reset button
   w_ButtonWidget = new QWidget(m_w_TopWidget);
   {
      QHBoxLayout  *ButtonLayout;
      ButtonLayout  = new  QHBoxLayout( w_ButtonWidget,2);

      m_w_ApplyButton = new QPushButton(tr("Apply"),w_ButtonWidget);
      m_w_ResetButton = new QPushButton(tr("Reset"),w_ButtonWidget);

      ButtonLayout->addWidget(new QtUtilSpacer(true,false,w_ButtonWidget));
      ButtonLayout->addWidget(m_w_ApplyButton);
      ButtonLayout->addWidget(m_w_ResetButton);
   }
   l_Layout->addWidget(w_ButtonWidget);

   // connect the button with the slots   
   connect( m_w_ApplyButton,
               SIGNAL(clicked()),
               SLOT(ApplyPressed()) );
   
   connect( m_w_ResetButton,
               SIGNAL(clicked()),
               SLOT(ResetPressed()) );

   // load default values
   LoadDefaults();               

} // SettingsBox()

//===========================================================================
//
//===========================================================================

void SettingsBox::ApplyPressed()
{
   QString      qs_CurrentEditText;
   bool         b_Success;
   bool         b_Value;
   int          i_SliderValue;
   

   // no base data available
   if(g_Application->m_pc_BaseData== NULL)
   {
      // then do nothing
      return;
   }

   
   // get the current text
   qs_CurrentEditText = m_w_NLSFile->GetFile();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_LANGUAGE_FILENAME,
                                  qs_CurrentEditText);

   qs_CurrentEditText = m_w_DefaultDirectory->GetFile();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_IMAGE_FILE_DIRECTORY,
                                  qs_CurrentEditText);

   i_SliderValue = m_w_JpegQualitySlider->GetValue();

   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_JPEG_QUALITY,
                                  i_SliderValue);

   qs_CurrentEditText = m_w_TempDirectory->GetFile();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_TEMP_DIRECTORY,
                                  qs_CurrentEditText);


   b_Value = m_w_AutosaveEnabledCheck->isChecked();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_AUTOSAVE_ENABLED,
                                  b_Value);
   i_SliderValue = m_w_AutosaveIntervalSlider->GetValue();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_AUTOSAVE_INTERVAL,
                                  i_SliderValue);
   i_SliderValue = m_w_AutosaveNumberOfImagesSlider->GetValue();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_AUTOSAVE_NUMBER_OF_IMAGES,
                                  i_SliderValue);
   b_Value = m_w_AutosaveTellAtStartup->isChecked();
   b_Success = g_Application->m_pc_BaseData->SetValue(
                                  BASEDATA_AUTOSAVE_TELL_AT_STARTUP,
                                  b_Value);

   // now write the new values into the basedata file                                  
   g_Application->m_pc_BaseData->SaveFile();   
   
} // ApplyPressed();

//===========================================================================
//
//===========================================================================

void SettingsBox::ResetPressed()
{
   LoadDefaults();
} // ResetPressed()

//===========================================================================
//
//===========================================================================

void SettingsBox::LoadDefaults()
{
   QString qs_LanguageFile;
   QString qs_ImageFileDirectory;
   QString qs_TempDirectory;
   bool    b_Success;
   bool    b_AutosaveEnabled;
   bool    b_AutosaveTellAtStartup;
   int     i_JpegQuality;
   int     i_AutosaveInterval;
   int     i_AutosaveNumber;
   

   // first set default values
   qs_LanguageFile         = "";
   qs_ImageFileDirectory   = "";
   i_JpegQuality           = 97;
   qs_TempDirectory        = "";
   b_AutosaveEnabled       = true;
   i_AutosaveInterval      = 5;
   i_AutosaveNumber        = 2;
   b_AutosaveTellAtStartup = true;


   if(g_Application->m_pc_BaseData != NULL)
   {
      // get the language filename
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_LANGUAGE_FILENAME,
                                  qs_LanguageFile);
      // get the image directory
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_IMAGE_FILE_DIRECTORY,
                                  qs_ImageFileDirectory);
      // get the default jpeg quality
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_JPEG_QUALITY,
                                  i_JpegQuality);

      // get the temp directory
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_TEMP_DIRECTORY,
                                  qs_TempDirectory);
      // get the autosave enabled
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_ENABLED,
                                  b_AutosaveEnabled);
      // get the autosave interval
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_INTERVAL,
                                  i_AutosaveInterval);
      // get the autosave number of images
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_NUMBER_OF_IMAGES,
                                  i_AutosaveNumber);
      // get the autosave number of images
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_TELL_AT_STARTUP,
                                  b_AutosaveTellAtStartup);
   }


   // now set the values into the fields;

   m_w_NLSFile->SetFile(qs_LanguageFile);
   m_w_DefaultDirectory->SetFile(qs_ImageFileDirectory);
   m_w_JpegQualitySlider->SetValue(i_JpegQuality);
   m_w_TempDirectory->SetFile(qs_TempDirectory);

   m_w_AutosaveEnabledCheck->setChecked(b_AutosaveEnabled);
   m_w_AutosaveIntervalSlider->SetValue( i_AutosaveInterval);
   m_w_AutosaveNumberOfImagesSlider->SetValue(i_AutosaveNumber);
   m_w_AutosaveTellAtStartup->setChecked(b_AutosaveTellAtStartup);
   
} // LoadDefaults()

//===========================================================================
//
//===========================================================================

