//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: SnapshotBox.c
//---------------------------------------------------------------------------
// Purpose:
//---------------------------------------------------------------------------

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qfiledialog.h>

#include "SnapshotBox.h"
#include "ApplicationStorage.h"
#include "BaseData.h"
#include "BaseDataStrings.h"
#include "PixelBuffer.h"
#include "FileLoader.h"
#include "QtUtilities.h"
#include "MainFrame.h"
#include "IconLibrary.h"



//===========================================================================
//
//===========================================================================

SnapshotBox::SnapshotBox(QWidget *parent ):
                        StandardDialog(parent,NULL)
{
   QBoxLayout   *l_Layout;
   QString       qs_Directory;
   QString       qs_LastChar;

   setCaption(tr("Snapshots"));
   m_b_AllSnapshots = false;
   m_qs_ImageName   = IMAGEINFO_DEFAULT;

   l_Layout     = new  QVBoxLayout( m_w_TopWidget,1);

   // checkbox for all or only to the current image
   m_w_AllSnapshotsCheck = new QCheckBox(tr("Show all Snapshots"),
                                     m_w_TopWidget);
   m_w_AllSnapshotsCheck->setChecked(m_b_AllSnapshots);
   l_Layout->addWidget(m_w_AllSnapshotsCheck);

   // file icon selection box
   m_pc_FileIconBox  = new QUFileIconWidget(m_w_TopWidget);
   l_Layout->addWidget(m_pc_FileIconBox);

   
   connect( m_w_AllSnapshotsCheck,
               SIGNAL(toggled(bool)),
               SLOT(AllSnapshotsChecked(bool )) );

   connect( m_pc_FileIconBox,
               SIGNAL(IconSelected(QString )),
               SLOT(IconSelected(QString )) );

   // calculate the temp directory
   qs_Directory = g_Application->m_qs_TempDirectory;

   // add a slash at the end
   qs_LastChar = qs_Directory.right(1);
   if(qs_LastChar.compare("/") != 0)
   {
      qs_Directory+= "/" ;
   }

   qs_Directory +="Snapshot";

   m_pc_FileIconBox->SetPath(qs_Directory);
   m_pc_FileIconBox->SetWildcard(m_qs_ImageName);

   m_pc_FileIconBox->ActicateSelection();

   SetTitle();
   
} // SnapshotBox()

//===========================================================================
//
//===========================================================================

SnapshotBox::~SnapshotBox( )
{
} // ~AutosaveBox()

//===========================================================================
//
//===========================================================================

void SnapshotBox::IconSelected(QString qs_Filename)
{
   MainFrame::NewPicture(qs_Filename);

} // IconSelected()

//===========================================================================
//
//===========================================================================

void  SnapshotBox::SetImageName(QString qs_Name)
{
   m_qs_ImageName = qs_Name;

   ActivateWildcard();
   
} // SetImageName()

//===========================================================================
//
//===========================================================================

void SnapshotBox::ActivateWildcard()
{
   QString qs_Wildcard;

   if(m_b_AllSnapshots)
   {
      qs_Wildcard = "*";
   }
   else
   {
      qs_Wildcard = m_qs_ImageName+"*";

   }
   
   m_pc_FileIconBox->SetWildcard(qs_Wildcard);

   // load files for the changed wildcard
   m_pc_FileIconBox->ActicateSelection();

   // adjust the box title   
   SetTitle();
} // ActivateWildcard()

//===========================================================================
//
//===========================================================================

void SnapshotBox::AllSnapshotsChecked(bool b_On)
{
   m_b_AllSnapshots = b_On;

   // it may take longer, so show a waiting cursor
   QtUtility::StartWaitingCursor();

   ActivateWildcard();

   QtUtility::EndWaitingCursor();

} // AllSnapshotsChecked()

//===========================================================================
//
//===========================================================================

void SnapshotBox::SetTitle()
{
   QString qs_What;

   // calculate the title string
   if(m_b_AllSnapshots)
   {
      qs_What = tr("All Snapshots");
   }
   else
   {
      qs_What = tr("Snapshots for")+ " " + m_qs_ImageName;
   }

   setCaption(qs_What);

} // SetTitle()

//===========================================================================
//
//===========================================================================

void SnapshotBox::Update()
{
   // it may take longer, so show a waiting cursor
   QtUtility::StartWaitingCursor();

   // load files for the changed wildcard
   m_pc_FileIconBox->ActicateSelection();

   // adjust the box title
   SetTitle();

   QtUtility::EndWaitingCursor();

} // Update()

//===========================================================================
//
//===========================================================================

