//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: StandardDialog.c
//---------------------------------------------------------------------------
// Purpose: default base class for dialogs
//---------------------------------------------------------------------------


#include "StandardDialog.h"
#include "QtUtilities.h"
#include <qmainwindow.h>
#include <qapplication.h>

#include "ApplicationStorage.h"
#include "BaseData.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

StandardDialog::StandardDialog( QWidget *parent,
                GlobalStorage *pc_Global,
                unsigned int ui_DialogOptions)
            : QDialog(parent,"StandardDialog",false)

{

   QBoxLayout   *l_DialogButtons;

   m_pc_GlobalStorage = pc_Global;
   m_qs_DialogName    = "";
   m_ui_DialogOptions = ui_DialogOptions;

   // create the box
   m_l_TopLayout = new QVBoxLayout( this,1);

   m_w_TopWidget = new QWidget(this);
   m_l_TopLayout->addWidget(m_w_TopWidget);


   // --- the close button
   l_DialogButtons = new QHBoxLayout( m_l_TopLayout,5);

   if(m_ui_DialogOptions & DialogHasNoCloseButton)
   {
      m_w_CloseButton = NULL;

   }
   else
   {
      m_w_CloseButton = new QPushButton( tr("Close"), this );
      l_DialogButtons->addWidget(new QtUtilSpacer(true,false,this) );
      l_DialogButtons->addWidget(m_w_CloseButton);
      connect( m_w_CloseButton, 
            SIGNAL(clicked()), 
            SLOT( ClosePressed()) );
   }


} // StandardDialog()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

StandardDialog::~StandardDialog()
{
   SavePosition();
} // ~StandardDialog()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void StandardDialog::ClosePressed()
{

   this->hide();
} // ClosePressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void StandardDialog::OnMessageReceiveCB(void *p_ThisData,
                                 QString qs_MessageType,
                                 void *p_Data)
{
   StandardDialog *p_This;

   p_This =(StandardDialog *) p_ThisData;
   p_This->OnMessageReceive(qs_MessageType,p_Data);
}

void StandardDialog::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   // virtual function, will get overloaded
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void StandardDialog::ResizeDisplay()
{


   resize(QSize(1,1));

   m_pc_GlobalStorage->mainApplication->flushX();
   // for QT2 I have also to use this call
   m_pc_GlobalStorage->mainApplication->sendPostedEvents();
   //   m_pc_Global->mainApplication->processEvents();

   resize(QSize(1,1));


} // ResizeDisplay()

//===========================================================================
//
//===========================================================================

void StandardDialog::LoadPosition(QString qs_Name)
{
   QString qs_QuestionName;
   QString qs_Value;
   bool    b_Success;
   int     i_X;
   int     i_Y;
   
   if(qs_Name.isEmpty())
   {
      return;
   }
   m_qs_DialogName = qs_Name;
   if(g_Application->m_pc_BaseData== NULL)
   {
      return;
   }

   qs_QuestionName = m_qs_DialogName+ ".Position";

   b_Success = g_Application->m_pc_BaseData->GetValue(
            qs_QuestionName, qs_Value);
   if(b_Success)
   {
      int i_ScanfResult;
      i_ScanfResult = sscanf((const char *) qs_Value,
                                 "%d,%d",
                                 &i_X,&i_Y);
      if(i_ScanfResult ==2)
      {
         // move the widget to the position;
         move(i_X,i_Y);
         
      } // if scan success
                  
   } // if value found
} // LoadPosition()

//===========================================================================
//
//===========================================================================

void StandardDialog::SavePosition()
{
   QString qs_QuestionName;
   QString qs_Value;
   bool    b_Success;

   if(m_qs_DialogName.isEmpty())
   {
      return;
   }
   qs_QuestionName = m_qs_DialogName+ ".Position";

   qs_Value.sprintf("%d,%d",x(),y());
   
   b_Success = g_Application->m_pc_BaseData->SetValue(
            qs_QuestionName, qs_Value);
   
} // SavePosition()

//===========================================================================
//
//===========================================================================

