//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: XenoHandler.c
//---------------------------------------------------------------------------
// Purpose: interface between perico datastructures and xenomorph
//---------------------------------------------------------------------------

#include <stdio.h>
#include <sys/stat.h>

#include "XenoHandler.h"

#include <qstring.h>
#include <qdir.h>
#include <qpopupmenu.h>
#include "ApplicationStorage.h"

//===========================================================================
//
//===========================================================================

XenoHandler::XenoHandler()
{
   m_pc_RedInBuffer    = NULL;
   m_pc_GreenInBuffer  = NULL;
   m_pc_BlueInBuffer   = NULL;

   m_pc_RedOutBuffer   = NULL;
   m_pc_GreenOutBuffer = NULL;
   m_pc_BlueOutBuffer  = NULL;

   LoadKernels();
   
} // XenoHandler -- Constructor

//===========================================================================
//
//===========================================================================

XenoHandler::~XenoHandler()
{

} // ~XenoHandler()

//===========================================================================
//
//===========================================================================

void XenoHandler::LoadKernels()
{
   // first look, whether the homedir does exists.
   struct stat s_StatBuffer;
   int i_Result;
   QString qs_Path;
   QString qs_Kernelpath;

   
   qs_Path = g_Application->GetTempDir();
   
   qs_Kernelpath = qs_Path + "/.KERNELS";

   // fprintf(stderr,"Kernelpath = %s\n",(const char *) qs_Kernelpath);

   i_Result = stat(qs_Kernelpath,&s_StatBuffer);
   if(i_Result <0)
   {
      // error, does not exist
      ExtractKernels(qs_Path);
   }

   m_qs_BasePath =qs_Path + "/.KERNELS/";
   m_i_BasePathLength = m_qs_BasePath.length();
   ScanDirectory(m_qs_BasePath);

   m_c_Kernelmanager.resolveAll();

} // LoadKernels()

//===========================================================================
//
//===========================================================================

void XenoHandler::ScanDirectory(QString rootpath)
{
   // Scan a directory for filters, read them and extend the filter menu
   QDir d_Directory(rootpath);
   
   if( d_Directory.exists() )
   {
      d_Directory.setFilter( QDir::Dirs | QDir::Files | QDir::NoSymLinks );
      d_Directory.setSorting( QDir::Name );

      const QFileInfoList *list = d_Directory.entryInfoList();
      QFileInfoListIterator it_FileIterator(*list);
      QFileInfoListIterator it_FileIterator2(*list);
      QFileInfo *fi_FileInfo;

      
      while ((fi_FileInfo=it_FileIterator.current())!=0)
      {
         if (fi_FileInfo->isDir())
         {
            if (   (fi_FileInfo->fileName()!="..")
                && (fi_FileInfo->fileName()!="." ) )
            {

               // recursive call
               ScanDirectory(fi_FileInfo->filePath().latin1());
            }
         }
         ++it_FileIterator;
      }

      while ((fi_FileInfo=it_FileIterator2.current())!=0)
      {
         if (!fi_FileInfo->isDir())
         {
            XenoKERNEL2D *pc_NewKernal;
            QString qs_KernalName;
            QString qs_KernelWithoutBasePath;
            // create a new kernal

            // load the kernal

            // fprintf(stderr,"Load kernel %s\n", (char*)fi_FileInfo->filePath().ascii() );
            qs_KernalName =   fi_FileInfo->filePath().ascii();

            pc_NewKernal = new XenoKERNEL2D(qs_KernalName,
                                            &m_c_Kernelmanager);
                                            
            pc_NewKernal->load( (const char*)qs_KernalName );

            // add kernal into the names list
            qs_KernelWithoutBasePath  = qs_KernalName.mid(m_i_BasePathLength);
            // fprintf(stderr,"<%s> <%s> <%s>\n",
            //       (const char * ) rootpath,
            //       (const char * ) qs_KernalName,
            //       (const char * ) qs_KernelWithoutBasePath
            //       );
            m_qsl_KernelNameList.push_back(qs_KernelWithoutBasePath);

         }
         ++it_FileIterator2;
      }
   }
} // scanDirectory()


//===========================================================================
//
//===========================================================================

QStringList * XenoHandler::GetKernalNames()
{
   return  &m_qsl_KernelNameList;
} // GetKernalName()

//===========================================================================
//
//===========================================================================

void XenoHandler::ProcessImage(PixelBuffer *pc_Buffer,
               QString qs_KernelName)
{
   QString qs_FullKernalName;
   if(   qs_KernelName.isEmpty()
      || pc_Buffer == NULL       )
   {
      return;
   }

   qs_FullKernalName = m_qs_BasePath + qs_KernelName;
      
   ConvertToDsp(pc_Buffer);

   StartFilter(qs_FullKernalName);
   

   ConvertFromDsp(pc_Buffer);

    m_c_Kernelmanager.clearCaches();
} // ProcessImage

//===========================================================================
//
//===========================================================================

void XenoHandler::ConvertToDsp(PixelBuffer *pc_Buffer)
{
   Pixel p_Pixel;
   int i_X;
   int i_Y;

   pc_Buffer->GetWidthHeight(m_i_Width,m_i_Height);

   m_pc_RedInBuffer   = new XenoDSP2D(m_i_Width,m_i_Height);
   m_pc_GreenInBuffer = new XenoDSP2D(m_i_Width,m_i_Height);
   m_pc_BlueInBuffer  = new XenoDSP2D(m_i_Width,m_i_Height);
   
   m_pc_RedOutBuffer   = new XenoDSP2D(m_i_Width,m_i_Height);
   m_pc_GreenOutBuffer = new XenoDSP2D(m_i_Width,m_i_Height);
   m_pc_BlueOutBuffer  = new XenoDSP2D(m_i_Width,m_i_Height);

   // now convert all pixels
   for(i_Y=0;i_Y<m_i_Height;i_Y++)
   {
      for(i_X=0; i_X <m_i_Width;i_X ++)
      {
         double d_Red;
         double d_Green;
         double d_Blue;
         p_Pixel = pc_Buffer->Get(i_X,i_Y);

         // make a double 0.0 ... 1.0
         d_Red   = PIXEL_RED(p_Pixel)   / 255.0;
         d_Green = PIXEL_GREEN(p_Pixel) / 255.0;
         d_Blue  = PIXEL_BLUE(p_Pixel)  / 255.0;

         // put it into the seperate buffers
         m_pc_RedInBuffer  ->set(i_X,i_Y,d_Red);
         m_pc_GreenInBuffer->set(i_X,i_Y,d_Green);
         m_pc_BlueInBuffer ->set(i_X,i_Y,d_Blue);
      } // for x
   } // for y
   
} // ConvertToDsp()

//===========================================================================
//
//===========================================================================


void XenoHandler::ConvertFromDsp(PixelBuffer *pc_Buffer)
{
   Pixel p_Pixel;
   int i_X;
   int i_Y;

   // now convert all pixels
   for(i_Y=0;i_Y<m_i_Height;i_Y++)
   {
      for(i_X=0; i_X <m_i_Width;i_X ++)
      {
         double d_Red;
         double d_Green;
         double d_Blue;

         int i_Red;
         int i_Green;
         int i_Blue;

         d_Red   = m_pc_RedOutBuffer   ->get(i_X,i_Y);
         d_Green = m_pc_GreenOutBuffer ->get(i_X,i_Y);
         d_Blue  = m_pc_BlueOutBuffer  ->get(i_X,i_Y);
         i_Red   = (int) (d_Red   * 255.0);
         i_Green = (int) (d_Green * 255.0);
         i_Blue  = (int) (d_Blue  * 255.0);
         if(i_Red> 255)
         {
            i_Red = 255;
         }
         else if(i_Red < 0)
         {
            i_Red = 0;
         }
         
         if(i_Green> 255)
         {
            i_Green = 255;
         }
         else if(i_Green < 0)
         {
            i_Green = 0;
         }

         if(i_Blue> 255)
         {
            i_Blue = 255;
         }
         else if(i_Blue < 0)
         {
            i_Blue = 0;
         }


         p_Pixel = PIXEL_MAKE(i_Red,i_Green,i_Blue);         
         pc_Buffer->Set(i_X,i_Y,p_Pixel);

      } // for x
   } // for y

   delete m_pc_RedInBuffer;
   delete m_pc_GreenInBuffer;
   delete m_pc_BlueInBuffer;

   delete m_pc_RedOutBuffer ;
   delete m_pc_GreenOutBuffer ;
   delete m_pc_BlueOutBuffer ;
   
   m_pc_RedInBuffer    = NULL;
   m_pc_GreenInBuffer  = NULL;
   m_pc_BlueInBuffer   = NULL;

   m_pc_RedOutBuffer   = NULL;
   m_pc_GreenOutBuffer = NULL;
   m_pc_BlueOutBuffer  = NULL;

} // ConvertFromDsp()

//===========================================================================
//
//===========================================================================

void XenoHandler::StartFilter(QString qs_KernelName)
{
   unsigned int i_PixelNumber;
   int          i_X;
   int          i_Y;

   i_PixelNumber = 0;

   m_pc_KernelRed   = m_c_Kernelmanager.resolve((const QString &) qs_KernelName);
   m_pc_KernelGreen = m_c_Kernelmanager.resolve((const QString &) qs_KernelName);
   m_pc_KernelBlue  = m_c_Kernelmanager.resolve((const QString &) qs_KernelName);

     
   // are kernals available ??
   if(   m_pc_KernelRed   == NULL
      || m_pc_KernelGreen == NULL
      || m_pc_KernelRed   == NULL)
   {
      // no, do nothing
      return ;
   }

   // for every pixel
   for(i_Y=0;i_Y<m_i_Height;i_Y++)
   {
      for(i_X=0; i_X <m_i_Width;i_X ++)
      {
         // pixel count up
         ++i_PixelNumber;

         // apply the kernel
         m_pc_KernelRed  ->filter(*m_pc_RedInBuffer,*m_pc_RedOutBuffer,i_X,i_Y);
         m_pc_KernelGreen->filter(*m_pc_GreenInBuffer,*m_pc_GreenOutBuffer,i_X,i_Y);
         m_pc_KernelBlue ->filter(*m_pc_BlueInBuffer,*m_pc_BlueOutBuffer,i_X,i_Y);

      } // for x
   } // for y

} // StartFilter()

//===========================================================================
//
//===========================================================================

extern unsigned int g_ui_KernelMemoryDataLength;
extern unsigned char g_uc_KernelMemoryData[];

void XenoHandler::ExtractKernels(QString qs_Path)
{
   FILE *fp_Outfile;
   QString qs_Outputname;
   QString qs_Command;

   // step 1 - write tgz file out of memory
   qs_Outputname = qs_Path + "/kernels.tgz";
	// fprintf(stderr,"Trying to open %s\n",(const char *) qs_Outputname);

   fp_Outfile = fopen((const char *) qs_Outputname,
                      "wb");

   if(fp_Outfile == NULL)
   {
      return;
   }

   fwrite( g_uc_KernelMemoryData,
           g_ui_KernelMemoryDataLength,
           1,
           fp_Outfile);
   
   fclose (fp_Outfile);

   // step 2 - uncompress the tgz file
   
   qs_Command.sprintf(" (cd %s ; tar -xzf kernels.tgz)",
                     (const char *) qs_Path);

   // fprintf(stderr,"Command <%s>\n", (const char *) qs_Command);

   system(qs_Command);                     
   
} // ExtractKernels()


//===========================================================================
//
//===========================================================================








