//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//
// Parts of this file is taken from the xenomorph program by Ewald Krmer.
// ewald72@yahoo.com.
// Original is at http://xenomorph.sourceforge.net/
// The orignal program is also GPL, so there should be no right problems.
//
//---------------------------------------------------------------------------
// Module: Xenolib.c
//---------------------------------------------------------------------------
// Purpose: Basic xenomorph filters.
//---------------------------------------------------------------------------



#include "XenoLib.h"

//===========================================================================
//
//===========================================================================

XenoDSP2D::XenoDSP2D(unsigned int i_Width, unsigned int i_Height)
{
   pd_Data=NULL;
   resize(i_Width,i_Height);
} // XenoDSP2D()

//===========================================================================
//
//===========================================================================

XenoDSP2D::~XenoDSP2D()
{
   if(pd_Data != NULL)
   {
      delete pd_Data;
      pd_Data = NULL;
   }
} // ~XenoDSP2D()

//===========================================================================
//
//===========================================================================

unsigned int XenoDSP2D::width ()
{
   return m_i_Width;
}

//===========================================================================
//
//===========================================================================

unsigned int XenoDSP2D::height()
{
   return m_i_Height;
}


//===========================================================================
//
//===========================================================================

// holen minimum und maximum aus dem gesamten buffer
void XenoDSP2D::getMinMax(int &i_Min, int &i_Max) 
{
   unsigned int i;
   
   if (   (m_i_Width>0)
       && (m_i_Height>0))
   {
      
      i_Min=(int)pd_Data[0];
      i_Max=(int)pd_Data[0];

      for (i=0;i<m_i_NumberOfPixel;++i)
      {
         if ((int)pd_Data[i]<i_Min)
         {
            i_Min=(int)pd_Data[i];
         }
         if ((int)pd_Data[i]>i_Max)
         {
            i_Max=(int)pd_Data[i];
         }
      }
   }
} // getMinMax()

//===========================================================================
//
//===========================================================================

// buffer neu dimensionieren
void XenoDSP2D::resize(unsigned int i_Width, unsigned int i_Height) 
{
   if (pd_Data!=NULL)
   {
      delete pd_Data;
   }
   
   m_i_Width =i_Width;
   m_i_Height=i_Height;
   
   if (   (m_i_Width >0)
       && (m_i_Height>0))
   {
      m_i_NumberOfPixel=m_i_Width*m_i_Height;
      pd_Data = new double[ m_i_NumberOfPixel ];
   }
   else
   {
      m_i_NumberOfPixel = 0;
      pd_Data           = NULL;
   }
} // resize()

//===========================================================================
//
//===========================================================================

// i-te dateneelement holen
double& XenoDSP2D::operator[](unsigned int i)
{
   return(pd_Data[i]);
}

//===========================================================================
//
//===========================================================================

// 2d einsetzen
void XenoDSP2D::set(int i_X, int i_Y, double d_Value)
{
   if (   (i_X>=0)
       && (i_Y>=0)
       && (i_X<(int)m_i_Width)
       && (i_Y<(int)m_i_Height))
   {
      pd_Data[i_X + m_i_Width*i_Y] = d_Value;
   }
}

//===========================================================================
//
//===========================================================================

// 2d holen
double XenoDSP2D::get(int i_X, int i_Y)
{
   if (    (i_X>=0)
        && (i_Y>=0)
        && (i_X<(int)m_i_Width)
        && (i_Y<(int)m_i_Height))
   {
      return(pd_Data[i_X + m_i_Width*i_Y]);
   }
   
   return 0;
}

//===========================================================================
//
//===========================================================================

// auf -1 setzen
void XenoDSP2D::resetPixelCounter()
{
   m_ul_PixelCounter = (unsigned int)-1;
}

//===========================================================================
//
//===========================================================================

// nchsten datenpunkt aus buffer holen
double XenoDSP2D::getNextPixel()
{
   return  pd_Data[ ++m_ul_PixelCounter];
}

//===========================================================================
//
//===========================================================================

// nchsten datenpunkt einschreiben
void   XenoDSP2D::setNextPixel(double d_Value)
{
   pd_Data[++m_ul_PixelCounter] = d_Value;
}

//===========================================================================
//
//===========================================================================

// alle datenpunkte mit dem wert v addieren
void XenoDSP2D::operator +=(double d_Value)
{
   unsigned int i;
   
   for (i=0; i<m_i_Width*m_i_Height; ++i)
   {
      pd_Data[i] += d_Value;
   }
}

//===========================================================================
//
//===========================================================================

// alle datenpunkte mit dem Wert v multiplizieren
void XenoDSP2D::operator *=(double d_Value)
{
   unsigned int i;

   for (i=0; i<m_i_Width*m_i_Height; ++i)
   {
      pd_Data[i] *= d_Value;
   }
}


//######################################################################
//######################################################################



XenoKERNEL2D::XenoKERNEL2D(XenoKERNEL_MANAGER *pc_Manager)
{
   morph_on=false;
   rank_on =false;	
   morph_select=0.0;
   rank_scale  =0.0;
   morph_select1=0;
   morph_weight1=0.0;
   morph_select2=0;
   morph_weight2=0.0;

   manager=pc_Manager;

   ch=new XenoCACHE2D();
   m_qs_Filename = "";
   m_qs_Name     = "";
   // fprintf(stderr,"checkin EMPTY\n");
   manager->checkin(m_qs_Name,this);
}

//===========================================================================
//
//===========================================================================


XenoKERNEL2D::XenoKERNEL2D(QString qs_Name, XenoKERNEL_MANAGER *pc_Manager)
{
   rank_on =false;
   morph_on=false;	
   morph_select=0.0;
   rank_scale  =0.0;
   morph_select1=0;
   morph_weight1=0.0;
   morph_select2=0;
   morph_weight2=0.0;

   manager=pc_Manager;
   ch=new XenoCACHE2D();
   m_qs_Filename = qs_Name;
   m_qs_Name     = "";
   
   // fprintf(stderr,"checkin <%s>\n",(const char *) m_qs_Name);   
   manager->checkin(m_qs_Name,this);
}

//===========================================================================
//
//===========================================================================

XenoKERNEL2D::~XenoKERNEL2D()
{
   delete ch;

   for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
        p!=FDescriptor.end();
        ++p)
   {
      for (set<XenoKFactor*>::iterator q=p->first.begin();
           q!=p->first.end();
           ++q)
      {
         delete *q;
      }
   }

}

//===========================================================================
//
//===========================================================================

double XenoKERNEL2D::getFilter(XenoDSP2D &source, int x, int y)
{
   long l_Position=x+source.m_i_Width*y;
   double d_Value ;

   if (   (x>=0)
       && (y>=0)
       && (x<(int)source.m_i_Width)
       && (y<(int)source.m_i_Height)
       && (ch->get(source,l_Position,d_Value)))
   {
      return d_Value;
   }
   
   if (morph_on)
   {
      d_Value=getMorph(source,x,y);
   }
   else	if (rank_on )
   {
      d_Value=getRank (source,x,y);
   }
   else
   {
      d_Value=getConv (source,x,y);
   }
   
   if (   (x>=0)
       && (y>=0)
       && (x<(int)source.m_i_Width)
       && (y<(int)source.m_i_Height))
   {
      ch->set(source,l_Position,d_Value);
   }
   
   return d_Value;
} // getFilter()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::filter(XenoDSP2D &source, XenoDSP2D &target, int x, int y)
{
   target.set(x,y,getFilter(source,x,y));
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::filter(XenoDSP2D &source, XenoDSP2D &target)
{
   for (int x=0;x<(int)source.m_i_Width;++x)
   {
      for (int y=0;y<(int)source.m_i_Height;++y)
      {
         filter(source,target,x,y);
     }
   }
}

//===========================================================================
//
//===========================================================================

double XenoKERNEL2D::getConv(XenoDSP2D &source, int x, int y)
{
   double r=0;

   for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
        p!=FDescriptor.end();
        ++p)
   {
      if (p->first.size()>0)
      {
         double prod=p->second;
         for (set<XenoKFactor*>::iterator q=p->first.begin();
              q!=p->first.end();
              ++q)
         {
            prod *= (*q)->get(source,x,y);
         }
         r+=prod;
      }
   }
   return r;
}

//===========================================================================
//
//===========================================================================

double XenoKERNEL2D::getMorph(XenoDSP2D &source, int x, int y)
{
   vector<double> result;
   for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
        p!=FDescriptor.end();
        ++p)
   {
      if (p->first.size()>0)
      {
         double prod=p->second;
         for (set<XenoKFactor*>::iterator q=p->first.begin();
              q!=p->first.end();
              ++q)
         {
            prod *= (*q)->get(source,x,y);
         }
         result.insert(result.end(),prod);
      }
   }

   
   double selection=result[0];
   if (morph_select==1.0)
   {
      for (vector<double>::iterator p=result.begin();
           p!=result.end();
           ++p)
      {
         if (selection<(*p))
         {
            selection=(*p);
         }
      }
   }
   else if (morph_select==0.0)
   {
      for (vector<double>::iterator p=result.begin();
           p!=result.end();
           ++p)
      {
         if (selection>(*p))
         {
            selection=(*p);
         }
      }
   }
   else
   {
      sort(result.begin(),result.end());
      if (morph_weight2==0)
      {
         selection=result[morph_select1];
      }
      else
      {
         selection=  result[morph_select1]*morph_weight1
                   + result[morph_select2]*morph_weight2;
      }
   }

   return selection;
} // getMorph()

//===========================================================================
//
//===========================================================================

double XenoKERNEL2D::getRank(XenoDSP2D &source, int x, int y)
{
   vector<double> result;
   double         d_Value;
   int            n;


   d_Value =0;
   n=0;
   
   for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
        p!=FDescriptor.end();
        ++p)
   {
      if (p->first.size()>0)
      {
         double prod=p->second;
         
         for ( set<XenoKFactor*>::iterator q=p->first.begin();
               q!=p->first.end();
               ++q)
         {
            prod *= (*q)->get(source,x,y);
         }
         result.insert(result.end(),prod);
      }
   }

   
   if (result.size()>0)
   {
      d_Value=result[0];
   }

   
   for (unsigned int i=0;i<result.size();++i)
   {
      if (result[i]>d_Value)
      {
         d_Value=result[i];
         ++n;
      }
   }
   
   d_Value = ((double)n) * rank_scale;
   return d_Value ;
} // getRank()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::convolve(XenoDSP2D &source,
                            XenoDSP2D &target,
                            int x,
                            int y)
{
   target.set(x,y,getConv(source,x,y));
   
} // convolve()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::convolve(XenoDSP2D &source,
                            XenoDSP2D &target)
{
   target.resize(source.m_i_Width,source.m_i_Height);
   
   for (int x=0;x<(int)source.m_i_Width;++x)
   {
      for (int y=0;y<(int)source.m_i_Height;++y)
      {
         convolve(source,target,x,y);
      }
   }
} // convolve()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::morph(XenoDSP2D &source,
                         XenoDSP2D &target,
                         int x,
                         int y)
{
   target.set(x,y,getMorph(source,x,y));
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::morph(XenoDSP2D &source,
                         XenoDSP2D &target)
{
   target.resize(source.m_i_Width,source.m_i_Height);
   
   for (int x=0;x<(int)source.m_i_Width;++x)
   {
      for (int y=0;y<(int)source.m_i_Height;++y)
      {
         morph(source,target,x,y);
      }
   }
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::rank(XenoDSP2D &source,
                        XenoDSP2D &target,
                        int x,
                        int y)
{
   target.set(x,y,getRank(source,x,y));
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::rank(XenoDSP2D &source,
                        XenoDSP2D &target)
{
   target.resize(source.m_i_Width,source.m_i_Height);
   
   for (int x=0;x<(int)source.m_i_Width;++x)
   {
      for (int y=0;y<(int)source.m_i_Height;++y)
      {
         rank(source,target,x,y);
      }
   }
}

//===========================================================================
//
//===========================================================================

QString XenoKERNEL2D::getName()
{
   return m_qs_Name;
} // getName()

//===========================================================================
//
//===========================================================================

QString XenoKERNEL2D::getFileName()
{
   return m_qs_Filename;
} // getFilename

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::insert(int x, int y)
{
   buffer.ins(x,y);
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::weight(double v)
{
   set<XenoKFactor*> fdset;
   
   XenoKFactor *fkt=new XenoKFactor("");

   *fkt=buffer;
   fdset.insert(fkt);

   FDescriptor.insert(map<set<XenoKFactor*>,double>::value_type(fdset,v));

   buffer.clear();
} // weight()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::clear()
{
   FDescriptor.clear();
   morph_on      = false;
  
   morph_select  = 0.0;
   morph_select1 = 0;
   morph_weight1 = 0.0;
   morph_select2 = 0;
   morph_weight2 = 0.0;
} // clear()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::load(QString qs_Filename)
{
   ifstream ifs((const char *) qs_Filename,ios::in);
   if (ifs)
   {
      clear();
      while (!ifs.eof())
      {
         char command=0;
         ifs >> command;

         if (command=='V')
         {
            double weight;
            ifs >> weight;
            XenoKFactor *kf=new XenoKFactor("");

            while ((ifs.peek()!='\n') && (!ifs.eof()))
            {
               int x,y;
               ifs >> x;
               ifs >> y;
               kf->ins(x,y);
            }
            set<XenoKFactor*> kfset;
            kfset.insert(kf);
            FDescriptor.insert(map<set<XenoKFactor*>,double>::value_type(kfset,weight));
         }
         if (command=='K')
         {
            QString qs_Name;
            string  s_Name;
            double weight;
            ifs >> weight;
            ifs >>  s_Name;

          
            qs_Name = QString( s_Name.c_str() );
            XenoKFactor *kf=new XenoKFactor(qs_Name);
            while ((ifs.peek()!='\n') && (!ifs.eof()))
            {
               int x,y;
               ifs >> x;
               ifs >> y;
               kf->ins(x,y);
            }
            set<XenoKFactor*> kfset;
            kfset.insert(kf);
            FDescriptor.insert(map<set<XenoKFactor*>,double>::value_type(kfset,weight));
         }
         if (command=='P')
         {
            double weight;
            ifs >> weight;
            int x,y;

            set<XenoKFactor*> kfset;
            XenoKFactor *kf=new XenoKFactor("");
            
            while ((ifs.peek()!='\n') && (!ifs.eof()))
            {
               string token;
               QString qs_Token;
               ifs >> token;
               
               if (sscanf(token.c_str(),"%d",&x)==1)
               {
                  ifs >> token;
                  if (sscanf(token.c_str(),"%d",&y)==1)
                  {
                     kf->ins(x,y);
                  }
               }
               else
               {
                  if (kf->size()>0)
                  {
                     kfset.insert(kf);
                  }
                  else
                  {
                     delete kf;
                  }
                  qs_Token =QString( token.c_str());
                  kf=new XenoKFactor(qs_Token);
               }
            }
            if (kf->size()>0)
            {
               kfset.insert(kf);
            }
            
            FDescriptor.insert(map<set<XenoKFactor*>,double>::value_type(kfset,weight));
         }
         if (command=='N')
         {
            QString qs_Name;
            string name;
            ifs >> name;
            qs_Name = QString( name.c_str());
            // fprintf(stderr,"Name of the Kernel is %s\n", (const char *) qs_Name);
            m_qs_Name = qs_Name;            
         }
         if (command=='M')
         {
            rank_on =false;
            morph_on=true;
            ifs >> morph_select;
         }
         if (command=='R')
         {
            morph_on=false;
            rank_on =true;
            ifs >> rank_scale;
         }
      } // while not eof
      
      ifs.close();
      clean();

      if (morph_on)
      {
         calcSel();
      }
   } // if ifs

} // load()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::save(const char* filename)
{
   ofstream ofs(filename,ios::out);
   if (ofs)
   {
      ofs << "N" << ((const char *) m_qs_Name) << endl;
      
      for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
           p!=FDescriptor.end();
           ++p)
      {
         ofs << "P " << p->second;
         for (set<XenoKFactor*>::iterator q=p->first.begin();
              q!=p->first.end();
              ++q)
         {
            (*q)->save(ofs);
         }
         ofs << endl;
      }
      if (morph_on)
      {
         ofs << "M " << morph_select << endl;
      }
      if (rank_on )
      {
         ofs << "R " << rank_scale   << endl;
      }
      ofs.close();

   }
} // save()

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::calcSel()
{
   double pos=1.0+((double)(FDescriptor.size()-1))*morph_select;
   
   if (pos==(int)pos)
   {
      morph_select1=(int)pos-1;
      morph_weight1=1.0;
      morph_select2=(int)pos;
      morph_weight2=0.0;
   }
   else
   {
      morph_select1=(int)pos-1;
      morph_weight1=1.0-pos+(double)((int)pos);
      morph_select2=(int)pos;
      morph_weight2=1.0-morph_weight1;
   }
} // calcSel()


//===========================================================================
//###########################################################################
//###########################################################################
//###########################################################################
//===========================================================================

void XenoKERNEL_MANAGER::clearCaches()
{
   unsigned int ui;
   for(ui=0;ui<m_l_KernelList.GetCount();ui++)
   {
      XenoKERNEL2D* pc_Kernel;
      pc_Kernel = m_l_KernelList.GetIndex(ui);
      pc_Kernel->clearCache();
   }
   
} // clearCaches()

//===========================================================================
//
//===========================================================================

void XenoKERNEL_MANAGER::checkin(QString &qs_Name, XenoKERNEL2D* pc_Kernel)
{
   m_l_KernelList.AddTail(pc_Kernel);
   changed=true;
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL_MANAGER::checkout(QString &qs_Name)
{
   kernels.erase(qs_Name);
   changed=true;
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL_MANAGER::resolveAll()
{
   unsigned int ui;
   if (changed)
   {
      changed=false;
      for(ui=0;ui<m_l_KernelList.GetCount();ui++)
      {
         XenoKERNEL2D* pc_Kernel;
         pc_Kernel = m_l_KernelList.GetIndex(ui);
         pc_Kernel->resolve();
      }
   }
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL_MANAGER::deleteAll()
{
   unsigned int ui;

   changed=true;

   for(ui=0;ui<m_l_KernelList.GetCount();ui++)
   {
      XenoKERNEL2D* pc_Kernel;
      pc_Kernel = m_l_KernelList.GetIndex(ui);
      delete pc_Kernel;
   }
   kernels.clear();
}

//===========================================================================
//
//===========================================================================

XenoKERNEL2D* XenoKERNEL_MANAGER::resolve(QString qs_Name)
{
   unsigned int ui;
   for(ui=0;ui<m_l_KernelList.GetCount();ui++)
   {
      XenoKERNEL2D* pc_Kernel;
      pc_Kernel = m_l_KernelList.GetIndex(ui);

/*
fprintf(stderr,"%d Name <%s>, Filename <%s>\n",
         ui,
         (const char *) pc_Kernel->getName(),
         (const char *) pc_Kernel->getFileName()
         );
*/

      if(pc_Kernel->getName().compare(qs_Name) == 0)
      {
         return pc_Kernel;
      }
      if(pc_Kernel->getFileName().compare(qs_Name) == 0)
      {
         return pc_Kernel;
      }
   }
  
   fprintf(stderr,"resolve(%s): NOTHING FOUND!!!\n",(const char *) qs_Name );
   return NULL ;
}

//===========================================================================
//###########################################################################
//###########################################################################
//###########################################################################
//===========================================================================


void XenoKERNEL2D::resolve()
{
   for (map<set<XenoKFactor*>,double>::iterator p=FDescriptor.begin();
        p !=FDescriptor.end();
        ++p)
   {
      for (set<XenoKFactor*>::iterator q=p->first.begin();
           q!=p->first.end();
           ++q)
      {
         
         (*q)->res(manager);
      }
   }
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::clearCache()
{
   if (ch!=NULL)
   {
      ch->clear();
   }
}

//===========================================================================
//
//===========================================================================

void XenoKERNEL2D::clean()
{
   if (!morph_on)
   {
      set<set<XenoKFactor*> >toDelete;
      
      map<set<XenoKFactor*>,double> fd=FDescriptor;
      
      FDescriptor.clear();

      for (map<set<XenoKFactor*>,double>::iterator p=fd.begin();p!=fd.end();++p)
      {
         if (p->second!=0.0)
         {
            FDescriptor.insert(*p);
         }
         else
         {
            toDelete.insert(p->first);
         }
      }
      
      for (set<set<XenoKFactor*> >::iterator p=toDelete.begin();p!=toDelete.end();++p)
      {
         for (set<XenoKFactor*>::iterator q=p->begin();q!=p->end();++q)
         {
            delete *q;
         }
      }
   }
}

//===========================================================================
//
//===========================================================================

//###########################################################################
//###########################################################################
//###########################################################################


XenoKFactor::XenoKFactor()
{
   name="";
   krn=NULL;
}

//===========================================================================
//
//===========================================================================

XenoKFactor::XenoKFactor(QString qs_Name)
{
   name= qs_Name;
   krn=NULL;
}

//===========================================================================
//
//===========================================================================

void XenoKFactor::ins(int x, int y)
{
   volterra.insert(make_pair(x,y));
}

//===========================================================================
//
//===========================================================================

void XenoKFactor::clear()
{
   volterra.clear();
}

//===========================================================================
//
//===========================================================================

double XenoKFactor::get(XenoDSP2D &source, int x, int y)
{
   double prod=1.0;
   
   if (krn==NULL)
   {
      for (multiset<pair<int,int> >::iterator p=volterra.begin();
           p!=volterra.end();
           ++p)
      {
         prod*=source.get(x+p->first,y+p->second);
      }
   }
   else
   {
      for (multiset<pair<int,int> >::iterator p=volterra.begin();
           p!=volterra.end();
           ++p)
      {
         prod*=krn->getFilter(source,x+p->first,y+p->second);
      }
   }
   return(prod);
}

//===========================================================================
//
//===========================================================================

void XenoKFactor::res(XenoKERNEL_MANAGER *m)
{
   if (name!="")
   {
      krn=m->resolve(name);
   }
   else
   {
      krn=NULL;
   }
}

//===========================================================================
//
//===========================================================================

void XenoKFactor::save(ofstream &ofs)
{
   ofs << " " << name;
   for (multiset<pair<int,int> >::iterator p=volterra.begin();p!=volterra.end();++p)
   {
      ofs << " " << p->first << " " << p->second;
   }
}

//===========================================================================
//
//===========================================================================

int XenoKFactor::size()
{
   return(volterra.size());
}

//===========================================================================
//###########################################################################
//###########################################################################
//###########################################################################
//===========================================================================

void XenoCACHE2D::set(XenoDSP2D &source, long pos, double v)
{
   if (cdata.count(&source)==0)
   {

      double *d= new double[ source.m_i_NumberOfPixel ];
      
      bool   *b= new bool  [source.m_i_NumberOfPixel ] ;
      
      for (long i=0;i<(long)source.m_i_NumberOfPixel;++i)
      {
         b[i]=false;
      }
      
      cdata.insert(map<XenoDSP2D*,pair<bool*,double*> >::value_type(&source,make_pair(b,d)));
   }
   
   if (    (pos>0)
        && (pos<(long)source.m_i_NumberOfPixel))
   {
      pair<bool*,double*> *p=&cdata[&source];
      p->first[pos]=true;
      p->second[pos]=v;
   }
} // set()

//===========================================================================
//
//===========================================================================

bool XenoCACHE2D::get(XenoDSP2D &source, long pos, double &v)
{
   if (cdata.count(&source)!=0)
   {
      if (   (pos>0)
          && (pos<(long)source.m_i_NumberOfPixel))
      {
         pair<bool*,double*> *p=&cdata[&source];
         v=p->second[pos];
         
         return p->first[pos];
      }
   }

   return false;
}

//===========================================================================
//
//===========================================================================

XenoCACHE2D::XenoCACHE2D()
{
}

//===========================================================================
//
//===========================================================================

XenoCACHE2D::~XenoCACHE2D()
{
   clear();
}

//===========================================================================
//
//===========================================================================

void XenoCACHE2D::clear()
{

   for (map<XenoDSP2D*,pair<bool*,double*> >::iterator p=cdata.begin();
        p!=cdata.end();
        ++p)
   {
      delete p->second.first;
      delete p->second.second;
   }
   cdata.clear();
}

//===========================================================================
//
//===========================================================================



